@@ -31,29 +31,29 @@ my_readlink()
 }
 
 no_malloc=""
-no_bfd=" -DUSE_BFD=FALSE"
-no_ah=" -DUSE_ACTIVEHARMONY=FALSE"
-no_ompt=" -DUSE_OMPT=FALSE"
-no_papi=" -DUSE_PAPI=FALSE"
-no_mpi=" -DUSE_MPI=FALSE"
-no_otf=" -DUSE_OTF2=FALSE"
-no_tau=" -DUSE_TAU=FALSE"
-yes_malloc=" -DUSE_JEMALLOC=TRUE"
-yes_bfd=" -DUSE_BFD=TRUE"
-yes_ah=" -DUSE_ACTIVEHARMONY=TRUE -DACTIVEHARMONY_ROOT=/usr/local/activeharmony/4.6 -DUSE_PLUGINS=TRUE"
-yes_otf=" -DUSE_OTF2=TRUE -DOTF2_ROOT=/usr/local/otf2/2.1"
-yes_ompt=" -DUSE_OMPT=TRUE -DOMPT_ROOT=/usr/local/ompt/5.0"
-#yes_mpi=" -DUSE_MPI=TRUE -DCMAKE_C_COMPILER=mpicc -DCMAKE_CXX_COMPILER=mpicxx"
-yes_mpi=" -DUSE_MPI=TRUE"
-yes_papi=" -DUSE_PAPI=TRUE -DPAPI_ROOT=/usr"
-yes_tau=" -DUSE_TAU=TRUE "
+no_bfd=" -DAPEX_WITH_BFD=FALSE"
+no_ah=" -DAPEX_WITH_ACTIVEHARMONY=FALSE"
+no_ompt=" -DAPEX_WITH_OMPT=FALSE"
+no_papi=" -DAPEX_WITH_PAPI=FALSE"
+no_mpi=" -DAPEX_WITH_MPI=FALSE"
+no_otf=" -DAPEX_WITH_OTF2=FALSE"
+no_tau=" -DAPEX_WITH_TAU=FALSE"
+yes_malloc=" -DAPEX_WITH_JEMALLOC=TRUE"
+yes_bfd=" -DAPEX_WITH_BFD=TRUE"
+yes_ah=" -DAPEX_WITH_ACTIVEHARMONY=TRUE -DACTIVEHARMONY_ROOT=/usr/local/activeharmony/4.6 -DAPEX_WITH_PLUGINS=TRUE"
+yes_otf=" -DAPEX_WITH_OTF2=TRUE -DOTF2_ROOT=/usr/local/otf2/2.1"
+yes_ompt=" -DAPEX_WITH_OMPT=TRUE -DOMPT_ROOT=/usr/local/ompt/5.0"
+#yes_mpi=" -DAPEX_WITH_MPI=TRUE -DCMAKE_C_COMPILER=mpicc -DCMAKE_CXX_COMPILER=mpicxx"
+yes_mpi=" -DAPEX_WITH_MPI=TRUE"
+yes_papi=" -DAPEX_WITH_PAPI=TRUE -DPAPI_ROOT=/usr"
+yes_tau=" -DAPEX_WITH_TAU=TRUE "
 if [ "$host" == "delphi" ] ; then
-  yes_bfd=" -DUSE_BFD=TRUE -DBFD_ROOT=/usr/local/packages/binutils/2.27"
-  yes_malloc=" -DUSE_JEMALLOC=TRUE -DJEMALLOC_ROOT=/usr/local/packages/jemalloc/5.0.1-gcc"
-  yes_ah=" -DUSE_ACTIVEHARMONY=TRUE -DACTIVEHARMONY_ROOT=/usr/local/packages/activeharmony/4.6.0-gcc -DUSE_PLUGINS=TRUE"
-  yes_otf=" -DUSE_OTF2=TRUE -DOTF2_ROOT=/usr/local/packages/otf2/2.1"
-  yes_ompt=" -DUSE_OMPT=TRUE -DOMPT_ROOT=/usr/local/packages/llvm-openmp/2020-03-25"
-  yes_papi=" -DUSE_PAPI=TRUE -DPAPI_ROOT=/usr/local/packages/papi/5.6.0"
+  yes_bfd=" -DAPEX_WITH_BFD=TRUE -DBFD_ROOT=/usr/local/packages/binutils/2.27"
+  yes_malloc=" -DAPEX_WITH_JEMALLOC=TRUE -DJEMALLOC_ROOT=/usr/local/packages/jemalloc/5.0.1-gcc"
+  yes_ah=" -DAPEX_WITH_ACTIVEHARMONY=TRUE -DACTIVEHARMONY_ROOT=/usr/local/packages/activeharmony/4.6.0-gcc -DAPEX_WITH_PLUGINS=TRUE"
+  yes_otf=" -DAPEX_WITH_OTF2=TRUE -DOTF2_ROOT=/usr/local/packages/otf2/2.1"
+  yes_ompt=" -DAPEX_WITH_OMPT=TRUE -DOMPT_ROOT=/usr/local/packages/llvm-openmp/2020-03-25"
+  yes_papi=" -DAPEX_WITH_PAPI=TRUE -DPAPI_ROOT=/usr/local/packages/papi/5.6.0"
 fi
 
 # set defaults
