@@ -1,7 +1,10 @@
-//  Copyright (c) 2014 University of Oregon
-//
-//  Distributed under the Boost Software License, Version 1.0. (See accompanying
-//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+/*
+ * Copyright (c) 2014-2021 Kevin Huck
+ * Copyright (c) 2014-2021 University of Oregon
+ *
+ * Distributed under the Boost Software License, Version 1.0. (See accompanying
+ * file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+ */
 
 /*  Copyright (c) 2014 University of Oregon
  *
@@ -20,6 +23,8 @@
  * The C API is required for HPX5 support.
  */
 
+#pragma once
+
 #ifndef APEX_H
 #define APEX_H
 
@@ -79,7 +84,7 @@ APEX_EXPORT const char * apex_dump(bool reset);
  \return No return value.
  \sa @ref apex_init
  */
-APEX_EXPORT void apex_finalize();
+APEX_EXPORT void apex_finalize(void);
 
 /**
  \brief Cleanup APEX.
@@ -93,7 +98,7 @@ APEX_EXPORT void apex_finalize();
  \return No return value.
  \sa @ref apex_finalize
  */
-APEX_EXPORT void apex_cleanup();
+APEX_EXPORT void apex_cleanup(void);
 
 /*
  * Functions for starting, stopping timers
@@ -118,7 +123,7 @@ APEX_EXPORT void apex_cleanup();
  \sa @ref apex_stop, @ref apex_resume, @ref apex_yield
  */
 APEX_EXPORT apex_profiler_handle apex_start(apex_profiler_type type,
-    void * identifier);
+    const void * identifier);
 
 /**
  \brief Start a timer.
@@ -140,7 +145,7 @@ APEX_EXPORT apex_profiler_handle apex_start(apex_profiler_type type,
  \sa @ref apex_stop, @ref apex_resume, @ref apex_yield
  */
 APEX_EXPORT apex_profiler_handle apex_start_guid(apex_profiler_type type,
-    void * identifier, uint64_t guid);
+    const void * identifier, uint64_t guid);
 
 /**
  \brief Stop a timer.
@@ -191,7 +196,7 @@ APEX_EXPORT void apex_yield(apex_profiler_handle profiler);
  \sa @ref apex_start, @ref apex_stop, @ref apex_yield
 */
 APEX_EXPORT apex_profiler_handle apex_resume(apex_profiler_type type,
-    void * identifier);
+    const void * identifier);
 
 /**
  \brief Resume a timer.
@@ -215,7 +220,7 @@ APEX_EXPORT apex_profiler_handle apex_resume(apex_profiler_type type,
  \sa @ref apex_start, @ref apex_stop, @ref apex_yield
 */
 APEX_EXPORT apex_profiler_handle apex_resume_guid(apex_profiler_type type,
-    void * identifier, uint64_t guid);
+    const void * identifier, uint64_t guid);
 
 /*
  * Functions for resetting timer values
@@ -234,7 +239,7 @@ APEX_EXPORT apex_profiler_handle apex_resume_guid(apex_profiler_type type,
  \return No return value.
  \sa @ref apex_get_profile
  */
-APEX_EXPORT void apex_reset(apex_profiler_type type, void * identifier);
+APEX_EXPORT void apex_reset(apex_profiler_type type, const void * identifier);
 
 /**
  \brief Set the thread state
@@ -277,7 +282,7 @@ APEX_EXPORT void apex_sample_value(const char * name, double value);
  \return No return value.
  */
 
-APEX_EXPORT void apex_new_task(apex_profiler_type type, void * identifier,
+APEX_EXPORT void apex_new_task(apex_profiler_type type, const void * identifier,
     uint64_t task_id);
 
 /**
@@ -406,7 +411,7 @@ APEX_EXPORT void apex_deregister_policy(apex_policy_handle * handle);
  \return The current profile for that timed function or sampled value.
  */
 APEX_EXPORT apex_profile * apex_get_profile(apex_profiler_type type,
-    void * identifier);
+    const void * identifier);
 
 /**
  \brief Get the current power reading
@@ -471,7 +476,7 @@ APEX_EXPORT int apex_setup_power_cap_throttling(void);      // initialize
  */
 
 APEX_EXPORT int apex_setup_timer_throttling(apex_profiler_type type,
-        void * identifier,
+        const void * identifier,
         apex_optimization_criteria_t criteria,
         apex_optimization_method_t method, unsigned long update_interval);
 
@@ -501,7 +506,7 @@ APEX_EXPORT int apex_setup_timer_throttling(apex_profiler_type type,
  */
 APEX_EXPORT int apex_setup_throughput_tuning(
         apex_profiler_type type,
-        void * identifier,
+        const void * identifier,
         apex_optimization_criteria_t criteria,
         apex_event_type event_type, int num_inputs, long ** inputs, long * mins,
         long * maxs, long * steps);
@@ -586,13 +591,14 @@ APEX_EXPORT uint64_t apex_hardware_concurrency (void);
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
 
-#define apex_macro(name, member_variable, type, default_value) \
+#define apex_macro(name, member_variable, type, default_value, description) \
 void apex_set_##member_variable (type inval); \
 type apex_get_##member_variable (void);
 FOREACH_APEX_OPTION(apex_macro)
+FOREACH_APEX_FLOAT_OPTION(apex_macro)
 #undef apex_macro
 
-#define apex_macro(name, member_variable, type, default_value) \
+#define apex_macro(name, member_variable, type, default_value, description) \
 void apex_set_##member_variable (type inval); \
 type apex_get_##member_variable (void);
 FOREACH_APEX_STRING_OPTION(apex_macro)
