@@ -1,10 +1,12 @@
-//  Copyright (c) 2014 University of Oregon
-//
-//  Distributed under the Boost Software License, Version 1.0. (See accompanying
-//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+/*
+ * Copyright (c) 2014-2021 Kevin Huck
+ * Copyright (c) 2014-2021 University of Oregon
+ *
+ * Distributed under the Boost Software License, Version 1.0. (See accompanying
+ * file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+ */
 
-#ifndef APEX_POLICIES_H
-#define APEX_POLICIES_H
+#pragma once
 
 #include "apex_export.h"
 
@@ -28,4 +30,3 @@ typedef enum {
 #define APEX_MIN_THREADS 1
 #define MAX_WINDOW_SIZE 3
 
-#endif // APEX_POLICIES_H
