@@ -1,7 +1,10 @@
-//  Copyright (c) 2014 University of Oregon
-//
-//  Distributed under the Boost Software License, Version 1.0. (See accompanying
-//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+/*
+ * Copyright (c) 2014-2021 Kevin Huck
+ * Copyright (c) 2014-2021 University of Oregon
+ *
+ * Distributed under the Boost Software License, Version 1.0. (See accompanying
+ * file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+ */
 
 #include "apex.hpp"
 #include "apex_api.hpp"
@@ -22,11 +25,12 @@
 
 #include "tau_listener.hpp"
 
-#define MAX_FUNCTIONS_IN_CHART 5
-
 using namespace std;
 
-std::vector<std::mutex*> _per_thread_mutex;
+std::vector<std::mutex*>& _per_thread_mutex() {
+    static std::vector<std::mutex*> ptm;
+    return ptm;
+}
 
 namespace apex {
 
@@ -53,9 +57,10 @@ concurrency_handler::~concurrency_handler () {
     for (auto tmp : _event_stack) {
         delete(tmp);
     }
-    for (auto tmp : _per_thread_mutex) {
-        delete(tmp);
-    }
+    // for some reason this causes shutdown crashes
+//    for (auto tmp : _per_thread_mutex()) {
+//        delete(tmp);
+//    }
     for (auto tmp : _states) {
         delete(tmp);
     }
@@ -92,7 +97,7 @@ bool concurrency_handler::_handler(void) {
     task_identifier func;
     if (tmp != nullptr && tmp->size() > 0) {
       {
-        std::lock_guard<std::mutex> l(*_per_thread_mutex[i]);
+        std::lock_guard<std::mutex> l(*(_per_thread_mutex()[i]));
         if (tmp->size() > 0) {
           func = tmp->top();
         } else {
@@ -121,10 +126,10 @@ bool concurrency_handler::_handler(void) {
   if (apex_options::use_tau()) {
     tau_listener::Tau_stop_wrapper("concurrency_handler::_handler");
   }
-#ifdef APEX_WITH_JUPYTER_SUPPORT
-  // update the timeout, if the user changed it.
-  set_timeout(apex_options::concurrency_period());
-#endif
+  if (apex_options::use_jupyter_support()) {
+    // update the timeout, if the user changed it.
+    set_timeout(apex_options::concurrency_period());
+  }
   return true;
 }
 
@@ -140,7 +145,7 @@ bool concurrency_handler::common_start(task_identifier *id) {
   if (!_terminate) {
     int i = thread_instance::get_id();
     stack<task_identifier>* my_stack = get_event_stack(i);
-    std::lock_guard<std::mutex> l(*_per_thread_mutex[i]);
+    std::lock_guard<std::mutex> l(*(_per_thread_mutex()[i]));
     my_stack->push(*id);
     return true;
   } else {
@@ -160,7 +165,7 @@ void concurrency_handler::common_stop(std::shared_ptr<profiler> &p) {
   if (!_terminate) {
     int i = thread_instance::get_id();
     stack<task_identifier>* my_stack = get_event_stack(i);
-    std::lock_guard<std::mutex> l(*_per_thread_mutex[i]);
+    std::lock_guard<std::mutex> l(*(_per_thread_mutex()[i]));
     if (!my_stack->empty()) {
       my_stack->pop();
     }
@@ -223,7 +228,7 @@ inline void concurrency_handler::add_thread(unsigned int tid) {
   std::lock_guard<std::mutex> l(_vector_mutex);
   while(_event_stack.size() <= tid) {
     _event_stack.push_back(new stack<task_identifier>);
-    _per_thread_mutex.push_back(new std::mutex());
+    _per_thread_mutex().push_back(new std::mutex());
   }
   _stack_count = _event_stack.size();
 }
@@ -250,7 +255,7 @@ bool path_has_suffix(const std::string &str)
 
 void concurrency_handler::output_samples(int node_id) {
   //cout << "HANDLER: writing samples " << endl;
-  cout << _states.size() << " samples seen:" << endl;
+  //cout << _states.size() << " samples seen:" << endl;
   ofstream myfile;
   stringstream datname;
   datname << apex_options::output_file_path();
@@ -286,7 +291,7 @@ void concurrency_handler::output_samples(int node_id) {
   for (vector<pair<task_identifier, int> >::iterator it=my_vec.begin();
     it!=my_vec.end(); ++it) {
     //if (top_x.size() < 15 && (*it).first != "APEX THREAD MAIN")
-    if (top_x.size() < MAX_FUNCTIONS_IN_CHART)
+    if (top_x.size() < (size_t)(apex_options::concurrency_max_timers()))
       top_x.insert((*it).first);
   }
 
