@@ -1,7 +1,10 @@
-//  Copyright (c) 2014 University of Oregon
-//
-//  Distributed under the Boost Software License, Version 1.0. (See accompanying
-//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+/*
+ * Copyright (c) 2014-2021 Kevin Huck
+ * Copyright (c) 2014-2021 University of Oregon
+ *
+ * Distributed under the Boost Software License, Version 1.0. (See accompanying
+ * file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+ */
 
 /* This is annoying and confusing.  We have to set a define so that the
  * HPX config file will be included, which will define APEX_HAVE_HPX
@@ -43,12 +46,13 @@ node_event_data::node_event_data(int node_id, int thread_id) {
 }
 
 sample_value_event_data::sample_value_event_data(int thread_id,
-    string counter_name, double counter_value) {
+    string counter_name, double counter_value, bool threaded) {
   this->event_type_ = APEX_SAMPLE_VALUE;
   this->is_counter = true;
   this->thread_id = thread_id;
   this->counter_name = new string(counter_name);
   this->counter_value = counter_value;
+  this->is_threaded = threaded;
 }
 
 sample_value_event_data::~sample_value_event_data() {
