@@ -1,13 +1,15 @@
-//  Copyright (c) 2014 University of Oregon
-//
-//  Distributed under the Boost Software License, Version 1.0. (See accompanying
-//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+/*
+ * Copyright (c) 2014-2021 Kevin Huck
+ * Copyright (c) 2014-2021 University of Oregon
+ *
+ * Distributed under the Boost Software License, Version 1.0. (See accompanying
+ * file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+ */
 
 /* Portable global constructor/destructor for GCC and MSVC.
  * I found this code here: http://www.gonwan.com/?p=8 */
 
-#ifndef GLOBAL_CONSTRUCTOR_DESTRUCTOR_H
-#define GLOBAL_CONSTRUCTOR_DESTRUCTOR_H
+#pragma once
 
 #include <stdlib.h>
 #if defined (_MSC_VER)
@@ -58,13 +60,3 @@
 #warning "Global constructors and destructors not defined!"
 #endif
 
-/*
-#ifdef HAS_CONSTRUCTORS
-extern "C" {
-DEFINE_CONSTRUCTOR(apex_init_static_void)
-DEFINE_DESTRUCTOR(apex_finalize_static_void)
-}
-#endif
-*/
-
-#endif // GLOBAL_CONSTRUCTOR_DESTRUCTOR_H
