@@ -1,7 +1,10 @@
-//  Copyright (c) 2014-2018 University of Oregon
-//
-//  Distributed under the Boost Software License, Version 1.0. (See accompanying
-//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+/*
+ * Copyright (c) 2014-2020 Kevin Huck
+ * Copyright (c) 2014-2020 University of Oregon
+ *
+ * Distributed under the Boost Software License, Version 1.0. (See accompanying
+ * file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+ */
 
 #if defined(APEX_HAVE_PROC)
 
@@ -34,6 +37,7 @@
 #define DATA_SIZE 512
 
 #include "tau_listener.hpp"
+#include "apex_dynamic.hpp"
 
 #ifdef APEX_HAVE_LM_SENSORS
 #include "sensor_data.hpp"
@@ -46,352 +50,13 @@
 
 using namespace std;
 
-namespace apex {
-
 #if defined(APEX_HAVE_PAPI)
+#include "proc_read_papi.cpp"
+#endif
 
-#include "papi.h"
-
-    static bool rapl_initialized = false;
-    static bool nvml_initialized = false;
-    static bool lms_initialized = false;
-    static int rapl_EventSet = PAPI_NULL;
-    static int nvml_EventSet = PAPI_NULL;
-    static int lms_EventSet = PAPI_NULL;
-    static std::vector<std::string> rapl_event_names;
-    static std::vector<std::string> nvml_event_names;
-    static std::vector<std::string> lms_event_names;
-    static std::vector<std::string> rapl_event_units;
-    static std::vector<std::string> nvml_event_units;
-    static std::vector<std::string> lms_event_units;
-    static std::vector<int> nvml_event_codes;
-    static std::vector<int> lms_event_codes;
-    static std::vector<int> rapl_event_data_type;
-    static std::vector<int> nvml_event_data_type;
-    static std::vector<int> lms_event_data_type;
-    static std::vector<double> rapl_event_conversion;
-    static std::vector<double> nvml_event_conversion;
-    static std::vector<double> lms_event_conversion;
-
-    typedef union {
-        long long ll;
-        double fp;
-    } event_result_t;
-
-    void initialize_papi_events(void) {
-        // get the PAPI components
-        int num_components = PAPI_num_components();
-        const PAPI_component_info_t *comp_info;
-        int retval = PAPI_OK;
-        // are there any components?
-        for (int component_id = 0 ; component_id < num_components ; component_id++) {
-            comp_info = PAPI_get_component_info(component_id);
-            if (comp_info == NULL) {
-                fprintf(stderr, "PAPI component info unavailable, no power measurements will be done.\n");
-                return;
-            }
-            // do we have the RAPL components?
-            if (strstr(comp_info->name, "rapl")) {
-                if (comp_info->num_native_events == 0) {
-                    fprintf(stderr, "PAPI RAPL component found, but ");
-                    fprintf(stderr, "no RAPL events found.\n");
-                    if (comp_info->disabled != 0) {
-                        fprintf(stderr, "%s.\n", comp_info->disabled_reason);
-                    }
-                } else {
-                    rapl_EventSet = PAPI_NULL;
-                    retval = PAPI_create_eventset(&rapl_EventSet);
-                    if (retval != PAPI_OK) {
-                        fprintf(stderr, "Error creating RAPL PAPI eventset.\n");
-                        fprintf(stderr, "PAPI error %d: %s\n", retval,
-                                PAPI_strerror(retval));
-                        return;
-                    }
-                    int code = PAPI_NATIVE_MASK;
-                    int event_modifier = PAPI_ENUM_FIRST;
-                    for ( int ii=0; ii< comp_info->num_native_events; ii++ ) {
-                        // get the event
-                        retval = PAPI_enum_cmp_event( &code, event_modifier, component_id );
-                        event_modifier = PAPI_ENUM_EVENTS;
-                        if ( retval != PAPI_OK ) {
-                            fprintf( stderr, "%s %d %s %d\n", __FILE__,
-                                    __LINE__, "RAPL PAPI_enum_cmp_event failed.", retval);
-                            fprintf(stderr, "PAPI error %d: %s\n", retval,
-                                    PAPI_strerror(retval));
-                            continue;
-                        }
-                        // get the event name
-                        char event_name[PAPI_MAX_STR_LEN];
-                        retval = PAPI_event_code_to_name( code, event_name );
-                        if (retval != PAPI_OK) {
-                            fprintf(stderr, "%s %d %s\n", __FILE__,
-                                    __LINE__, "RAPL PAPI_event_code_to_name failed");
-                            fprintf(stderr, "PAPI error %d: %s\n", retval,
-                                    PAPI_strerror(retval));
-                            continue;
-                        }
-                        // skip the counter events...
-                        if (strstr(event_name, "_CNT") != NULL) { continue; }
-                        // get the event info
-                        PAPI_event_info_t evinfo;
-                        retval = PAPI_get_event_info(code,&evinfo);
-                        if (retval != PAPI_OK) {
-                            fprintf(stderr, "%s %d %s\n", __FILE__,
-                                    __LINE__, "Error getting RAPL event info");
-                            fprintf(stderr, "PAPI error %d: %s\n", retval,
-                                    PAPI_strerror(retval));
-                            continue;
-                        }
-                        // get the event units
-                        char unit[PAPI_MAX_STR_LEN] = {0};
-                        strncpy(unit,evinfo.units,PAPI_MAX_STR_LEN);
-                        // save the event info
-                        //printf("Found event '%s (%s)'\n", event_name, unit);
-                        if(strcmp(unit, "nJ") == 0) {
-                            rapl_event_units.push_back(std::string("J"));
-                            rapl_event_conversion.push_back(1.0e-9);
-                        } else {
-                            rapl_event_units.push_back(std::string(unit));
-                            rapl_event_conversion.push_back(1.0);
-                        }
-                        rapl_event_data_type.push_back(evinfo.data_type);
-                        rapl_event_names.push_back(std::string(event_name));
-
-                        retval = PAPI_add_event(rapl_EventSet, code);
-                        if (retval != PAPI_OK) {
-                            fprintf(stderr, "Error adding RAPL event.\n");
-                            fprintf(stderr, "PAPI error %d: %s\n", retval,
-                                    PAPI_strerror(retval));
-                            return;
-                        }
-                    }
-                    retval = PAPI_start(rapl_EventSet);
-                    if (retval != PAPI_OK) {
-                        fprintf(stderr, "Error starting PAPI RAPL eventset.\n");
-                        return;
-                    }
-                    rapl_initialized = true;
-                }
-            }
-            // do we have the NVML (cuda) components?
-            if (strstr(comp_info->name, "nvml")) {
-                if (comp_info->num_native_events == 0) {
-                    fprintf(stderr, "PAPI NVML component found, but ");
-                    fprintf(stderr, "no NVML events found.\n");
-                    if (comp_info->disabled != 0) {
-                        fprintf(stderr, "%s.\n", comp_info->disabled_reason);
-                    }
-                } else {
-                    nvml_EventSet = PAPI_NULL;
-                    retval = PAPI_create_eventset(&nvml_EventSet);
-                    if (retval != PAPI_OK) {
-                        fprintf(stderr, "Error creating NVML PAPI eventset.\n");
-                        fprintf(stderr, "PAPI error %d: %s\n", retval,
-                                PAPI_strerror(retval));
-                        return;
-                    }
-                    int code = PAPI_NATIVE_MASK;
-                    int event_modifier = PAPI_ENUM_FIRST;
-                    for ( int ii=0; ii< comp_info->num_native_events; ii++ ) {
-                        // get the event
-                        retval = PAPI_enum_cmp_event( &code, event_modifier, component_id );
-                        event_modifier = PAPI_ENUM_EVENTS;
-                        if ( retval != PAPI_OK ) {
-                            fprintf( stderr, "%s %d %s %d\n", __FILE__,
-                                    __LINE__, "NVML PAPI_event_code_to_name", retval );
-                            fprintf(stderr, "PAPI error %d: %s\n", retval,
-                                    PAPI_strerror(retval));
-                            continue;
-                        }
-                        // get the event name
-                        char event_name[PAPI_MAX_STR_LEN];
-                        retval = PAPI_event_code_to_name( code, event_name );
-                        if (retval != PAPI_OK) {
-                            fprintf(stderr, "%s %d %s %d\n", __FILE__,
-                                    __LINE__, "NVML Error getting event name\n",retval);
-                            fprintf(stderr, "PAPI error %d: %s\n", retval,
-                                    PAPI_strerror(retval));
-                            continue;
-                        }
-                        // get the event info
-                        PAPI_event_info_t evinfo;
-                        retval = PAPI_get_event_info(code,&evinfo);
-                        if (retval != PAPI_OK) {
-                            fprintf(stderr, "%s %d %s %d\n", __FILE__,
-                                    __LINE__, "Error getting NVML event info\n",retval);
-                            fprintf(stderr, "PAPI error %d: %s\n", retval,
-                                    PAPI_strerror(retval));
-                            continue;
-                        }
-                        // get the event units
-                        char unit[PAPI_MAX_STR_LEN] = {0};
-                        strncpy(unit,evinfo.units,PAPI_MAX_STR_LEN);
-                        // save the event info
-                        //printf("Found event '%s (%s)'\n", event_name, unit);
-                        nvml_event_codes.push_back(code);
-                        if(strcmp(unit, "mW") == 0) {
-                            nvml_event_units.push_back(std::string("W"));
-                            nvml_event_conversion.push_back(0.0001);
-                        } else {
-                            nvml_event_units.push_back(std::string(unit));
-                            nvml_event_conversion.push_back(1.0);
-                        }
-                        nvml_event_data_type.push_back(evinfo.data_type);
-                        nvml_event_names.push_back(std::string(event_name));
-                        retval = PAPI_add_event(nvml_EventSet, code);
-                        if (retval != PAPI_OK) {
-                            fprintf(stderr, "Error adding NVML event.\n");
-                            fprintf(stderr, "PAPI error %d: %s\n", retval,
-                                    PAPI_strerror(retval));
-                            return;
-                        }
-                    }
-                    retval = PAPI_start(nvml_EventSet);
-                    if (retval != PAPI_OK) {
-                        fprintf(stderr, "Error starting PAPI NVML eventset.\n");
-                        fprintf(stderr, "PAPI error %d: %s\n", retval,
-                                PAPI_strerror(retval));
-                        return;
-                    }
-                    nvml_initialized = true;
-                }
-            }
-            if (strstr(comp_info->name, "lmsensors")) {
-                if (comp_info->num_native_events == 0) {
-                    fprintf(stderr, "PAPI lmsensors component found, but ");
-                    fprintf(stderr, "no lmsensors events found.\n");
-                    if (comp_info->disabled != 0) {
-                        fprintf(stderr, "%s.\n", comp_info->disabled_reason);
-                    }
-                } else {
-                    lms_EventSet = PAPI_NULL;
-                    retval = PAPI_create_eventset(&lms_EventSet);
-                    if (retval != PAPI_OK) {
-                        fprintf(stderr, "Error creating PAPI lmsensors eventset.\n");
-                        fprintf(stderr, "PAPI error %d: %s\n", retval,
-                                PAPI_strerror(retval));
-                        return;
-                    }
-                    int code = PAPI_NATIVE_MASK;
-                    int event_modifier = PAPI_ENUM_FIRST;
-                    for ( int ii=0; ii< comp_info->num_native_events; ii++ ) {
-                        // get the event
-                        retval = PAPI_enum_cmp_event( &code, event_modifier, component_id );
-                        event_modifier = PAPI_ENUM_EVENTS;
-                        if ( retval != PAPI_OK ) {
-                            fprintf( stderr, "%s %d %s %d\n",
-                                    __FILE__, __LINE__, "lmsensors PAPI_event_code_to_name", retval );
-                            fprintf(stderr, "PAPI error %d: %s\n", retval,
-                                    PAPI_strerror(retval));
-                        }
-                        // get the event name
-                        char event_name[PAPI_MAX_STR_LEN];
-                        retval = PAPI_event_code_to_name( code, event_name );
-                        if (retval != PAPI_OK) {
-                            fprintf(stderr, "%s %d %s %d\n", __FILE__,
-                                    __LINE__, "Error getting lmsensors event name\n",retval);
-                            fprintf(stderr, "PAPI error %d: %s\n", retval,
-                                    PAPI_strerror(retval));
-                            continue;
-                        }
-                        // get the event info
-                        PAPI_event_info_t evinfo;
-                        retval = PAPI_get_event_info(code,&evinfo);
-                        if (retval != PAPI_OK) {
-                            fprintf(stderr, "%s %d %s %d\n",
-                                    __FILE__, __LINE__, "Error getting lmsensors event info\n",retval);
-                            fprintf(stderr, "PAPI error %d: %s\n", retval,
-                                    PAPI_strerror(retval));
-                        }
-                        // get the event units
-                        char unit[PAPI_MAX_STR_LEN] = {0};
-                        strncpy(unit,evinfo.units,PAPI_MAX_STR_LEN);
-                        // add all LMSensors events!  Except the "Core" specific ones.  Can be too many.
-                        //char *Core_sub = strstr(event_name, ".Core ");
-                        // ignore the values that don't change over time, either.
-                        char *max_sub = strstr(event_name, "_max");
-                        char *crit_sub = strstr(event_name, "_crit");
-                        char *interval_sub = strstr(event_name, "_interval");
-                        if (max_sub == NULL &&
-                                crit_sub == NULL && interval_sub == NULL) {
-                            // save the event info
-                            //printf("Found event '%s (%s)'\n", event_name, unit);
-                            lms_event_codes.push_back(code);
-                            lms_event_units.push_back(std::string(unit));
-                            lms_event_data_type.push_back(evinfo.data_type);
-                            lms_event_names.push_back(std::string(event_name));
-                            retval = PAPI_add_event(lms_EventSet, code);
-                            if (retval != PAPI_OK) {
-                                fprintf(stderr, "Error adding lmsensors event.\n");
-                                fprintf(stderr, "PAPI error %d: %s\n", retval,
-                                        PAPI_strerror(retval));
-                                continue;
-                            }
-                        }
-                    }
-                }
-                retval = PAPI_start(lms_EventSet);
-                if (retval != PAPI_OK) {
-                    fprintf(stderr, "Error starting PAPI lmsensors eventset.\n");
-                    fprintf(stderr, "PAPI error %d: %s\n", retval,
-                            PAPI_strerror(retval));
-                    return;
-                }
-                lms_initialized = true;
-            }
-        }
-    }
-
-    void read_papi_components(ProcData * data) {
-        if (rapl_initialized) {
-            long long * rapl_values = (long long *)calloc(rapl_event_names.size(), sizeof(long long));
-            int retval = PAPI_read(rapl_EventSet, rapl_values);
-            if (retval != PAPI_OK) {
-                fprintf(stderr, "Error reading PAPI RAPL eventset.\n");
-                fprintf(stderr, "PAPI error %d: %s\n", retval,
-                        PAPI_strerror(retval));
-            } else {
-                for (size_t i = 0 ; i < rapl_event_names.size() ; i++) {
-                    data->rapl_metrics.push_back(rapl_values[i]);
-                }
-            }
-            free(rapl_values);
-        }
-        if (nvml_initialized) {
-            long long * nvml_values = (long long *)calloc(nvml_event_names.size(), sizeof(long long));
-            int retval = PAPI_read(nvml_EventSet, nvml_values);
-            if (retval != PAPI_OK) {
-                fprintf(stderr, "Error reading PAPI NVML eventset.\n");
-                fprintf(stderr, "PAPI error %d: %s\n", retval,
-                        PAPI_strerror(retval));
-            } else {
-                for (size_t i = 0 ; i < nvml_event_names.size() ; i++) {
-                    data->nvml_metrics.push_back(nvml_values[i]);
-                }
-            }
-            free(nvml_values);
-        }
-        if (lms_initialized) {
-            long long * lms_values = (long long *)calloc(lms_event_names.size(), sizeof(long long));
-            int retval = PAPI_read(lms_EventSet, lms_values);
-            if (retval != PAPI_OK) {
-                fprintf(stderr, "Error reading PAPI lmsensors eventset.\n");
-                fprintf(stderr, "PAPI error %d: %s\n", retval,
-                        PAPI_strerror(retval));
-            } else {
-                for (size_t i = 0 ; i < lms_event_names.size() ; i++) {
-                    data->lms_metrics.push_back(lms_values[i]);
-                }
-            }
-            free(lms_values);
-        }
-        return;
-    }
-
-#endif // defined(APEX_HAVE_PAPI)
-
-    std::atomic<bool> proc_data_reader::done(false);
+#include <dirent.h>
 
+namespace apex {
 
     void get_popen_data(char *cmnd) {
         FILE *pf;
@@ -423,6 +88,50 @@ namespace apex {
         return;
     }
 
+    /* NB: We have to use C functions for File I/O because the C++ functions
+       seem to be caching the data from the virtual filesystem. So it never
+       gets refreshed... */
+    void read_cray_power(
+        std::unordered_map<std::string,std::string>& cray_power_units,
+        std::unordered_map<std::string,uint64_t>& cray_power_values) {
+        struct dirent *entry = nullptr;
+        DIR *dp = nullptr;
+        std::set<std::string> skip = { "startup", "version", "raw_scan_hz", "generation", "freshness" };
+
+        dp = opendir("/sys/cray/pm_counters");
+        if (dp != nullptr) {
+            while ((entry = readdir(dp))) {
+                if (entry->d_type == DT_DIR) continue;
+                std::string name(entry->d_name);
+                if (strstr(entry->d_name,"_cap") != 0) continue;
+                if (skip.count(name) != 0) continue;
+                uint64_t tmpint;
+                std::string tmpstr;
+                std::string fname("/sys/cray/pm_counters/");
+                fname+=name;
+                FILE * fp = fopen(fname.c_str(), "r");
+                if (fp != NULL) {
+                    char *line = NULL;
+                    size_t len = 0;
+                    ssize_t nread;
+                    while ((nread = getline(&line, &len, fp)) != -1) {
+                        if (nread == -1) break;
+                        std::istringstream iline{line};
+                        while (iline >> tmpint >> tmpstr) {
+                            cray_power_units[name] = tmpstr;
+                            cray_power_values[name] = tmpint;
+                            break;
+                        }
+                    }
+                }
+                fclose(fp);
+            }
+        }
+        closedir(dp);
+        return;
+    }
+
+
     ProcData* parse_proc_stat(void) {
         if (!apex_options::use_proc_stat()) return nullptr;
 
@@ -469,18 +178,11 @@ namespace apex {
         }
         fclose (pFile);
 #if defined(APEX_HAVE_CRAY_POWER)
-        procData->power = read_power();
-        procData->power_cap = read_power_cap();
-        procData->energy = read_energy();
-        procData->freshness = read_freshness();
-        procData->generation = read_generation();
+        read_cray_power(procData->cray_power_units, procData->cray_power_values);
 #endif
 #if defined(APEX_HAVE_POWERCAP_POWER)
         procData->package0 = read_package0();
         procData->dram = read_dram();
-#endif
-#if defined(APEX_HAVE_PAPI)
-        read_papi_components(procData);
 #endif
         return procData;
     }
@@ -515,35 +217,25 @@ namespace apex {
         d->procs_running = procs_running - rhs.procs_running;
         d->procs_blocked = procs_blocked - rhs.procs_blocked;
 #if defined(APEX_HAVE_CRAY_POWER)
-        d->power = power;
-        d->power_cap = power_cap;
-        d->energy = energy - rhs.energy;
-        d->freshness = freshness;
-        d->generation = generation;
+        d->cray_power_units = rhs.cray_power_units;
+        /*
+        d->cray_power_values = rhs.cray_power_values;
+        */
+        // We want relative energy values, so take a diff from the last reading.
+        for (auto it : cray_power_units) {
+            std::string name(it.first);
+            std::string unit(it.second);
+            uint64_t value{rhs.cray_power_values.find(name)->second};
+            if (unit.compare("J") == 0) {
+                d->cray_power_values[name] = cray_power_values[name] - value;
+            } else {
+                d->cray_power_values[name] = value;
+            }
+        }
 #endif
 #if defined(APEX_HAVE_POWERCAP_POWER)
         d->package0 = package0 - rhs.package0;
         d->dram = dram - rhs.dram;
-#endif
-#if defined(APEX_HAVE_PAPI)
-        if (rapl_initialized) {
-            // reading might have failed, so only copy if there's data
-            for (size_t i = 0 ; i < rapl_metrics.size() ; i++) {
-                d->rapl_metrics.push_back(rapl_metrics[i]);
-            }
-        }
-        if (nvml_initialized) {
-            // reading might have failed, so only copy if there's data
-            for (size_t i = 0 ; i < nvml_metrics.size() ; i++) {
-                d->nvml_metrics.push_back(nvml_metrics[i]);
-            }
-        }
-        if (lms_initialized) {
-            // reading might have failed, so only copy if there's data
-            for (size_t i = 0 ; i < lms_metrics.size() ; i++) {
-                d->lms_metrics.push_back(lms_metrics[i]);
-            }
-        }
 #endif
         return d;
     }
@@ -723,62 +415,20 @@ namespace apex {
             }
         }
 #if defined(APEX_HAVE_CRAY_POWER)
-        sample_value("Power", power);
-        sample_value("Power Cap", power_cap);
-        sample_value("Energy", energy);
-        sample_value("Freshness", freshness);
-        sample_value("Generation", generation);
+        for (auto it : cray_power_units) {
+            // Ignore zero energy values, they're misleading
+            if (cray_power_values[it.first] > 0) {
+                std::string name(it.first);
+                name += " (";
+                name += it.second;
+                name += ")";
+                sample_value(name, cray_power_values[it.first]);
+            }
+        }
 #endif
 #if defined(APEX_HAVE_POWERCAP_POWER)
         sample_value("Package-0 Energy", package0);
         sample_value("DRAM Energy", dram);
-#endif
-#if defined(APEX_HAVE_PAPI)
-        if (rapl_initialized) {
-            // reading might have failed, so only iterate over the data
-            for (size_t i = 0 ; i < rapl_metrics.size() ; i++) {
-                stringstream ss;
-                ss << rapl_event_names[i];
-                if (rapl_event_units[i].length() > 0) {
-                    ss << "(" << rapl_event_units[i] << ")";
-                }
-                if (rapl_event_names[i].find("ENERGY") == string::npos &&
-                        rapl_event_data_type[i] == PAPI_DATATYPE_FP64) {
-                    event_result_t tmp;
-                    tmp.ll = rapl_metrics[i];
-                    sample_value(ss.str().c_str(), tmp.fp * rapl_event_conversion[i]);
-                } else {
-                    double tmp = (double)rapl_metrics[i];
-                    sample_value(ss.str().c_str(), tmp * rapl_event_conversion[i]);
-                }
-            }
-        }
-        if (nvml_initialized) {
-            // reading might have failed, so only iterate over the data
-            for (size_t i = 0 ; i < nvml_metrics.size() ; i++) {
-                stringstream ss;
-                ss << nvml_event_names[i];
-                if (nvml_event_units[i].length() > 0) {
-                    ss << "(" << nvml_event_units[i] << ")";
-                }
-                if (nvml_event_data_type[i] == PAPI_DATATYPE_FP64) {
-                    event_result_t tmp;
-                    tmp.ll = nvml_metrics[i];
-                    sample_value(ss.str().c_str(), tmp.fp * nvml_event_conversion[i]);
-                } else {
-                    double tmp = nvml_metrics[i];
-                    sample_value(ss.str().c_str(), tmp * nvml_event_conversion[i]);
-                }
-            }
-        }
-        if (lms_initialized) {
-            // reading might have failed, so only iterate over the data
-            for (size_t i = 0 ; i < lms_metrics.size() ; i++) {
-                // PAPI scales LM sensor data by 1000,
-                // because it doesn't have floating point values..
-                sample_value(lms_event_names[i].c_str(), (double)lms_metrics[i]/1000.0);
-            }
-        }
 #endif
     }
 
@@ -866,6 +516,12 @@ namespace apex {
                     char* pEnd;
                     double d1 = strtod (value.c_str(), &pEnd);
                     string mname("meminfo:" + name);
+                    if (pEnd != NULL) {
+                        int len = strlen(pEnd);
+                        if( pEnd[len-1] == '\n' )
+                            pEnd[len-1] = 0;
+                        mname.append(pEnd);
+                    }
                     if (pEnd) { sample_value(mname, d1); }
                 }
             }
@@ -897,6 +553,12 @@ namespace apex {
                         char* pEnd;
                         double d1 = strtod (value.c_str(), &pEnd);
                         string mname("status:" + name);
+                        if (pEnd != NULL) {
+                        int len = strlen(pEnd);
+                            if( pEnd[len-1] == '\n' )
+                                pEnd[len-1] = 0;
+                            mname.append(pEnd);
+                        }
                         if (pEnd) { sample_value(mname, d1); }
                     }
                 }
@@ -911,6 +573,12 @@ namespace apex {
                         char* pEnd;
                         double d1 = strtod (value.c_str(), &pEnd);
                         string mname("status:" + name);
+                        if (pEnd != NULL) {
+                        int len = strlen(pEnd);
+                            if( pEnd[len-1] == '\n' )
+                                pEnd[len-1] = 0;
+                            mname.append(pEnd);
+                        }
                         if (pEnd) { sample_value(mname, d1); }
                     }
                 }
@@ -925,6 +593,12 @@ namespace apex {
                         char* pEnd;
                         double d1 = strtod (value.c_str(), &pEnd);
                         string mname("status:" + name);
+                        if (pEnd != NULL) {
+                        int len = strlen(pEnd);
+                            if( pEnd[len-1] == '\n' )
+                                pEnd[len-1] = 0;
+                            mname.append(pEnd);
+                        }
                         if (pEnd) { sample_value(mname, d1); }
                     }
                 }
@@ -1092,11 +766,15 @@ namespace apex {
     }
 
     /* This is the main function for the reader thread. */
-    void* proc_data_reader::read_proc(void * _ptw) {
-        pthread_wrapper* ptw = (pthread_wrapper*)_ptw;
+    //void* proc_data_reader::read_proc(void * _ptw) {
+    void* proc_data_reader::read_proc() {
+        in_apex prevent_deadlocks;
+        // when tracking memory allocations, ignore these
+        in_apex prevent_nonsense;
+        //pthread_wrapper* ptw = (pthread_wrapper*)_ptw;
         // make sure APEX knows this is not a worker thread
-        thread_instance::instance(false);
-        ptw->_running = true;
+        thread_instance::instance(false).set_worker(false);
+        //ptw->_running = true;
         if (apex_options::pin_apex_threads()) {
             set_thread_affinity();
         }
@@ -1119,6 +797,9 @@ namespace apex {
         sensor_data * mysensors = new sensor_data();
 #endif
         ProcData *oldData = parse_proc_stat();
+#if defined(APEX_HAVE_PAPI)
+        read_papi_components();
+#endif
         // disabled for now - not sure that it is useful
         parse_proc_cpuinfo(); // do this once, it won't change.
         parse_proc_meminfo(); // some things change, others don't...
@@ -1127,13 +808,33 @@ namespace apex {
         parse_proc_loadavg(); // this will change
         parse_proc_netdev();
 #ifdef APEX_HAVE_LM_SENSORS
-        mysensors->read_sensors();
+        if (apex_options::use_lm_sensors()) {
+            mysensors->read_sensors();
+        }
 #endif
         ProcData *newData = nullptr;
         ProcData *periodData = nullptr;
-
-        while(ptw->wait()) {
+        if (apex_options::monitor_gpu()) {
+            dynamic::nvml::query();
+        }
+        if (apex_options::monitor_gpu()) {
+            dynamic::rsmi::query();
+        }
+        if (apex_options::use_hip_profiler()) {
+            dynamic::rocprofiler::query();
+        }
+        // release the main thread to continue
+        while(!done /*&& ptw->wait()*/) {
+            incrementPeriod();
+            //usleep(apex_options::proc_period());
+            std::unique_lock<std::mutex> lk(proc_data_reader::cv_m);
+            // if we've been interrupted by the main thread, break and exit
+            if(cv.wait_for(lk, apex_options::proc_period()*1us) ==
+               std::cv_status::no_timeout) {
+               break;
+            }
             if (done) break;
+            reading = true;
             if (apex_options::use_tau()) {
                 tau_listener::Tau_start_wrapper("proc_data_reader::read_proc: main loop");
             }
@@ -1154,25 +855,44 @@ namespace apex {
             parse_proc_self_status();
             parse_proc_self_io();
             parse_proc_netdev();
-
-#ifdef APEX_HAVE_LM_SENSORS
-            mysensors->read_sensors();
+#if defined(APEX_HAVE_PAPI)
+            read_papi_components();
 #endif
 
+#ifdef APEX_HAVE_LM_SENSORS
+            if (apex_options::use_lm_sensors()) {
+                mysensors->read_sensors();
+            }
+#endif
+            if (apex_options::monitor_gpu()) {
+                dynamic::nvml::query();
+                dynamic::rsmi::query();
+            }
+            if (apex_options::use_hip_profiler()) {
+                dynamic::rocprofiler::query();
+            }
             if (apex_options::use_tau()) {
                 tau_listener::Tau_stop_wrapper("proc_data_reader::read_proc: main loop");
             }
+            reading = false;
         }
+        // in case we broke from while loop
+        reading = false;
 #ifdef APEX_HAVE_LM_SENSORS
         delete(mysensors);
 #endif
 
+        if (apex_options::monitor_gpu()) {
+            dynamic::rsmi::stop();
+        }
+        if (apex_options::use_hip_profiler()) {
+            dynamic::rocprofiler::stop();
+        }
         if (apex_options::use_tau()) {
             tau_listener::Tau_stop_wrapper("proc_data_reader::read_proc");
         }
         delete(oldData);
-        thread_instance::delete_instance();
-        ptw->_running = false;
+        //ptw->_running = false;
         return nullptr;
     }
 
@@ -1244,6 +964,38 @@ namespace apex {
         return line;
     }
 
+    std::atomic<uint64_t> proc_data_reader::sample_period{0};
+
+
+std::array<double,2> getAvailableMemory() {
+    std::array<double,2> values{0,0};
+    /* Get the CPU memory */
+        FILE *f = fopen("/proc/meminfo", "r");
+        if (f) {
+            char line[4096] = {0};
+            while ( fgets( line, 4096, f)) {
+                string tmp(line);
+                const REGEX_NAMESPACE::regex separator(":");
+                REGEX_NAMESPACE::sregex_token_iterator token(tmp.begin(), tmp.end(),
+                        separator, -1);
+                REGEX_NAMESPACE::sregex_token_iterator end;
+                string name = *token++;
+                if (token != end && name.find("MemFree") != name.npos) {
+                    string value = *token;
+                    char* pEnd;
+                    double d1 = strtod (value.c_str(), &pEnd);
+                    if (pEnd) { values[0] =  d1; }
+                    break;
+                }
+            }
+            fclose(f);
+        }
+        if (apex_options::monitor_gpu()) {
+            dynamic::rsmi::getAvailableMemory();
+        }
+    return values;
+}
+
 } // namespace
 
 #endif // APEX_HAVE_PROC
