@@ -1,13 +1,17 @@
-//  Copyright (c) 2014 University of Oregon
-//
-//  Distributed under the Boost Software License, Version 1.0. (See accompanying
-//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+/*
+ * Copyright (c) 2014-2021 Kevin Huck
+ * Copyright (c) 2014-2021 University of Oregon
+ *
+ * Distributed under the Boost Software License, Version 1.0. (See accompanying
+ * file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+ */
 
 #pragma once
 
 #if APEX_HAVE_PROC
 
 #include <stdio.h>
+#include <unistd.h>
 #include <vector>
 #include <iostream>
 #include <fstream>
@@ -19,7 +23,7 @@
 #include <thread>
 #include <string>
 #include <memory>
-#include "pthread_wrapper.hpp"
+//#include "pthread_wrapper.hpp"
 #include "apex_options.hpp"
 
 namespace apex {
@@ -42,25 +46,47 @@ typedef std::vector<CPUStat*> CPUs;
 
 class proc_data_reader {
 private:
-    pthread_wrapper * worker_thread;
-    static std::atomic<bool> done;
+    //pthread_wrapper * worker_thread;
+    std::atomic<bool> done;
+    std::atomic<bool> reading;
+    std::thread worker_thread;
+    std::condition_variable cv;
+    std::mutex cv_m;
+    static std::atomic<uint64_t> sample_period;
 public:
+    /*
     static void* read_proc(void * _pdr);
     proc_data_reader(void) {
         worker_thread = new pthread_wrapper(&proc_data_reader::read_proc,
         (void*)(this), apex_options::proc_period());
     };
+    */
+    void* read_proc(void);
+    proc_data_reader(void) : done(false), reading(false) {
+        worker_thread = std::thread(&proc_data_reader::read_proc, this);
+        worker_thread.detach();
+    }
 
     void stop_reading(void) {
         done = true;
-        worker_thread->stop_thread();
+        cv.notify_all();
+        //worker_thread->stop_thread();
+        if (worker_thread.joinable()) {
+            worker_thread.join();
+        }
+        if(reading) {
+            // this is helpful if we are sampling frequently
+            usleep(apex_options::proc_period());
+        }
     }
 
     ~proc_data_reader(void) {
         stop_reading();
-        delete worker_thread;
+        //delete worker_thread;
     }
     static std::string get_command_line(void);
+    static uint64_t getPeriod() { return sample_period; }
+    static void incrementPeriod() { sample_period++; }
 };
 
 class ProcData {
@@ -79,6 +105,8 @@ public:
   long power_cap;
   long startup;
   long version;
+  std::unordered_map<std::string,std::string> cray_power_units;
+  std::unordered_map<std::string,uint64_t> cray_power_values;
   std::unordered_map<std::string,std::string> cpuinfo;
   std::unordered_map<std::string,std::string> meminfo;
   std::unordered_map<std::string,double> netdev;
@@ -86,11 +114,6 @@ public:
 #if defined(APEX_HAVE_POWERCAP_POWER)
   long long package0;
   long long dram;
-#endif
-#if defined(APEX_HAVE_PAPI)
-  std::vector<long long> rapl_metrics;
-  std::vector<long long> nvml_metrics;
-  std::vector<long long> lms_metrics;
 #endif
   //softirq 10953997190 0 1380880059 1495447920 1585783785 ...
   //        15525789 0 12 661586214 0 1519806115
@@ -137,6 +160,10 @@ bool parse_sensor_data();
     macro(version,"/sys/cray/pm_counters/version") \
 
 #if defined(APEX_HAVE_CRAY_POWER)
+void read_cray_power(
+  std::unordered_map<std::string,std::string>& cray_power_units,
+  std::unordered_map<std::string,uint64_t>& cray_power_values);
+
 #define apex_macro(name,filename) \
 inline int read_##name (void) { \
   int tmpint; \
@@ -170,7 +197,7 @@ FOREACH_APEX_XC30_VALUE(apex_macro)
  *
  * This was a quick hack to get basic support for KNL.
  */
-inline long long read_package0 (void) {
+inline long long read_package0 (bool convert = true) {
   long long tmplong;
   FILE *fff;
   fff=fopen("/sys/class/powercap/intel-rapl/intel-rapl:0/energy_uj","r");
@@ -178,10 +205,16 @@ inline long long read_package0 (void) {
     //std::cerr << "Error opening package0!" << std::endl;
     tmplong = 0LL;
   } else {
-    fscanf(fff,"%lld",&tmplong);
+    if (fscanf(fff,"%lld",&tmplong) == EOF) {
+        std::cerr << "Error reading package0!" << std::endl;
+    }
     fclose(fff);
   }
-  return tmplong/1000000;
+  if (convert) {
+    return tmplong * 1e-6;
+  } else {
+    return tmplong;
+  }
 }
 
 inline long long  read_dram (void) {
@@ -195,17 +228,19 @@ inline long long  read_dram (void) {
     //std::cerr << "Error opening dram!" << std::endl;
     tmplong = 0LL;
   } else {
-    fscanf(fff,"%lld",&tmplong);
+    if (fscanf(fff,"%lld",&tmplong) == EOF) {
+        std::cerr << "Error reading dram!" << std::endl;
+    }
     fclose(fff);
   }
-  return tmplong/1000000;
+  return tmplong * 1e-6;
 }
 
 #endif
 
 #if defined(APEX_HAVE_PAPI)
 void initialize_papi_events(void);
-void read_papi_components(ProcData * data);
+void read_papi_components(void);
 #endif
 
 #ifdef APEX_HAVE_MSR
@@ -214,6 +249,8 @@ void apex_finalize_msr(void);
 double msr_current_power_high(void);
 #endif
 
+std::array<double,2> getAvailableMemory();
+
 }
 
 #endif
