@@ -1,7 +1,10 @@
-//  Copyright (c) 2014 University of Oregon
-//
-//  Distributed under the Boost Software License, Version 1.0. (See accompanying
-//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+/*
+ * Copyright (c) 2014-2021 Kevin Huck
+ * Copyright (c) 2014-2021 University of Oregon
+ *
+ * Distributed under the Boost Software License, Version 1.0. (See accompanying
+ * file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+ */
 
 #pragma once
 
@@ -52,14 +55,17 @@ using moodycamel::ConcurrentQueue;
 #include "task_dependency.hpp"
 #include <sys/stat.h>
 #if !defined(_MSC_VER)
-#include <unistd.h>
-#include <sys/file.h>
+//#include <unistd.h>
+//#include <sys/file.h>
 #else // not _MSC_VER
 #if !defined(__APPLE__)
 #include <io.h>
 #endif // __APPLE__
 #endif // _MSC_VER
 #include <fcntl.h>
+#include <iostream>
+#include <iomanip>
+#include <ctime>
 
 #define INITIAL_NUM_THREADS 2
 
@@ -69,7 +75,7 @@ class profiler_queue_t : public ConcurrentQueue<std::shared_ptr<profiler> > {
 public:
   profiler_queue_t() {}
   virtual ~profiler_queue_t() {
-      finalize();
+      //finalize();
   }
 };
 
@@ -77,33 +83,36 @@ class dependency_queue_t : public ConcurrentQueue<task_dependency*> {
 public:
   dependency_queue_t() {}
   virtual ~dependency_queue_t() {
-      finalize();
+      //finalize();
   }
 };
 
-static const char * task_scatterplot_sample_filename = "apex_task_samples.csv";
+static const char * task_scatterplot_sample_filename = "apex_task_samples.";
+static const char * counter_scatterplot_sample_filename = "apex_counter_samples.";
 
 class profiler_listener : public event_listener {
 private:
   void _init(void);
   bool _initialized;
+  bool _main_timer_stopped;
   std::atomic<bool> _done;
   std::atomic<int> active_tasks;
-  std::shared_ptr<profiler> main_timer; // not a shared pointer, yet...
-  void write_one_timer(task_identifier &task_id, profile * p,
+  std::shared_ptr<profiler> main_timer;
+  void write_one_timer(std::string &name, profile * p,
                        std::stringstream &screen_output,
-                       std::stringstream &csv_output,
                        double &total_accumulated,
-                       double &total_main, bool timer);
-  void finalize_profiles(dump_event_data &data);
+                       double &total_main, double &wall_main, bool include_stops,
+                       bool include_papi);
+  void finalize_profiles(dump_event_data &data, std::map<std::string, apex_profile*>& profiles);
   void write_taskgraph(void);
+  void write_tasktree(void);
   void write_profile(void);
   void delete_profiles(void);
 #ifdef APEX_HAVE_HPX
   void schedule_process_profiles(void);
 #endif
   unsigned int process_profile(std::shared_ptr<profiler> &p, unsigned int tid);
-  unsigned int process_profile(profiler* p, unsigned int tid);
+  unsigned int process_profile(profiler& p, unsigned int tid);
   unsigned int process_dependency(task_dependency* td);
   int node_id;
   std::mutex _mtx;
@@ -112,6 +121,7 @@ private:
   void _common_stop(std::shared_ptr<profiler> &p,
     bool is_yield); // internal, inline function
   void push_profiler(int my_tid, std::shared_ptr<profiler> &p);
+  void push_profiler(int my_tid, profiler &p);
   std::unordered_map<task_identifier, profile*> task_map;
   std::mutex _task_map_mutex;
   std::unordered_map<task_identifier, std::unordered_map<task_identifier,
@@ -126,79 +136,39 @@ private:
   dependency_queue_t * _construct_dependency_queue(void);
   dependency_queue_t * dependency_queue(void);
   //ConcurrentQueue<task_dependency*> dependency_queue;
-#if defined(APEX_THROTTLE)
   std::unordered_set<task_identifier> throttled_tasks;
-#endif
-#if APEX_HAVE_PAPI
   int num_papi_counters;
-  std::vector<int> event_sets;
   std::vector<std::string> metric_names;
+#if APEX_HAVE_PAPI
   void initialize_PAPI(bool first_time);
 #endif
 #ifndef APEX_HAVE_HPX
   std::thread * consumer_thread;
 #endif
   semaphore queue_signal;
-  //std::ofstream task_scatterplot_sample_file;
-  int task_scatterplot_sample_file;
+  std::ofstream _task_scatterplot_sample_file;
+  std::ofstream _counter_scatterplot_sample_file;
   std::stringstream task_scatterplot_samples;
+  std::stringstream counter_scatterplot_samples;
+  std::string timestamp_started;
 public:
   void set_node_id(int node_id, int node_count) {
     APEX_UNUSED(node_count);
     this->node_id = node_id;
   }
-  profiler_listener (void) : _initialized(false), _done(false),
-                             node_id(0), task_map()
-#if APEX_HAVE_PAPI
-                             , num_papi_counters(0), event_sets(8),
+  profiler_listener (void) : _initialized(false), _main_timer_stopped(false), _done(false),
+                             node_id(0), task_map() , num_papi_counters(0),
                              metric_names(0)
-#endif
   {
-#if APEX_HAVE_PAPI
-      num_papi_counters = 0;
-#endif
       if (apex_options::task_scatterplot()) {
-        // check if the samples file exists
-        struct stat buffer;
-        std::stringstream ss;
-        ss << apex_options::output_file_path();
-        ss << filesystem_separator();
-        ss << task_scatterplot_sample_filename;
-        if (stat (ss.str().c_str(), &buffer) == 0) {
-            struct tm *timeinfo = localtime(&buffer.st_mtime);
-            time_t filetime = mktime(timeinfo);
-            time_t nowish;
-            time(&nowish);
-            double seconds = difftime(nowish, filetime);
-            /* if the file exists, was it recently created? */
-            if (seconds > 10) {
-                /* create the file */
-                std::ofstream tmp;
-                tmp.open(ss.str());
-                tmp << "#timestamp value   name" << std::endl << std::flush;
-                /* yes, close the file because we will use some
-                   low-level calls to have concurrent access
-                   across processes/threads. */
-                tmp.close();
-            }
-        } else {
-            /* create the file */
-            std::ofstream tmp;
-            tmp.open(ss.str());
-            tmp << "#timestamp value   name" << std::endl << std::flush;
-            /* yes, close the file because we will use some
-            low-level calls to have concurrent access
-            across processes/threads. */
-            tmp.close();
-        }
-        // open the file
-        task_scatterplot_sample_file = open(ss.str().c_str(), O_APPEND | O_WRONLY );
-        if (task_scatterplot_sample_file < 0) {
-            perror("opening scatterplot sample file");
-        }
-        APEX_ASSERT(task_scatterplot_sample_file >= 0);
         profiler::get_global_start();
       }
+      // get a timestamp for the start of execution
+      auto t = std::time(nullptr);
+      auto tm = *std::localtime(&t);
+      std::ostringstream oss;
+      oss << std::put_time(&tm, "%d/%m/%Y %H:%M:%S");
+      timestamp_started = oss.str();
   };
   ~profiler_listener (void);
   void async_thread_setup(void);
@@ -206,6 +176,7 @@ public:
   void on_startup(startup_event_data &data);
   void on_dump(dump_event_data &data);
   void on_reset(task_identifier * id);
+  void on_pre_shutdown(void);
   void on_shutdown(shutdown_event_data &data);
   void on_new_node(node_event_data &data);
   void on_new_thread(new_thread_event_data &data);
@@ -221,13 +192,53 @@ public:
   void on_send(message_event_data &data);
   void on_recv(message_event_data &data);
   // other methods
+  std::ofstream& task_scatterplot_sample_file() {
+      if (!_task_scatterplot_sample_file.is_open()) {
+        std::stringstream ss;
+        ss << apex_options::output_file_path();
+        ss << filesystem_separator();
+        ss << task_scatterplot_sample_filename << node_id << ".csv";
+        // open the file
+        _task_scatterplot_sample_file.open(ss.str(), std::ofstream::out | std::ofstream::app);
+        if (!_task_scatterplot_sample_file.is_open()) {
+            perror("opening scatterplot sample file");
+        }
+        APEX_ASSERT(_task_scatterplot_sample_file.is_open());
+      }
+      return _task_scatterplot_sample_file;
+  }
+  std::ofstream& counter_scatterplot_sample_file () {
+      if (!_counter_scatterplot_sample_file.is_open()) {
+        std::stringstream ss;
+        ss << apex_options::output_file_path();
+        ss << filesystem_separator();
+        ss << counter_scatterplot_sample_filename << node_id << ".csv";
+        // open the file
+        _counter_scatterplot_sample_file.open(ss.str(), std::ofstream::out | std::ofstream::app);
+        if (!_counter_scatterplot_sample_file.is_open()) {
+            perror("opening scatterplot sample file");
+        }
+        APEX_ASSERT(_counter_scatterplot_sample_file.is_open());
+      }
+      return _counter_scatterplot_sample_file;
+  }
   void reset(task_identifier * id);
   void reset_all(void);
-  profile * get_profile(task_identifier &id);
+  profile * get_profile(const task_identifier &id);
   double get_non_idle_time(void);
   profile * get_idle_time(void);
   profile * get_idle_rate(void);
-  //std::vector<std::string> get_available_profiles();
+  std::vector<task_identifier>& get_available_profiles() {
+    static std::vector<task_identifier> ids;
+    std::lock_guard<std::mutex> lock(_task_map_mutex);
+    if (task_map.size() > ids.size()) {
+        ids.clear();
+        for (auto kv : task_map) {
+           ids.push_back(kv.first);
+        }
+    }
+    return ids;
+  }
   void process_profiles(void);
   static void process_profiles_wrapper(void);
   static void consumer_process_profiles_wrapper(void);
@@ -236,6 +247,10 @@ public:
   std::vector<std::string>& get_metric_names(void) { return metric_names; };
 #endif
   void stop_main_timer(void);
+  void yield_main_timer(void);
+  void resume_main_timer(void);
+  void increment_main_timer_allocations(double bytes);
+  void increment_main_timer_frees(double bytes);
   void push_profiler_public(std::shared_ptr<profiler> &p);
 };
 
