@@ -1,7 +1,10 @@
-//  Copyright (c) 2014 University of Oregon
-//
-//  Distributed under the Boost Software License, Version 1.0. (See accompanying
-//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+/*
+ * Copyright (c) 2014-2021 Kevin Huck
+ * Copyright (c) 2014-2021 University of Oregon
+ *
+ * Distributed under the Boost Software License, Version 1.0. (See accompanying
+ * file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+ */
 
 #pragma once
 
@@ -14,6 +17,8 @@
 #include <utility>
 #include <cstddef>
 
+constexpr char APEX_MAIN_STR[] = "APEX MAIN";
+
 namespace apex {
 
 class task_identifier {
@@ -59,7 +64,14 @@ public:
 
   static task_identifier * get_task_id (apex_function_address a);
   static task_identifier * get_task_id (const std::string& n);
+  static task_identifier * get_main_task_id () {
+    static const std::string apex_main_str{APEX_MAIN_STR};
+    return get_task_id(apex_main_str);
+  };
   std::string get_name(bool resolve = true);
+  std::string get_short_name();
+  std::string get_tree_name();
+  std::string get_group();
   ~task_identifier() { }
   // requried for using this class as a key in an unordered map.
   // the hash function is defined below.
@@ -100,7 +112,8 @@ namespace std {
     {
       std::size_t h1 = std::hash<std::size_t>()(k.address);
       std::size_t h2 = std::hash<std::string>()(k.name);
-      return h1 ^ (h2 << 1);; // instead of boost::hash_combine
+      std::size_t h3 = h1 ^ (h2 << 1);
+      return h3; // instead of boost::hash_combine
     }
   };
 
