@@ -1,7 +1,10 @@
-//  Copyright (c) 2014-2018 University of Oregon
-//
-//  Distributed under the Boost Software License, Version 1.0. (See accompanying
-//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+/*
+ * Copyright (c) 2014-2021 Kevin Huck
+ * Copyright (c) 2014-2021 University of Oregon
+ *
+ * Distributed under the Boost Software License, Version 1.0. (See accompanying
+ * file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+ */
 
 #pragma once
 
@@ -19,6 +22,17 @@
 #include <cxxabi.h>
 #endif
 #include <atomic>
+#include <regex>
+
+#include "apex_types.h"
+#include "apex_options.hpp"
+
+#ifdef DEBUG
+#define DEBUG_PRINT(...) if (::apex::apex_options::use_verbose()) { \
+fprintf( stderr, __VA_ARGS__ ); fflush(stderr); }
+#else
+#define DEBUG_PRINT(...) do{ } while ( false )
+#endif
 
 namespace apex {
 
@@ -47,6 +61,16 @@ inline std::string& trim(std::string& s, const char* t = " \t\n\r\f\v")
     return ltrim(rtrim(s, t), t);
 }
 
+// remove substrings
+inline void eraseSubStr(std::string &mainStr, const std::string toErase) {
+    // Search for the substring in string
+    size_t pos = mainStr.find(toErase);
+    if (pos != std::string::npos) {
+        // If found then erase it from string
+        mainStr.erase(pos, toErase.length());
+    }
+}
+
 // copying versions
 
 inline std::string ltrim_copy(std::string s, const char* t = " \t\n\r\f\v")
@@ -208,13 +232,20 @@ class reference_counter {
 #define APEX_UTIL_REPORT_STATS
 #endif
 
+std::vector<uint32_t> parseDiscreteValues(std::string inputString);
+std::string getCpusAllowed(const char * filename);
+
 inline unsigned int my_hardware_concurrency()
 {
 #if defined(_MSC_VER)
     return std::thread::hardware_concurrency();
-#else
+#elif defined(__APPLE__)
     unsigned int cores = std::thread::hardware_concurrency();
     return cores ? cores : sysconf(_SC_NPROCESSORS_ONLN);
+#else
+    static std::vector<uint32_t>
+        cores{parseDiscreteValues(getCpusAllowed("/proc/self/status"))};
+    return cores.size();
 #endif
 }
 
@@ -224,7 +255,7 @@ inline unsigned int hardware_concurrency()
     return(hwc);
 }
 
-std::string* demangle(const std::string& timer_name);
+std::string demangle(const std::string& timer_name);
 
 void set_thread_affinity(void);
 void set_thread_affinity(int core);
@@ -243,6 +274,50 @@ inline char filesystem_separator()
 
 uint64_t test_for_MPI_comm_rank(uint64_t commrank);
 uint64_t test_for_MPI_comm_size(uint64_t commsize);
+void handle_delayed_start(void);
+
+std::string activity_to_string(apex_async_activity_t activity);
+
+class node_color {
+public:
+    double red;
+    double green;
+    double blue;
+    node_color() : red(1.0), green(1.0), blue(1.0) {}
+    int convert(double in) { return (int)(in * 255.0); }
+} ;
+
+bool isGPUTimer(std::string name);
+node_color * get_node_color_visible(double v, double vmin, double vmax, std::string name);
+node_color * get_node_color(double v,double vmin,double vmax);
+
+/* Defined in memory_wrapper_shudown.cpp */
+void enable_memory_wrapper(void);
+void disable_memory_wrapper(void);
+
+#include <sys/syscall.h>
+
+class in_apex {
+    public:
+        static size_t& get();
+        in_apex();
+        ~in_apex();
+};
+
+void rank0_print(const char * fmt, ...);
+
+std::string getCommandLine(void);
 
 }
 
+template <typename I>
+std::string hex2str(I w, size_t hex_len = sizeof(I)<<1) {
+    static const char* digits = "0123456789ABCDEF";
+    std::string rc(hex_len,'0');
+    for (size_t i=0, j=(hex_len-1)*4 ; i<hex_len; ++i,j-=4)
+        rc[i] = digits[(w>>j) & 0x0f];
+    std::string full = "0x"+rc;
+    return full;
+}
+
+
