@@ -13,7 +13,6 @@ void c() {
 }
 
 int main (void) {
- 	apex_init(__func__, 0, 1);
     apex_set_use_screen_output(1);
 #pragma omp parallel sections
     {
@@ -24,5 +23,4 @@ int main (void) {
     #pragma omp section
         c();
     }
-    apex_finalize();
 }
\ No newline at end of file
