@@ -1,6 +1,8 @@
 #include "apex_api.hpp"
 
 int main (int argc, char** argv) {
+  APEX_UNUSED(argc);
+  APEX_UNUSED(argv);
   apex::init("apex::cleanup unit test", 0, 1);
   apex::apex_options::use_screen_output(true);
   apex::finalize();
