@@ -3,7 +3,7 @@
 #include <stdio.h>
 
 int foo(int i) {
-  apex_profiler_handle profiler = apex_start(APEX_FUNCTION_ADDRESS, &foo);
+  apex_profiler_handle profiler = apex_start(APEX_FUNCTION_ADDRESS, (const void *)&foo);
   int j = i * i;
   apex_stop(profiler);
   return j;
@@ -14,7 +14,7 @@ int main (int argc, char** argv) {
   apex_init("apex_print_options unit test", 0, 1);
   printf("APEX Version : %s\n", apex_version());
   apex_print_options();
-  apex_profiler_handle profiler = apex_start(APEX_FUNCTION_ADDRESS, &main);
+  apex_profiler_handle profiler = apex_start(APEX_FUNCTION_ADDRESS, (const void *)&main);
   int i,j = 0;
   for (i = 0 ; i < 3 ; i++)
     j += foo(i);
