@@ -5,6 +5,7 @@ from mpi4py import MPI
 
 comm = MPI.COMM_WORLD
 
-print ("Hello! I'm rank %d from %d running in total..." % (comm.rank, comm.size))
+processor_name = MPI.Get_processor_name()
+print ("Hello! I'm rank %d from %d running on %s ..." % (comm.rank, comm.size, processor_name))
 
 comm.Barrier()   # wait for everybody to synchronize _here_
