@@ -8,6 +8,7 @@
 #include <math.h>
 #include <iostream>
 #include <limits.h>
+#include <Profile/CuptiNVTX.h>
 
 //#define TAU_CUPTI_DEBUG_COUNTERS
 //#define TAU_DEBUG_CUPTI
@@ -146,7 +147,7 @@ extern "C" void Tau_cupti_enter_memcpy_event(
 						uint32_t streamId,
 						uint32_t contextId,
 						uint32_t correlationId,
-						int bytes_copied,
+						size_t bytes_copied,
 						int memcpy_type,
 						int taskId);
 
@@ -156,7 +157,7 @@ extern "C" void Tau_cupti_exit_memcpy_event(
 						uint32_t streamId,
 						uint32_t contextId,
 						uint32_t correlationId,
-						int bytes_copied,
+						size_t bytes_copied,
 						int memcpy_type,
 						int taskId);
 
@@ -280,7 +281,7 @@ void Tau_cupti_init(void);
 
 void __attribute__ ((destructor)) Tau_cupti_onunload(void);
 
-void get_values_from_memcpy(const CUpti_CallbackData *info, CUpti_CallbackId id, CUpti_CallbackDomain domain, int &kind, int &count);
+void get_values_from_memcpy(const CUpti_CallbackData *info, CUpti_CallbackId id, CUpti_CallbackDomain domain, int &kind, size_t &count);
 
 int getMemcpyType(int kind);
 int getUnifmemType(int kind);
@@ -375,7 +376,7 @@ int get_task_from_id(int id, int task);
 	str_##n << device->n; \
 	int string_length_##n = strlen(str_##n.str().c_str()) + 1; \
 	char *stored_name_##n = (char*) malloc(sizeof(char)*string_length_##n); \
-	strcpy(stored_name_##n, str_##n.str().c_str()); \
+	strncpy(stored_name_##n,  str_##n.str().c_str(), sizeof(char)*string_length_##n);  \
 	metadata[id].name = (char*)("GPU " SX(n)); \
 	metadata[id].value = stored_name_##n; \
 	id++
