@@ -13,46 +13,36 @@
 #include <string>
 #include <vector>
 
-#include <Profile/L0/i915_utils.h>
+#include <level_zero/layers/zel_tracing_api.h>
+
 #include <Profile/L0/utils.h>
 #include <Profile/L0/ze_utils.h>
 #include <Profile/TauEnv.h>
 
-namespace ze_kernel_collector {
-  const uint32_t kKernelLength = 10;
-  const uint32_t kCallsLength = 12;
-  const uint32_t kSimdLength = 5;
-  const uint32_t kTransferredLength = 20;
-  const uint32_t kTimeLength = 20;
-  const uint32_t kPercentLength = 10;
-} // namespace ze_kernel_collector
-
-struct KernelInstance {
+struct ZeKernelCommand {
   std::string name;
   size_t simd_width;
-  size_t bytes_transferred;
   ze_event_pool_handle_t event_pool;
   ze_event_handle_t event;
-  uint64_t append_time;
-  uint64_t submit_time;
+  uint64_t timer_frequency;
+  uint64_t timer_mask;
 };
 
-struct Kernel {
+struct ZeKernelInfo {
   uint64_t total_time;
   uint64_t min_time;
   uint64_t max_time;
   uint64_t call_count;
   size_t simd_width;
-  size_t bytes_transferred;
 
-  bool operator>(const Kernel& r) const {
+  bool operator>(const ZeKernelInfo& r) const {
     if (total_time != r.total_time) {
       return total_time > r.total_time;
     }
     return call_count > r.call_count;
   }
 
-  bool operator!=(const Kernel& r) const {
+  bool operator!=(const ZeKernelInfo& r) const {
     if (total_time == r.total_time) {
       return call_count != r.call_count;
     }
@@ -60,19 +50,25 @@ struct Kernel {
   }
 };
 
-struct CommandListInfo {
-  std::vector<KernelInstance*> kernel_list;
+struct ZeKernelInterval {
+  std::string name;
+  uint64_t start;
+  uint64_t end;
+};
+
+struct ZeCommandListInfo {
+  std::vector<const ZeKernelCommand*> kernel_command_list;
+  ze_context_handle_t context;
+  ze_device_handle_t device;
   bool immediate;
 };
 
-using KernelInfoMap = std::map<std::string, Kernel>;
-using KernelNameMap = std::map<ze_kernel_handle_t, std::string>;
-using KernelTimePoint = std::chrono::time_point<std::chrono::steady_clock>;
-using CommandListMap = std::map<ze_command_list_handle_t, CommandListInfo>;
+using ZeKernelInfoMap = std::map<std::string, ZeKernelInfo>;
+using ZeKernelIntervalList = std::vector<ZeKernelInterval>;
+using ZeCommandListMap = std::map<ze_command_list_handle_t, ZeCommandListInfo>;
 
 typedef void (*OnKernelFinishCallback)(
     void* data, const std::string& name,
-    uint64_t appended, uint64_t submitted,
     uint64_t started, uint64_t ended);
 
 class ZeKernelCollector {
@@ -80,27 +76,23 @@ class ZeKernelCollector {
 
   static ZeKernelCollector* Create(
       ze_driver_handle_t driver,
-      KernelTimePoint base_time = std::chrono::steady_clock::now(),
       OnKernelFinishCallback callback = nullptr,
       void* callback_data = nullptr) {
-    PTI_ASSERT(driver != nullptr);
-
+    PTI_ASSERT(utils::ze::GetVersion() != ZE_API_VERSION_1_0);
+    
     ze_context_handle_t context = utils::ze::GetContext(driver);
     PTI_ASSERT(context != nullptr);
 
-    ZeKernelCollector* collector = new ZeKernelCollector(
-        context, base_time, callback, callback_data);
+    ZeKernelCollector* collector = new ZeKernelCollector(context, callback, callback_data);
     PTI_ASSERT(collector != nullptr);
 
     ze_result_t status = ZE_RESULT_SUCCESS;
-    zet_tracer_exp_desc_t tracer_desc = {
-        ZET_STRUCTURE_TYPE_TRACER_EXP_DESC, nullptr, collector};
-    zet_tracer_exp_handle_t tracer = nullptr;
-    status = zetTracerExpCreate(context, &tracer_desc, &tracer);
+    zel_tracer_desc_t tracer_desc = {
+        ZEL_STRUCTURE_TYPE_TRACER_EXP_DESC, nullptr, collector};
+    zel_tracer_handle_t tracer = nullptr;
+    status = zelTracerCreate(&tracer_desc, &tracer);
     if (status != ZE_RESULT_SUCCESS) {
-      std::cerr <<
-        "[WARNING] Unable to create Level Zero tracer for target context" <<
-        std::endl;
+      std::cerr << "[WARNING] Unable to create Level Zero tracer" << std::endl;
       delete collector;
       return nullptr;
     }
@@ -109,13 +101,13 @@ class ZeKernelCollector {
     return collector;
   }
 
-  static void PrintKernelsTable(const KernelInfoMap& kernel_info_map) {
-    std::set< std::pair<std::string, Kernel>,
+  static void PrintKernelsTable(const ZeKernelInfoMap& kernel_info_map) {
+    std::set< std::pair<std::string, ZeKernelInfo>,
               utils::Comparator > sorted_list(
         kernel_info_map.begin(), kernel_info_map.end());
 
     uint64_t total_duration = 0;
-    size_t max_name_length = ze_kernel_collector::kKernelLength;
+    size_t max_name_length = kKernelLength;
     for (auto& value : sorted_list) {
       total_duration += value.second.total_time;
       if (value.first.size() > max_name_length) {
@@ -128,79 +120,55 @@ class ZeKernelCollector {
     }
 
     std::cerr << std::setw(max_name_length) << "Kernel" << "," <<
-      std::setw(ze_kernel_collector::kCallsLength) << "Calls" << "," <<
-      std::setw(ze_kernel_collector::kSimdLength) << "SIMD" << "," <<
-      std::setw(ze_kernel_collector::kTransferredLength) <<
-        "Transferred (bytes)" << "," <<
-      std::setw(ze_kernel_collector::kTimeLength) << "Time (ns)" << "," <<
-      std::setw(ze_kernel_collector::kPercentLength) << "Time (%)" << "," <<
-      std::setw(ze_kernel_collector::kTimeLength) << "Average (ns)" << "," <<
-      std::setw(ze_kernel_collector::kTimeLength) << "Min (ns)" << "," <<
-      std::setw(ze_kernel_collector::kTimeLength) << "Max (ns)" << std::endl;
+      std::setw(kCallsLength) << "Calls" << "," <<
+      std::setw(kSimdLength) << "SIMD" << "," <<
+      std::setw(kTimeLength) << "Time (ns)" << "," <<
+      std::setw(kPercentLength) << "Time (%)" << "," <<
+      std::setw(kTimeLength) << "Average (ns)" << "," <<
+      std::setw(kTimeLength) << "Min (ns)" << "," <<
+      std::setw(kTimeLength) << "Max (ns)" << std::endl;
 
     for (auto& value : sorted_list) {
       const std::string& function = value.first;
       uint64_t call_count = value.second.call_count;
       size_t simd_width = value.second.simd_width;
-      size_t bytes_transferred = value.second.bytes_transferred;
       uint64_t duration = value.second.total_time;
       uint64_t avg_duration = duration / call_count;
       uint64_t min_duration = value.second.min_time;
       uint64_t max_duration = value.second.max_time;
       float percent_duration = 100.0f * duration / total_duration;
       std::cerr << std::setw(max_name_length) << function << "," <<
-        std::setw(ze_kernel_collector::kCallsLength) << call_count << "," <<
-        std::setw(ze_kernel_collector::kSimdLength) << simd_width << "," <<
-        std::setw(ze_kernel_collector::kTransferredLength) <<
-          bytes_transferred << "," <<
-        std::setw(ze_kernel_collector::kTimeLength) << duration << "," <<
-        std::setw(ze_kernel_collector::kPercentLength) << std::setprecision(2) <<
+        std::setw(kCallsLength) << call_count << "," <<
+        std::setw(kSimdLength) << simd_width << "," <<
+        std::setw(kTimeLength) << duration << "," <<
+        std::setw(kPercentLength) << std::setprecision(2) <<
           std::fixed << percent_duration << "," <<
-        std::setw(ze_kernel_collector::kTimeLength) << avg_duration << "," <<
-        std::setw(ze_kernel_collector::kTimeLength) << min_duration << "," <<
-        std::setw(ze_kernel_collector::kTimeLength) << max_duration << std::endl;
+        std::setw(kTimeLength) << avg_duration << "," <<
+        std::setw(kTimeLength) << min_duration << "," <<
+        std::setw(kTimeLength) << max_duration << std::endl;
     }
   }
 
   ~ZeKernelCollector() {
-    ze_result_t status = ZE_RESULT_SUCCESS;
-    PTI_ASSERT(kernel_instance_list_.size() == 0);
-
     if (tracer_ != nullptr) {
-      status = zetTracerExpDestroy(tracer_);
+      ze_result_t status = zelTracerDestroy(tracer_);
       PTI_ASSERT(status == ZE_RESULT_SUCCESS);
     }
-
-    PTI_ASSERT(context_ != nullptr);
-    status = zeContextDestroy(context_);
-    PTI_ASSERT(status == ZE_RESULT_SUCCESS);
   }
 
   void DisableTracing() {
     PTI_ASSERT(tracer_ != nullptr);
     ze_result_t status = ZE_RESULT_SUCCESS;
-    status = zetTracerExpSetEnabled(tracer_, false);
+    status = zelTracerSetEnabled(tracer_, false);
     PTI_ASSERT(status == ZE_RESULT_SUCCESS);
   }
 
-  const KernelInfoMap& GetKernelInfoMap() const {
+  const ZeKernelInfoMap& GetKernelInfoMap() const {
     return kernel_info_map_;
   }
 
- private: // Implementation
-
-  ZeKernelCollector(ze_context_handle_t context,
-                    KernelTimePoint base_time,
-                    OnKernelFinishCallback callback,
-                    void* callback_data)
-      : context_(context), base_time_(base_time),
-        callback_(callback), callback_data_(callback_data),
-        timer_frequency_(utils::i915::GetGpuTimerFrequency()) {
-    PTI_ASSERT(context_ != nullptr);
-    PTI_ASSERT(timer_frequency_ > 0);
-    if (callback_ != nullptr) {
-      SetCpuGpuTimestamps();
-    }
+  const ZeKernelIntervalList& GetKernelIntervalList() const {
+    return kernel_interval_list_;
   }
 
   uint64_t GetTimestamp() const {
@@ -209,7 +177,19 @@ class ZeKernelCollector {
     return timestamp.count();
   }
 
-  void EnableTracing(zet_tracer_exp_handle_t tracer) {
+ private: // Implementation
+
+  ZeKernelCollector(ze_context_handle_t context,
+                    OnKernelFinishCallback callback,
+                    void* callback_data):
+                     context_(context),  callback_(callback), callback_data_(callback_data) {
+                     
+                     PTI_ASSERT(context_ != nullptr);
+
+    if (callback_ != nullptr) {TAU_VERBOSE("TAU: <kernel>: callback_ != nullptr");}
+    }
+
+  void EnableTracing(zel_tracer_handle_t tracer) {
     PTI_ASSERT(tracer != nullptr);
     tracer_ = tracer;
 
@@ -217,6 +197,7 @@ class ZeKernelCollector {
     zet_core_callbacks_t epilogue_callbacks{};
 
     prologue_callbacks.Event.pfnDestroyCb = OnEnterEventDestroy;
+    epilogue_callbacks.Event.pfnHostSynchronizeCb = OnExitEventHostSynchronize;
 
     prologue_callbacks.Event.pfnHostResetCb = OnEnterEventHostReset;
 
@@ -228,187 +209,155 @@ class ZeKernelCollector {
     epilogue_callbacks.CommandList.pfnAppendLaunchKernelCb =
       OnExitCommandListAppendLaunchKernel;
 
-    prologue_callbacks.CommandList.pfnAppendMemoryCopyCb =
-      OnEnterCommandListAppendMemoryCopy;
-    epilogue_callbacks.CommandList.pfnAppendMemoryCopyCb =
-      OnExitCommandListAppendMemoryCopy;
-
-    prologue_callbacks.CommandQueue.pfnExecuteCommandListsCb =
-      OnEnterCommandQueueExecuteCommandLists;
-
     epilogue_callbacks.CommandList.pfnCreateCb =
       OnExitCommandListCreate;
     epilogue_callbacks.CommandList.pfnCreateImmediateCb =
-      OnExitCommandListImmediateCreate;
+      OnExitCommandListCreateImmediate;
     epilogue_callbacks.CommandList.pfnDestroyCb =
       OnExitCommandListDestroy;
     epilogue_callbacks.CommandList.pfnResetCb =
       OnExitCommandListReset;
 
-    epilogue_callbacks.Kernel.pfnCreateCb = OnExitKernelCreate;
-
-    epilogue_callbacks.Kernel.pfnDestroyCb = OnExitKernelDestroy;
-
+    epilogue_callbacks.CommandQueue.pfnExecuteCommandListsCb =
+      OnExitCommandQueueExecuteCommandLists;
     epilogue_callbacks.CommandQueue.pfnSynchronizeCb =
       OnExitCommandQueueSynchronize;
     epilogue_callbacks.CommandQueue.pfnDestroyCb =
       OnExitCommandQueueDestroy;
 
     ze_result_t status = ZE_RESULT_SUCCESS;
-    status = zetTracerExpSetPrologues(tracer_, &prologue_callbacks);
+    status = zelTracerSetPrologues(tracer_, &prologue_callbacks);
     PTI_ASSERT(status == ZE_RESULT_SUCCESS);
-    status = zetTracerExpSetEpilogues(tracer_, &epilogue_callbacks);
+    status = zelTracerSetEpilogues(tracer_, &epilogue_callbacks);
     PTI_ASSERT(status == ZE_RESULT_SUCCESS);
-    status = zetTracerExpSetEnabled(tracer_, true);
+    status = zelTracerSetEnabled(tracer_, true);
     PTI_ASSERT(status == ZE_RESULT_SUCCESS);
   }
 
-  void AddKernelName(ze_kernel_handle_t kernel, std::string name) {
-    PTI_ASSERT(kernel != nullptr);
-    PTI_ASSERT(!name.empty());
-
-    const std::lock_guard<std::mutex> lock(lock_);
-    PTI_ASSERT(kernel_name_map_.count(kernel) == 0);
-    kernel_name_map_[kernel] = name;
-  }
-
-  void RemoveKernelName(ze_kernel_handle_t kernel) {
-    PTI_ASSERT(kernel != nullptr);
-
-    const std::lock_guard<std::mutex> lock(lock_);
-    PTI_ASSERT(kernel_name_map_.count(kernel) == 1);
-    kernel_name_map_.erase(kernel);
-  }
-
-  std::string GetKernelName(ze_kernel_handle_t kernel) {
-    PTI_ASSERT(kernel != nullptr);
-
-    const std::lock_guard<std::mutex> lock(lock_);
-    if (kernel_name_map_.count(kernel) == 1) {
-      return kernel_name_map_[kernel];
-    }
-
-    return std::string();
-  }
-
-  void AddKernelInstance(ze_command_list_handle_t command_list,
-                         const KernelInstance& instance) {
+  void AddKernelCommand(
+      ze_command_list_handle_t command_list,
+      const ZeKernelCommand* command) {
     PTI_ASSERT(command_list != nullptr);
+    PTI_ASSERT(command != nullptr);
+
     const std::lock_guard<std::mutex> lock(lock_);
-    kernel_instance_list_.push_back(instance);
-    KernelInstance* kernel_instance = &kernel_instance_list_.back();
 
     PTI_ASSERT(command_list_map_.count(command_list) == 1);
-    CommandListInfo& command_list_info = command_list_map_[command_list];
-    if (command_list_info.immediate) {
-      kernel_instance->submit_time = kernel_instance->append_time;
-    }
-    command_list_info.kernel_list.push_back(kernel_instance);
+    ZeCommandListInfo& info = command_list_map_[command_list];
+    info.kernel_command_list.push_back(command);
   }
 
-  void ProcessInstance(ze_event_handle_t event) {
-    PTI_ASSERT(event != nullptr);
+  void AddKernelCall(const ZeKernelCommand* call) {
+    PTI_ASSERT(call != nullptr);
+    const std::lock_guard<std::mutex> lock(lock_);
+    kernel_call_list_.push_back(call);
+  }
+
+  void AddKernelCalls(
+      ze_command_list_handle_t command_list) {
+    PTI_ASSERT(command_list != nullptr);
+
     const std::lock_guard<std::mutex> lock(lock_);
 
-    for (auto it = kernel_instance_list_.begin();
-         it != kernel_instance_list_.end(); ++it) {
-      if ((*it).event == event) {
-        ProcessInstance(*it);
-        kernel_instance_list_.erase(it);
+    PTI_ASSERT(command_list_map_.count(command_list) == 1);
+    ZeCommandListInfo& info = command_list_map_[command_list];
+    PTI_ASSERT(!info.immediate);
+
+    for (const ZeKernelCommand* command : info.kernel_command_list) {
+      kernel_call_list_.push_back(command);
+    }
+  }
+
+  void ProcessCall(ze_event_handle_t event) {
+    PTI_ASSERT(event != nullptr);
+
+    const std::lock_guard<std::mutex> lock(lock_);
+
+    for (auto it = kernel_call_list_.begin();
+         it != kernel_call_list_.end(); ++it) {
+      const ZeKernelCommand* call = *it;
+      PTI_ASSERT(call != nullptr);
+      if (call->event == event) {
+        ProcessCall(call);
+        kernel_call_list_.erase(it);
         break;
       }
     }
   }
 
-  void ProcessInstance(const KernelInstance& instance) {
+  void ProcessCall(const ZeKernelCommand* call) {
+    PTI_ASSERT(call != nullptr);
+
     ze_result_t status = ZE_RESULT_SUCCESS;
-    status = zeEventQueryStatus(instance.event);
+    status = zeEventQueryStatus(call->event);
     PTI_ASSERT(status == ZE_RESULT_SUCCESS);
 
     ze_kernel_timestamp_result_t timestamp{};
-    status = zeEventQueryKernelTimestamp(instance.event, &timestamp);
+    status = zeEventQueryKernelTimestamp(call->event, &timestamp);
     PTI_ASSERT(status == ZE_RESULT_SUCCESS);
 
-
     uint64_t start = timestamp.global.kernelStart;
     uint64_t end = timestamp.global.kernelEnd;
-    uint64_t time = 0;
+    uint64_t freq = call->timer_frequency;
+    PTI_ASSERT(freq > 0);
+    uint64_t mask = call->timer_mask;
+    PTI_ASSERT(mask > 0);
+
+    uint64_t time = 0, start_ns = 0, end_ns = 0;
+
+    start_ns = start *
+      static_cast<uint64_t>(NSEC_IN_SEC) / freq;
     if (start < end) {
-      time = (end - start) *
-        static_cast<uint64_t>(NSEC_IN_SEC) / timer_frequency_;
+      end_ns = end *
+        static_cast<uint64_t>(NSEC_IN_SEC) / freq;
     } else { // 32-bit timer overflow
       PTI_ASSERT(start < (1ULL << 32));
-      time = ((1ULL << 32) - start + end) *
-        static_cast<uint64_t>(NSEC_IN_SEC) / timer_frequency_;
+      end_ns = ((1ULL << 32) + end) *
+        static_cast<uint64_t>(NSEC_IN_SEC) / freq;
     }
-    AddKernelInfo(instance.name, time,
-                  instance.simd_width,
-                  instance.bytes_transferred);
-
-    if (callback_ != nullptr) {
-      PTI_ASSERT(instance.append_time > 0);
-      PTI_ASSERT(instance.submit_time > 0);
-
-      uint64_t cpu_start = 0, cpu_end = 0;
-      std::chrono::duration<uint64_t, std::nano> time_shift =
-        cpu_timestamp_ - base_time_;
-
-      if (gpu_timestamp_ < start) {
-        cpu_start = (start - gpu_timestamp_) *
-          static_cast<uint64_t>(NSEC_IN_SEC) / timer_frequency_ +
-          time_shift.count();
-        cpu_end = cpu_start + time;
-      } else { // 32-bit timer overflow
-        SetCpuGpuTimestamps();
-        time_shift = cpu_timestamp_ - base_time_;
-        PTI_ASSERT(start < gpu_timestamp_);
-        cpu_start = time_shift.count() - (gpu_timestamp_ - start) *
-          static_cast<uint64_t>(NSEC_IN_SEC) / timer_frequency_;
-        cpu_end = cpu_start + time;
-      }
-
-      callback_(callback_data_, instance.name,
-                instance.append_time, instance.submit_time,
-                cpu_start, cpu_end);
-
-    }
-
-    if (instance.event_pool != nullptr) {
-      ze_result_t status = ZE_RESULT_SUCCESS;
-      status = zeEventDestroy(instance.event);
-      PTI_ASSERT(status == ZE_RESULT_SUCCESS);
-      status = zeEventPoolDestroy(instance.event_pool);
-      PTI_ASSERT(status == ZE_RESULT_SUCCESS);
+    time = end_ns - start_ns;
+    //mask -> max value before overflow, appears as 343597383600 nanoseconds, aprox 343 seconds
+    //printf("%lu %lu %lu %lu %lu %lu\n", mask, mask*static_cast<uint64_t>(NSEC_IN_SEC) / freq, (mask*static_cast<uint64_t>(NSEC_IN_SEC) / freq)/NSEC_IN_SEC, freq, start_ns, end_ns);
+    AddKernelInfo(call->name, time, call->simd_width);
+    AddKernelInterval(call->name, start_ns, end_ns);
+    
+    if(callback_ != nullptr){
+    	callback_(callback_data_, call->name,
+                start_ns, end_ns);
     }
+    
+    
   }
 
-  void ProcessInstances() {
+  void ProcessCalls() {
     ze_result_t status = ZE_RESULT_SUCCESS;
+
     const std::lock_guard<std::mutex> lock(lock_);
 
-    auto it = kernel_instance_list_.begin();
-    while (it != kernel_instance_list_.end()) {
-      PTI_ASSERT(it->event != nullptr);
-      status = zeEventQueryStatus(it->event);
+    auto it = kernel_call_list_.begin();
+    while (it != kernel_call_list_.end()) {
+      const ZeKernelCommand* call = *it;
+      PTI_ASSERT(call != nullptr);
+      PTI_ASSERT(call->event != nullptr);
+      status = zeEventQueryStatus(call->event);
       if (status == ZE_RESULT_NOT_READY) {
         ++it;
       } else if (status == ZE_RESULT_SUCCESS) {
-        ProcessInstance(*it);
-        it = kernel_instance_list_.erase(it);
+        ProcessCall(call);
+        it = kernel_call_list_.erase(it);
       } else {
         PTI_ASSERT(0);
       }
     }
   }
 
-  void AddKernelInfo(std::string name, uint64_t time,
-                     size_t simd_width, size_t bytes_transferred) {
+  void AddKernelInfo(
+      std::string name, uint64_t time, size_t simd_width) {
     PTI_ASSERT(!name.empty());
     if (kernel_info_map_.count(name) == 0) {
-      kernel_info_map_[name] = {
-        time, time, time, 1, simd_width, bytes_transferred};
+      kernel_info_map_[name] = {time, time, time, 1, simd_width};
     } else {
-      Kernel& kernel = kernel_info_map_[name];
+      ZeKernelInfo& kernel = kernel_info_map_[name];
       kernel.total_time += time;
       if (time > kernel.max_time) {
         kernel.max_time = time;
@@ -417,78 +366,91 @@ class ZeKernelCollector {
         kernel.min_time = time;
       }
       kernel.call_count += 1;
-      kernel.bytes_transferred += bytes_transferred;
-      PTI_ASSERT(kernel.simd_width == simd_width);
+      kernel.simd_width = std::max(kernel.simd_width, simd_width);
     }
   }
 
-  void AddCommandList(ze_command_list_handle_t command_list, bool immediate) {
+  void AddKernelInterval(std::string name, uint64_t start, uint64_t end) {
+    PTI_ASSERT(!name.empty());
+    PTI_ASSERT(start < end);
+    kernel_interval_list_.push_back({name, start, end});
+  }
+
+  void AddCommandList(
+      ze_command_list_handle_t command_list,
+      ze_context_handle_t context,
+      ze_device_handle_t device,
+      bool immediate) {
     PTI_ASSERT(command_list != nullptr);
+    PTI_ASSERT(context != nullptr);
     const std::lock_guard<std::mutex> lock(lock_);
     PTI_ASSERT(command_list_map_.count(command_list) == 0);
-    command_list_map_[command_list] = {std::vector<KernelInstance*>(), immediate};
+    command_list_map_[command_list] = {
+        std::vector<const ZeKernelCommand*>(), context, device, immediate};
+  }
+
+  void RemoveKernelCommands(ze_command_list_handle_t command_list) {
+    PTI_ASSERT(command_list != nullptr);
+
+    PTI_ASSERT(command_list_map_.count(command_list) == 1);
+    ZeCommandListInfo& info = command_list_map_[command_list];
+    for (const ZeKernelCommand* command : info.kernel_command_list) {
+      if (command->event_pool != nullptr) {
+        ze_result_t status = ZE_RESULT_SUCCESS;
+        status = zeEventDestroy(command->event);
+        PTI_ASSERT(status == ZE_RESULT_SUCCESS);
+        status = zeEventPoolDestroy(command->event_pool);
+        PTI_ASSERT(status == ZE_RESULT_SUCCESS);
+      }
+
+      for (const ZeKernelCommand* call : kernel_call_list_) {
+        PTI_ASSERT(call != command);
+      }
+
+      delete command;
+    }
+    info.kernel_command_list.clear();
   }
 
   void RemoveCommandList(ze_command_list_handle_t command_list) {
     PTI_ASSERT(command_list != nullptr);
     const std::lock_guard<std::mutex> lock(lock_);
-    PTI_ASSERT(command_list_map_.count(command_list) == 1);
+    RemoveKernelCommands(command_list);
     command_list_map_.erase(command_list);
   }
 
   void ResetCommandList(ze_command_list_handle_t command_list) {
     PTI_ASSERT(command_list != nullptr);
     const std::lock_guard<std::mutex> lock(lock_);
-    PTI_ASSERT(command_list_map_.count(command_list) == 1);
-    command_list_map_[command_list].kernel_list.clear();
+    RemoveKernelCommands(command_list);
   }
 
-  void SetKernelInstanceSubmitTime(ze_command_list_handle_t command_list,
-                                   uint64_t submit_time) {
+  ze_context_handle_t GetCommandListContext(
+      ze_command_list_handle_t command_list) {
     PTI_ASSERT(command_list != nullptr);
     const std::lock_guard<std::mutex> lock(lock_);
     PTI_ASSERT(command_list_map_.count(command_list) == 1);
-    CommandListInfo& command_list_info = command_list_map_[command_list];
-    if (!command_list_info.immediate) {
-      std::vector<KernelInstance*>& kernel_list =
-        command_list_info.kernel_list;
-      for (size_t i = 0; i < kernel_list.size(); ++i) {
-        kernel_list[i]->submit_time = submit_time;
-      }
-    }
+    return command_list_map_[command_list].context;
   }
 
-  void SetCpuGpuTimestamps() {
-    gpu_timestamp_ = utils::i915::GetGpuTimestamp() & 0x0FFFFFFFF;
-    cpu_timestamp_ = std::chrono::steady_clock::now();
+  ze_device_handle_t GetCommandListDevice(
+      ze_command_list_handle_t command_list) {
+    PTI_ASSERT(command_list != nullptr);
+    const std::lock_guard<std::mutex> lock(lock_);
+    PTI_ASSERT(command_list_map_.count(command_list) == 1);
+    return command_list_map_[command_list].device;
+  }
+
+  bool IsCommandListImmediate(ze_command_list_handle_t command_list) {
+    PTI_ASSERT(command_list != nullptr);
+    const std::lock_guard<std::mutex> lock(lock_);
+    PTI_ASSERT(command_list_map_.count(command_list) == 1);
+    ZeCommandListInfo& command_list_info = command_list_map_[command_list];
+    return command_list_info.immediate;
   }
 
  private: // Callbacks
 
-  static void OnExitKernelCreate(
-      ze_kernel_create_params_t *params, ze_result_t result,
-      void *global_data, void **instance_data) {
-    if (result == ZE_RESULT_SUCCESS) {
-      ZeKernelCollector* collector =
-        reinterpret_cast<ZeKernelCollector*>(global_data);
-      PTI_ASSERT(collector != nullptr);
-      collector->AddKernelName(
-          **(params->pphKernel), (*(params->pdesc))->pKernelName);
-    }
-  }
-
-  static void OnExitKernelDestroy(ze_kernel_destroy_params_t *params,
-                                  ze_result_t result,
-                                  void *global_data,
-                                  void **instance_data) {
-    if (result == ZE_RESULT_SUCCESS) {
-      ZeKernelCollector* collector =
-        reinterpret_cast<ZeKernelCollector*>(global_data);
-      PTI_ASSERT(collector != nullptr);
-      collector->RemoveKernelName(*(params->phKernel));
-    }
-  }
-
   static void OnEnterEventPoolCreate(ze_event_pool_create_params_t *params,
                                      ze_result_t result,
                                      void *global_data,
@@ -506,7 +468,9 @@ class ZeKernelCollector {
     profiling_desc->stype = desc->stype;
     // PTI_ASSERT(profiling_desc->stype == ZE_STRUCTURE_TYPE_EVENT_POOL_DESC);
     profiling_desc->pNext = desc->pNext;
-    profiling_desc->flags = (desc->flags | ZE_EVENT_POOL_FLAG_KERNEL_TIMESTAMP);
+    profiling_desc->flags = desc->flags;
+    profiling_desc->flags |= ZE_EVENT_POOL_FLAG_KERNEL_TIMESTAMP;
+    profiling_desc->flags |= ZE_EVENT_POOL_FLAG_HOST_VISIBLE;
     profiling_desc->count = desc->count;
 
     *(params->pdesc) = profiling_desc;
@@ -532,7 +496,7 @@ class ZeKernelCollector {
       ZeKernelCollector* collector =
         reinterpret_cast<ZeKernelCollector*>(global_data);
       PTI_ASSERT(collector != nullptr);
-      collector->ProcessInstance(*(params->phEvent));
+      collector->ProcessCall(*(params->phEvent));
     }
   }
 
@@ -544,7 +508,20 @@ class ZeKernelCollector {
       ZeKernelCollector* collector =
         reinterpret_cast<ZeKernelCollector*>(global_data);
       PTI_ASSERT(collector != nullptr);
-      collector->ProcessInstance(*(params->phEvent));
+      collector->ProcessCall(*(params->phEvent));
+    }
+  }
+
+ static void OnExitEventHostSynchronize(ze_event_host_synchronize_params_t *params,
+                                    ze_result_t result,
+                                    void *global_data,
+                                    void **instance_data) {
+    if (*(params->phEvent) != nullptr) {
+      ZeKernelCollector* collector =
+        reinterpret_cast<ZeKernelCollector*>(global_data);
+      PTI_ASSERT(collector != nullptr);
+    collector->ProcessCall(*(params->phEvent));
+    collector->ProcessCalls();
     }
   }
 
@@ -556,7 +533,8 @@ class ZeKernelCollector {
 
     ze_event_pool_desc_t event_pool_desc = {
         ZE_STRUCTURE_TYPE_EVENT_POOL_DESC, nullptr,
-        ZE_EVENT_POOL_FLAG_KERNEL_TIMESTAMP, 1};
+        ZE_EVENT_POOL_FLAG_KERNEL_TIMESTAMP | ZE_EVENT_POOL_FLAG_HOST_VISIBLE,
+        1};
     status = zeEventPoolCreate(
         context, &event_pool_desc, 0, nullptr, &event_pool);
     PTI_ASSERT(status == ZE_RESULT_SUCCESS);
@@ -568,92 +546,87 @@ class ZeKernelCollector {
     PTI_ASSERT(status == ZE_RESULT_SUCCESS);
   }
 
-  static void OnEnterCommandListAppendLaunchKernel(
-      ze_command_list_append_launch_kernel_params_t* params,
-      ze_result_t result, void* global_data, void** instance_data) {
+  static void OnEnterKernelAppend(
+      std::string name, size_t simd_width,
+      ze_event_handle_t& signal_event, ze_command_list_handle_t command_list,
+      void* global_data, void** instance_data) {
+    PTI_ASSERT(!name.empty());
+
     ZeKernelCollector* collector =
         reinterpret_cast<ZeKernelCollector*>(global_data);
     PTI_ASSERT(collector != nullptr);
 
-    if (*(params->phKernel) ==  nullptr) {
+    if (command_list == nullptr) {
       return;
     }
 
-    KernelInstance* instance = new KernelInstance;
-    PTI_ASSERT(instance != nullptr);
-    instance->name = collector->GetKernelName(*(params->phKernel));
-    PTI_ASSERT(!instance->name.empty());
+    ZeKernelCommand* command = new ZeKernelCommand;
+    PTI_ASSERT(command != nullptr);
 
-    ze_kernel_properties_t props{};
-    ze_result_t status = zeKernelGetProperties(*(params->phKernel), &props);
-    PTI_ASSERT(status == ZE_RESULT_SUCCESS);
-    instance->simd_width = props.maxSubgroupSize;
-    instance->bytes_transferred = 0;
-    instance->append_time = collector->GetTimestamp();
-    instance->submit_time = 0;
+    command->name = name;
+    command->simd_width = simd_width;
 
-    if (*(params->phSignalEvent) == nullptr) {
-      CreateEvent(collector->context_, instance->event_pool, instance->event);
-      *(params->phSignalEvent) = instance->event;
+    ze_device_handle_t device = collector->GetCommandListDevice(command_list);
+    PTI_ASSERT(device != nullptr);
+    command->timer_frequency = utils::ze::GetDeviceTimerFrequency(device);
+    PTI_ASSERT(command->timer_frequency > 0);
+      command->timer_mask = utils::ze::GetDeviceTimestampMask(device);
+    PTI_ASSERT(command->timer_frequency > 0);
+
+    if (signal_event == nullptr) {
+      ze_context_handle_t context =
+        collector->GetCommandListContext(command_list);
+      CreateEvent(context, command->event_pool, command->event);
+      signal_event = command->event;
     } else {
-      instance->event_pool = nullptr;
-      instance->event = *(params->phSignalEvent);
+      command->event_pool = nullptr;
+      command->event = signal_event;
     }
 
-    *instance_data = static_cast<void*>(instance);
+    *instance_data = static_cast<void*>(command);
   }
 
-  static void OnEnterCommandListAppendMemoryCopy(
-      ze_command_list_append_memory_copy_params_t* params,
+  static void OnEnterCommandListAppendLaunchKernel(
+      ze_command_list_append_launch_kernel_params_t* params,
       ze_result_t result, void* global_data, void** instance_data) {
-    ZeKernelCollector* collector =
-        reinterpret_cast<ZeKernelCollector*>(global_data);
-    PTI_ASSERT(collector != nullptr);
-
-    KernelInstance* instance = new KernelInstance;
-    instance->name = "zeCommandListAppendMemoryCopy";
-    instance->bytes_transferred = *(params->psize);
-    instance->simd_width = 0;
-    instance->append_time = collector->GetTimestamp();
-    instance->submit_time = 0;
-
-    if (*(params->phSignalEvent) == nullptr) {
-      CreateEvent(collector->context_, instance->event_pool, instance->event);
-      *(params->phSignalEvent) = instance->event;
-    } else {
-      instance->event_pool = nullptr;
-      instance->event = *(params->phSignalEvent);
-    }
-
-    *instance_data = static_cast<void*>(instance);
+    OnEnterKernelAppend(
+        utils::ze::GetKernelName(*(params->phKernel)),
+        utils::ze::GetKernelMaxSubgroupSize(*(params->phKernel)),
+        *(params->phSignalEvent), *(params->phCommandList),
+        global_data, instance_data);
   }
 
-  static void OnExitKernelAppend(ze_command_list_handle_t command_list,
-                                 void* global_data, void** instance_data,
-                                 ze_result_t result) {
+  static void OnExitKernelAppend(
+      ze_command_list_handle_t command_list,
+      void* global_data, void** instance_data,
+      ze_result_t result) {
     PTI_ASSERT(command_list != nullptr);
 
-    KernelInstance* instance = static_cast<KernelInstance*>(*instance_data);
-    if (instance == nullptr) {
+    ZeKernelCommand* command =
+      static_cast<ZeKernelCommand*>(*instance_data);
+    if (command == nullptr) {
       return;
     }
 
     if (result != ZE_RESULT_SUCCESS) {
-      if (instance->event_pool != nullptr) {
+      if (command->event_pool != nullptr) {
         ze_result_t status = ZE_RESULT_SUCCESS;
-        status = zeEventDestroy(instance->event);
+        status = zeEventDestroy(command->event);
         PTI_ASSERT(status == ZE_RESULT_SUCCESS);
-        status = zeEventPoolDestroy(instance->event_pool);
+        status = zeEventPoolDestroy(command->event_pool);
         PTI_ASSERT(status == ZE_RESULT_SUCCESS);
       }
+
+      delete command;
     } else {
       ZeKernelCollector* collector =
         reinterpret_cast<ZeKernelCollector*>(global_data);
       PTI_ASSERT(collector != nullptr);
-      collector->AddKernelInstance(command_list, *instance);
+      collector->AddKernelCommand(command_list, command);
+      if (collector->IsCommandListImmediate(command_list)) {
+        collector->AddKernelCall(command);
+      }
     }
-
-    delete instance;
   }
 
   static void OnExitCommandListAppendLaunchKernel(
@@ -664,14 +637,6 @@ class ZeKernelCollector {
                        instance_data, result);
   }
 
-  static void OnExitCommandListAppendMemoryCopy(
-      ze_command_list_append_memory_copy_params_t* params,
-      ze_result_t result, void* global_data, void** instance_data) {
-    PTI_ASSERT(*(params->phSignalEvent) != nullptr);
-    OnExitKernelAppend(*params->phCommandList, global_data,
-                       instance_data, result);
-  }
-
   static void OnExitCommandListCreate(
       ze_command_list_create_params_t* params,
       ze_result_t result, void* global_data, void** instance_data) {
@@ -680,11 +645,15 @@ class ZeKernelCollector {
       ZeKernelCollector* collector =
         reinterpret_cast<ZeKernelCollector*>(global_data);
       PTI_ASSERT(collector != nullptr);
-      collector->AddCommandList(**params->pphCommandList, false);
+      collector->AddCommandList(
+          **(params->pphCommandList),
+          *(params->phContext),
+          *(params->phDevice),
+          false);
     }
   }
 
-  static void OnExitCommandListImmediateCreate(
+  static void OnExitCommandListCreateImmediate(
       ze_command_list_create_immediate_params_t* params,
       ze_result_t result, void* global_data, void** instance_data) {
     if (result == ZE_RESULT_SUCCESS) {
@@ -692,7 +661,11 @@ class ZeKernelCollector {
       ZeKernelCollector* collector =
         reinterpret_cast<ZeKernelCollector*>(global_data);
       PTI_ASSERT(collector != nullptr);
-      collector->AddCommandList(**params->pphCommandList, true);
+      collector->AddCommandList(
+          **(params->pphCommandList),
+          *(params->phContext),
+          *(params->phDevice),
+          true);
     }
   }
 
@@ -704,7 +677,7 @@ class ZeKernelCollector {
       ZeKernelCollector* collector =
         reinterpret_cast<ZeKernelCollector*>(global_data);
       PTI_ASSERT(collector != nullptr);
-      collector->ProcessInstances();
+      collector->ProcessCalls();
       collector->RemoveCommandList(*params->phCommandList);
     }
   }
@@ -717,23 +690,26 @@ class ZeKernelCollector {
       ZeKernelCollector* collector =
         reinterpret_cast<ZeKernelCollector*>(global_data);
       PTI_ASSERT(collector != nullptr);
-      collector->ProcessInstances();
+      collector->ProcessCalls();
       collector->ResetCommandList(*params->phCommandList);
     }
   }
 
-  static void OnEnterCommandQueueExecuteCommandLists(
+  static void OnExitCommandQueueExecuteCommandLists(
       ze_command_queue_execute_command_lists_params_t* params,
       ze_result_t result, void* global_data, void** instance_data) {
-    ZeKernelCollector* collector =
-      reinterpret_cast<ZeKernelCollector*>(global_data);
-    PTI_ASSERT(collector != nullptr);
+    if (result == ZE_RESULT_SUCCESS) {
+      ZeKernelCollector* collector =
+        reinterpret_cast<ZeKernelCollector*>(global_data);
+      PTI_ASSERT(collector != nullptr);
 
-    uint64_t submit_time = collector->GetTimestamp();
-    uint32_t command_list_count = *params->pnumCommandLists;
-    ze_command_list_handle_t* command_lists = *params->pphCommandLists;
-    for (uint32_t i = 0; i < command_list_count; ++i) {
-      collector->SetKernelInstanceSubmitTime(command_lists[i], submit_time);
+      uint32_t command_list_count = *params->pnumCommandLists;
+      ze_command_list_handle_t* command_lists = *params->pphCommandLists;
+      for (uint32_t i = 0; i < command_list_count; ++i) {
+        if (!collector->IsCommandListImmediate(command_lists[i])) {
+          collector->AddKernelCalls(command_lists[i]);
+        }
+      }
     }
   }
 
@@ -744,7 +720,7 @@ class ZeKernelCollector {
       ZeKernelCollector* collector =
         reinterpret_cast<ZeKernelCollector*>(global_data);
       PTI_ASSERT(collector != nullptr);
-      collector->ProcessInstances();
+      collector->ProcessCalls();
     }
   }
 
@@ -755,29 +731,30 @@ class ZeKernelCollector {
       ZeKernelCollector* collector =
         reinterpret_cast<ZeKernelCollector*>(global_data);
       PTI_ASSERT(collector != nullptr);
-      collector->ProcessInstances();
+      collector->ProcessCalls();
     }
   }
+  
 
  private: // Data
-
+  zel_tracer_handle_t tracer_ = nullptr;
   ze_context_handle_t context_ = nullptr;
-  zet_tracer_exp_handle_t tracer_ = nullptr;
-
-  uint64_t timer_frequency_ = 0;
-  KernelTimePoint base_time_;
-
+  
   OnKernelFinishCallback callback_ = nullptr;
   void* callback_data_ = nullptr;
 
-  KernelTimePoint cpu_timestamp_;
-  uint64_t gpu_timestamp_ = 0;
-
   std::mutex lock_;
-  KernelInfoMap kernel_info_map_;
-  KernelNameMap kernel_name_map_;
-  std::list<KernelInstance> kernel_instance_list_;
-  CommandListMap command_list_map_;
+  ZeKernelInfoMap kernel_info_map_;
+  ZeKernelIntervalList kernel_interval_list_;
+  std::list<const ZeKernelCommand*> kernel_call_list_;
+  ZeCommandListMap command_list_map_;
+  std::chrono::time_point<std::chrono::steady_clock> base_time_;
+
+  static const uint32_t kKernelLength = 10;
+  static const uint32_t kCallsLength = 12;
+  static const uint32_t kSimdLength = 5;
+  static const uint32_t kTimeLength = 20;
+  static const uint32_t kPercentLength = 10;
 };
 
 #endif // PTI_SAMPLES_ZE_HOT_KERNELS_ZE_KERNEL_COLLECTOR_H_
