@@ -21,6 +21,11 @@
 #include <string>
 #include <functional>
 #include <utility>
+#include <vector>
+#include <mutex>
+#include <atomic>
+#include <unordered_set>
+using namespace std;
 
 //////////////////////////////////////////////////////////////////////
 //
@@ -31,20 +36,35 @@
 // the node id, thread id etc. When Threads are implemented, myThread()
 // method should return the thread id in {0..N-1} where N is the total
 // number of threads. All interaction with the outside world should be
-// restrained to this class. 
+// restrained to this class.
 //////////////////////////////////////////////////////////////////////
 
 typedef std::map<std::string, TauGroup_t, std::less<std::string> > ProfileMap_t;
 
 
+
 double TauWindowsUsecD(void);
 
 class RtsLayer {
   // Layer for Profiler to interact with the Runtime System
 public:
- 	
+
   RtsLayer () { }  // defaults
-  ~RtsLayer () { } 
+  ~RtsLayer () { }
+
+struct TAULocks{
+    int lockDBCount=0;
+    int lockEnvCount=0;
+};
+ struct LockList : vector<TAULocks *>{
+      LockList(){
+         //printf("Creating CapiThreadList at %p\n", this);
+      }
+     virtual ~LockList(){
+         //printf("Destroying CapiThreadList at %p, with size %ld\n", this, this->size());
+         Tau_destructor_trigger();
+     }
+   };
 
   static int _createThread(void);
   static int createThread(void);
@@ -56,6 +76,14 @@ public:
 #endif
 
   static TauGroup_t & TheProfileMask(void);
+  static TauGroup_t & TheProfileBlackMask(void);
+  // Enum to define the spatical exclusion mode 
+  enum class SpatialExclusionMode { UNSET, INCLUDE, EXCLUDE };
+  static std::atomic<bool>& TheExcludeDefaultGroup(void);
+  static std::unordered_set<int>& TheRankExclusionSet(void);
+  static SpatialExclusionMode& TheRankExclusionMode(void);
+  static std::atomic<uint64_t>& TheRankExclusionVersion();
+  static void InvalidateRankCache()   { TheRankExclusionVersion().fetch_add(1, std::memory_order_relaxed); }
   static bool& TheEnableInstrumentation(void);
   static bool& TheShutdown(void);
   static int& TheNode(void);
@@ -72,18 +100,96 @@ public:
   static TauGroup_t disableAllGroups(void) ;
   static TauGroup_t resetProfileGroup(void) ;
   static int setAndParseProfileGroups (char *prog, char *str) ;
-  static bool isEnabled(TauGroup_t  ProfileGroup) ; 
+  static bool isEnabled(TauGroup_t  ProfileGroup) ;
   static void ProfileInit(int& argc, char**& argv);
   static std::string PrimaryGroup(const char *ProfileGroupName);
   static bool isCtorDtor(const char *name);
 
-  static std::string GetRTTI(const char *name); 
+  static std::string GetRTTI(const char *name);
   inline static const char * CheckNotNull(const char * str) {
     if (str) return str;
     else return "  ";
   }
 
-	static void Initialize(void);
+  static LockList & TheLockList() {
+    static LockList threadList;
+    return threadList;
+}
+  static std::mutex DBVectorMutex;
+
+  //static thread_local int local_lock_tid = RtsLayer::myThread();
+  //static thread_local TAULocks* TL_cache=0;
+  static std::atomic<int> maxLockTid;
+
+  static inline void checkLockVector(unsigned int tid){
+    if(maxLockTid>=0 && (unsigned int)maxLockTid>=tid){
+        //printf("Tid: %d vs max: %d. No check needed?\n",tid,maxLockTid);
+        return;
+    }
+
+      if(TheLockList().size()<=tid){
+      std::lock_guard<std::mutex> guard(RtsLayer::DBVectorMutex);
+      maxLockTid=tid;
+      while(TheLockList().size()<=tid){
+          TheLockList().push_back(new TAULocks());
+
+      }
+      }
+  }
+
+  inline static int getLockVecSize(){
+      return TheLockList().size();
+  }
+
+  static int getDBLock(int tid){
+      checkLockVector(tid);
+      std::lock_guard<std::mutex> guard(RtsLayer::DBVectorMutex);
+    return TheLockList()[tid]->lockDBCount;
+  }
+
+  inline static void setDBLock(int tid, int value){
+      checkLockVector(tid);
+      std::lock_guard<std::mutex> guard(RtsLayer::DBVectorMutex);
+    TheLockList()[tid]->lockDBCount=value;
+  }
+
+  inline static void incrementDBLock(int tid){
+      checkLockVector(tid);
+      std::lock_guard<std::mutex> guard(RtsLayer::DBVectorMutex);
+    TheLockList()[tid]->lockDBCount++;
+  }
+
+  inline static void decrementDBLock(int tid){
+      checkLockVector(tid);
+      std::lock_guard<std::mutex> guard(RtsLayer::DBVectorMutex);
+    TheLockList()[tid]->lockDBCount--;
+  }
+
+  inline static int getEnvLock(int tid){
+      checkLockVector(tid);
+      std::lock_guard<std::mutex> guard(RtsLayer::DBVectorMutex);
+    return TheLockList()[tid]->lockEnvCount;
+  }
+
+  inline static void setEnvLock(int tid, int value){
+      checkLockVector(tid);
+      std::lock_guard<std::mutex> guard(RtsLayer::DBVectorMutex);
+    TheLockList()[tid]->lockEnvCount=value;
+  }
+
+  inline static void incrementEnvLock(int tid){
+      checkLockVector(tid);
+      std::lock_guard<std::mutex> guard(RtsLayer::DBVectorMutex);
+    TheLockList()[tid]->lockEnvCount++;
+  }
+
+  inline static void decrementEnvLock(int tid){
+      checkLockVector(tid);
+      std::lock_guard<std::mutex> guard(RtsLayer::DBVectorMutex);
+    TheLockList()[tid]->lockEnvCount--;
+  }
+
+  static void Initialize(void);
 
   static int 	SetEventCounter(void);
   static double GetEventCounter(void);
@@ -96,8 +202,8 @@ public:
 
   static int setMyContext(int ContextId);
 
-  static const char* getSingleCounterName(); 
-  static const char* getCounterName(int i); 
+  static const char* getSingleCounterName();
+  static const char* getCounterName(int i);
 
   // Return the number of the 'current' node.
   static int myNode(void);
@@ -109,8 +215,8 @@ public:
   static int myThread(void);
 
   static int unsafeThreadId(void);
- 
- 	// Return the local thread id (ignoring tasks) This is a 
+
+ 	// Return the local thread id (ignoring tasks) This is a
 	// low-overhead call but DO NOT use this call when
 	// accessing Profiler stack or the FunctionInfo DB.
   static int localThreadId(void);
@@ -118,13 +224,14 @@ public:
 
   static int getPid();
   static int getTid();
+  static bool isMainThread();
 
 #ifdef KTAU_NG
   static int getLinuxKernelTid();
 #endif /* KTAU_NG */
 
   static int RegisterThread();
-	
+
   static void RegisterFork(int nodeid, enum TauFork_t opcode);
 
   // This ensure that the FunctionDB (global) is locked while updating
@@ -147,8 +254,8 @@ private:
   static void threadLockEnv(void);
   static void threadUnLockEnv(void);
 
-  static int lockDBCount[TAU_MAX_THREADS];
-  static int lockEnvCount[TAU_MAX_THREADS];
+  static int& lockDBCount();
+  static int& lockEnvCount();
 
   static bool initLocks();
   static bool initEnvLocks();
@@ -163,5 +270,5 @@ extern "C" int Tau_RtsLayer_TheEnableInstrumentation();
 /***************************************************************************
  * $RCSfile: RtsLayer.h,v $   $Author: amorris $
  * $Revision: 1.34 $   $Date: 2010/04/08 23:07:41 $
- * POOMA_VERSION_ID: $Id: RtsLayer.h,v 1.34 2010/04/08 23:07:41 amorris Exp $ 
+ * POOMA_VERSION_ID: $Id: RtsLayer.h,v 1.34 2010/04/08 23:07:41 amorris Exp $
  ***************************************************************************/
