@@ -136,6 +136,16 @@
 #define TAU_ENABLE_INSTRUMENTATION()		Tau_enable_instrumentation();
 #define TAU_DISABLE_INSTRUMENTATION()		Tau_disable_instrumentation();
 
+/* Runtime Control */
+#define TAU_INCLUDE_FUNCTION(timer)                 Tau_include_function(timer)
+#define TAU_EXCLUDE_FUNCTION(timer)                 Tau_exclude_function(timer)
+#define TAU_INCLUDE_FUNCTION_BY_NAME(timer_name)        Tau_include_function_by_name(timer_name)
+#define TAU_EXCLUDE_FUNCTION_BY_NAME(timer_name)        Tau_exclude_function_by_name(timer_name)
+#define TAU_ENABLE_FUNCTION_EXCLUSION()					Tau_enable_function_exclusion()
+#define TAU_DISABLE_FUNCTION_EXCLUSION()				Tau_disable_function_exclusion()
+#define TAU_EXCLUDE_DEFAULT_GROUP()						Tau_exclude_default_group()
+#define TAU_INCLUDE_DEFAULT_GROUP()						Tau_include_default_group()
+
 /* DB Access */
 #define TAU_DB_DUMP()                           Tau_dump();
 #define TAU_DB_MERGED_DUMP()                    Tau_mergeProfiles_MPI();
@@ -254,7 +264,7 @@
 
 #define TAU_METADATA(name, value)               Tau_metadata(name, value);
 #define TAU_METADATA_ITERATION(name,iteration,value) {char meta_buf[1024]; \
-        sprintf(meta_buf,"%s_|_%d",name,iteration); \
+        snprintf(meta_buf, sizeof(meta_buf), "%s_|_%d",name,iteration); \
         TAU_METADATA(meta_buf,value);}
 
 #define TAU_CONTEXT_METADATA(name, value)       Tau_context_metadata(name, value);
@@ -276,7 +286,7 @@
 #define TAU_METADATA_STRING(name, value)   Tau_metadata_value_t* name = NULL; \
         Tau_metadata_create_value(&name, TAU_METADATA_TYPE_STRING); \
         name->data.cval = malloc((sizeof(char))*(strlen(value))); \
-        strcpy(name->data.cval, value);
+        strncpy(name->data.cval,  value, strlen(value)); 
 
 #define TAU_METADATA_INTEGER(name, value)       Tau_metadata_value_t* name = NULL; \
         Tau_metadata_create_value(&name, TAU_METADATA_TYPE_INTEGER); \
@@ -448,6 +458,7 @@ int TAUDECL Tau_get_current_stack_depth(int tid);
 int TAUDECL Tau_get_local_tid(void);
 int TAUDECL Tau_get_thread(void);
 int TAUDECL Tau_get_node(void);
+int TAUDECL tau_totalnodes(int set_or_get, int value);
 int  Tau_create_task(void);
 void Tau_destructor_trigger();
 
@@ -456,6 +467,21 @@ void Tau_profile_set_type(void *ptr, const char *type);
 void Tau_profile_set_group(void *ptr, TauGroup_t group);
 void Tau_profile_set_group_name(void *ptr, const char *groupname);
 
+void Tau_exclude_function(void *ptr);
+void Tau_include_function(void *ptr);
+void Tau_exclude_function_by_name(const char *event_name);
+void Tau_include_function_by_name(const char *event_name);
+void Tau_enable_function_exclusion(); //This turns on function exclusion. No function will be excluded until this is called (but they can be set for exclusion first).
+void Tau_disable_function_exclusion(); //This fully deactivates function exclusion. All excluded functions are recorded again (but their excluded status is retained and they can be re-excluded by Tau_enable_function_exclusion)
+void Tau_exclude_default_group(); //Exclude every function in TAU_DEFAULT. This is most events, except threading/communication and specially recognized API's. It should result in fairly lightweight instrumentation.
+void Tau_include_default_group(); //Record events from TAU_DEFAULT again.
+void Tau_set_rank_exclusion_list(const char* rank_list_str);
+  void Tau_exclude_rank_list();
+  void Tau_include_rank_list();
+  void Tau_ignore_rank_list();
+  void Tau_tracing_off();
+  void Tau_tracing_on();
+
 const char *Tau_profile_get_group_name(void *ptr);
 const char *Tau_profile_get_name(void *ptr);
 const char *Tau_profile_get_type(void *ptr);
@@ -480,8 +506,6 @@ void *Tau_query_parent_event(void *event);
 void Tau_disable_context_event(void *event);
 void Tau_enable_context_event(void *event);
 
-void Tau_pure_context_userevent(void **u, const char *n);
-
 void Tau_the_function_list(const char ***functionList, int *num);
 int Tau_dump_prefix(const char *prefix);
 int Tau_dump_prefix_task(const char *prefix, int taskid);
