@@ -155,6 +155,7 @@
 
 
 #ifdef __cplusplus
+#include <mutex>
 //=============================================================================
 // Allocation record for heap allocation made by TAU or by the system.
 // Manages guarded (de)allocation and memory profiling events.
@@ -221,7 +222,7 @@ public:
   static TauAllocation * FindContaining(void * ptr);
 
   // Records memory used at the current time
-  static void TriggerHeapMemoryUsageEvent(void);
+  static void TriggerHeapMemoryUsageEvent(const char * prefix = nullptr);
 
   // Records memory remaining at the current time
   static void TriggerMemoryHeadroomEvent(void);
@@ -257,6 +258,9 @@ private:
   // Bytes of memory protection overhead
   static size_t & __bytes_overhead();
 
+  // internal lock for memory management
+  static std::mutex mtx;
+
 // ----------------------------------------------------------------------------
 // Public instance members
 //
