@@ -19,7 +19,9 @@
 #include "TauMetaDataTypes.h"
 
 #if defined (TAU_USE_OMPT_TR6) || defined (TAU_USE_OMPT_TR7) || defined (TAU_USE_OMPT_5_0)
+#if ((!defined(TAU_DISABLE_OMPT_5_0_IN_C_COMPILER)) || defined(__cplusplus))
 #define TAU_PLUGIN_OMPT_ON
+#endif /* TAU_DISABLE_OMPT_5_0_IN_C_COMPILER */
 #endif /*  defined (TAU_USE_OMPT_TR6) || defined (TAU_USE_OMPT_TR7) || defined (TAU_USE_OMPT_5_0) */
 
 #ifdef TAU_PLUGIN_OMPT_ON
@@ -66,6 +68,28 @@ typedef struct Tau_plugin_event_metadata_registration_data {
    int tid;
 } Tau_plugin_event_metadata_registration_data_t;
 
+/* GPU EVENTS BEGIN */
+typedef struct Tau_plugin_event_gpu_init_data {
+   int tid;
+} Tau_plugin_event_gpu_init_data_t;
+
+typedef struct Tau_plugin_event_gpu_finalize_data {
+   int tid;
+} Tau_plugin_event_gpu_finalize_data_t;
+
+typedef struct Tau_plugin_event_gpu_kernel_exec_data {
+   int tid;
+   unsigned long int time;
+} Tau_plugin_event_gpu_kernel_exec_data_t;
+
+typedef struct Tau_plugin_event_gpu_memcpy_data {
+   int tid;
+   unsigned long int time;
+   unsigned long int size;
+   unsigned short int kind;
+} Tau_plugin_event_gpu_memcpy_data_t;
+/* GPU EVENTS END */
+
 typedef struct Tau_plugin_event_post_init_data {
    int dummy;
    int tid;
@@ -95,6 +119,7 @@ typedef struct Tau_plugin_event_function_exit_data {
    unsigned int func_id;
    int tid;
    long unsigned int timestamp;
+   double * metrics;
 } Tau_plugin_event_function_exit_data_t;
 
 typedef struct Tau_plugin_event_phase_entry_data {
@@ -362,6 +387,50 @@ typedef struct Tau_plugin_event_ompt_mutex_released_data {
 #endif /* TAU_PLUGIN_OMPT_ON */
 } Tau_plugin_event_ompt_mutex_released_data_t;
 
+typedef struct Tau_plugin_event_ompt_device_initialize_data {
+#ifdef TAU_PLUGIN_OMPT_ON
+    int device_num;
+    const char *type;
+    ompt_device_t *device;
+    ompt_function_lookup_t lookup;
+    const char *documentation;
+#else /* TAU_PLUGIN_OMPT_ON */
+   /* This is here for the sole purpose of preventing a warning saying that
+    * empty struct have a size of 0 in C but 1 in C++.
+    * This struct should never * be used if OMPT is not enabled */
+    int null;
+#endif /* TAU_PLUGIN_OMPT_ON */
+} Tau_plugin_event_ompt_device_initialize_data_t;
+
+typedef struct Tau_plugin_event_ompt_device_finalize_data {
+#ifdef TAU_PLUGIN_OMPT_ON
+    int device_num;
+#else /* TAU_PLUGIN_OMPT_ON */
+   /* This is here for the sole purpose of preventing a warning saying that
+    * empty struct have a size of 0 in C but 1 in C++.
+    * This struct should never * be used if OMPT is not enabled */
+    int null;
+#endif /* TAU_PLUGIN_OMPT_ON */
+} Tau_plugin_event_ompt_device_finalize_data_t;
+
+typedef struct Tau_plugin_event_ompt_device_load_data {
+#ifdef TAU_PLUGIN_OMPT_ON
+    int device_num;
+    const char *filename;
+    int64_t offset_in_file;
+    void *vma_in_file;
+    size_t bytes;
+    void *host_addr;
+    void *device_addr;
+    uint64_t module_id;
+#else /* TAU_PLUGIN_OMPT_ON */
+   /* This is here for the sole purpose of preventing a warning saying that
+    * empty struct have a size of 0 in C but 1 in C++.
+    * This struct should never * be used if OMPT is not enabled */
+   int null;
+#endif /* TAU_PLUGIN_OMPT_ON */
+} Tau_plugin_event_ompt_device_load_data_t;
+
 typedef struct Tau_plugin_event_ompt_target_data {
 #ifdef TAU_PLUGIN_OMPT_ON
    ompt_target_t kind;
@@ -397,6 +466,28 @@ typedef struct Tau_plugin_event_ompt_target_data_op_data {
 #endif /* TAU_PLUGIN_OMPT_ON */
 } Tau_plugin_event_ompt_target_data_op_data_t;
 
+typedef struct Tau_plugin_event_ompt_target_data_op_emi_data {
+#ifdef TAU_PLUGIN_OMPT_ON
+  ompt_scope_endpoint_t endpoint; 
+  ompt_data_t *target_task_data;
+  ompt_data_t *target_data;
+  ompt_id_t *host_op_id;
+  ompt_target_data_op_t optype;
+  void *src_addr;
+  int src_device_num;
+  void *dest_addr;
+  int dest_device_num;
+  size_t bytes;
+  const void *codeptr_ra;
+#else /* TAU_PLUGIN_OMPT_ON */
+   /* This is here for the sole purpose of preventing a warning saying that
+    * empty struct have a size of 0 in C but 1 in C++.
+    * This struct should never * be used if OMPT is not enabled */
+   int null;
+#endif /* TAU_PLUGIN_OMPT_ON */
+} Tau_plugin_event_ompt_target_data_op_emi_data_t;
+
+
 typedef struct Tau_plugin_event_ompt_target_submit_data {
 #ifdef TAU_PLUGIN_OMPT_ON
    ompt_id_t target_id;
@@ -451,10 +542,20 @@ typedef int (*Tau_plugin_ompt_sync_region)(Tau_plugin_event_ompt_sync_region_dat
 typedef int (*Tau_plugin_ompt_mutex_acquire)(Tau_plugin_event_ompt_mutex_acquire_data_t*);
 typedef int (*Tau_plugin_ompt_mutex_acquired)(Tau_plugin_event_ompt_mutex_acquired_data_t*);
 typedef int (*Tau_plugin_ompt_mutex_released)(Tau_plugin_event_ompt_mutex_released_data_t*);
+typedef int (*Tau_plugin_ompt_device_initialize)(Tau_plugin_event_ompt_device_initialize_data_t*);
+typedef int (*Tau_plugin_ompt_device_finalize)(Tau_plugin_event_ompt_device_finalize_data_t*);
+typedef int (*Tau_plugin_ompt_device_load)(Tau_plugin_event_ompt_device_load_data_t*);
 typedef int (*Tau_plugin_ompt_target)(Tau_plugin_event_ompt_target_data_t*);
 typedef int (*Tau_plugin_ompt_target_data_op)(Tau_plugin_event_ompt_target_data_op_data_t*);
+typedef int (*Tau_plugin_ompt_target_data_op_emi)(Tau_plugin_event_ompt_target_data_op_emi_data_t*);
 typedef int (*Tau_plugin_ompt_target_submit)(Tau_plugin_event_ompt_target_submit_data_t*);
 typedef int (*Tau_plugin_ompt_finalize)(Tau_plugin_event_ompt_finalize_data_t*);
+/* GPU EVENTS BEGIN */
+typedef int (*Tau_plugin_gpu_init)(Tau_plugin_event_gpu_init_data_t*);
+typedef int (*Tau_plugin_gpu_finalize)(Tau_plugin_event_gpu_finalize_data_t*);
+typedef int (*Tau_plugin_gpu_kernel_exec)(Tau_plugin_event_gpu_kernel_exec_data_t*);
+typedef int (*Tau_plugin_gpu_memcpy)(Tau_plugin_event_gpu_memcpy_data_t*);
+/* GPU EVENTS END */
 
 
 /*Define the callback structure*/
@@ -492,10 +593,20 @@ typedef struct Tau_plugin_callbacks {
    Tau_plugin_ompt_mutex_acquire OmptMutexAcquire;
    Tau_plugin_ompt_mutex_acquired OmptMutexAcquired;
    Tau_plugin_ompt_mutex_released OmptMutexReleased;
+   Tau_plugin_ompt_device_initialize OmptDeviceInitialize;
+   Tau_plugin_ompt_device_finalize OmptDeviceFinalize;
+   Tau_plugin_ompt_device_load OmptDeviceLoad;
    Tau_plugin_ompt_target OmptTarget;
    Tau_plugin_ompt_target_data_op OmptTargetDataOp;
+   Tau_plugin_ompt_target_data_op_emi OmptTargetDataOpEmi;
    Tau_plugin_ompt_target_submit OmptTargetSubmit;
    Tau_plugin_ompt_finalize OmptFinalize;
+/* GPU EVENTS BEGIN */
+   Tau_plugin_gpu_init GpuInit;
+   Tau_plugin_gpu_finalize GpuFinalize;
+   Tau_plugin_gpu_kernel_exec GpuKernelExec;
+   Tau_plugin_gpu_memcpy GpuMemcpy;
+/* GPU EVENTS END */
 } Tau_plugin_callbacks_t;
 
 /*Define all the events currently supported*/
@@ -533,10 +644,19 @@ typedef enum Tau_plugin_event {
    TAU_PLUGIN_EVENT_OMPT_MUTEX_ACQUIRE,
    TAU_PLUGIN_EVENT_OMPT_MUTEX_ACQUIRED,
    TAU_PLUGIN_EVENT_OMPT_MUTEX_RELEASED,
+   TAU_PLUGIN_EVENT_OMPT_DEVICE_INITIALIZE,
+   TAU_PLUGIN_EVENT_OMPT_DEVICE_FINALIZE,
+   TAU_PLUGIN_EVENT_OMPT_DEVICE_LOAD,
    TAU_PLUGIN_EVENT_OMPT_TARGET,
    TAU_PLUGIN_EVENT_OMPT_TARGET_DATA_OP,
    TAU_PLUGIN_EVENT_OMPT_TARGET_SUBMIT,
    TAU_PLUGIN_EVENT_OMPT_FINALIZE,
+   /* GPU KERNEL START */
+   TAU_PLUGIN_EVENT_GPU_INIT,
+   TAU_PLUGIN_EVENT_GPU_FINALIZE,
+   TAU_PLUGIN_EVENT_GPU_KERNEL_EXEC,
+   TAU_PLUGIN_EVENT_GPU_MEMCPY,
+   /* GPU KERNEL STOP */
 
    /* Max for number of events */
    NB_TAU_PLUGIN_EVENTS
@@ -577,10 +697,20 @@ typedef struct Tau_plugin_callbacks_active {
     unsigned int ompt_mutex_acquire;
     unsigned int ompt_mutex_acquired;
     unsigned int ompt_mutex_released;
+    unsigned int ompt_device_initialize;
+    unsigned int ompt_device_finalize;
+    unsigned int ompt_device_load;
     unsigned int ompt_target;
     unsigned int ompt_target_data_op;
+    unsigned int ompt_target_data_op_emi;
     unsigned int ompt_target_submit;
     unsigned int ompt_finalize;
+    /* GPU KERNEL START */
+    unsigned int gpu_init;
+    unsigned int gpu_finalize;
+    unsigned int gpu_kernel_exec;
+    unsigned int gpu_memcpy;
+    /* GPU KERNEL STOP */
 } Tau_plugin_callbacks_active_t;
 
 /*Deprecated*/
