@@ -53,6 +53,7 @@
 #define TAU_ADIOS2_ONE_FILE_DEFAULT false
 #define TAU_ADIOS2_ENGINE "BPFile"
 #define TAU_ADIOS2_CONFIG_FILE_DEFAULT "./adios2.xml"
+#define TAU_ADIOS2_SELECT_FILE_DEFAULT "./select.tau"
 
 // This will enable some checking to make sure we don't have call stack violations.
 // #define DO_VALIDATION
@@ -65,11 +66,11 @@ int Tau_plugin_adios2_function_exit(
 void Tau_dump_ADIOS2_metadata(adios2::IO& bpIO, int tid);
 
 static bool enabled{false};
-static bool done{false};
+static bool plugin_done{false};
 static bool _threaded{false};
 static int global_comm_size = 1;
 static int global_comm_rank = 0;
-std::mutex _my_mutex;    // controlling access to local and TAU variables
+static std::mutex _my_mutex;    // controlling access to local and TAU variables
 
 /* These two variables are used to create something like a "counting semaphore".
  * When dumping, we want to prevent any threads from adding events to the buffers.
@@ -82,8 +83,8 @@ static atomic<bool> dumping{false};
 static atomic<bool> in_async_write{false};
 static atomic<size_t> active_threads{0};
 
-std::condition_variable _my_cond;
-std::thread * worker_thread = nullptr;
+static std::condition_variable _my_cond;
+static std::thread * worker_thread = nullptr;
 
 //typedef unsigned long tau_data_t;
 typedef uint64_t tau_data_t;
@@ -106,7 +107,7 @@ char *_program_path()
     std::string tmp(path);
     size_t i = tmp.rfind('/', tmp.length());
     if (i != string::npos) {
-        sprintf(executable, "%s", tmp.substr(i+1, ((tmp.length() - i) - 1)).c_str());
+        snprintf(executable, PATH_MAX,  "%s", tmp.substr(i+1, ((tmp.length() - i) - 1)).c_str());
     }
     return executable;
 #endif
@@ -121,7 +122,8 @@ class plugin_options {
             env_filename(TAU_ADIOS2_FILENAME),
             env_one_file(TAU_ADIOS2_ONE_FILE_DEFAULT),
             env_engine(TAU_ADIOS2_ENGINE),
-            env_config_file("")
+            env_config_file(""),
+            env_select_file("")
             {}
     public:
         int env_periodic;
@@ -131,6 +133,7 @@ class plugin_options {
         int env_one_file;
         std::string env_engine;
         std::string env_config_file;
+        std::string env_select_file;
         std::set<std::string> included_timers;
         std::set<std::string> excluded_timers;
         std::set<std::string> included_timers_with_wildcards;
@@ -162,7 +165,7 @@ bool& inPlugin() {
 
 void Tau_ADIOS2_parse_environment_variables(void);
 void Tau_ADIOS2_parse_selection_file(const char * filename);
-bool Tau_ADIOS2_contains(std::set<std::string>& myset,
+bool Tau_ADIOS2_contains(std::unordered_set<std::string>& myset,
         const char * key, bool if_empty);
 
 void * Tau_ADIOS2_thread_function(void);
@@ -235,6 +238,18 @@ void Tau_ADIOS2_parse_environment_variables(void) {
           thePluginOptions().env_config_file = "";
       }
     }
+    tmp = getenv("TAU_ADIOS2_SELECT_FILE");
+    if (tmp != NULL) {
+      thePluginOptions().env_select_file = strdup(tmp);
+    } else {
+      if( access(TAU_ADIOS2_SELECT_FILE_DEFAULT, F_OK ) != -1 ) {
+          // file exists
+          thePluginOptions().env_select_file = strdup(TAU_ADIOS2_SELECT_FILE_DEFAULT);
+      } else {
+          // file doesn't exist
+          thePluginOptions().env_select_file = "";
+      }
+    }
 }
 
 void Tau_ADIOS2_parse_selection_file(const char * filename) {
@@ -255,7 +270,11 @@ void Tau_ADIOS2_parse_selection_file(const char * filename) {
             continue;
         }
         // skip comments
-        if (str.find("#", 0) == 0) {
+        if ((str.find("#", 0) == 0) &&
+            !including_timers &&
+            !excluding_timers &&
+            !including_counters &&
+            !excluding_counters) {
             continue;
         }
         if (str.compare("BEGIN_INCLUDE_TIMERS") == 0) {
@@ -340,6 +359,28 @@ class adios {
         int total_valid;
         int time_index;
         int max_threads;
+        // This is a stuct (one per thread)
+        // of vectors (one per timestamp)
+        // of pairs (timestamp, values)...
+        class adiosThread{
+        public:
+        timer_values_array_t timer_values;
+        counter_values_array_t counter_values;
+        comm_values_array_t comm_values;
+        // for validation
+#ifdef DO_VALIDATION
+        std::stack<tau_data_t> timer_stack;
+        std::stack<tau_data_t> pre_timer_stack;
+#endif
+        tau_data_t previous_timestamp;
+        adiosThread() : previous_timestamp(0L) {
+            timer_values.reserve(1024);
+            counter_values.reserve(1024);
+            comm_values.reserve(1024);
+        };
+        };
+        //A per-thread vector of those structs.
+        vector<adiosThread *> adiosThreadVector;
     public:
         std::unordered_map<std::string, int> prog_names;
         std::unordered_map<std::string, int> value_names;
@@ -363,6 +404,7 @@ class adios {
             initialize();
             open();
             define_variables();
+            adiosThreadVector.reserve(256);
         };
         ~adios() {
             close();
@@ -387,26 +429,24 @@ class adios {
         int get_event_type_count(void) { return event_types.size(); }
         int get_counter_count(void) { return counters.size(); }
         int get_thread_count(void) { return max_threads+1; } // zero-indexed.
-        // This is an array (one per thread)
-        // of vectors (one per timestamp)
-        // of pairs (timestamp, values)...
-        timer_values_array_t timer_values_array[TAU_MAX_THREADS];
-        counter_values_array_t counter_values_array[TAU_MAX_THREADS];
-        comm_values_array_t comm_values_array[TAU_MAX_THREADS];
-        // for validation
-#ifdef DO_VALIDATION
-        std::stack<tau_data_t> timer_stack[TAU_MAX_THREADS];
-        std::stack<tau_data_t> pre_timer_stack[TAU_MAX_THREADS];
-#endif
-        tau_data_t previous_timestamp[TAU_MAX_THREADS];
+
+        adiosThread* getAdiosThread(int tid){
+            static std::mutex _mtx;
+            std::unique_lock<std::mutex> lk(_mtx);
+            while(adiosThreadVector.size()<=tid){
+                adiosThreadVector.push_back(new adiosThread);
+            }
+            return adiosThreadVector[tid];
+        }
 };
 
 void adios::initialize() {
 #if TAU_MPI
     // Get the rank and size in the original communicator
     int world_rank, world_size;
-    PMPI_Comm_rank(MPI_COMM_WORLD, &world_rank);
-    PMPI_Comm_size(MPI_COMM_WORLD, &world_size);
+    world_rank = RtsLayer::myNode();
+    world_size = tau_totalnodes(0,1);
+
     MPI_Comm adios_comm = MPI_COMM_NULL;
 
     // If we are using one file for the whole communicator, copy it
@@ -417,24 +457,24 @@ void adios::initialize() {
     if (thePluginOptions().env_config_file != "") {
         // if not using one file, we don't need MPI in ADIOS2
         if (!thePluginOptions().env_one_file) {
-            ad = adios2::ADIOS(thePluginOptions().env_config_file, true);
+            ad = adios2::ADIOS(thePluginOptions().env_config_file);
         } else {
-            ad = adios2::ADIOS(thePluginOptions().env_config_file, adios_comm, true);
+            ad = adios2::ADIOS(thePluginOptions().env_config_file, adios_comm);
         }
     } else {
         // if not using one file, we don't need MPI in ADIOS2
         if (!thePluginOptions().env_one_file) {
-            ad = adios2::ADIOS(true);
+            ad = adios2::ADIOS();
         } else {
-            ad = adios2::ADIOS(adios_comm, true);
+            ad = adios2::ADIOS(adios_comm);
         }
     }
 #else
     /** ADIOS class factory of IO class objects, DebugON is recommended */
     if (thePluginOptions().env_config_file != "") {
-        ad = adios2::ADIOS(thePluginOptions().env_config_file, true);
+        ad = adios2::ADIOS(thePluginOptions().env_config_file);
     } else {
-        ad = adios2::ADIOS(true);
+        ad = adios2::ADIOS();
     }
 #endif
     /*** IO class object: settings and factory of Settings: Variables,
@@ -447,12 +487,16 @@ void adios::initialize() {
         _bpIO.SetEngine(thePluginOptions().env_engine);
         // bpIO.SetParameters({{"num_threads", "2"}});
         // don't wait on readers to connect
-        _bpIO.SetParameters({{"RendezvousReaderCount", "0"}});
-        _bpIO.SetParameters({{"num_threads", "1"}});
-
-        // ISO-POSIX file output is the default transport (called "File")
-        // Passing parameters to the transport
-        _bpIO.AddTransport("File", {{"Library", "POSIX"}});
+        if (thePluginOptions().env_engine.compare("SST") == 0) {
+            _bpIO.SetParameters({{"RendezvousReaderCount", "1"}});
+            _bpIO.SetParameters({{"QueueFullPolicy", "Block"}});
+        } else {
+            _bpIO.SetParameters({{"RendezvousReaderCount", "0"}});
+            _bpIO.SetParameters({{"num_threads", "1"}});
+            // ISO-POSIX file output is the default transport (called "File")
+            // Passing parameters to the transport
+            _bpIO.AddTransport("File", {{"Library", "POSIX"}});
+        }
     }
     Tau_global_decr_insideTAU();
 }
@@ -529,9 +573,10 @@ void event_gets_control() {
      */
     _my_mutex.lock();
     size_t safety{0};
+    if (dumping) TAU_VERBOSE("ADIOS2 trace plugin: %s: waiting...\n", __func__);
     while(dumping) {
-        if (++safety > 1000000000) {
-            TAU_VERBOSE("%s: Broke deadlock!\n", __func__);
+        if (++safety > 100000000000) {
+            TAU_VERBOSE("ADIOS2 trace plugin: %s: Broke deadlock!\n", __func__);
             abort();
             //PMPI_Abort(MPI_COMM_WORLD, 99);
         }
@@ -541,7 +586,13 @@ void event_gets_control() {
     _my_mutex.unlock();
 }
 
+void writer_gives_control() {
+    TAU_VERBOSE("%s: Done writing.\n", __func__);
+    dumping = false;
+}
+
 void writer_gets_control() {
+    TAU_VERBOSE("%s: Ready to write...\n", __func__);
     /* There's a tiny chance of a race condition because we have to update
      * and check two variables.  So acquire the lock before waiting until it's
      * safe to proceed with the dump.
@@ -579,7 +630,7 @@ void adios::write_variables(void)
     writer_gets_control();
 
     try{
-    TAU_VERBOSE("%s: Merging %lu timers...\n", __func__, timer_values_array[0].size());
+    TAU_VERBOSE("%s: Merging %lu timers from 0...\n", __func__, getAdiosThread(0)->timer_values.size());
 #if 0
     /* sort into one big vector from all threads */
     std::vector<event5_t> merged_timers(timer_values_array[0]);
@@ -626,10 +677,10 @@ void adios::write_variables(void)
 #endif
     }
 #endif
-    size_t num_timer_values = timer_values_array[0].size();
+    size_t num_timer_values = getAdiosThread(0)->timer_values.size();
     for (int t = 1 ; t < threads ; t++) {
-        TAU_VERBOSE("%s: %d has %lu timers...\n", __func__, t, timer_values_array[t].size());
-        num_timer_values += timer_values_array[t].size();
+        TAU_VERBOSE("%s: %d has %lu timers...\n", __func__, t, getAdiosThread(t)->timer_values.size());
+        num_timer_values += getAdiosThread(t)->timer_values.size();
     }
     static tau_data_t * all_timers = nullptr;
     static uint64_t prev_all_timers_bytes{0};
@@ -655,7 +706,7 @@ void adios::write_variables(void)
         }
         for (int t = 0 ; t < threads ; t++) {
             TAU_VERBOSE("%s: thread %d...\n", __func__, t);
-            for (auto it : timer_values_array[t]) {
+            for (auto it : getAdiosThread(t)->timer_values) {
                 all_timers[i++] = it.second[0];
                 all_timers[i++] = it.second[1];
                 all_timers[i++] = it.second[2];
@@ -668,17 +719,18 @@ void adios::write_variables(void)
 
     TAU_VERBOSE("%s: Freeing timer_values_array...\n", __func__);
     for (int t = 0 ; t < threads ; t++) {
-        timer_values_array[t].clear();
+        getAdiosThread(t)->timer_values.clear();
+        getAdiosThread(t)->timer_values.reserve(1024);
     }
 
-    TAU_VERBOSE("%s: Merging %llu counters from %d...\n", __func__, counter_values_array[0].size(), 0);
+    TAU_VERBOSE("%s: Merging %llu counters from %d...\n", __func__, getAdiosThread(0)->counter_values.size(), 0);
     /* sort into one big vector from all threads */
-    std::vector<event5_t> merged_counters(counter_values_array[0]);
+    std::vector<event5_t> merged_counters(getAdiosThread(0)->counter_values);
     for (int t = 1 ; t < threads ; t++) {
-        TAU_VERBOSE("%s: Merging %llu counters from %d...\n", __func__, counter_values_array[t].size(), t);
+        TAU_VERBOSE("%s: Merging %llu counters from %d...\n", __func__, getAdiosThread(t)->counter_values.size(), t);
         merged_counters.insert(merged_counters.end(),
-            counter_values_array[t].begin(),
-            counter_values_array[t].end());
+            getAdiosThread(t)->counter_values.begin(),
+            getAdiosThread(t)->counter_values.end());
     }
     std::sort(merged_counters.begin(), merged_counters.end());
     size_t num_counter_values = merged_counters.size();
@@ -697,17 +749,18 @@ void adios::write_variables(void)
     }
 
     for (int t = 0 ; t < threads ; t++) {
-        counter_values_array[t].clear();
+        getAdiosThread(t)->counter_values.clear();
+        getAdiosThread(t)->counter_values.reserve(1024);
     }
 
-    TAU_VERBOSE("%s: Merging %llu comms from %d...\n", __func__, comm_values_array[0].size(), 0);
+    TAU_VERBOSE("%s: Merging %llu comms from %d...\n", __func__, getAdiosThread(0)->comm_values.size(), 0);
     /* sort into one big vector from all threads */
-    std::vector<event7_t> merged_comms(comm_values_array[0]);
+    std::vector<event7_t> merged_comms(getAdiosThread(0)->comm_values);
     for (int t = 1 ; t < threads ; t++) {
-        TAU_VERBOSE("%s: Merging %llu comms from %d...\n", __func__, comm_values_array[t].size(), t);
+        TAU_VERBOSE("%s: Merging %llu comms from %d...\n", __func__, getAdiosThread(t)->comm_values.size(), t);
         merged_comms.insert(merged_comms.end(),
-            comm_values_array[t].begin(),
-            comm_values_array[t].end());
+            getAdiosThread(t)->comm_values.begin(),
+            getAdiosThread(t)->comm_values.end());
     }
     std::sort(merged_comms.begin(), merged_comms.end());
     size_t num_comm_values = merged_comms.size();
@@ -727,29 +780,28 @@ void adios::write_variables(void)
     }
 
     for (int t = 0 ; t < threads ; t++) {
-        comm_values_array[t].clear();
+        getAdiosThread(t)->comm_values.clear();
+        getAdiosThread(t)->comm_values.reserve(1024);
     }
 
     // Need to release the "dumping" flag so that ADIOS2 calls that
     // result in TAU timers won't deadlock when they enter this
     // plugin
-    dumping = false;
+    writer_gives_control();
 
     tau_plugin::inPlugin() = true;
 
-    TAU_VERBOSE("%s: Writing step... ", __func__);
+    TAU_VERBOSE("%s: Writing step...\n", __func__);
+    _my_mutex.lock();
     bpWriter.BeginStep();
 
     // do this first, so other threads don't define new attributes yet
-    TAU_VERBOSE("%s: Writing %lu attributes...", __func__, attributes_to_define.size());
-    _my_mutex.lock();
+    TAU_VERBOSE("%s: Writing %lu attributes...\n", __func__, attributes_to_define.size());
     for (auto a : attributes_to_define) {
         _bpIO.DefineAttribute<std::string>(a.first, a.second);
     }
-    // safe to clear...
-    attributes_to_define.clear();
-    _my_mutex.unlock();
 
+    TAU_VERBOSE("%s: Writing scalar values...\n", __func__);
     bpWriter.Put(program_count, &programs);
     bpWriter.Put(comm_size, &comm_ranks);
     bpWriter.Put(thread_count, &threads);
@@ -760,6 +812,7 @@ void adios::write_variables(void)
     bpWriter.Put(counter_event_count, &num_counter_values);
     bpWriter.Put(comm_count, &num_comm_values);
 
+    TAU_VERBOSE("%s: Writing %lu timers...\n", __func__, num_timer_values);
     if (num_timer_values > 0) {
         event_timestamps.SetShape({num_timer_values, 6});
         /* These dimensions need to change for 1-file case! */
@@ -770,6 +823,7 @@ void adios::write_variables(void)
         bpWriter.Put(event_timestamps, all_timers);
     }
 
+    TAU_VERBOSE("%s: Writing %lu counters...\n", __func__, num_counter_values);
     if (num_counter_values > 0) {
         counter_values.SetShape({num_counter_values, 6});
         /* These dimensions need to change for 1-file case! */
@@ -780,6 +834,7 @@ void adios::write_variables(void)
         bpWriter.Put(counter_values, all_counters.data());
     }
 
+    TAU_VERBOSE("%s: Writing %lu communicators...\n", __func__, num_comm_values);
     if (num_comm_values > 0) {
         comm_timestamps.SetShape({num_comm_values, 8});
         /* These dimensions need to change for 1-file case! */
@@ -790,7 +845,14 @@ void adios::write_variables(void)
         bpWriter.Put(comm_timestamps, all_comms.data());
     }
 
+    TAU_VERBOSE("%s: Ending step...\n", __func__);
     bpWriter.EndStep();
+    TAU_VERBOSE("%s: Clearing attributes...\n", __func__);
+    // safe to clear...
+    attributes_to_define.clear();
+    TAU_VERBOSE("%s: Unlocking...\n", __func__);
+    _my_mutex.unlock();
+    TAU_VERBOSE("%s: leaving plugin...\n", __func__);
     tau_plugin::inPlugin() = false;
     //TAU_VERBOSE("Freeing all_timers.\n");
     //free(all_timers);
@@ -813,6 +875,7 @@ void adios::write_variables(void)
 
     /* Keep a map of program names to indexes */
     int adios::check_prog_name(char * prog_name) {
+        //std::unique_lock<std::mutex> lk(_my_mutex);
         if (prog_names.count(prog_name) == 0) {
             std::stringstream ss;
             int num = prog_names.size();
@@ -825,6 +888,7 @@ void adios::write_variables(void)
 
     /* Keep a map of event types to indexes */
     int adios::check_event_type(const std::string& event_type) {
+        //std::unique_lock<std::mutex> lk(_my_mutex);
         if (event_types.count(event_type) == 0) {
             std::stringstream ss;
             int num = event_types.size();
@@ -857,6 +921,7 @@ void adios::write_variables(void)
                 define_attribute(ss.str(), tmp, _bpIO, false);
             }
         }
+        //std::unique_lock<std::mutex> lk(_my_mutex);
         return timers[tmp];
     }
 
@@ -879,6 +944,7 @@ void adios::write_variables(void)
                 define_attribute(ss.str(), tmp, _bpIO, false);
             }
         }
+        std::unique_lock<std::mutex> lk(_my_mutex);
         return counters[tmp];
     }
 
@@ -892,7 +958,7 @@ void adios::write_variables(void)
 /* Necessary to use const char * because UserEvents use TauSafeString objects,
  * not std::string. We use the "if_empty" parameter to tell us how to treat
  * an empty set.  For exclude lists, it's false, for include lists, it's true */
-bool Tau_ADIOS2_contains(std::set<std::string>& myset,
+bool Tau_ADIOS2_contains(std::unordered_set<std::string>& myset,
         const char * key, bool if_empty) {
     // if the set has contents, and we are in the set, then return true.
     std::string _key(key);
@@ -930,7 +996,7 @@ int Tau_plugin_adios2_dump(Tau_plugin_event_dump_data_t* data) {
 void Tau_ADIOS2_stop_worker(void) {
     if (!enabled) return;
     if (!_threaded) return;
-    done = true;
+    plugin_done = true;
     if (tau_plugin::thePluginOptions().env_periodic && _threaded) {
         TAU_VERBOSE("TAU ADIOS2 thread joining...\n"); fflush(stderr);
         while(in_async_write) {/* wait for the async thread to finish writing, if necessary */}
@@ -945,12 +1011,12 @@ void Tau_ADIOS2_stop_worker(void) {
 void * Tau_ADIOS2_thread_function(void) {
 	Tau_register_thread();
     std::chrono::microseconds period(tau_plugin::thePluginOptions().env_period);
-    while (!done) {
+    while (!plugin_done) {
         in_async_write = false;
         {
             // scoped region for lock
             std::unique_lock<std::mutex> lk(_my_mutex);
-            if (_my_cond.wait_for(lk, period, [] {return done == true;})) {
+            if (_my_cond.wait_for(lk, period, [] {return plugin_done == true;})) {
                 // done executing
                 break;
             }
@@ -975,6 +1041,8 @@ int do_teardown(bool pre) {
     TAU_VERBOSE("%d:%d %s...\n", global_comm_rank, RtsLayer::myThread(), __func__); fflush(stdout);
     /* Don't do these instructions more than once */
     done_once = true;
+    /* Prevent any other timers from triggering a write */
+    plugin_done = true;
     Tau_ADIOS2_stop_worker();
     /* Stop any outstanding timers on the stack(s) */
     if (pre) {
@@ -993,6 +1061,7 @@ int do_teardown(bool pre) {
             double CurrentTime[TAU_MAX_COUNTERS] = { 0 };
             RtsLayer::getUSecD(tid, CurrentTime);
             exit_data.timestamp = (x_uint64)CurrentTime[0];    // USE COUNTER1 for tracing
+            exit_data.metrics = &(CurrentTime[0]);
             //printf("%d Stopping %s\n", tid, exit_data.timer_name);
             Tau_plugin_adios2_function_exit(&exit_data);
         }
@@ -1014,7 +1083,7 @@ int do_teardown(bool pre) {
 int Tau_plugin_adios2_pre_end_of_execution(Tau_plugin_event_pre_end_of_execution_data_t* data) {
     if (!enabled || data->tid != 0) return 0;
     TAU_VERBOSE("TAU PLUGIN ADIOS2 Pre-Finalize\n"); fflush(stdout);
-    return do_teardown(false);
+    return do_teardown(true);
 }
 
 /* This happens from Profiler.cpp, when data is written out. */
@@ -1148,12 +1217,12 @@ int Tau_plugin_adios2_send(Tau_plugin_event_send_data_t* data) {
     tmparray[5] = (tau_data_t)(data->destination);
     tmparray[6] = (tau_data_t)(data->bytes_sent);
 
-    tau_data_t ts = my_adios().previous_timestamp[data->tid] >
-        data->timestamp ? my_adios().previous_timestamp[data->tid] :
+    tau_data_t ts = my_adios().getAdiosThread(data->tid)->previous_timestamp >
+        data->timestamp ? my_adios().getAdiosThread(data->tid)->previous_timestamp :
             data->timestamp;
-    my_adios().previous_timestamp[data->tid] = ts;
+    my_adios().getAdiosThread(data->tid)->previous_timestamp = ts;
 
-    auto &tmp = my_adios().comm_values_array[data->tid];
+    auto &tmp = my_adios().getAdiosThread(data->tid)->comm_values;
     tau_plugin::event_gets_control();
     tmp.push_back(
         std::make_pair(
@@ -1181,12 +1250,12 @@ int Tau_plugin_adios2_recv(Tau_plugin_event_recv_data_t* data) {
     tmparray[5] = (tau_data_t)(data->source);
     tmparray[6] = (tau_data_t)(data->bytes_received);
 
-    tau_data_t ts = my_adios().previous_timestamp[data->tid] >
-        data->timestamp ? my_adios().previous_timestamp[data->tid] :
+    tau_data_t ts = my_adios().getAdiosThread(data->tid)->previous_timestamp >
+        data->timestamp ? my_adios().getAdiosThread(data->tid)->previous_timestamp :
             data->timestamp;
-    my_adios().previous_timestamp[data->tid] = ts;
+    my_adios().getAdiosThread(data->tid)->previous_timestamp = ts;
 
-    auto &tmp = my_adios().comm_values_array[data->tid];
+    auto &tmp = my_adios().getAdiosThread(data->tid)->comm_values;
     tau_plugin::event_gets_control();
     tmp.push_back(
         std::make_pair(
@@ -1220,15 +1289,15 @@ int Tau_plugin_adios2_function_entry(Tau_plugin_event_function_entry_data_t* dat
     tmparray[3] = (tau_data_t)(event_index);
     tmparray[4] = (tau_data_t)(timer_index);
 
-    tau_data_t ts = my_adios().previous_timestamp[data->tid] > data->timestamp ?
-        my_adios().previous_timestamp[data->tid] : data->timestamp;
-    my_adios().previous_timestamp[data->tid] = ts;
+    tau_data_t ts = my_adios().getAdiosThread(data->tid)->previous_timestamp > data->timestamp ?
+        my_adios().getAdiosThread(data->tid)->previous_timestamp : data->timestamp;
+    my_adios().getAdiosThread(data->tid)->previous_timestamp = ts;
 #ifdef DO_VALIDATION
-    my_adios().pre_timer_stack[data->tid].push(timer_index);
+    my_adios().getAdiosThread(data->tid)->pre_timer_stack.push(timer_index);
     TAU_VERBOSE("Enter:   %d %d %lu %lu %s %s\n", global_comm_rank, data->tid, data->timestamp, ts, (data->timestamp != ts ? "FIXED" : ""), data->timer_name);
 #endif
     tau_plugin::event_gets_control();
-    auto &tmp = my_adios().timer_values_array[data->tid];
+    auto &tmp = my_adios().getAdiosThread(data->tid)->timer_values;
     tmp.push_back(
         std::make_pair(
             ts,
@@ -1239,6 +1308,27 @@ int Tau_plugin_adios2_function_entry(Tau_plugin_event_function_entry_data_t* dat
     return 0;
 }
 
+typedef struct metric_names_count {
+    const char **counterNames;
+    int numCounters;
+    int counter_index[TAU_MAX_COUNTERS] = {0};
+    int firstIndex;
+} metric_names_count_t;
+
+metric_names_count& get_metric_names_count() {
+    static metric_names_count_t the_data;
+    TauMetrics_getCounterList(&the_data.counterNames, &the_data.numCounters);
+    for (int i = 0 ; i < the_data.numCounters ; i++) {
+        the_data.counter_index[i] = my_adios().check_counter(the_data.counterNames[i]);
+    }
+    the_data.firstIndex = 1;
+    // if the first metric isn't time, then we should output it
+    if (strstr(the_data.counterNames[0], "TIME") == NULL) {
+        the_data.firstIndex = 0;
+    }
+    return the_data;
+}
+
 /* This happens on Tau_stop() */
 int Tau_plugin_adios2_function_exit(Tau_plugin_event_function_exit_data_t* data) {
     if (!enabled) return 0;
@@ -1258,10 +1348,10 @@ int Tau_plugin_adios2_function_exit(Tau_plugin_event_function_exit_data_t* data)
     tmparray[3] = (tau_data_t)(event_index);
     tmparray[4] = (tau_data_t)(timer_index);
 
-    tau_data_t ts = my_adios().previous_timestamp[data->tid] >
-        data->timestamp ? my_adios().previous_timestamp[data->tid] :
+    tau_data_t ts = my_adios().getAdiosThread(data->tid)->previous_timestamp >
+        data->timestamp ? my_adios().getAdiosThread(data->tid)->previous_timestamp :
             data->timestamp;
-    my_adios().previous_timestamp[data->tid] = ts;
+    my_adios().getAdiosThread(data->tid)->previous_timestamp = ts;
 #ifdef DO_VALIDATION
     TAU_VERBOSE("Exit:    %d %d %lu %lu %s %s\n", global_comm_rank, data->tid, data->timestamp, ts, (data->timestamp != ts ? "FIXED" : ""), data->timer_name);
     if (my_adios().pre_timer_stack[data->tid].size() == 0) {
@@ -1271,7 +1361,7 @@ int Tau_plugin_adios2_function_exit(Tau_plugin_event_function_exit_data_t* data)
       active_threads--;
       return 0;
     } else {
-        tau_data_t lhs = (tau_data_t)(my_adios().pre_timer_stack[data->tid].top());
+        tau_data_t lhs = (tau_data_t)(my_adios().getAdiosThread(data->tid)->pre_timer_stack.top());
         tau_data_t rhs = (tau_data_t)(timer_index);
         if (lhs != rhs) {
             TAU_VERBOSE("Pre: Stack violation. %s\n", data->timer_name);
@@ -1282,7 +1372,7 @@ int Tau_plugin_adios2_function_exit(Tau_plugin_event_function_exit_data_t* data)
     }
 #endif
     tau_plugin::event_gets_control();
-    auto &tmp = my_adios().timer_values_array[data->tid];
+    auto &tmp = my_adios().getAdiosThread(data->tid)->timer_values;
     tmp.push_back(
         std::make_pair(
             ts,
@@ -1290,32 +1380,42 @@ int Tau_plugin_adios2_function_exit(Tau_plugin_event_function_exit_data_t* data)
         )
     );
     active_threads--;
+    /* Write the PAPI counters for this timer */
+    static metric_names_count_t metric_data = get_metric_names_count();
+    // Iterate over the metrics
+    auto &tmp2 = my_adios().getAdiosThread(data->tid)->counter_values;
+    for (int i = metric_data.firstIndex ; i < metric_data.numCounters ; i++) {
+        std::array<tau_data_t, 5> tmparray2;
+        tmparray2[0] = 0UL;
+        tmparray2[1] = (tau_data_t)(global_comm_rank);
+        tmparray2[2] = (tau_data_t)(data->tid);
+        tmparray2[3] = (tau_data_t)(metric_data.counter_index[i]);
+        tmparray2[4] = (tau_data_t)(data->metrics[i]);
+        tau_plugin::event_gets_control();
+        tmp2.push_back( std::make_pair( ts, std::move(tmparray2)));
+        active_threads--;
+    }
     // initialize to a time in the future
     using namespace std::chrono;
     static time_point<steady_clock> next_write(steady_clock::now() +
         microseconds(tau_plugin::thePluginOptions().env_period));
     static std::mutex timer_lock;
+    static std::atomic<bool> writing{false};
     if (tau_plugin::thePluginOptions().env_periodic &&
         !tau_plugin::thePluginOptions().env_one_file) {
-        // is it time to write?
-        if (steady_clock::now() > next_write) {
-            bool mine = false;
-            // only let one thread do this
-            timer_lock.lock();
-            if (steady_clock::now() > next_write) {
-                // increment to a time in the future
-                next_write = next_write +
-                    microseconds(tau_plugin::thePluginOptions().env_period);
-                mine = true;
-            }
-            // don't hold the lock while writing, deadlock can happen, apparently.
-            timer_lock.unlock();
-            if (mine) {
-                TAU_VERBOSE("%d Sending data from exit event...\n", RtsLayer::myNode()); fflush(stderr);
-                Tau_plugin_event_dump_data_t dummy_data;
-                Tau_plugin_adios2_dump(&dummy_data);
-                TAU_VERBOSE("%d Done.\n", RtsLayer::myNode()); fflush(stderr);
-            }
+        // is it time to write? (and thread 0, and not in an MPI or ADIOS call)
+        if (steady_clock::now() > next_write && !plugin_done &&
+            !writing && data->tid == 0 &&
+            strstr(data->timer_name, "MPI_") == NULL) {
+            writing = true;
+            TAU_VERBOSE("%d Sending data from exit event %s...\n", RtsLayer::myNode(), data->timer_name); fflush(stderr);
+            Tau_plugin_event_dump_data_t dummy_data;
+            Tau_plugin_adios2_dump(&dummy_data);
+            TAU_VERBOSE("%d Done.\n", RtsLayer::myNode()); fflush(stderr);
+            // increment to a time in the future
+            next_write = next_write +
+                microseconds(tau_plugin::thePluginOptions().env_period);
+            writing = false;
         }
     }
     return 0;
@@ -1340,15 +1440,15 @@ int Tau_plugin_adios2_atomic_trigger(Tau_plugin_event_atomic_event_trigger_data_
     tmparray[3] = (tau_data_t)(counter_index);
     tmparray[4] = (tau_data_t)(data->value);
 
-    tau_data_t ts = my_adios().previous_timestamp[data->tid] >
-        data->timestamp ? my_adios().previous_timestamp[data->tid] :
+    tau_data_t ts = my_adios().getAdiosThread(data->tid)->previous_timestamp >
+        data->timestamp ? my_adios().getAdiosThread(data->tid)->previous_timestamp :
             data->timestamp;
-    my_adios().previous_timestamp[data->tid] = ts;
+    my_adios().getAdiosThread(data->tid)->previous_timestamp = ts;
 #ifdef DO_VALIDATION
     TAU_VERBOSE("Counter: %d %d %lu %lu %s %s\n", global_comm_rank, data->tid, data->timestamp, ts, (data->timestamp != ts ? "FIXED" : ""), data->counter_name);
 #endif
 
-    auto &tmp = my_adios().counter_values_array[data->tid];
+    auto &tmp = my_adios().getAdiosThread(data->tid)->counter_values;
     tau_plugin::event_gets_control();
     tmp.push_back(
         std::make_pair(
@@ -1434,9 +1534,12 @@ extern "C" int Tau_plugin_init_func(int argc, char **argv, int id) {
     Tau_plugin_callbacks_t cb;
     TAU_VERBOSE("TAU PLUGIN ADIOS2 Init\n"); fflush(stdout);
     tau_plugin::Tau_ADIOS2_parse_environment_variables();
+    if (tau_plugin::thePluginOptions().env_select_file != "") {
+        tau_plugin::Tau_ADIOS2_parse_selection_file(tau_plugin::thePluginOptions().env_select_file.c_str());
+    }
 #if TAU_MPI
-    PMPI_Comm_size(MPI_COMM_WORLD, &global_comm_size);
-    PMPI_Comm_rank(MPI_COMM_WORLD, &global_comm_rank);
+    global_comm_rank = RtsLayer::myNode();
+    global_comm_size = tau_totalnodes(0,1);
 #endif
     /* Create the callback object */
     TAU_UTIL_INIT_TAU_PLUGIN_CALLBACKS(&cb);
@@ -1541,11 +1644,23 @@ bool skip_timer(const char * key) {
     if (!thePluginOptions().env_use_selection) {
         return false;
     }
+    /* We use thread-local copies of the sets to speed things up.
+       First, We want to modify these sets, and we don't want to lock.
+       Also, after we've checked against all the wildcards we want
+       to explicitly include or exclude the timer after the first
+       time we see it. */
+    static thread_local std::unordered_set<std::string>
+        my_included_timers{std::begin(thePluginOptions().included_timers),
+                           std::end(thePluginOptions().included_timers)};
+    static thread_local std::unordered_set<std::string>
+        my_excluded_timers{std::begin(thePluginOptions().excluded_timers),
+                           std::end(thePluginOptions().excluded_timers)};
+
     // check to see if this label is excluded
-    if (Tau_ADIOS2_contains(thePluginOptions().excluded_timers, key, false)) {
+    if (Tau_ADIOS2_contains(my_excluded_timers, key, false)) {
         return true;
     // check to see if this label is included
-    } else if (Tau_ADIOS2_contains(thePluginOptions().included_timers, key, false)) {
+    } else if (Tau_ADIOS2_contains(my_included_timers, key, false)) {
         return false;
     } else {
         // check to see if it's in the excluded wildcards
@@ -1554,7 +1669,7 @@ bool skip_timer(const char * key) {
              it!=thePluginOptions().excluded_timers_with_wildcards.end(); ++it) {
             if (strmatch(key, it->c_str(), strlen(key), it->length())) {
                 // make the lookup faster next time
-                thePluginOptions().excluded_timers.insert(key);
+                my_excluded_timers.insert(key);
                 return true;
             }
         }
@@ -1564,7 +1679,7 @@ bool skip_timer(const char * key) {
              it!=thePluginOptions().included_timers_with_wildcards.end(); ++it) {
             if (strmatch(key, it->c_str(), strlen(key), it->length())) {
                 // make the lookup faster next time
-                thePluginOptions().included_timers.insert(key);
+                my_included_timers.insert(key);
                 return false;
             }
         }
@@ -1573,9 +1688,13 @@ bool skip_timer(const char * key) {
     // do we have an inclusion list? If so, then skip (because we didn't match it).
     if (!thePluginOptions().included_timers.empty() ||
         !thePluginOptions().included_timers_with_wildcards.empty()) {
+        // make the lookup faster next time
+        my_excluded_timers.insert(key);
         return true;
     }
     // by default, don't skip it.
+    // make the lookup faster next time
+    my_included_timers.insert(key);
     return false;
 }
 
@@ -1584,11 +1703,23 @@ bool skip_counter(const char * key) {
     if (!thePluginOptions().env_use_selection) {
         return false;
     }
+    /* We use thread-local copies of the sets to speed things up.
+       First, We want to modify these sets, and we don't want to lock.
+       Also, after we've checked against all the wildcards we want
+       to explicitly include or exclude the timer after the first
+       time we see it. */
+    static thread_local std::unordered_set<std::string>
+        my_included_counters{std::begin(thePluginOptions().included_counters),
+                           std::end(thePluginOptions().included_counters)};
+    static thread_local std::unordered_set<std::string>
+        my_excluded_counters{std::begin(thePluginOptions().excluded_counters),
+                           std::end(thePluginOptions().excluded_counters)};
+
     // check to see if this label is excluded
-    if (Tau_ADIOS2_contains(thePluginOptions().excluded_counters, key, false)) {
+    if (Tau_ADIOS2_contains(my_excluded_counters, key, false)) {
         return true;
     // check to see if this label is included
-    } else if (Tau_ADIOS2_contains(thePluginOptions().included_counters, key, false)) {
+    } else if (Tau_ADIOS2_contains(my_included_counters, key, false)) {
         return false;
     } else {
         // check to see if it's in the excluded wildcards
@@ -1597,7 +1728,7 @@ bool skip_counter(const char * key) {
              it!=thePluginOptions().excluded_counters_with_wildcards.end(); ++it) {
             if (strmatch(key, it->c_str(), strlen(key), it->length())) {
                 // make the lookup faster next time
-                thePluginOptions().excluded_counters.insert(key);
+                my_excluded_counters.insert(key);
                 return true;
             }
         }
@@ -1607,7 +1738,7 @@ bool skip_counter(const char * key) {
              it!=thePluginOptions().included_counters_with_wildcards.end(); ++it) {
             if (strmatch(key, it->c_str(), strlen(key), it->length())) {
                 // make the lookup faster next time
-                thePluginOptions().included_counters.insert(key);
+                my_included_counters.insert(key);
                 return false;
             }
         }
@@ -1616,8 +1747,12 @@ bool skip_counter(const char * key) {
     // do we have an inclusion list? If so, then skip (because we didn't match it).
     if (!thePluginOptions().included_counters.empty() ||
         !thePluginOptions().included_counters_with_wildcards.empty()) {
+        // make the lookup faster next time
+        my_excluded_counters.insert(key);
         return true;
     }
+    // make the lookup faster next time
+    my_included_counters.insert(key);
     return false;
 }
 
