@@ -133,7 +133,7 @@ size_t store_trial() {
     /* Get the executable name from the TAU metadata */
     Tau_metadata_key key;
     key.name = strdup("Executable");
-    const char * execname = Tau_metadata_getMetaData(0)[key]->data.cval;
+    const char * execname = Tau_metadata_getMetaData(0).at(key)->data.cval;
     sql << "insert into trial (name) values ('" << execname << "');";
     rc = sqlite3_exec(db, sql.str().c_str(), callback, 0, &zErrMsg);
 
@@ -511,10 +511,9 @@ void close_database() {
 void write_profile_to_database() {
     if (done) { return; }
     if (RtsLayer::myThread() != 0) { return; }
-    Tau_metadata_writeEndingTimeStamp();
     TauProfiler_updateAllIntermediateStatistics();
     size_t trial_id = 0;
-    if (comm_rank == 0) {
+    if (comm_rank <= 0) {
         create_database();
         trial_id = store_profile(0UL);
         close_database();
@@ -564,10 +563,8 @@ int Tau_plugin_metadata_registration_complete_sqlite3(Tau_plugin_event_metadata_
 
 int Tau_plugin_event_post_init_sqlite3(Tau_plugin_event_post_init_data_t* data) {
     //printf("NULL PLUGIN %s\n", __func__);
-#ifdef TAU_MPI
-    MPI_Comm_rank(MPI_COMM_WORLD, &comm_rank);
-    MPI_Comm_size(MPI_COMM_WORLD, &comm_size);
-#endif
+    comm_rank = RtsLayer::myNode();
+    comm_size = tau_totalnodes(0,1);
     return 0;
 }
 
