@@ -25,11 +25,11 @@ int Tau_plugin_test_event_end_of_execution(Tau_plugin_event_end_of_execution_dat
 /*This is the init function that gets invoked by the plugin mechanism inside TAU.
  * Every plugin MUST implement this function to register callbacks for various events 
  * that the plugin is interested in listening to*/
-extern "C" int Tau_plugin_init_func(int argc, char **argv) {
+extern "C" int Tau_plugin_init_func(int argc, char **argv, int id) {
   Tau_plugin_callbacks * cb = (Tau_plugin_callbacks*)malloc(sizeof(Tau_plugin_callbacks));
   TAU_UTIL_INIT_TAU_PLUGIN_CALLBACKS(cb);
   cb->EndOfExecution = Tau_plugin_test_event_end_of_execution;
-  TAU_UTIL_PLUGIN_REGISTER_CALLBACKS(cb);
+  TAU_UTIL_PLUGIN_REGISTER_CALLBACKS(cb, id);
 
   return 0;
 }
