@@ -128,6 +128,7 @@ int& TheUsingCompInst()
 //////////////////////////////////////////////////////////////////////
 
 
+
 static char *strip_tau_group(const char *ProfileGroupName) {
   char *source = strdup(ProfileGroupName);
   const char *find = "TAU_GROUP_";
@@ -145,10 +146,20 @@ static char *strip_tau_group(const char *ProfileGroupName) {
   return source;
 }
 
+// The next ID number for a FunctionInfo instance.
+// This is used for the thread-local cache of FunctionMetrics.
+std::atomic<uint64_t> FunctionInfo::next_id{0};
+
+//thread_local FunctionInfo::FMetricListVector FunctionInfo::MetricThreadCache; //#Fixes opari bug, breaks pthreads
+thread_local FunctionInfo::FMetricListVector_local FunctionInfo::MetricThreadCache;     //vector<FunctionInfo::FunctionMetrics*>* FunctionInfo::MetricThreadCache=new vector<FunctionInfo::FunctionMetrics*>; // One entry per instance
+bool FunctionInfo::use_metric_tls = true;
+//bool FunctionInfo::destructed = false;
+//thread_local bool FunctionInfo::destructed_local = false;
+
 //////////////////////////////////////////////////////////////////////
 // FunctionInfoInit is called by all four forms of FunctionInfo ctor
 //////////////////////////////////////////////////////////////////////
-void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup, const char *ProfileGroupName, bool InitData, int tid)
+void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup, const char *ProfileGroupName, bool InitData)
 {
   /* Make sure TAU is initialized */
   static bool flag = true;
@@ -166,7 +177,7 @@ void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup, const char *Profile
   }
 #ifdef TAU_SCOREP
   if (Tau_global_getLightsOut()) {
-    TAU_VERBOSE("TAU<%d,%d>: FunctionInfoInit: Lights out... \n",RtsLayer::myNode(), tid);
+    TAU_VERBOSE("TAU<%d,%d>: FunctionInfoInit: Lights out... \n",RtsLayer::myNode(), RtsLayer::myThread());
     return;
   }
 #endif /* TAU_SCOREP */
@@ -178,6 +189,8 @@ void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup, const char *Profile
   // Use LockDB to avoid a possible race condition.
   RtsLayer::LockDB();
 
+  function_info_id = ++next_id;
+
   //Need to keep track of all the groups this function is a member of.
   AllGroups = strip_tau_group(ProfileGroupName);
 
@@ -197,10 +210,12 @@ void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup, const char *Profile
 
   //Add function name to the name list.
   TauProfiler_theFunctionList(NULL, NULL, true, (const char *)GetName());
-
+/*
+ //TODO: Confirm that this init is no longer needed
   if (InitData) {
     for (int i = 0; i < TAU_MAX_THREADS; i++) {
-      SetAlreadyOnStack(false, i);
+      SetAlreadyOnStack(false, i); 
+      SetCalls(i,0);
       NumCalls[i] = 0;
       NumSubrs[i] = 0;
       for (int j = 0; j < Tau_Global_numCounters; j++) {
@@ -210,14 +225,17 @@ void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup, const char *Profile
         dumpInclusiveValues[i][j] = 0;
       }
     }
-  }
-
+  } 
+*/
   MyProfileGroup_ = ProfileGroup;
 
   // While accessing the global function database, lock it to ensure
   // an atomic operation in the push_back and size() operations.
   // Important in the presence of concurrent threads.
   TheFunctionDB().push_back(this);
+  //DEBUG object address
+  //printf("Address of function %s is %p !\n",(const char *)GetName(),(TheFunctionDB().back()));
+  //DEBUG
   FunctionId = RtsLayer::GenerateUniqueId();
 
   StartAddr = 0;
@@ -239,7 +257,10 @@ void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup, const char *Profile
       !strstr(ProfileGroupName, "TAU_SAMPLE_CONTEXT") &&
       //!strstr(ProfileGroupName, "TAU_OMP_STATE") &&
       !strstr(ProfileGroupName, "TAU_UNWIND"))
-  {
+ {//TODO: DYNAPROF. We need to flag this to do at initial thread initializations since we don't have all our threads at startup in a dynamic implementation
+      setPathHistograms=true;
+  }
+    /*{
     for (int i = 0; i < TAU_MAX_THREADS; i++) {
       pathHistogram[i] = new TauPathHashTable<TauPathAccumulator>(i);
     }
@@ -247,7 +268,7 @@ void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup, const char *Profile
     for (int i = 0; i < TAU_MAX_THREADS; i++) {
       pathHistogram[i] = NULL;
     }
-  }
+  }*/
 
   // Initialization of CallSite discovery structures.
   isCallSite = false;
@@ -291,7 +312,7 @@ void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup, const char *Profile
 #endif
 
   DEBUGPROFMSG("nct "<< RtsLayer::myNode() <<","
-      << RtsLayer::myContext() << ", " << tid
+      << RtsLayer::myContext() << ", " << RtsLayer::myThread()
       << " FunctionInfo::FunctionInfo(n,t) : Name : "<< GetName()
       << " Group :  " << GetProfileGroup()
       << " Id :  " << GetFunctionId()
@@ -299,8 +320,9 @@ void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup, const char *Profile
 
 #ifdef TAU_PROFILEMEMORY
   {
-    char * buff = new char[strlen(Name)+strlen(Type)+100];
-    sprintf(buff, "%s %s - Heap Memory Used (KB)", Name, Type);
+    const int len = strlen(Name)+strlen(Type)+100;
+    char * buff = new char[len];
+    snprintf(buff, len,  "%s %s - Heap Memory Used (KB)", Name, Type);
     MemoryEvent = new tau::TauUserEvent(buff);
     delete buff;
   }
@@ -310,6 +332,8 @@ void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup, const char *Profile
   HeadroomEvent = new TauUserEvent(string(string(Name)+" "+Type+" - Memory Headroom Available (MB)").c_str());
 #endif /* TAU_PROFILEHEADROOM */
 
+//Already NULL by default
+/*
 #ifdef RENCI_STFF
   for (int t=0; t < TAU_MAX_THREADS; t++) {
     for (int i=0; i < TAU_MAX_COUNTERS; i++) {
@@ -317,12 +341,13 @@ void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup, const char *Profile
     }
   }
 #endif //RENCI_STFF
+*/
 
   /*Invoke plugins only if both plugin path and plugins are specified*/
   if(Tau_plugins_enabled.function_registration) {
     Tau_plugin_event_function_registration_data_t plugin_data;
     plugin_data.function_info_ptr = this;
-    plugin_data.tid = tid;
+    plugin_data.tid = RtsLayer::myThread();
     Tau_util_invoke_callbacks(TAU_PLUGIN_EVENT_FUNCTION_REGISTRATION, GetName(), &plugin_data);
   }
 
@@ -332,50 +357,48 @@ void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup, const char *Profile
 
 //////////////////////////////////////////////////////////////////////
 FunctionInfo::FunctionInfo(const char *name, const char *type, TauGroup_t ProfileGroup,
-    const char *ProfileGroupName, bool InitData, int tid)
+    const char *ProfileGroupName, bool InitData)
 {
   DEBUGPROFMSG("FunctionInfo::FunctionInfo: MyProfileGroup_ = " << ProfileGroup << " Mask = " << RtsLayer::TheProfileMask() <<endl;);
   Name = strdup(name);
   Type = strdup(type);
   FullName = NULL;
   DEBUGPROFMSG("FunctionInfo::FunctionInfo: MyProfileGroup_ = " << ProfileGroup << " Mask = " << RtsLayer::TheProfileMask() <<endl;);
-  FunctionInfoInit(ProfileGroup, ProfileGroupName, InitData, tid);
+  FunctionInfoInit(ProfileGroup, ProfileGroupName, InitData);
 }
 
 //////////////////////////////////////////////////////////////////////
 
 FunctionInfo::FunctionInfo(const char *name, const string& type, TauGroup_t ProfileGroup,
-    const char *ProfileGroupName, bool InitData, int tid)
+    const char *ProfileGroupName, bool InitData)
 {
   Name = strdup(name);
   Type = strdup(type.c_str());
   FullName = NULL;
-  FunctionInfoInit(ProfileGroup, ProfileGroupName, InitData, tid);
+  FunctionInfoInit(ProfileGroup, ProfileGroupName, InitData);
 }
 
 //////////////////////////////////////////////////////////////////////
 
 FunctionInfo::FunctionInfo(const string& name, const char * type,
-	TauGroup_t ProfileGroup , const char *ProfileGroupName, bool InitData,
-	int tid) {
+	TauGroup_t ProfileGroup , const char *ProfileGroupName, bool InitData) {
   Name = strdup(name.c_str());
   Type = strdup(type);
   FullName = NULL;
 
-  FunctionInfoInit(ProfileGroup, ProfileGroupName, InitData, tid);
+  FunctionInfoInit(ProfileGroup, ProfileGroupName, InitData);
 }
 
 //////////////////////////////////////////////////////////////////////
 
 FunctionInfo::FunctionInfo(const string& name, const string& type,
-	TauGroup_t ProfileGroup , const char *ProfileGroupName, bool InitData,
-	int tid) {
+	TauGroup_t ProfileGroup , const char *ProfileGroupName, bool InitData) {
 
   Name = strdup(name.c_str());
   Type = strdup(type.c_str());
   FullName = NULL;
 
-  FunctionInfoInit(ProfileGroup, ProfileGroupName, InitData, tid);
+  FunctionInfoInit(ProfileGroup, ProfileGroupName, InitData);
 }
 
 //////////////////////////////////////////////////////////////////////
@@ -394,8 +417,9 @@ FunctionInfo::~FunctionInfo()
   Name = Type = GroupName = AllGroups = NULL;
 #ifndef TAU_WINDOWS
 #ifndef _AIX
-  for (int i = 0; i < TAU_MAX_THREADS; i++) {
-    delete pathHistogram[i];
+  int maxThreads=getPathHistogramSize();
+  for (int i = 0; i < maxThreads; i++) {
+    delete GetPathHistogram(i);//pathHistogram[i];// TODO: DYNAPROF
   }
 #endif /* _AIX */
 #endif /* TAU_WINDOWS */
@@ -405,7 +429,7 @@ FunctionInfo::~FunctionInfo()
 double * FunctionInfo::GetExclTime(int tid){
   double * tmpCharPtr = (double *) malloc( sizeof(double) * Tau_Global_numCounters);
   for(int i=0;i<Tau_Global_numCounters;i++){
-    tmpCharPtr[i] = ExclTime[tid][i];
+    tmpCharPtr[i] = GetExclTimeForCounter(tid,i);
   }
   return tmpCharPtr;
 }
@@ -413,35 +437,11 @@ double * FunctionInfo::GetExclTime(int tid){
 double * FunctionInfo::GetInclTime(int tid){
   double * tmpCharPtr = (double *) malloc( sizeof(double) * Tau_Global_numCounters);
   for(int i=0;i<Tau_Global_numCounters;i++){
-    tmpCharPtr[i] = InclTime[tid][i];
+    tmpCharPtr[i] = GetInclTimeForCounter(tid,i);
   }
   return tmpCharPtr;
 }
 
-
-double *FunctionInfo::getInclusiveValues(int tid) {
-  printf ("TAU: Warning, potentially evil function called\n");
-  return InclTime[tid];
-}
-
-double *FunctionInfo::getExclusiveValues(int tid) {
-  printf ("TAU: Warning, potentially evil function called\n");
-  return ExclTime[tid];
-}
-
-void FunctionInfo::getInclusiveValues(int tid, double *values) {
-  for(int i=0; i<Tau_Global_numCounters; i++) {
-    values[i] = InclTime[tid][i];
-  }
-}
-
-void FunctionInfo::getExclusiveValues(int tid, double *values) {
-  for(int i=0; i<Tau_Global_numCounters; i++) {
-    values[i] = ExclTime[tid][i];
-  }
-}
-
-
 //////////////////////////////////////////////////////////////////////
 x_uint64 FunctionInfo::GetFunctionId(void) {
   // To avoid data races, we use a lock if the id has not been created
@@ -465,8 +465,8 @@ void FunctionInfo::ResetExclTimeIfNegative(int tid) {
      and multiple counters */
   int i;
   for (i=0; i < Tau_Global_numCounters; i++) {
-    if (ExclTime[tid][i] < 0) {
-      ExclTime[tid][i] = 0.0; /* set each negative counter to zero */
+    if (GetExclTimeForCounter(tid,i) < 0) {
+      SetExclTimeForCounter(0.0, tid, i); /* set each negative counter to zero */
     }
   }
   return;
@@ -475,7 +475,8 @@ void FunctionInfo::ResetExclTimeIfNegative(int tid) {
 
 
 //////////////////////////////////////////////////////////////////////
-void tauCreateFI(void **ptr, const char *name, const char *type, TauGroup_t ProfileGroup, const char *ProfileGroupName)
+void tauCreateFI(void **ptr, const char *name, const char *type,
+    TauGroup_t ProfileGroup, const char *ProfileGroupName)
 {
   /* This is the entry point into TAU from PDT-instrumented C++ codes, so
    * make sure that TAU is ready to go before doing anything else! */
@@ -503,8 +504,8 @@ void tauCreateFI(void **ptr, const char *name, const char *type, TauGroup_t Prof
   }
 }
 
-void tauCreateFI(void **ptr, const char *name, const string& type, TauGroup_t ProfileGroup,
-    const char *ProfileGroupName)
+void tauCreateFI(void **ptr, const char *name, const string& type,
+    TauGroup_t ProfileGroup, const char *ProfileGroupName)
 {
   /* This is the entry point into TAU from PDT-instrumented C++ codes, so
    * make sure that TAU is ready to go before doing anything else! */
@@ -532,23 +533,15 @@ void tauCreateFI(void **ptr, const char *name, const string& type, TauGroup_t Pr
   }
 }
 
-void tauCreateFI_signalSafe(void **ptr, const string& name, const char *type, TauGroup_t ProfileGroup,
-    const char *ProfileGroupName)
+void tauCreateFI_signalSafe(void **ptr, const string& name, const char *type,
+    TauGroup_t ProfileGroup, const char *ProfileGroupName)
 {
-  /* This is the entry point into TAU from PDT-instrumented C++ codes, so
-   * make sure that TAU is ready to go before doing anything else! */
-  static int do_this_once = Tau_init_initializeTAU();
+  /* This is NOT the entry point into TAU from PDT-instrumented C++ codes!
+   * this comes from creating the top level timer in TauCAPI.cpp. */
   if (*ptr == 0) {
     // Protect TAU from itself
     TauInternalFunctionGuard protects_this_function;
 
-#ifdef TAU_CHARM
-    if (RtsLayer::myNode() != -1)
-    RtsLayer::LockEnv();
-#else
-    RtsLayer::LockEnv();
-#endif
-    if (*ptr == 0) {
     /* KAH - Whoops!! We can't call "new" here, because malloc is not
      * safe in signal handling. therefore, use the special memory
      * allocation routines */
@@ -562,19 +555,12 @@ void tauCreateFI_signalSafe(void **ptr, const string& name, const char *type, Ta
     new(*ptr) FunctionInfo(name, type, ProfileGroup, ProfileGroupName);
 #else
     *ptr = (void *) new FunctionInfo(name, type, ProfileGroup, ProfileGroupName);
-#endif
-    }
-#ifdef TAU_CHARM
-    if (RtsLayer::myNode() != -1)
-    RtsLayer::UnLockEnv();
-#else
-    RtsLayer::UnLockEnv();
 #endif
   }
 }
 
-void tauCreateFI(void **ptr, const string& name, const char *type, TauGroup_t ProfileGroup,
-    const char *ProfileGroupName)
+void tauCreateFI(void **ptr, const string& name, const char *type,
+    TauGroup_t ProfileGroup, const char *ProfileGroupName)
 {
   /* This is the entry point into TAU from PDT-instrumented C++ codes, so
    * make sure that TAU is ready to go before doing anything else! */
@@ -656,8 +642,8 @@ void FunctionInfo::addPcSample(unsigned long *pcStack, int tid, double interval[
 {
   // Add to the mmap-ed histogram. We start with a temporary conversion. This
   //   becomes unnecessary once we stop using the vector.
-  if (pathHistogram[tid] == NULL) return; 
-  TauPathAccumulator * accumulator = pathHistogram[tid]->get(pcStack);
+  if (GetPathHistogram(tid) == NULL) return;
+  TauPathAccumulator * accumulator = GetPathHistogram(tid)->get(pcStack);
   if (accumulator == NULL) {
     /* KAH - Whoops!! We can't call "new" here, because malloc is not
      * safe in signal handling. therefore, use the special memory
@@ -671,7 +657,7 @@ void FunctionInfo::addPcSample(unsigned long *pcStack, int tid, double interval[
     accumulator = (TauPathAccumulator*)Tau_MemMgr_malloc(tid, sizeof(TauPathAccumulator));
     new (accumulator) TauPathAccumulator(1, interval);
 
-    bool success = pathHistogram[tid]->insert(pcStack, *accumulator);
+    bool success = GetPathHistogram(tid)->insert(pcStack, *accumulator);//pathHistogram[tid]->insert(pcStack, *accumulator);//TODO: DYNAPROF
     if (!success) {
       fprintf(stderr, "addPcSample: Failed to insert sample.\n");
     }
