@@ -79,9 +79,15 @@ extern "C" {
 
 bool PapiLayer::papiInitialized = false;
 double PapiLayer::scalingFactor = 0.0;
-ThreadValue * PapiLayer::ThreadList[TAU_MAX_THREADS] = { 0 };
 int PapiLayer::numCounters = 0;
 int PapiLayer::counterList[MAX_PAPI_COUNTERS];
+bool PapiLayer::destroyed=false;
+std::mutex PapiLayer::papiVectorMutex;
+PapiLayer::PapiThreadList & PapiLayer::ThePapiThreadList() {
+    static PapiLayer::PapiThreadList threadList;
+    return threadList;
+}
+
 
 int tauSampEvent = 0;
 
@@ -180,35 +186,38 @@ void Tau_child(void)
 
   rc = PAPI_thread_init((unsigned long (*)(void))(RtsLayer::unsafeThreadId));
 
-if(  tid >= TAU_MAX_THREADS) {
-    fprintf (stderr, "TAU: Exceeded max thread count of TAU_MAX_THREADS\n");
-  }
-
+  PapiThreadValue* localThreadValue=PapiLayer::getThreadValue(tid);
   /* Check ThreadList */
-  if (PapiLayer::ThreadList[tid] == 0) {
-    PapiLayer::ThreadList[tid] = new ThreadValue;
+  if (localThreadValue == 0) {
+    localThreadValue=new PapiThreadValue;
+    PapiLayer::setThreadValue(tid, localThreadValue);
   }
-  PapiLayer::ThreadList[tid]->ThreadID = tid;
-  PapiLayer::ThreadList[tid]->CounterValues = new long long[MAX_PAPI_COUNTERS];
+  localThreadValue->ThreadID = tid;
+  localThreadValue->CounterValues = new long long[MAX_PAPI_COUNTERS];
   for (i = 0; i < MAX_PAPI_COUNTERS; i++) {
-    PapiLayer::ThreadList[tid]->CounterValues[i] = 0L;
+    localThreadValue->CounterValues[i] = 0L;
   }
 
   for (i = 0; i < TAU_PAPI_MAX_COMPONENTS; i++) {
-    PapiLayer::ThreadList[tid]->NumEvents[i] = 0;
-    PapiLayer::ThreadList[tid]->EventSet[i] = PAPI_NULL;
-    rc = PAPI_create_eventset(&(PapiLayer::ThreadList[tid]->EventSet[i]));
+    localThreadValue->NumEvents[i] = 0;
+    localThreadValue->EventSet[i] = PAPI_NULL;
+    rc = PAPI_create_eventset(&(localThreadValue->EventSet[i]));
     if (rc != PAPI_OK) {
       fprintf(stderr, "TAU: Error creating PAPI event set: %s\n", PAPI_strerror(rc));
       return;
     }
     if(TauEnv_get_papi_multiplexing()) {
-      rc = PAPI_assign_eventset_component( PapiLayer::ThreadList[tid]->EventSet[i], 0 );
+      rc = PAPI_multiplex_init();
+      if ( PAPI_OK != rc ) {
+        fprintf(stderr, "PAPI_multiplex_init failed (%s)\n", PAPI_strerror(rc));
+        exit(1);
+      }
+      rc = PAPI_assign_eventset_component( localThreadValue->EventSet[i], 0 );
       if ( PAPI_OK != rc ) {
         fprintf(stderr, "PAPI_assign_eventset_component failed (%s)\n", PAPI_strerror(rc));
         return;
       }
-      rc = PAPI_set_multiplex(PapiLayer::ThreadList[tid]->EventSet[i]);
+      rc = PAPI_set_multiplex(localThreadValue->EventSet[i]);
       if ( PAPI_OK != rc ) {
         fprintf(stderr, "PAPI_set_multiplex failed (%s)\n", PAPI_strerror(rc));
         return;
@@ -219,24 +228,24 @@ if(  tid >= TAU_MAX_THREADS) {
   /* PAPI 3 support goes here */
   for (i = 0; i < numCounters; i++) {
     int comp = PAPI_COMPONENT_INDEX (PapiLayer::counterList[i]);
-    rc = PAPI_add_event(PapiLayer::ThreadList[tid]->EventSet[comp], PapiLayer::counterList[i]);
+    rc = PAPI_add_event(localThreadValue->EventSet[comp], PapiLayer::counterList[i]);
     if (rc != PAPI_OK) {
-      fprintf(stderr, "pid=%d, TAU: Error adding PAPI events: %s\n", RtsLayer::getPid(), PAPI_strerror(rc));
+      fprintf(stderr, "pid=%d, TAU (tau_child): Error adding PAPI events: %s\n", RtsLayer::getPid(), PAPI_strerror(rc));
       return;
     }
 
     // this creates a mapping from 'component', and index in that component back    // to the original index, since we return just a single array of values
-    PapiLayer::ThreadList[tid]->Comp2Metric[comp][PapiLayer::ThreadList[tid]->NumEvents[comp]++] = i;
+    localThreadValue->Comp2Metric[comp][localThreadValue->NumEvents[comp]++] = i;
   }
 
   for (i = 0; i < TAU_PAPI_MAX_COMPONENTS; i++) {
-    if (PapiLayer::ThreadList[tid]->NumEvents[i] >= 1) {    // if there were active counters for this component
-      rc = PAPI_start(PapiLayer::ThreadList[tid]->EventSet[i]);
+    if (localThreadValue->NumEvents[i] >= 1) {    // if there were active counters for this component
+      rc = PAPI_start(localThreadValue->EventSet[i]);
     }
   }
 
   if (rc != PAPI_OK) {
-    fprintf(stderr, "pid=%d: TAU: Error calling PAPI_start: %s, tid = %d\n", RtsLayer::getPid(), PAPI_strerror(rc), tid);
+    fprintf(stderr, "pid=%d: TAU (tau_child2): Error calling PAPI_start: %s, tid = %d\n", RtsLayer::getPid(), PAPI_strerror(rc), tid);
     return;
   }
 
@@ -292,50 +301,41 @@ int PapiLayer::initializeThread(int tid)
 {
   int rc;
 
-  if (tid >= TAU_MAX_THREADS) {
-    fprintf (stderr, "TAU: Exceeded max thread count of TAU_MAX_THREADS\n");
-    return -1;
-  }
-
-  if (!ThreadList[tid]) {
+  if (!getThreadValue(tid)){
     RtsLayer::LockDB();
-    if (!ThreadList[tid]) {
+    if (!getThreadValue(tid)){
       dmesg(1, "TAU: PAPI: Initializing Thread Data for TID = %d\n", tid);
 
       /* Task API does not have a real thread associated with it. It is fake */
       if (Tau_is_thread_fake(tid) == 1) tid = 0;
-
-      ThreadList[tid] = new ThreadValue;
-      ThreadList[tid]->ThreadID = tid;
-      ThreadList[tid]->CounterValues = new long long[MAX_PAPI_COUNTERS];
-      memset(ThreadList[tid]->CounterValues, 0, MAX_PAPI_COUNTERS*sizeof(long long));
-
+      PapiThreadValue* localThreadValue = new PapiThreadValue;
+      setThreadValue(tid,localThreadValue);
+      localThreadValue->ThreadID = tid;
+      localThreadValue->CounterValues = new long long[MAX_PAPI_COUNTERS];
+      memset(localThreadValue->CounterValues, 0, MAX_PAPI_COUNTERS*sizeof(long long));
+      
+	  if(TauEnv_get_papi_multiplexing()) {
+	      rc = PAPI_multiplex_init();
+          if ( PAPI_OK != rc ) {
+            fprintf(stderr, "PAPI_multiplex_init failed (%s)\n", PAPI_strerror(rc));
+            exit(1);
+          }
+      }
+	  
       for (int i=0; i<TAU_PAPI_MAX_COMPONENTS; i++) {
-        ThreadList[tid]->NumEvents[i] = 0;
-        ThreadList[tid]->EventSet[i] = PAPI_NULL;
-        rc = PAPI_create_eventset(&(ThreadList[tid]->EventSet[i]));
+        localThreadValue->NumEvents[i] = 0;
+        localThreadValue->EventSet[i] = PAPI_NULL;
+        rc = PAPI_create_eventset(&(localThreadValue->EventSet[i]));
         if (rc != PAPI_OK) {
           fprintf (stderr, "TAU: Error creating PAPI event set: %s\n", PAPI_strerror(rc));
           RtsLayer::UnLockDB();
           return -1;
         }
-        if(TauEnv_get_papi_multiplexing()) {
-          rc = PAPI_assign_eventset_component( ThreadList[tid]->EventSet[i], 0 );
-          if ( PAPI_OK != rc ) {
-            fprintf(stderr, "PAPI_assign_eventset_component failed (%s)\n", PAPI_strerror(rc));
-            exit(1);
-          }
-          rc = PAPI_set_multiplex(ThreadList[tid]->EventSet[i]);
-          if ( PAPI_OK != rc ) {
-            fprintf(stderr, "PAPI_set_multiplex failed (%s)\n", PAPI_strerror(rc));
-            return -1;
-          }
-        }
       }
 
 #ifndef PAPI_VERSION
       /* PAPI 2 support goes here */
-      rc = PAPI_add_events(&(ThreadList[tid]->EventSet[0]), counterList, numCounters);
+      rc = PAPI_add_events(&(localThreadValue->EventSet[0]), counterList, numCounters);
       if (rc != PAPI_OK) {
         fprintf (stderr, "TAU: Error adding PAPI events: %s\n", PAPI_strerror(rc));
         RtsLayer::UnLockDB();
@@ -344,17 +344,60 @@ int PapiLayer::initializeThread(int tid)
 #elif (PAPI_VERSION_MAJOR(PAPI_VERSION) >= 3)
       /* PAPI 3 support goes here */
       for (int i=0; i<numCounters; i++) {
-        int comp = PAPI_COMPONENT_INDEX (counterList[i]);
-        rc = PAPI_add_event(ThreadList[tid]->EventSet[comp], counterList[i]);
+	    int comp = PAPI_COMPONENT_INDEX (counterList[i]);
+        if(comp > TAU_PAPI_MAX_COMPONENTS) {
+          fprintf(stderr, "TAU: PAPI component index %d greater than maximum %d; increase TAU_PAPI_MAX_COMPONENTS\n", comp, TAU_PAPI_MAX_COMPONENTS);
+          RtsLayer::UnLockDB();
+          return -1;
+        }
+		if(TauEnv_get_papi_multiplexing() && localThreadValue->NumEvents[comp] == 0) {
+		  rc = PAPI_assign_eventset_component( localThreadValue->EventSet[comp], comp );
+          if ( PAPI_OK != rc ) {
+            fprintf(stderr, "PAPI_assign_eventset_component failed (%s)\n", PAPI_strerror(rc));
+			RtsLayer::UnLockDB();
+            return -1;
+          }
+          // Not all components support multiplexing, so we need to check
+          // whether this component does before enabling it for the EventSet
+          // that corresponds to a given component.
+          const PAPI_component_info_t * comp_info = PAPI_get_component_info(comp);
+          bool supports_multiplex = false;
+          const char * comp_name = NULL;
+          if(comp_info != NULL) {
+            supports_multiplex = comp_info->kernel_multiplex;
+            comp_name = comp_info->name;
+          }
+          TAU_VERBOSE("TAU: PAPI: Configuring PAPI component %d \"%s\"\n", comp, comp_name);
+          if(supports_multiplex) {
+            TAU_VERBOSE("TAU: PAPI: Component %d \"%s\" supports multiplexing, will enable multiplexing\n", comp, comp_name);
+            rc = PAPI_set_multiplex(localThreadValue->EventSet[comp]);
+            if ( PAPI_OK != rc ) {
+                fprintf(stderr, "PAPI_set_multiplex failed for component %d: (%s)\n", comp, PAPI_strerror(rc));
+                RtsLayer::UnLockDB();
+                return -1;
+            }  
+          } else {
+            TAU_VERBOSE("TAU: PAPI: Component %d \"%s\" does not support multiplexing, not enabling multiplexing\n", comp, comp_name);
+          }
+		}
+		  
+
+        rc = PAPI_add_event(localThreadValue->EventSet[comp], counterList[i]);
         if (rc != PAPI_OK) {
-          fprintf (stderr, "TAU: Error adding PAPI events: %s\n", PAPI_strerror(rc));
+          char event_name[PAPI_MAX_STR_LEN];
+          int name_rc = PAPI_event_code_to_name(counterList[i], event_name);
+          if(name_rc == PAPI_OK) {
+            fprintf(stderr, "TAU: Error adding PAPI event named \"%s\": %s\n", event_name, PAPI_strerror(rc));
+          } else {
+            fprintf (stderr, "TAU: Error adding PAPI event (unknown name): %s\n", PAPI_strerror(rc));
+          }
           RtsLayer::UnLockDB();
           return -1;
         }
 
         // this creates a mapping from 'component', and index in that component back
         // to the original index, since we return just a single array of values
-        ThreadList[tid]->Comp2Metric[comp][ThreadList[tid]->NumEvents[comp]++] = i;
+        localThreadValue->Comp2Metric[comp][localThreadValue->NumEvents[comp]++] = i;
       }
 
 
@@ -364,7 +407,7 @@ int PapiLayer::initializeThread(int tid)
           int comp = PAPI_COMPONENT_INDEX (tauSampEvent);
           int threshold = TauEnv_get_ebs_period();
           TAU_VERBOSE("TAU: Setting PAPI overflow handler\n");
-          rc = PAPI_overflow(ThreadList[tid]->EventSet[comp], tauSampEvent, threshold, 0, Tau_sampling_papi_overflow_handler);
+          rc = PAPI_overflow(localThreadValue->EventSet[comp], tauSampEvent, threshold, 0, Tau_sampling_papi_overflow_handler);
           if (rc != PAPI_OK) {
             fprintf (stderr, "TAU Sampling Warning: Error adding PAPI overflow handler: %s. Threshold=%d\n", PAPI_strerror(rc), threshold);
             tauSampEvent = 0; // Make sampling use itimer instead. We can disable it later.
@@ -379,20 +422,21 @@ int PapiLayer::initializeThread(int tid)
 #endif
 
       for (int i=0; i<TAU_PAPI_MAX_COMPONENTS; i++) {
-        if (ThreadList[tid]->NumEvents[i] > 0) { // if there were active counters for this component
-          rc = PAPI_start(ThreadList[tid]->EventSet[i]);
+        if (localThreadValue->NumEvents[i] > 0) { // if there were active counters for this component
+          TAU_VERBOSE("TAU: PAPI: Starting event set %d\n", i);
+          rc = PAPI_start(localThreadValue->EventSet[i]);
           if (rc != PAPI_OK) {
-            fprintf (stderr, "pid=%d: TAU: Error calling PAPI_start: %s, tid = %d\n", RtsLayer::getPid(), PAPI_strerror(rc), tid);
+            fprintf (stderr, "pid=%d: TAU(initializeThread): Error calling PAPI_start: %s, tid = %d\n", RtsLayer::getPid(), PAPI_strerror(rc), tid);
             RtsLayer::UnLockDB();
             return -1;
           }
         }
       }
+    dmesg(10, "ThreadList[%d] = %p\n", tid, localThreadValue);
     } /*if (!ThreadList[tid]) */
     RtsLayer::UnLockDB();
   } /*if (!ThreadList[tid]) */
 
-  dmesg(10, "ThreadList[%d] = %p\n", tid, ThreadList[tid]);
   return 0;
 }
 
@@ -402,10 +446,10 @@ int PapiLayer::initializeThread(int tid)
 /////////////////////////////////////////////////
 long long *PapiLayer::getAllCounters(int tid, int *numValues) {
   int rc=0;
-  long long tmpCounters[MAX_PAPI_COUNTERS];
+  long long tmpCounters[MAX_PAPI_COUNTERS] = {0};
 
   /* Task API does not have a real thread associated with it. It is fake */
-  if (Tau_is_thread_fake(tid) == 1) tid = 0;
+  if (Tau_is_thread_fake(tid) == 1) { return NULL; }
 
   if (!papiInitialized) {
     if (initializePapiLayer()) {
@@ -418,52 +462,55 @@ long long *PapiLayer::getAllCounters(int tid, int *numValues) {
     return NULL;
   }
 
-  if (ThreadList[tid] == NULL) {
+  PapiThreadValue* localThreadValue=getThreadValue(tid);
+  if (localThreadValue==NULL) {
     if(initializeThread(tid)) {
       return NULL;
     }
+    localThreadValue=getThreadValue(tid);
   }
 
   *numValues = numCounters;
-
+  //ThreadValue* localThreadValue=getThreadValue(tid);
+  
 #ifdef TAU_PAPI_DEBUG
   long long previousCounters[MAX_PAPI_COUNTERS];
   for (int i=0; i<numCounters; i++) {
-    previousCounters[i] = ThreadList[tid]->CounterValues[i];
+    previousCounters[i] = localThreadValue->CounterValues[i];
   }
 #endif
 
 #ifdef PTHREADS
   if (tid != RtsLayer::myThread() ) {
     //printf("Returning values for %d instead of %d\n", tid, RtsLayer::myThread());
-    return ThreadList[tid]->CounterValues;
+    return localThreadValue->CounterValues;
   }
 #endif /* PTHREADS */
 
   for (int comp=0; comp<TAU_PAPI_MAX_COMPONENTS; comp++) {
-    if (ThreadList[tid]->NumEvents[comp] > 0) { // if there were active counters for this component
+    if (localThreadValue->NumEvents[comp] > 0) { // if there were active counters for this component
       // read eventset for this component and reset counters
-      if (PAPI_read(ThreadList[tid]->EventSet[comp], tmpCounters) != PAPI_OK)
+      if (PAPI_read(localThreadValue->EventSet[comp], tmpCounters) != PAPI_OK)
         break;
-      if (PAPI_reset(ThreadList[tid]->EventSet[comp]) != PAPI_OK)
+      if (PAPI_reset(localThreadValue->EventSet[comp]) != PAPI_OK)
         break;
       // map back to original indices
-      for (int j=0; j<ThreadList[tid]->NumEvents[comp]; j++) {
-	int index = ThreadList[tid]->Comp2Metric[comp][j];
-	ThreadList[tid]->CounterValues[index] += tmpCounters[j];
-        dmesg(10, "ThreadList[%d]->CounterValues[%d] = %lld\n", tid, index, ThreadList[tid]->CounterValues[index]);
+      for (int j=0; j<localThreadValue->NumEvents[comp]; j++) {
+        int index = localThreadValue->Comp2Metric[comp][j];
+        localThreadValue->CounterValues[index] += tmpCounters[j];
+        dmesg(10, "ThreadList[%d]->CounterValues[%d] = %lld\n", tid, index, localThreadValue->CounterValues[index]);
       }
     }
   }
 
 #ifdef TAU_PAPI_DEBUG
   for (int i=0; i<numCounters; i++) {
-    long long difference = ThreadList[tid]->CounterValues[i] - previousCounters[i];
+    long long difference = localThreadValue->CounterValues[i] - previousCounters[i];
     dmesg(10, "TAU: PAPI: Difference[%d] = %lld\n", i, difference);
     if (difference < 0) {
       dmesg(0, "TAU: PAPI: Counter running backwards?\n");
       dmesg(0, "TAU: PAPI: Previous value[%d] = %lld\n", i, previousCounters[i]);
-      dmesg(0, "TAU: PAPI: Current  value[%d] = %lld\n", i, ThreadList[tid]->CounterValues[i]);
+      dmesg(0, "TAU: PAPI: Current  value[%d] = %lld\n", i, localThreadValue->CounterValues[i]);
       dmesg(0, "TAU: PAPI: Difference    [%d] = %lld\n", i, difference);
     }
   }
@@ -473,15 +520,14 @@ long long *PapiLayer::getAllCounters(int tid, int *numValues) {
     fprintf (stderr, "pid=%d, TAU: Error reading PAPI counters: %s\n", RtsLayer::getPid(), PAPI_strerror(rc));
     return NULL;
   }
-
-  return ThreadList[tid]->CounterValues;
+  return localThreadValue->CounterValues;
 }
 
 
 
 /////////////////////////////////////////////////
 int PapiLayer::reinitializePAPI() {
-  dmesg(1, "TAU: PapiLayer::reinitializePAPI\n");
+  dmesg(1, "%s", "TAU: PapiLayer::reinitializePAPI\n");
 
   // This function is called from the fork() handler
   // We need to clean up the ThreadList and then reinitialize PAPI
@@ -491,12 +537,13 @@ int PapiLayer::reinitializePAPI() {
     RtsLayer::LockDB();
     if (papiInitialized) {
       TAU_VERBOSE("Reinitializing papi...");
-      for(int i=0; i<TAU_MAX_THREADS; i++){
-        if (ThreadList[i] != NULL) {
-          delete ThreadList[i]->CounterValues;
-          delete ThreadList[i];
+      for(int i=0; i<ThePapiThreadList().size(); i++){
+	    PapiThreadValue* localThreadValue=getThreadValue(i);
+        if (localThreadValue != NULL) {
+          delete localThreadValue->CounterValues;
+          delete localThreadValue;
         }
-        ThreadList[i] = NULL;
+        setThreadValue(i,NULL);
       }
       TauMetrics_init();
       rc = initializePAPI();
@@ -512,8 +559,6 @@ int PapiLayer::reinitializePAPI() {
 // note, this only works on linux
 #include <sys/types.h>
 #include <linux/unistd.h>
-_syscall0(pid_t,gettid)
-pid_t gettid(void);
 
 unsigned long papi_thread_gettid(void) {
 #ifdef SYS_gettid
@@ -552,10 +597,6 @@ int PapiLayer::initializePAPI() {
 #endif /* TAU_MPI */
 #endif /* TAU_AT_FORK */
 
-  for (int i=0; i<TAU_MAX_THREADS; i++) {
-    ThreadList[i] = NULL;
-  }
-
   // Initialize PAPI
   if (Tau_initialize_papi_library() != PAPI_VER_CURRENT) {
     return -1;
@@ -698,18 +739,18 @@ int PapiLayer::initializeAndCheckRAPL(int tid) {
   if (!papiInitialized)
     initializePapiLayer();
 
-  if (!ThreadList[tid]) {
+  if (!getThreadValue(tid)) { 
     RtsLayer::LockDB();
-    if (!ThreadList[tid]) {
+    if (!getThreadValue(tid)) {
       dmesg(1, "TAU: PAPI: Initializing Thread Data for TID = %d\n", tid);
 
       /* Task API does not have a real thread associated with it. It is fake */
       if (Tau_is_thread_fake(tid) == 1) tid = 0;
-
-      ThreadList[tid] = new ThreadValue;
-      ThreadList[tid]->ThreadID = tid;
-      ThreadList[tid]->CounterValues = new long long[MAX_PAPI_COUNTERS];
-      memset(ThreadList[tid]->CounterValues, 0, MAX_PAPI_COUNTERS*sizeof(long long));
+      PapiThreadValue* localThreadValue = new PapiThreadValue;
+      setThreadValue(tid,localThreadValue);
+      localThreadValue->ThreadID = tid;
+      localThreadValue->CounterValues = new long long[MAX_PAPI_COUNTERS];
+      memset(localThreadValue->CounterValues, 0, MAX_PAPI_COUNTERS*sizeof(long long));
     }
     RtsLayer::UnLockDB();
   }
@@ -745,15 +786,15 @@ int PapiLayer::initializePerfRAPL(int tid) {
     fprintf(stderr,"PAPI_set_granularity\n");
     exit(1);
   }
-
-  ThreadList[tid]->EventSet[rapl_cid] = PAPI_NULL;
-  ret = PAPI_create_eventset(&(ThreadList[tid]->EventSet[rapl_cid]));
+  PapiThreadValue* localThreadValue=getThreadValue(tid);
+  localThreadValue->EventSet[rapl_cid] = PAPI_NULL;
+  ret = PAPI_create_eventset(&(localThreadValue->EventSet[rapl_cid]));
   if (PAPI_OK != ret) {
     fprintf(stderr,"PAPI_create_eventset.\n");
     exit(1);
   }
-  opt.eventset = ThreadList[tid]->EventSet[rapl_cid];
-  ret = PAPI_assign_eventset_component( ThreadList[tid]->EventSet[rapl_cid], 1 );
+  opt.eventset = localThreadValue->EventSet[rapl_cid];
+  ret = PAPI_assign_eventset_component( localThreadValue->EventSet[rapl_cid], 1 );
 
   if ( PAPI_OK != ret ) {
     fprintf(stderr, "PAPI_assign_eventset_component failed (%s)\n", PAPI_strerror(ret));
@@ -767,7 +808,7 @@ int PapiLayer::initializePerfRAPL(int tid) {
   }
 
   if(TauEnv_get_papi_multiplexing()) {
-    ret = PAPI_set_multiplex(ThreadList[tid]->EventSet[rapl_cid]);
+    ret = PAPI_set_multiplex(localThreadValue->EventSet[rapl_cid]);
     if ( PAPI_OK != ret ) {
       fprintf(stderr, "PAPI_set_multiplex failed (%s)\n", PAPI_strerror(ret));
       exit(1);
@@ -786,19 +827,19 @@ int PapiLayer::initializePerfRAPL(int tid) {
   fclose(para);
 
   numCounters = 0;
-  ret = PAPI_add_named_event(ThreadList[tid]->EventSet[rapl_cid], (char*)"rapl::RAPL_ENERGY_CORES");
+  ret = PAPI_add_named_event(localThreadValue->EventSet[rapl_cid], (char*)"rapl::RAPL_ENERGY_CORES");
   if (PAPI_OK != ret) {
 #ifdef DEBUG_PROF
     fprintf(stderr,"Error: PAPI_add_named_event(RAPL_ENERGY_CORES) because %s.\nPlease ensure that /proc/sys/kernel/perf_event_paranoid has a -1 and your system has /sys/devices/power/events/energy-pkg.scale.\n", PAPI_strerror(ret));
 #endif /* DEBUG_PROF */
     // don't exit(1);
   } else {
-    sprintf(Tau_rapl_event_names[numCounters], "rapl::RAPL_ENERGY_CORES");
-    sprintf(Tau_rapl_units[numCounters], "Joules");
+    snprintf(Tau_rapl_event_names[numCounters], sizeof(Tau_rapl_event_names[numCounters]),  "rapl::RAPL_ENERGY_CORES");
+    snprintf(Tau_rapl_units[numCounters], sizeof(Tau_rapl_units[numCounters]),  "Joules");
     numCounters++;
   }
 
-  ret = PAPI_add_named_event(ThreadList[tid]->EventSet[rapl_cid], (char*)"rapl::RAPL_ENERGY_PKG");
+  ret = PAPI_add_named_event(localThreadValue->EventSet[rapl_cid], (char*)"rapl::RAPL_ENERGY_PKG");
   if (PAPI_OK != ret) {
 
 #ifdef DEBUG_PROF
@@ -806,32 +847,32 @@ int PapiLayer::initializePerfRAPL(int tid) {
 // don't    exit(1);
 #endif /* DEBUG_PROF */
   } else {
-    sprintf(Tau_rapl_event_names[numCounters], "rapl::RAPL_ENERGY_PKG");
-    sprintf(Tau_rapl_units[numCounters], "Joules");
+    snprintf(Tau_rapl_event_names[numCounters], sizeof(Tau_rapl_event_names[numCounters]),  "rapl::RAPL_ENERGY_PKG");
+    snprintf(Tau_rapl_units[numCounters], sizeof(Tau_rapl_units[numCounters]),  "Joules");
     numCounters++;
 #ifdef TAU_BEACON
     TauBeaconInit();
 #endif /* TAU_BEACON */
   }
 
-  ret = PAPI_add_named_event(ThreadList[tid]->EventSet[rapl_cid], (char*)"rapl::RAPL_ENERGY_GPU");
+  ret = PAPI_add_named_event(localThreadValue->EventSet[rapl_cid], (char*)"rapl::RAPL_ENERGY_GPU");
   if (PAPI_OK != ret) {
 #ifdef DEBUG_PROF
     fprintf(stderr,"Error: PAPI_add_named_event(RAPL_ENERGY_GPU) because %s.\nPlease ensure that /proc/sys/kernel/perf_event_paranoid has a -1 and your system has /sys/devices/power/events/energy-pkg.scale.\n", PAPI_strerror(ret));
 #endif /* DEBUG_PROF */
     // exit(1);
   } else {
-    sprintf(Tau_rapl_event_names[numCounters], "rapl::RAPL_ENERGY_GPU");
-    sprintf(Tau_rapl_units[numCounters], "Joules");
+    snprintf(Tau_rapl_event_names[numCounters], sizeof(Tau_rapl_event_names[numCounters]),  "rapl::RAPL_ENERGY_GPU");
+    snprintf(Tau_rapl_units[numCounters], sizeof(Tau_rapl_units[numCounters]),  "Joules");
     numCounters++;
   }
 
-  ret = PAPI_add_named_event(ThreadList[tid]->EventSet[rapl_cid], (char*)"rapl::RAPL_ENERGY_DRAM");
+  ret = PAPI_add_named_event(localThreadValue->EventSet[rapl_cid], (char*)"rapl::RAPL_ENERGY_DRAM");
   if (PAPI_OK != ret) {
    // OK: this event is only available on servers
   } else {
-    sprintf(Tau_rapl_event_names[numCounters], "rapl::RAPL_ENERGY_DRAM");
-    sprintf(Tau_rapl_units[numCounters], "Joules");
+    snprintf(Tau_rapl_event_names[numCounters], sizeof(Tau_rapl_event_names[numCounters]),  "rapl::RAPL_ENERGY_DRAM");
+    snprintf(Tau_rapl_units[numCounters], sizeof(Tau_rapl_units[numCounters]),  "Joules");
     numCounters++;
   }
 
@@ -847,9 +888,10 @@ int PapiLayer::initializePerfRAPL(int tid) {
      printf("%s: /sys/devices/power/events/energy-pkg.scale doesn't contain a double", line);
      exit(1);
   }
-  ThreadList[tid]->NumEvents[rapl_cid] = numCounters;
 
-  if (PAPI_start(ThreadList[tid]->EventSet[rapl_cid]) != PAPI_OK) {
+  localThreadValue->NumEvents[rapl_cid] = numCounters;
+
+  if (PAPI_start(localThreadValue->EventSet[rapl_cid]) != PAPI_OK) {
     printf("TAU PERF: Error in PAPI_Start\n");
     return -1;
   }
@@ -864,10 +906,11 @@ int PapiLayer::initializeRAPL(int tid) {
   PAPI_event_info_t evinfo;
   const PAPI_component_info_t *cinfo = NULL;
   int num_events = 0;
-
+ 
   initializeAndCheckRAPL(tid);
-
+  PapiThreadValue* localThreadValue=getThreadValue(tid);
   ncomponents = PAPI_num_components();
+
   for (i=0; i < ncomponents; i++) {
     if ((cinfo = PAPI_get_component_info(i)) == NULL) {
       printf("PAPI_get_component_info returns null. PAPI was not configured with --components=rapl and hence RAPL events for power cannot be measured.\n");
@@ -882,20 +925,20 @@ int PapiLayer::initializeRAPL(int tid) {
         return -1;
       } /* rapl is disabled */
       /* create event set */
-      ThreadList[tid]->EventSet[rapl_cid] = PAPI_NULL;
-      ret = PAPI_create_eventset(&(ThreadList[tid]->EventSet[rapl_cid]));
+      localThreadValue->EventSet[rapl_cid] = PAPI_NULL;
+      ret = PAPI_create_eventset(&(localThreadValue->EventSet[rapl_cid]));
       if (ret != PAPI_OK) {
         printf("WARNING: TAU couldn't create a PAPI eventset. Please check the LD_LIBRARY_PATH and ensure that there is no mismatch between the version of papi.h and the papi library that is loaded\n");
         return -1;
       }
 
       if(TauEnv_get_papi_multiplexing()) {
-        ret = PAPI_assign_eventset_component( ThreadList[tid]->EventSet[rapl_cid], 0 );
+        ret = PAPI_assign_eventset_component( localThreadValue->EventSet[rapl_cid], 0 );
         if ( PAPI_OK != ret ) {
           fprintf(stderr, "PAPI_assign_eventset_component failed (%s)\n", PAPI_strerror(ret));
           return -1;
         }
-        ret = PAPI_set_multiplex(ThreadList[tid]->EventSet[rapl_cid]);
+        ret = PAPI_set_multiplex(localThreadValue->EventSet[rapl_cid]);
         if ( PAPI_OK != ret ) {
           fprintf(stderr, "PAPI_set_multiplex failed (%s)\n", PAPI_strerror(ret));
           return -1;
@@ -927,14 +970,14 @@ int PapiLayer::initializeRAPL(int tid) {
         if ((evinfo.units[0] == 'n') && (evinfo.units[1] == 'J')) {
           scalingFactor = 1.0e-9; /* nano Joules */
           strncpy(Tau_rapl_units[num_events], evinfo.units, PAPI_MIN_STR_LEN);
-          ret = PAPI_add_event( (ThreadList[tid]->EventSet[rapl_cid]), code);
+          ret = PAPI_add_event( (localThreadValue->EventSet[rapl_cid]), code);
           if (ret != PAPI_OK) {
             printf("PAPI_add_event is not OK!\n");
             break; /* hit an event limit */
           }
-          dmesg(1,"Added PAPI event %s successfully, rapl_cid = %d, EventSet=%d\n", Tau_rapl_event_names[num_events], rapl_cid, ThreadList[tid]->EventSet[rapl_cid]);
-          ThreadList[tid]->Comp2Metric[rapl_cid][ThreadList[tid]->NumEvents[rapl_cid]++] = numCounters;
-          ThreadList[tid]->CounterValues[num_events] = 0;
+          dmesg(1,"Added PAPI event %s successfully, rapl_cid = %d, EventSet=%d\n", Tau_rapl_event_names[num_events], rapl_cid, localThreadValue->EventSet[rapl_cid]);
+          localThreadValue->Comp2Metric[rapl_cid][localThreadValue->NumEvents[rapl_cid]++] = numCounters;
+          localThreadValue->CounterValues[num_events] = 0;
           num_events++;
 	  numCounters++;
           ret = PAPI_enum_cmp_event(&code, PAPI_ENUM_EVENTS, rapl_cid);
@@ -947,7 +990,7 @@ int PapiLayer::initializeRAPL(int tid) {
     } /* if rapl */
   } /* for ncomponents */
 
-  if (PAPI_start(ThreadList[tid]->EventSet[rapl_cid]) != PAPI_OK) {
+  if (PAPI_start(localThreadValue->EventSet[rapl_cid]) != PAPI_OK) {
     printf("PAPI RAPL: Error in PAPI_Start\n");
     return -1;
   }
@@ -978,9 +1021,9 @@ void PapiLayer::triggerRAPLPowerEvents(bool in_signal_handler) {
   double elapsedTimeInSecs = 0.0;
   long long curtime;
   char ename[1024];
-
+  PapiThreadValue* localThreadValue=getThreadValue(tid);
   /* Have we initialized on this thread yet? */
-  if (ThreadList[tid] == 0) return;
+  if (localThreadValue == 0) return;
   for (i=0; i<numCounters; i++) {
     tmpCounters[i] = 0;
     dmesg(1,"Tau_rapl_event_names = %s, Tau_rapl_units=%s, numCounters=%d\n",
@@ -988,39 +1031,39 @@ void PapiLayer::triggerRAPLPowerEvents(bool in_signal_handler) {
   }
 
   if (rapl_cid != -1) {
-    dmesg(1, "Inside PapiLayer::triggerRAPLPowerEvents()\n");
+    dmesg(1, "%s", "Inside PapiLayer::triggerRAPLPowerEvents()\n");
 
     curtime = PAPI_get_real_nsec();
     if (firsttime) {
        firsttime = false;
-       ThreadList[tid]->CounterValues[numCounters - 1] = curtime;
-       return ;
+       localThreadValue->CounterValues[numCounters - 1] = curtime;
+       return;
     }
-
+   
     // NOTE: We store the curtime in the numCounters index.
-    dmesg(1,"curtime = %lld, EventSet=%d, numEvents=%d\n", curtime, ThreadList[tid]->EventSet[rapl_cid], ThreadList[tid]->NumEvents[rapl_cid]);
-    if (ThreadList[tid]->NumEvents[rapl_cid] > 0) { // active counters
+    dmesg(1,"curtime = %lld, EventSet=%d, numEvents=%d\n", curtime, localThreadValue->EventSet[rapl_cid], localThreadValue->NumEvents[rapl_cid]);
+    if (localThreadValue->NumEvents[rapl_cid] > 0) { // active counters
       // read eventset for this component and reset counters
-      if (PAPI_stop(ThreadList[tid]->EventSet[rapl_cid], tmpCounters)
+      if (PAPI_stop(localThreadValue->EventSet[rapl_cid], tmpCounters)
 	!= PAPI_OK) {
         printf("Node %d, Thread %d:Error reading counters in PapiLayer::triggerRAPLPowerEvents\n", RtsLayer::myNode(), tid);
         return;
       }
       tmpCounters[numCounters - 1] = curtime;
 
-      elapsedTimeInSecs = (curtime - ThreadList[tid]->CounterValues[numCounters-1])/1.0e9;
-      ThreadList[tid]->CounterValues[numCounters - 1] = curtime;
+      elapsedTimeInSecs = (curtime - localThreadValue->CounterValues[numCounters-1])/1.0e9;
+      localThreadValue->CounterValues[numCounters - 1] = curtime;
 
       for (i = 0; i < numCounters-1; i++) {
 	dmesg(1,"Before subtracting: Counter: %s: tmp value= %.8f, units = %s, old value=%.8f, time elapsed=%.4f seconds\n",
-	Tau_rapl_event_names[i], (double) tmpCounters[i], Tau_rapl_units[i], (double) ThreadList[tid]->CounterValues[i], elapsedTimeInSecs);
-
+	Tau_rapl_event_names[i], (double) tmpCounters[i], Tau_rapl_units[i], (double) localThreadValue->CounterValues[i], elapsedTimeInSecs);
+	
       }
       for(i=0; i < numCounters -1; i++) {
         double value = (((double) tmpCounters[i]) *scalingFactor)/elapsedTimeInSecs;
 	dmesg(1,"Counter: %s: value %.9f, units = W\n", Tau_rapl_event_names[i], value);
 	if (value > 1e-5) {
-	  sprintf(ename,"%s (CPU Socket Power in Watts)", Tau_rapl_event_names[i]);
+	  snprintf(ename, sizeof(ename), "%s (CPU Socket Power in Watts)", Tau_rapl_event_names[i]);
       if (in_signal_handler) {
           TAU_REGISTER_EVENT(ue, ename);
           Tau_userevent_thread(ue, value, 0);
@@ -1033,8 +1076,7 @@ void PapiLayer::triggerRAPLPowerEvents(bool in_signal_handler) {
         }
       }
 
-
-      if (PAPI_start(ThreadList[tid]->EventSet[rapl_cid]) != PAPI_OK) {
+      if (PAPI_start(localThreadValue->EventSet[rapl_cid]) != PAPI_OK) {
         printf("Node %d, Thread %d:Error starting counters in PapiLayer::triggerRAPLPowerEvents\n", RtsLayer::myNode(), tid);
         return;
       }
