@@ -114,6 +114,9 @@ using namespace std;
 // Important for Porting to other platforms and frameworks.
 /////////////////////////////////////////////////////////////////////////
 
+std::mutex RtsLayer::DBVectorMutex;
+
+std::atomic<int> RtsLayer::maxLockTid(-1);
 /////////////////////////////////////////////////////////////////////////
 TauGroup_t& RtsLayer::TheProfileMask(void) {
   // to avoid initialization problems of non-local static variables
@@ -122,6 +125,43 @@ TauGroup_t& RtsLayer::TheProfileMask(void) {
   return ProfileMask;
 }
 
+/////////////////////////////////////////////////////////////////////////
+TauGroup_t& RtsLayer::TheProfileBlackMask(void) {
+  // to avoid initialization problems of non-local static variables
+  // This is set to TAU_EXCLUDE if an event is deactivated
+  static TauGroup_t ProfileBlackMask = 0;
+
+  return ProfileBlackMask;
+}
+
+/////////////////////////////////////////////////////////////////////////
+std::atomic<bool>& RtsLayer::TheExcludeDefaultGroup(void) {
+  static std::atomic<bool> is_default_group_excluded = { false };
+  return is_default_group_excluded;
+}
+
+// --- For Thread Exclusion ---
+//static std::unordered_set<int> gex_thread_set;
+
+//static SpatialExclusion gex_thread_mode = SpatialExclusionMode::UNSET;
+
+// --- For MPI Rank Exclusion ---
+std::unordered_set<int>& RtsLayer::TheRankExclusionSet(void)
+{
+	static std::unordered_set<int> exclusion_rank_set;
+	return exclusion_rank_set;
+}
+
+RtsLayer::SpatialExclusionMode& RtsLayer::TheRankExclusionMode(void){
+	static SpatialExclusionMode rankex_mode=SpatialExclusionMode::UNSET;
+	return rankex_mode;
+}
+
+std::atomic<uint64_t>& RtsLayer::TheRankExclusionVersion() {
+    static std::atomic<uint64_t> version{0};
+    return version;
+}
+
 /////////////////////////////////////////////////////////////////////////
 bool& RtsLayer::TheEnableInstrumentation(void) {
   // to avoid initialization problems of non-local static variables
@@ -162,6 +202,22 @@ int Tau_test_for_MPI_comm_rank() {
         Tau_set_usesMPI(1);
 		return commrank;
     }
+    // PMIx (Process Management Interface for Exascale)
+    tmpvar = getenv("PMIX_RANK");
+    if (tmpvar != NULL) {
+        commrank = atoi(tmpvar);
+	//printf("Found the rank! '%s', %d\n", tmpvar, commrank);
+        Tau_set_usesMPI(1);
+        return commrank;
+    }
+    // PALS (Cray Parallel Application Launch Service)
+    tmpvar = getenv("PALS_RANKID");
+    if (tmpvar != NULL) {
+        commrank = atoi(tmpvar);
+	//printf("Found the rank! '%s', %d\n", tmpvar, commrank);
+        Tau_set_usesMPI(1);
+        return commrank;
+    }
     // ALPS on Cray
     tmpvar = getenv("ALPS_APP_PE");
     if (tmpvar != NULL) {
@@ -177,13 +233,6 @@ int Tau_test_for_MPI_comm_rank() {
 		// printf("Changing openMPI rank to %lu\n", commrank);
         Tau_set_usesMPI(1);
 		return commrank;
-    }
-	// PBS/Torque
-    tmpvar = getenv("PBS_TASKNUM");
-	if (tmpvar != NULL) {
-        commrank = atoi(tmpvar);
-        Tau_set_usesMPI(1);
-		return commrank;
     }
 	// ALPS on Cray
     tmpvar = getenv("ALPS_APP_PE");
@@ -275,6 +324,12 @@ TauGroup_t RtsLayer::enableProfileGroupName(char const * ProfileGroup) {
 TauGroup_t RtsLayer::generateProfileGroup(void) {
   static TauGroup_t key =  0x00000001;
   key = key << 1;
+  /*
+   * TAU_EXCLUDE is reserved for shutting off events. Do not use it as a general id. 
+   */
+  if (key == TAU_EXCLUDE) {
+    key = key << 1;
+  }
   if (!key) key = 0x1; // cycle
   return key;
 }
@@ -627,7 +682,8 @@ int RtsLayer::setAndParseProfileGroups(char *prog, char *str) {
 	  RtsLayer::enableProfileGroup(TAU_GROUP_4);
 	  break;
 	case '5' :
-	  RtsLayer::enableProfileGroup(TAU_GROUP_5);
+           printf("WARNING: TAU_GROUP_5 is unavailable. Reserved for TAU_EXCLUDE\n");
+	  //RtsLayer::enableProfileGroup(TAU_GROUP_5);
 	  break;
 	case '6' :
 	  RtsLayer::enableProfileGroup(TAU_GROUP_6);
