@@ -13,6 +13,7 @@
 #include <string.h>
 #include <vector>
 #include <set>
+#include <mutex>
 
 #include <TAU.h>
 #include <Profile/TauBfd.h>
@@ -24,7 +25,7 @@
 #define PACKAGE_VERSION 2.25
 #endif
 #include <bfd.h>
-#if TAU_BFD >= 022300
+#ifdef TAU_ELF_BFD
 #include <elf-bfd.h>
 #endif
 #include <dirent.h>
@@ -69,6 +70,9 @@
 #include <errno.h>
 #include <dwarf.h>
 #include <libdwarf.h>
+#ifdef DW_LIBDWARF_VERSION
+#define TAU_USE_NEW_LIBDWARF
+#endif
 #endif
 
 /* When BFD 2.34 was released, some API calls were replaced. */
@@ -131,7 +135,7 @@ struct TauBfdModule
       return (bfdOpen = false);
     }
 
-#if TAU_BFD >= 022200
+#if defined(BFD_DECOMPRESS)
     // Decompress sections
     bfdImage->flags |= BFD_DECOMPRESS;
 #endif
@@ -664,6 +668,11 @@ static unsigned long getProbeAddr(bfd * bfdImage, unsigned long pc) {
 // Probe for BFD information given a single address.
 bool Tau_bfd_resolveBfdInfo(tau_bfd_handle_t handle, unsigned long probeAddr, TauBfdInfo & info)
 {
+  // BFD is not thread safe, and we call this function from lots of places.
+  static std::mutex mtx;
+  // a unique lock will unlock when it goes out of scope.
+  std::lock_guard<std::mutex> lck (mtx);
+
   if (!TauEnv_get_bfd_lookup() || !Tau_bfd_checkHandle(handle)) {
     info.secure(probeAddr);
     return false;
@@ -801,14 +810,14 @@ bool Tau_bfd_resolveBfdInfo(tau_bfd_handle_t handle, unsigned long probeAddr, Ta
   if ((info.funcname == NULL) && (info.filename != NULL) && (info.lineno > 0)) {
     info.probeAddr = probeAddr;
     info.funcname = (char*)malloc(32);
-    sprintf((char*)info.funcname, "anonymous");
+    snprintf((char*)info.funcname, 32,  "anonymous");
     return true;
   }
 
   // Couldn't resolve the address so fill in fields as best we can.
   if (info.funcname == NULL) {
     info.funcname = (char*)malloc(128);
-    sprintf((char*)info.funcname, "addr=<%lx>", probeAddr);
+    snprintf((char*)info.funcname, 128,  "addr=<%lx>", probeAddr);
   }
   if (info.filename == NULL) {
     if (matchingIdx != -1) {
@@ -998,7 +1007,7 @@ static char const * Tau_bfd_internal_getExecutablePath()
     RtsLayer::LockEnv();
     if (!init) {
 #if defined(TAU_AIX)
-      sprintf(path, "/proc/%d/object/a.out", RtsLayer::getPid());
+      snprintf(path, sizeof(path),  "/proc/%d/object/a.out", RtsLayer::getPid());
 #elif defined(TAU_BGP)
       if (Tau_bfd_internal_getBGPExePath(path) != 0) {
         fprintf(stderr, "Tau_bfd_internal_getExecutablePath: "
@@ -1006,7 +1015,7 @@ static char const * Tau_bfd_internal_getExecutablePath()
             "symbols will not be resolved\n", path);
       }
 #elif defined(TAU_BGQ)
-      sprintf(path, "%s", "/proc/self/exe");
+      snprintf(path, sizeof(path),  "%s", "/proc/self/exe");
 #elif defined(__APPLE__)
       uint32_t size = sizeof(path);
       _NSGetExecutablePath(path, &size);
@@ -1014,7 +1023,7 @@ static char const * Tau_bfd_internal_getExecutablePath()
       GetModuleFileName(NULL, path, sizeof(path));
 #else
       // Default: Linux systems
-      sprintf(path, "%s", "/proc/self/exe");
+      snprintf(path, sizeof(path),  "%s", "/proc/self/exe");
 #endif
       init = true;
     }
@@ -1059,7 +1068,7 @@ static void Tau_bfd_internal_locateAddress(bfd * bfdptr, asection * section, voi
   // TauBfdInfo fields without an extra copy.  This also means
   // that the pointers in TauBfdInfo must never be deleted
   // since they point directly into the module's BFD.
-#if (TAU_BFD >= 022200)
+#if defined(bfd_find_nearest_line_discriminator)
   data.found = bfd_find_nearest_line_discriminator(bfdptr, section,
       data.module->syms, (data.info.probeAddr - vma),
       &data.info.filename, &data.info.funcname,
@@ -1079,8 +1088,13 @@ static void
 simple_error_handler(Dwarf_Error error, Dwarf_Ptr errarg)
 {
     (void)errarg; // intentionally unused
+#ifdef TAU_USE_NEW_LIBDWARF
+    printf("\nTAU: libdwarf error detected: 0x%lx %s\n",
+        (unsigned long) dwarf_errno(error),dwarf_errmsg(error));
+#else
     printf("\nTAU: libdwarf error detected: 0x%" DW_PR_DUx " %s\n",
-        dwarf_errno(error),dwarf_errmsg(error));
+            dwarf_errno(error),dwarf_errmsg(error));
+#endif
     return;
 }
 
@@ -1235,13 +1249,21 @@ static void Tau_get_dwarf_symbols(tau_bfd_handle_t bfd_handle, const char * file
     Dwarf_Ptr errarg = (Dwarf_Ptr)1;
     Dwarf_Error * errp = 0;
 
+#ifndef TAU_USE_NEW_LIBDWARF
     fd = open(filepath, O_RDONLY);
     if(fd < 0) {
         TAU_VERBOSE("TAU_BFD: Unable to open file %s\n", filepath);
         return;
     }
+#endif
 
+#ifdef TAU_USE_NEW_LIBDWARF
+    TAU_VERBOSE("TAU_DWARF: Initializing libdwarf using the new API version (DWARF v5) for %s\n", filepath);
+    res = dwarf_init_path(filepath, NULL, 0, DW_GROUPNUMBER_ANY, errhand, errarg, &dbg, errp);
+#else
+    TAU_VERBOSE("TAU_DWARF: Initializing libdwarf using the old API version (DWARF v4) for %s\n", filepath);
     res = dwarf_init(fd, DW_DLC_READ, errhand, errarg, &dbg, errp);
+#endif
 
     if(res != DW_DLV_OK) {
         fprintf(stderr, "Error opening DWARF session: %d\n", res);
@@ -1292,16 +1314,22 @@ static void Tau_get_dwarf_symbols(tau_bfd_handle_t bfd_handle, const char * file
     }
 
 
+#ifdef TAU_USE_NEW_LIBDWARF
+    res = dwarf_finish(dbg);
+#else
     res = dwarf_finish(dbg, errp);
+#endif
     if(res != DW_DLV_OK) {
         fprintf(stderr, "TAU: Error closing DWARF session: %d\n", res);
         return;
     }
 
+#ifndef TAU_USE_NEW_LIBDWARF
     res = close(fd);
     if(res != 0) {
         fprintf(stderr, "TAU: Error closing dwarf file\n");
     }
+#endif
 }
 
 #endif // TAU_DWARF
@@ -1491,7 +1519,50 @@ char * Tau_demangle_name(const char * name) {
     if (dem_name == NULL) {
         dem_name = strdup(name);
     }
-    TAU_VERBOSE("Demangled: '%s'\n", dem_name);
+    //TAU_VERBOSE("Demangled: '%s'\n", dem_name);
+    return dem_name;
+}
+/* If no bfd + demangle, and we have C++ support, use it */
+#elif defined(__GNUC__)
+#include <cxxabi.h>
+char * Tau_demangle_name(const char * name) {
+    int status;
+    char * dem_name = abi::__cxa_demangle(name, 0, 0, &status);
+    if (status != 0 || dem_name == nullptr) {
+        switch (status) {
+            case 0:
+                TAU_VERBOSE("The demangling operation succeeded, but realname is NULL\n");
+                break;
+            case -1:
+                TAU_VERBOSE("The demangling operation failed:");
+                TAU_VERBOSE(" A memory allocation failiure occurred.\n");
+                break;
+            case -2:
+                /* This happens when the name isn't mangled, so no reason to pollute the output... */
+                /*
+                TAU_VERBOSE("The demangling operation failed:");
+                TAU_VERBOSE(" '%s' is not a valid", name);
+                TAU_VERBOSE(" name under the C++ ABI mangling rules.\n");
+                */
+                break;
+            case -3:
+                TAU_VERBOSE("The demangling operation failed: One of the");
+                TAU_VERBOSE(" arguments is invalid.\n");
+                break;
+            default:
+                TAU_VERBOSE("The demangling operation failed: Unknown error.\n");
+                break;
+        }
+		dem_name = strdup(name);
+    }
+    //TAU_VERBOSE("Demangled: '%s'\n", dem_name);
+    return dem_name;
+}
+/* No support for either, just return the name */
+#else
+char * Tau_demangle_name(const char * name) {
+    TAU_VERBOSE("Warning: No demangling support provided...\n");
+    char * dem_name = strdup(name);
     return dem_name;
 }
 #endif // #if defined(HAVE_GNU_DEMANGLE)
