@@ -129,12 +129,18 @@ struct callsiteKey2IdMap_t : public map<TAU_CALLSITE_KEY_ID_MAP_TYPE>
     finalizeCallSites_if_necessary();
   }
 };
-
+static std::mutex KeyVectorMutex;
 static callsiteKey2IdMap_t& TheCallSiteKey2IdMap(void)
 {
-  static callsiteKey2IdMap_t callsiteKey2IdMap[TAU_MAX_THREADS];
+  static vector<callsiteKey2IdMap_t*> callsiteKey2IdMap;//[TAU_MAX_THREADS];
   int tid = RtsLayer::myThread();
-  return callsiteKey2IdMap[tid];
+  if(callsiteKey2IdMap.size()<=tid){
+      std::lock_guard<std::mutex> guard(KeyVectorMutex);
+      while(callsiteKey2IdMap.size()<=tid){
+        callsiteKey2IdMap.push_back(new callsiteKey2IdMap_t());
+      }
+    }
+  return *callsiteKey2IdMap[tid];
 }
 
 struct callsiteId2KeyVec_t : public vector<tau_cs_info_t *>
@@ -147,11 +153,18 @@ struct callsiteId2KeyVec_t : public vector<tau_cs_info_t *>
   }
 };
 
+static std::mutex IDVectorMutex;
 static callsiteId2KeyVec_t& TheCallSiteIdVector(void)
 {
-  static callsiteId2KeyVec_t callsiteId2KeyVec[TAU_MAX_THREADS];
+  static vector<callsiteId2KeyVec_t*> callsiteId2KeyVec;//[TAU_MAX_THREADS];
   int tid = RtsLayer::myThread();
-  return callsiteId2KeyVec[tid];
+  if(callsiteId2KeyVec.size()<=tid){
+      std::lock_guard<std::mutex> guard(IDVectorMutex);
+      while(callsiteId2KeyVec.size()<=tid){
+        callsiteId2KeyVec.push_back(new callsiteId2KeyVec_t());
+      }
+    }
+  return *callsiteId2KeyVec[tid];
 }
 
 struct callsiteFirstKeyMap_t : public map<TAU_CALLSITE_FIRSTKEY_MAP_TYPE>
@@ -183,22 +196,51 @@ struct callsitePathMap_t : public map<TAU_CALLSITE_PATH_MAP_TYPE>
   }
 };
 
+static std::mutex PathMapVectorMutex;
 static callsitePathMap_t& TheCallSitePathMap(void)
 {
   // to avoid initialization problems of non-local static variables
-  static callsitePathMap_t callsitePathMap[TAU_MAX_THREADS];
+  static vector<callsitePathMap_t*> callsitePathMap;//[TAU_MAX_THREADS];
   int tid = RtsLayer::myThread();
-  return callsitePathMap[tid];
+  if(callsitePathMap.size()<=tid){
+      std::lock_guard<std::mutex> guard(PathMapVectorMutex);
+      while(callsitePathMap.size()<=tid){
+        callsitePathMap.push_back(new callsitePathMap_t());
+      }
+    }
+  return *callsitePathMap[tid];
 }
 
-static unsigned long callSiteId[TAU_MAX_THREADS];
+static vector<unsigned long> callSiteId;//[TAU_MAX_THREADS];
+static std::mutex CallSiteVectorMutex;
+static inline void checkCallSiteVector(int tid){
+    if(callSiteId.size()<=tid){
+      std::lock_guard<std::mutex> guard(CallSiteVectorMutex);
+      while(callSiteId.size()<=tid){
+        callSiteId.push_back(0);
+      }
+    }
+}
+static inline unsigned long getCallSiteId(int tid){
+    checkCallSiteVector(tid);
+    return callSiteId[tid];
+}
+static inline void setCallSiteId(int tid, unsigned long value){
+    checkCallSiteVector(tid);
+    callSiteId[tid]=value;
+}
+static inline void incrementCallSiteId(int tid){
+    checkCallSiteVector(tid);
+    callSiteId[tid]++;
+}
 
 void initializeCallSiteDiscoveryIfNecessary()
 {
   static bool initialized = false;
   if (!initialized) {
-    for (int i = 0; i < TAU_MAX_THREADS; i++) {
-      callSiteId[i] = 0;
+    int vecSize=callSiteId.size();
+    for (int i = 0; i < vecSize; i++) {
+      setCallSiteId(i, 0);
     }
     initialized = true;
   }
@@ -258,14 +300,14 @@ char * Tau_callsite_resolveCallSite(unsigned long addr)
 #else
     char *demangled_funcname = strdup(resolvedInfo.funcname);
 #endif
-    sprintf(resolvedBuffer, "[%s] [{%s} {%d}]",
+    snprintf(resolvedBuffer, length * sizeof(char),  "[%s] [{%s} {%d}]",
         demangled_funcname, resolvedInfo.filename, resolvedInfo.lineno);
     free(demangled_funcname);
   } else {
     // this should be enough...
     length = strlen(mapName) + 32;
     resolvedBuffer = (char*)malloc(length * sizeof(char));
-    sprintf(resolvedBuffer, "[%s] UNRESOLVED ADDR", mapName);
+    snprintf(resolvedBuffer, length * sizeof(char),  "[%s] UNRESOLVED ADDR", mapName);
   }
 
   return resolvedBuffer;
@@ -373,9 +415,19 @@ size_t trimwhitespace(char *out, size_t len, const char *str)
 //         Also look for "tau*/include/" where * has no "/".
 bool nameInTau(const char *name)
 {
-  if (strstr(name, "UNRESOLVED ADDR") != NULL)
+  //if (strstr(name, "UNRESOLVED ADDR") != NULL)
+    //return false;
+  const char * tmp_name = strchr(name, '{');
+  // no leading '{'?
+  if (tmp_name == nullptr) {
+    tmp_name = strchr(name, '[');
+  }
+  // no leading '[', either? Then return false.
+  if (tmp_name == nullptr) {
     return false;
-  name = strchr(name, '{') + 1;
+  }
+  tmp_name = tmp_name+1;
+
   static char const * libprefix[] = {"libtau", "libTAU", NULL};
   static char const * libsuffix[] = {".a", ".so", ".dylib", NULL};
   int offset = 0;
@@ -383,7 +435,7 @@ bool nameInTau(const char *name)
   // Check libTAU and varients.
   char const * prefix;
   for (char const ** p=libprefix; (prefix = *p); ++p) {
-    char const * head = strstr(name, prefix);
+    char const * head = strstr(tmp_name, prefix);
     if (!head) continue;
     char const * suffix;
     for (char const ** s=libsuffix; (suffix = *s); ++s) {
@@ -394,7 +446,7 @@ bool nameInTau(const char *name)
     }
   }
   // Pretty ugly hack, I foresee much trouble ahead.
-  const char *strPtr = strstr(name, "tau");
+  const char *strPtr = strstr(tmp_name, "tau");
   if (strPtr != NULL) {
     length = strlen(strPtr);
     offset = strcspn(strPtr, "/");
@@ -640,7 +692,7 @@ void Profiler::CallSiteStart(int tid, x_uint64 TraceTimeStamp)
       // *CWL* - It is important to make a copy of the callsiteKey for registration.
       unsigned long * callsiteKeyCopy = (unsigned long*)malloc(sizeof(callsiteKey));
       memcpy(callsiteKeyCopy, callsiteKey, sizeof(callsiteKey));
-      callsiteKeyId = callSiteId[tid];
+      callsiteKeyId = getCallSiteId(tid);
       TheCallSiteKey2IdMap().insert(map<TAU_CALLSITE_KEY_ID_MAP_TYPE>::value_type(callsiteKeyCopy, callsiteKeyId));
       tau_cs_info_t *callSiteInfo = (tau_cs_info_t *)malloc(sizeof(tau_cs_info_t));
       callSiteInfo->key = callsiteKeyCopy;
@@ -649,7 +701,7 @@ void Profiler::CallSiteStart(int tid, x_uint64 TraceTimeStamp)
       callSiteInfo->hasName = false;
       callSiteInfo->resolvedName = NULL;
       TheCallSiteIdVector().push_back(callSiteInfo);
-      callSiteId[tid]++;
+      incrementCallSiteId(tid);
     } else {
       // We've seen this callsite key before.
       callsiteKeyId = (*itCs).second;
@@ -807,14 +859,28 @@ static string getNameAndType(FunctionInfo *fi)
 }
 */
 
+ struct CallsiteFinalThreadList : vector<bool>{
+      CallsiteFinalThreadList(){
+         //printf("Creating CallsiteFinalThreadList at %p\n", this);
+      }
+     virtual ~CallsiteFinalThreadList(){
+         //printf("Destroying CallsiteFinalThreadList at %p, with size %ld\n", this, this->size());
+         Tau_destructor_trigger();
+     }
+   };
+
 extern "C" void finalizeCallSites_if_necessary()
 {
+  //static std::mutex FinCaSiVectorMutex;
   static bool callsiteFinalizationSetup = false;
-  static bool callsiteThreadFinalized[TAU_MAX_THREADS];
+  static CallsiteFinalThreadList callsiteThreadFinalized;//[TAU_MAX_THREADS];
   if (!callsiteFinalizationSetup) {
-    for (int i = 0; i < TAU_MAX_THREADS; i++) {
-      callsiteThreadFinalized[i] = false;
+    int vecSize=RtsLayer::getTotalThreads();
+    //std::lock_guard<std::mutex> guard(FinCaSiVectorMutex);
+    while(callsiteThreadFinalized.size()<=vecSize){
+      callsiteThreadFinalized.push_back(false);
     }
+
     callsiteFinalizationSetup = true;
   }
   int tid = RtsLayer::myThread();
@@ -834,7 +900,7 @@ extern "C" void finalizeCallSites_if_necessary()
 #endif /* TAU_BFD */
 
   string delimiter = string(" --> ");
-  for (unsigned int i = 0; i < callSiteId[tid]; i++) {
+  for (unsigned int i = 0; i < getCallSiteId(tid); i++) {
     tau_cs_info_t *callsiteInfo = TheCallSiteIdVector()[i];
     if (callsiteInfo && callsiteInfo->hasName) {
       // We've already done this in the discovery phase.
