@@ -24,6 +24,9 @@
 #include <iostream>
 #include <iomanip>
 #include <Profile/OpenMPLayer.h>
+#ifdef TAU_OPENMP
+#include <Profile/TauGpuAdapterOpenMP.h>
+#endif
 #include <stdlib.h>
 #include <map>
 #include <set>
@@ -86,12 +89,15 @@ int counted_unifmems = 0;
 
 extern "C" void metric_set_gpu_timestamp(int tid, double value);
 extern "C" void Tau_set_thread_fake(int tid);
+extern "C" void Tau_set_fake_thread_use_cpu_metric(int tid);
 
 extern "C" void Tau_create_top_level_timer_if_necessary_task(int tid);
 extern "C" void Tau_stop_top_level_timer_if_necessary_task(int tid);
 
 extern "C" int Tau_is_thread_fake(int tid);
 
+extern "C" x_uint64 TauTraceGetTimeStamp(int tid);
+
 #include<map>
 using namespace std;
 
@@ -105,6 +111,11 @@ static inline void record_context_event(TauContextUserEvent * e, TAU_EVENT_DATAT
   }
 }
 
+void set_first_ts_cupti()
+{
+  cupti_init_timestamp = TauTraceGetTimeStamp(0);
+}
+
 void check_gpu_event(int gpuTask)
 {
   static std::set<int> seen;
@@ -112,11 +123,11 @@ void check_gpu_event(int gpuTask)
 #ifdef DEBUG_PROF
     cerr << "first gpu event" << endl;
 #endif
-    if (gpuTask >= TAU_MAX_THREADS) {
+    /*if (gpuTask >= TAU_MAX_THREADS) {
       cerr << "TAU ERROR: The number of GPU entities exceeds the maximum: " << TAU_MAX_THREADS
           << ". Please reconfigure TAU with '-useropt=-DTAU_MAX_THREADS=<larger number>.'" << endl;
       exit(1);
-    }
+    }*/
     if (seen.count(gpuTask) == 0) {
         Tau_create_top_level_timer_if_necessary_task(gpuTask);
         number_of_top_level_task_events++;
@@ -140,7 +151,7 @@ void Tau_gpu_enter_event(const char* name)
   Tau_start_timer(handle, 0, Tau_get_thread());
 }
 
-void Tau_gpu_enter_memcpy_event(const char *functionName, GpuEvent *device, int transferSize, int memcpyType)
+void Tau_gpu_enter_memcpy_event(const char *functionName, GpuEvent *device, size_t transferSize, int memcpyType)
 {
 #ifdef DEBUG_PROF
   //TAU_VERBOSE("entering Memcpy event type: %d.\n", memcpyType);
@@ -197,7 +208,7 @@ void Tau_gpu_enter_memcpy_event(const char *functionName, GpuEvent *device, int
   }
 }
 
-void Tau_gpu_enter_unifmem_event(const char *functionName, GpuEvent *device, int transferSize, int unifmemType)
+void Tau_gpu_enter_unifmem_event(const char *functionName, GpuEvent *device, size_t transferSize, int unifmemType)
 {
 #ifdef DEBUG_PROF
   //TAU_VERBOSE("entering Memcpy event type: %d.\n", memcpyType);
@@ -277,9 +288,9 @@ void Tau_gpu_exit_memcpy_event(const char * functionName, GpuEvent *device, int
   // time.
 
   //Tau_pure_stop(functionName);
-  //void* handle = Tau_get_function_info(functionName, "", TAU_USER, "TAU_GPU_MEMORY_COPY");
-  //Tau_stop_timer(handle, Tau_get_thread());
-  Tau_stop_current_timer_task(Tau_get_thread());
+  void* handle = Tau_get_function_info(functionName, "", TAU_USER, "TAU_GPU_MEMORY_COPY");
+  Tau_stop_timer(handle, Tau_get_thread());
+  //Tau_stop_current_timer_task(Tau_get_thread());
 
 }
 
@@ -306,9 +317,9 @@ void Tau_gpu_exit_unifmem_event(const char * functionName, GpuEvent *device, int
   // time.
 
   //Tau_pure_stop(functionName);
-  //void* handle = Tau_get_function_info(functionName, "", TAU_USER, "TAU_GPU_UNIFIED_MEMORY");
-  //Tau_stop_timer(handle, Tau_get_thread());
-  Tau_stop_current_timer_task(Tau_get_thread());
+  void* handle = Tau_get_function_info(functionName, "", TAU_USER, "TAU_GPU_UNIFIED_MEMORY");
+  Tau_stop_timer(handle, Tau_get_thread());
+  //Tau_stop_current_timer_task(Tau_get_thread());
 
 }
 
@@ -318,9 +329,9 @@ void Tau_gpu_exit_event(const char *name)
   TAU_VERBOSE("exit cu event: %s.\n", name);
 #endif
   //Tau_pure_stop(name);
-  //void* handle = Tau_get_function_info(name, "", TAU_USER, "TAU_GPU_API");
-  //Tau_stop_timer(handle, Tau_get_thread());
-  Tau_stop_current_timer_task(Tau_get_thread());
+  void* handle = Tau_get_function_info(name, "", TAU_USER, "TAU_GPU_API");
+  Tau_stop_timer(handle, Tau_get_thread());
+  //Tau_stop_current_timer_task(Tau_get_thread());
 
 }
 
@@ -330,7 +341,6 @@ void stage_gpu_event(const char *name, int gpuTask, double start_time, FunctionI
   cerr << "setting gpu timestamp for start " << setprecision(16) << start_time << endl;
 #endif
   metric_set_gpu_timestamp(gpuTask, start_time);
-
   check_gpu_event(gpuTask);
   if (TauEnv_get_callpath()) {
     //printf("Profiler: %s \n", parent->GetName());
@@ -347,8 +357,8 @@ void break_gpu_event(const char *name, int gpuTask, double stop_time, FunctionIn
   cerr << "setting gpu timestamp for stop: " << setprecision(16) << stop_time << endl;
 #endif
   metric_set_gpu_timestamp(gpuTask, stop_time);
-  //Tau_pure_stop_task(name, gpuTask);
-  Tau_stop_current_timer_task(gpuTask);
+  Tau_pure_stop_task(name, gpuTask);
+  //Tau_stop_current_timer_task(gpuTask);
   if (TauEnv_get_callpath()) {
     //printf("Profiler: %s \n", parent->GetName());
     //double totalTime = 0;
@@ -357,6 +367,7 @@ void break_gpu_event(const char *name, int gpuTask, double stop_time, FunctionIn
     }
   }
 }
+
 int get_task(GpuEvent *new_task)
 {
   int task = 0;
@@ -369,6 +380,16 @@ int get_task(GpuEvent *new_task)
     TheGpuEventMap.get().insert(pair<GpuEvent *, int>(create_task, task));
     number_of_tasks++;
     Tau_set_thread_fake(task);
+#ifdef TAU_OPENMP
+    /* if we have an openmp tools virtual thread, it shouldn't use the
+       GPU counter or any offset */
+    OpenMPGpuEvent* isOMPT = dynamic_cast<OpenMPGpuEvent*> (new_task);
+    if (isOMPT != nullptr) {
+        Tau_set_fake_thread_use_cpu_metric(task);
+    }
+#endif
+    metric_set_gpu_timestamp(task, (double) cupti_init_timestamp);
+    Tau_create_top_level_timer_if_necessary_task(task);
     //TAU_CREATE_TASK(task);
     //printf("new task: %s id: %d.\n", create_task->gpuIdentifier(), task);
   } else {
@@ -438,9 +459,10 @@ void Tau_gpu_register_sync_event(GpuEvent *id, double startTime, double endTime)
   break_gpu_event(id->getName(), task, syncEndTime, id->getCallingSite());
 }
 
-void Tau_gpu_register_memcpy_event(GpuEvent *id, double startTime, double endTime, int transferSize, int memcpyType,
+void Tau_gpu_register_memcpy_event(GpuEvent *id, double startTime, double endTime, size_t transferSize, int memcpyType,
 				   int direction)
 {
+
   int task = get_task(id);
   const char* functionName = id->getName();
   if (strcmp(functionName, TAU_GPU_USE_DEFAULT_NAME) == 0) {
@@ -453,7 +475,7 @@ void Tau_gpu_register_memcpy_event(GpuEvent *id, double startTime, double endTim
     }
     //printf("using default name: %s.\n", functionName);
   }
-
+  
 #ifdef DEBUG_PROF
   TAU_VERBOSE("recording memcopy event.\n");
   TAU_VERBOSE("time is: %f:%f.\n", startTime, endTime);
@@ -582,7 +604,7 @@ void Tau_gpu_register_envt_event(GpuEvent *event, double startTime, double endTi
 	}
 }
 
-void Tau_gpu_register_unifmem_event(GpuEvent *id, double startTime, double endTime, int transferSize, int unifmemType,
+void Tau_gpu_register_unifmem_event(GpuEvent *id, double startTime, double endTime, size_t transferSize, int unifmemType,
     int direction)
 {
   int task = get_task(id);
@@ -698,20 +720,20 @@ void Tau_gpu_register_imix_event(GpuEvent *event, double startTime, double endTi
   printf("IMIX type is %d, contextId: %i.\n", dataType, task);
   const char* functionName = event->getName();
   if (strcmp(functionName, TAU_GPU_USE_DEFAULT_NAME) == 0)
-    {
-      if (dataType == FlPtOps) {
-	functionName = "Floating Point Operations";
-      }
-      else if (dataType == MemOps)
-	{
-	  functionName = "Memory Operations";
-	}
-      else if (dataType == CtrlOps)
-	{
-	  functionName = "Control Operations";
-	}
-      printf("using default name: %s.\n", functionName);
+  {
+    if (dataType == FlPtOps) {
+      functionName = "Floating Point Operations";
     }
+    else if (dataType == MemOps)
+    {
+      functionName = "Memory Operations";
+    }
+    else if (dataType == CtrlOps)
+    {
+      functionName = "Control Operations";
+    }
+      //printf("using default name: %s.\n", functionName);
+  }
 #ifdef DEBUG_PROF
   TAU_VERBOSE("recording instruction mix event.\n");
   TAU_VERBOSE("time is: %f:%f.\n", startTime, endTime);
