@@ -38,15 +38,15 @@ using namespace tau;
 
 extern "C" void tau_dyninst_init(int isMPI);
 
-#ifndef __ia64
+/*#ifndef __ia64
 int TheFlag[TAU_MAX_THREADS] ;
 #define TAU_MONITOR_ENTER(tid) if (TheFlag[tid] == 0) {TheFlag[tid] = 1;}  else {return; }
 #define TAU_MONITOR_EXIT(tid) TheFlag[tid] = 0
-#else /* FOR IA64 */
-vector<int> TheFlag(TAU_MAX_THREADS);
-#define TAU_MONITOR_ENTER(tid)  if (TheFlag[tid] == 0) {TheFlag[tid] = 1;}  else {return; }
+#else // FOR IA64 */
+vector<int> TheFlag;//(TAU_MAX_THREADS)
+#define TAU_MONITOR_ENTER(tid)  while(TheFlag.size()<=tid){TheFlag.push_back(0);} if (TheFlag[tid] == 0) {TheFlag[tid] = 1;}  else {return; }
 #define TAU_MONITOR_EXIT(tid) TheFlag[tid] = 0
-#endif /* IA64 */
+//#endif /* IA64 */
 
 
 vector<string> TauLoopNames; /* holds just names of loops */
@@ -112,7 +112,7 @@ void TauInitCode(char *arg, int isMPI)
        the global vector of FI pointers */
 #ifdef TAUDYNVEC
     FunctionInfo *taufi = new
-	FunctionInfo(name, " " , TAU_DEFAULT, "TAU_DEFAULT", true, tid);
+	FunctionInfo(name, " " , TAU_DEFAULT, "TAU_DEFAULT", true);
     if (taufi == (FunctionInfo *) NULL) {
       printf("ERROR: new returns NULL in TauInitCode\n"); exit(1);
     }
@@ -291,11 +291,31 @@ int TauRenameTimer(char *oldName, char *newName)
 }
 
 
-static int tauFiniID = -1;
-static int tauDyninstEnabled[TAU_MAX_THREADS];
+static int tauFiniID = -1; 
+static vector <int> tauDyninstEnabled;//[TAU_MAX_THREADS];
+static int defaultDyEn=0;
+std::mutex DyEnVectorMutex;
+static inline void checkDyEnVector(int tid){
+    if(tauDyninstEnabled.size()<=tid){
+      std::lock_guard<std::mutex> guard(DyEnVectorMutex);
+      while(tauDyninstEnabled.size()<=tid){
+        tauDyninstEnabled.push_back(defaultDyEn);
+      }
+    }
+ }
+static inline void setTauDyninstEnabled(int tid, int val){
+    checkDyEnVector(tid);
+    tauDyninstEnabled[tid]=val;
+}
+
+static inline int getTauDyninstEnabled(int tid){
+    checkDyEnVector(tid);
+    return tauDyninstEnabled[tid];
+}
 
 void trace_register_func(char *origname, int id)
 {
+
   static int invocations = 0;
   int i;
   int tid = RtsLayer::myThread();
@@ -312,8 +332,9 @@ void trace_register_func(char *origname, int id)
       }
     }
     char *dem = Tau_demangle_name(mirror);
-    char *newname = (char *) malloc(strlen(dem)+funclen-i+3);
-    sprintf(newname, "%s %s", dem, &func[i-1]);
+    const int len = strlen(dem)+funclen-i+3;
+    char *newname = (char *) malloc(len);
+    snprintf(newname, len,  "%s %s", dem, &func[i-1]);
     TAU_VERBOSE("name=%s, newname = %s\n", func, newname);
     free(mirror);
     free(dem);
@@ -321,7 +342,7 @@ void trace_register_func(char *origname, int id)
   }
   TAU_VERBOSE("trace_register_func: func = %s, id = %d\n", func, id);
   if (invocations == 0) {
-    if (!tauDyninstEnabled[tid]) {
+    if (!getTauDyninstEnabled(tid)) {
 #ifdef TAU_MPI
       tau_dyninst_init(1);
 #else
@@ -348,9 +369,7 @@ void trace_register_func(char *origname, int id)
     TAU_VERBOSE("func=%s, after chopping off the bracket! \n", func);
   }
 
-
-
-  if (!tauDyninstEnabled[tid]) return;
+  if (!getTauDyninstEnabled(tid)) return;
 
   void *taufi;
   TAU_PROFILER_CREATE(taufi, func, " ", TAU_DEFAULT);
@@ -392,11 +411,30 @@ void trace_register_func(char *origname, int id)
   TAU_VERBOSE("Exiting trace_register_func\n");
 }
 
+#ifdef DYNINSTARMLINUX64
+void trace_register_func_dyn(char *id_origname)
+{
+  
+  int id = std::stoi(strtok(id_origname, ","));
+  char *origname = strtok(NULL, ",");
+#else
+void trace_register_func_dyn(char *origname, int id)
+{
+#endif
+  trace_register_func(origname, id);  
+}
+
+
+
 void traceEntry(int id)
 {
+  TAU_VERBOSE("Inside traceEntry: id = %d\n", id);
+  TAU_QUERY_DECLARE_EVENT(curr);
+  TAU_QUERY_GET_CURRENT_EVENT(curr);
+
   int tid = RtsLayer::myThread();
   if ( !RtsLayer::TheEnableInstrumentation()) return;
-  if (!tauDyninstEnabled[tid]) return;
+  if (!getTauDyninstEnabled(tid)) return;
   void *fi = TheTauBinDynFI()[id];
 
   // Additional sanity checks
@@ -410,9 +448,6 @@ void traceEntry(int id)
     return;
   }
 
-  TAU_QUERY_DECLARE_EVENT(curr);
-  TAU_QUERY_GET_CURRENT_EVENT(curr);
-
   if ( curr && ((Profiler *)curr)->ThisFunction &&
      ((Profiler *)curr)->ThisFunction->GetProfileGroup() == TAU_GROUP_31) {
     TAU_VERBOSE("TARG on the stack \n");
@@ -445,12 +480,12 @@ void traceExit(int id)
 {
   //const char *strcurr;
   //const char *strbin;
-  //TAU_VERBOSE("Inside traceExit: id = %d\n", id);
+  TAU_VERBOSE("Inside traceExit: id = %d\n", id);
 
   if ( !RtsLayer::TheEnableInstrumentation()) return;
   int tid = RtsLayer::myThread();
 
-  if (!tauDyninstEnabled[tid]) return;
+  if (!getTauDyninstEnabled(tid)) return;
   void *fi = TheTauBinDynFI()[id];
 #if 0
   if (fi)
@@ -483,7 +518,7 @@ void traceExit(int id)
     printf("ERROR: traceExit: fi = null!\n");
   }
   if(disableinstr) {
-    tauDyninstEnabled[tid] = false;
+    setTauDyninstEnabled(tid, false);
   }
 
 }
@@ -498,8 +533,8 @@ void my_otf_init(int isMPI)
     TAU_PROFILE_SET_NODE(0);
   }
   int tid = RtsLayer::myThread();
-  if (!tauDyninstEnabled[tid]) {
-    tauDyninstEnabled[tid] = 1;
+  if (!getTauDyninstEnabled(tid)) {
+    setTauDyninstEnabled(tid, 1);
   }
 }
 
@@ -518,10 +553,12 @@ void tau_dyninst_init(int isMPI)
     TAU_PROFILE_SET_NODE(0);
   }
   int tid = RtsLayer::myThread();
-  if (!tauDyninstEnabled[tid]) {
+  if (!getTauDyninstEnabled(tid)) {
+    defaultDyEn=1;
     RtsLayer::LockDB();
-    for (int i = 0; i < TAU_MAX_THREADS; i++)
-      tauDyninstEnabled[i] = 1;
+    int vecSize=tauDyninstEnabled.size();
+    for (int i = 0; i < vecSize; i++)
+      setTauDyninstEnabled(i,1);
     RtsLayer::UnLockDB();
   }
 }
@@ -531,15 +568,17 @@ void tau_dyninst_cleanup()
   TAU_VERBOSE("Inside tau_dyninst_cleanup\n");
 }
 
+
 void  tau_register_func(char **func, char** file, int* lineno,
   int id) {
     char * tmpstr = Tau_demangle_name(*func);
     if (*file == NULL){
       TAU_VERBOSE("TAU: tau_register_func: name = %s, id = %d\n", *func, id);
+      
       trace_register_func(tmpstr, id);
     } else {
       char funcname[2048];
-      sprintf(funcname, "%s [{%s}{%d}]", tmpstr, *file, *lineno);
+      snprintf(funcname, sizeof(funcname),  "%s [{%s}{%d}]", tmpstr, *file, *lineno);
       trace_register_func(funcname, id);
       TAU_VERBOSE("TAU : tau_register_func: name = %s, id = %d\n", funcname, id);
     }
@@ -556,6 +595,22 @@ void tau_trace_exit(int id) {
   traceExit(id);
 }
 
+void tau_trace_lib_entry(const char * func_name)
+{
+  if(!RtsLayer::TheEnableInstrumentation())
+	  return;
+  TAU_VERBOSE("TAU: tau_trace_lib_entry %s\n", func_name);
+  TAU_START(func_name);
+}
+
+void tau_trace_lib_exit(const char * func_name)
+{
+  if(!RtsLayer::TheEnableInstrumentation())
+	  return;
+  TAU_VERBOSE("TAU: tau_trace_lib_exit %s\n", func_name);
+  TAU_STOP(func_name);
+}
+
 void tau_loop_trace_entry(int id) {
   TAU_VERBOSE("TAU: tau_loop_trace_entry: id = %d\n", id);
   TAU_START(TauLoopNames[id].c_str());
@@ -596,13 +651,12 @@ void  tau_trace_register_loop(int id, char *loopname) {
 
 void  tau_register_loop(char **func, char** file, int* lineno,
   int id) {
-
   char lname[2048];
   char *loopname;
   if (((*file) != (char *)NULL) && (*lineno != 0)) {
-    sprintf(lname, "Loop: %s [{%s}{%d}]", *func, *file, *lineno);
+    snprintf(lname, sizeof(lname),  "Loop: %s [{%s}{%d}]", *func, *file, *lineno);
   } else {
-    sprintf(lname, "Loop: %s ",*func);
+    snprintf(lname, sizeof(lname),  "Loop: %s ",*func);
   }
   loopname = strdup(lname);
   tau_register_func(&loopname, file, lineno, id);
