@@ -24,9 +24,9 @@
 #include <Profile/TauMetrics.h>
 #include <Profile/Profiler.h>
 #include <Profile/TauTrace.h>
-#ifdef CUPTI
+#ifdef TAU_CUPTI
 #include <Profile/CuptiLayer.h>
-#endif //CUPTI
+#endif //TAU_CUPTI
 
 #ifdef TAUKTAU_SHCTR
 #include "Profile/KtauCounters.h"
@@ -63,6 +63,7 @@ struct ScopedArray {
 
 void metric_read_nullClock(int tid, int idx, double values[]);
 void metric_write_userClock(int tid, double value);
+int metric_get_num_clocks();
 void metric_read_userClock(int tid, int idx, double values[]);
 void metric_read_logicalClock(int tid, int idx, double values[]);
 void metric_read_gettimeofday(int tid, int idx, double values[]);
@@ -104,7 +105,7 @@ int Tau_Global_numGPUCounters = 0;
 
 static TauUserEvent **traceCounterEvents;
 
-typedef void (*function)(int, int, double[]);
+typedef void (*tau_function)(int, int, double[]);
 
 static char *metricv[TAU_MAX_METRICS];
 static int nmetrics = 0;
@@ -120,7 +121,7 @@ static int nfunctions = 0;
 static int traceMetric = 0;
 
 /* array of function pointers used to get metric data */
-static function functionArray[TAU_MAX_METRICS];
+static tau_function functionArray[TAU_MAX_METRICS];
 
 /* gtod based initial timestamp, used for snapshots and other stuff */
 static x_uint64 initialTimeStamp = 0L;
@@ -130,7 +131,7 @@ static x_uint64 finalTimeStamp = 0L;
 /* flags for atomic metrics */
 char *TauMetrics_atomicMetrics[TAU_MAX_METRICS] = { NULL };
 
-#ifdef CUPTI
+#ifdef TAU_CUPTI
 static int cuda_device_count() {
     int deviceCount;
     CUresult result = cuDeviceGetCount(&deviceCount);
@@ -175,7 +176,7 @@ static void metricv_add(const char *name) {
 
     check_max_metrics();
 
-#ifdef CUPTI
+#ifdef TAU_CUPTI
     char const * const tau_cuda_device_name = TauEnv_get_cuda_device_name();
 
     // Get events required to compute CUPTI metric
@@ -240,11 +241,11 @@ static void metricv_add(const char *name) {
                 exit(EXIT_FAILURE);
             }
             if (numMetricEvents == 1) {
-                sprintf(buff, "%s", name);
+                snprintf(buff, sizeof(buff),  "%s", name);
             } else {
                 // some events don't have proper names, so just use event id instead
                 if (std::string(buff).compare("event_name") == 0) {
-                    sprintf(buff, "%s.%d", name, event);
+                    snprintf(buff, sizeof(buff),  "%s.%d", name, event);
                     //sprintf(buff, "CUpti_EventID.%s", name);
                 }
             }
@@ -279,7 +280,7 @@ static void metricv_add(const char *name) {
             }
         } // for (event)
     } // for (dev)
-#endif //CUPTI
+#endif //TAU_CUPTI
 
     check_max_metrics();
     metricv[nmetrics] = strdup(name);
@@ -381,7 +382,7 @@ static void read_env_vars() {
     } else {
         char counterName[256];
         for (int i = 1; i < 26; i++) {
-            sprintf(counterName, "COUNTER%d", i);
+            snprintf(counterName, sizeof(counterName),  "COUNTER%d", i);
             char *metric = getenv(counterName);
             if (metric && strlen(metric) == 0) {
                 metric = NULL;
@@ -454,7 +455,7 @@ static int is_likwid_metric(char *str) {
     return 0;
 }
 
-#ifdef CUPTI
+#ifdef TAU_CUPTI
 /*********************************************************************
  * Query if a string is a CUPTI event
  ********************************************************************/
@@ -487,7 +488,7 @@ static int is_cupti_metric(char const * const str)
     return 0;
 }
 
-#endif //CUPTI
+#endif //TAU_CUPTI
 
 /*********************************************************************
  * Initialize the function array
@@ -580,7 +581,7 @@ static void initialize_functionArray() {
             functionArray[pos++] = metric_read_craytimers;
         } else if (strcasecmp(metricv[i], "TAU_MPI_MESSAGE_SIZE") == 0) {
             functionArray[pos++] = metric_read_messagesize;
-#ifdef CUPTI
+#ifdef TAU_CUPTI
         } else if (is_cupti_event(metricv[i])) {
             /* CUPTI handled separately */
             /* setup CUPTI metrics */
@@ -590,7 +591,7 @@ static void initialize_functionArray() {
             functionArray[pos++] = metric_read_cupti;
         } else if(is_cupti_metric(metricv[i])) {
             /* Cupti metrics handled separately */
-#endif //CUPTI
+#endif //TAU_CUPTI
 #ifdef TAU_PAPI
         } else if (strcasecmp(metricv[i], "P_WALL_CLOCK_TIME") == 0) {
             usingPAPI = 1;
@@ -759,7 +760,7 @@ static void initialize_functionArray() {
  ********************************************************************/
 extern "C" const char *TauMetrics_getMetricName(int metric) {
     char const * metric_name = metricv[metric];
-#ifdef CUPTI
+#ifdef TAU_CUPTI
     // Don't bother checking if it's a CUPTI counter if it's obviously not.
     if( (strncmp(metric_name, "TAU", 3) == 0)
             || (strncmp(metric_name, "TIME", 4) == 0)
@@ -771,7 +772,7 @@ extern "C" const char *TauMetrics_getMetricName(int metric) {
         event_id < Tau_CuptiLayer_get_num_events()) {
         return Tau_CuptiLayer_get_event_name(event_id);
     }
-#endif
+#endif //TAU_CUPTI
     return metric_name;
 }
 
@@ -804,11 +805,11 @@ const char* TauMetrics_getMetricAtomic(int metric) {
  * Get id of time metric
  ********************************************************************/
 int TauMetrics_getTimeMetric() {
-#ifdef CUPTI
+#ifdef TAU_CUPTI
     char const * const time = "TAUGPU_TIME";
 #else
     char const * const time = "TIME";
-#endif
+#endif //TAU_CUPTI
     for (int i = 0; i < nmetrics; i++) {
         if (strcasecmp(metricv[i], time) == 0)
             return i;
@@ -931,10 +932,11 @@ int TauMetrics_init() {
             metricv_add (TauEnv_get_ebs_source());}
     }
 
-        /* Set the user clock values to 0 */
-    for (i = 0; i < TAU_MAX_THREADS; i++) {
-        metric_write_userClock(i, 0);
-    }
+		/* Set the user clock values to 0 */
+    int numClocks=metric_get_num_clocks();
+	for (i = 0; i < numClocks; i++) {
+		metric_write_userClock(i, 0);
+	}
 
     read_env_vars();
     //for(i = 0; i < nmetrics; i++)
