@@ -20,7 +20,7 @@
  #if defined(TAU_IBM_OMPT)
   #define OMPT_VERSION 1
   #include <lomp/omp.h>
- #elif defined(__ICC) || defined(__INTEL_COMPILER)
+ #elif defined(__ICC) || defined(TAU_INTEL_COMPILER)
   // check for intel second
   #define OMPT_VERSION 3 // someday we will update this, but in the meantime...
   #define STATES_ARE_TYPE_INT
@@ -106,6 +106,7 @@ struct Tau_collector_status_flags {
  * This is very important with timers, as all threads are entering timers
  * at the same time, and every thread will invalidate the cache line
  * otherwise. */
+ /*
 #if defined __INTEL__COMPILER
 __declspec (align(64)) static struct Tau_collector_status_flags Tau_collector_flags[TAU_MAX_THREADS] = {0};
 #elif defined __GNUC__
@@ -113,6 +114,23 @@ static struct Tau_collector_status_flags Tau_collector_flags[TAU_MAX_THREADS] __
 #else
 static struct Tau_collector_status_flags Tau_collector_flags[TAU_MAX_THREADS] = {0};
 #endif
+*/
+
+static vector<Tau_collector_status_flags*> Tau_collector_flags;
+
+inline void checkTCFVector(int tid){
+	while(Tau_collector_flags.size()<=tid){
+        RtsLayer::LockDB();
+		Tau_collector_flags.push_back(new Tau_collector_status_flags());
+        RtsLayer::UnLockDB();
+	}
+}
+
+static inline Tau_collector_status_flags& getTauCollectorFlags(int tid)
+{
+    checkTCFVector(tid);
+    return *Tau_collector_flags[tid];
+}
 
 // this is map of region names, indexed by region id.
 static std::map<unsigned long, char*> * region_names = NULL;
@@ -244,12 +262,12 @@ void region_name_cleanup(unsigned long parallel_id) {
 void Tau_get_region_id(int tid) {
     // if not available, return something useful
     if (Tau_collector_api == NULL) {
-        Tau_collector_flags[tid].regionid = 0UL;
+        getTauCollectorFlags(tid).regionid = 0UL;
         return;
     }
     // query the thread state
-    int rc = (Tau_collector_api)(Tau_collector_flags[tid].region_message);
-    Tau_collector_flags[tid].regionid = (unsigned long)Tau_collector_flags[tid].region_message[4];
+    int rc = (Tau_collector_api)(getTauCollectorFlags(tid).region_message);
+    getTauCollectorFlags(tid).regionid = (unsigned long)getTauCollectorFlags(tid).region_message[4];
     if (rc !=0) {
       TAU_VERBOSE("Error getting region id from ORA!\n");
     }
@@ -259,12 +277,12 @@ void Tau_get_region_id(int tid) {
 void Tau_get_task_id(int tid) {
     // if not available, return something useful
     if (Tau_collector_api == NULL) {
-        Tau_collector_flags[tid].taskid = 0UL;
+        getTauCollectorFlags(tid).taskid = 0UL;
         return;
     }
     // query the thread state
-    int rc = (Tau_collector_api)(Tau_collector_flags[tid].task_message);
-    Tau_collector_flags[tid].taskid = (unsigned long)Tau_collector_flags[tid].task_message[4];
+    int rc = (Tau_collector_api)(getTauCollectorFlags(tid).task_message);
+    getTauCollectorFlags(tid).taskid = (unsigned long)getTauCollectorFlags(tid).task_message[4];
     if (rc !=0) {
       TAU_VERBOSE("Error getting task id from ORA!\n");
     }
@@ -351,8 +369,9 @@ char * get_proxy_name(unsigned long ip) {
     tau_bfd_handle_t & OmpbfdUnitHandle = OmpTheBfdUnitHandle();
     if (ip == 0) {
         //printf("IP IS ZERO!!!\n"); fflush(stdout); //abort();
-        location = (char*)malloc(strlen(__UNKNOWN_ADDR__)+1);
-        strcpy(location, __UNKNOWN_ADDR__);
+        const int len = strlen(__UNKNOWN_ADDR__)+1;
+        location = (char*)malloc(len);
+        strncpy(location,  __UNKNOWN_ADDR__, len); 
         return location;
     }
     //TAU_OPENMP_SET_LOCK;
@@ -382,13 +401,13 @@ char * get_proxy_name(unsigned long ip) {
             // Build routine name for TAU function info
             unsigned int size = strlen(node->info.funcname) + strlen(node->info.filename) + 128;
             routine = (char*)malloc(size);
-            sprintf(routine, "%s [{%s} {%d,0}]", node->info.funcname, node->info.filename, node->info.lineno);
+            snprintf(routine, size,  "%s [{%s} {%d,0}]", node->info.funcname, node->info.filename, node->info.lineno);
         } else {
         char addrString[64];
-        sprintf(addrString, "%p", (void*)ip);
+        snprintf(addrString, sizeof(addrString),  "%p", (void*)ip);
             unsigned int size = strlen(__BFD_UNKNOWN__) + strlen(addrString) + 2;
             routine = (char*)malloc(size);
-            sprintf(routine, "%s %s", __BFD_UNKNOWN__, addrString);
+            snprintf(routine, size,  "%s %s", __BFD_UNKNOWN__, addrString);
         }
     node->location = routine;
         TAU_OPENMP_SET_LOCK;
@@ -396,8 +415,9 @@ char * get_proxy_name(unsigned long ip) {
         TAU_OPENMP_UNSET_LOCK;
     }
 
-    location = (char*)malloc(strlen(node->location)+1);
-    strcpy(location, node->location);
+    const int len = strlen(node->location)+1;
+    location = (char*)malloc(len);
+    strncpy(location,  node->location, len); 
     return location;
 }
 #else /* defined (TAU_BFD) */
@@ -406,12 +426,13 @@ char * get_proxy_name(unsigned long ip) {
     char * location = NULL;
     if (ip == 0) {
         //printf("IP IS ZERO!!!\n"); fflush(stdout); //abort();
-        location = (char*)malloc(strlen(__UNKNOWN_ADDR__)+1);
-        strcpy(location, __UNKNOWN_ADDR__);
+        const int len = strlen(__UNKNOWN_ADDR__)+1;
+        location = (char*)malloc(len);
+        strncpy(location,  __UNKNOWN_ADDR__, len); 
         return location;
     }
     location = (char*)malloc(128);
-    sprintf(location, "UNRESOLVED ADDR %p", (void*)ip);
+    snprintf(location, 128,  "UNRESOLVED ADDR %p", (void*)ip);
     return location;
 }
 #endif /* defined (TAU_BFD) */
@@ -452,13 +473,13 @@ char * show_backtrace (int tid, int offset) {
                 // Build routine name for TAU function info
                 unsigned int size = strlen(node->info.funcname) + strlen(node->info.filename) + 128;
                 routine = (char*)malloc(size);
-                sprintf(routine, "%s [{%s} {%d,0}]", node->info.funcname, node->info.filename, node->info.lineno);
+                snprintf(routine, size,  "%s [{%s} {%d,0}]", node->info.funcname, node->info.filename, node->info.lineno);
               } else {
             char addrString[64];
-            sprintf(addrString, "%p", (void*)ip);
+            snprintf(addrString, sizeof(addrString),  "%p", (void*)ip);
                 unsigned int size = strlen(__BFD_UNKNOWN__) + strlen(addrString) + 2;
                 routine = (char*)malloc(size);
-                sprintf(routine, "%s %s", __BFD_UNKNOWN__, addrString);
+                snprintf(routine, size,  "%s %s", __BFD_UNKNOWN__, addrString);
               }
               node->location = routine;
             //TAU_OPENMP_SET_LOCK;
@@ -478,7 +499,7 @@ char * show_backtrace (int tid, int offset) {
                 else if (strncmp(node->info.funcname,"__ompc_", 7) == 0) { // in OpenUH runtime
                     continue; // keep unwinding
                 }
-#elif defined (__INTEL_COMPILER)
+#elif defined (TAU_INTEL_COMPILER)
                 else if (strncmp(node->info.funcname,"my_parallel_region_create", 25) == 0) { // in OMPT wraper (see below)
                     continue; // keep unwinding
                 } else if (strncmp(node->info.funcname,"__kmp", 5) == 0) { // in Intel runtime
@@ -502,8 +523,9 @@ char * show_backtrace (int tid, int offset) {
                 // stop unwinding
                 basedepth = index;
             }
-            location = (char*)malloc(strlen(node->location)+1);
-            strcpy(location, node->location);
+            const int len = strlen(node->location)+1;
+            location = (char*)malloc(len);
+            strncpy(location,  node->location, len); 
             break;
         }
     }
@@ -549,12 +571,12 @@ defined (__GNUC_PATCHLEVEL__)) // IBM OMPT and Generic ORA support requires unwi
     if (task) {
         //TAU_VERBOSE("Task %lu has name %s\n", Tau_collector_flags[tid].taskid, tmpStr);
         TAU_OPENMP_SET_LOCK;
-        (*task_names)[Tau_collector_flags[tid].taskid] = strdup(tmpStr);
+        (*task_names)[getTauCollectorFlags(tid).taskid] = strdup(tmpStr);
         TAU_OPENMP_UNSET_LOCK;
     } else {
         //TAU_VERBOSE("Region %lu has name %s\n", Tau_collector_flags[tid].regionid, tmpStr);
         TAU_OPENMP_SET_LOCK;
-        (*region_names)[Tau_collector_flags[tid].regionid] = strdup(tmpStr);
+        (*region_names)[getTauCollectorFlags(tid).regionid] = strdup(tmpStr);
         TAU_OPENMP_UNSET_LOCK;
     }
     free (tmpStr);
@@ -614,7 +636,7 @@ extern "C" void Tau_pure_stop_openmp_task(const char * n, int tid);
     char * tmpStr = Tau_get_my_region_context(tid, forking, task);
     contextLength = strlen(tmpStr);
     regionIDstr = (char*)malloc(contextLength + 32);
-    sprintf(regionIDstr, "%s: %s", state, tmpStr);
+    snprintf(regionIDstr, contextLength,  "%s: %s", state, tmpStr);
     //fprintf(stderr, "%d: start '%s'\n", tid, regionIDstr); fflush(stderr);
     Tau_pure_start_openmp_task(regionIDstr, tid);
 #if defined(TAU_USE_OMPT) 
@@ -640,7 +662,7 @@ extern "C" void Tau_pure_stop_openmp_task(const char * n, int tid);
         char * tmpStr = Tau_get_my_region_context(tid, 0, task);
         contextLength = strlen(tmpStr);
         regionIDstr = (char*)malloc(contextLength + 32);
-        sprintf(regionIDstr, "%s: %s", state, tmpStr);
+        snprintf(regionIDstr, contextLength,  "%s: %s", state, tmpStr);
     //fprintf(stderr, "%d: stop '%s'\n", tid, regionIDstr); fflush(stderr);
         Tau_pure_stop_openmp_task(regionIDstr, tid);
         free(regionIDstr); 
@@ -675,31 +697,31 @@ extern "C" void Tau_omp_event_handler(OMP_COLLECTORAPI_EVENT event) {
         case OMP_EVENT_FORK:
             Tau_get_current_region_context(tid, 0LU, false);
             Tau_omp_start_timer("OpenMP_PARALLEL_REGION", tid, 1, 1, false);
-            Tau_collector_flags[tid].parallel++;
+            getTauCollectorFlags(tid).parallel++;
             break;
         case OMP_EVENT_JOIN:
-            if (Tau_collector_flags[tid].parallel>0) {
+            if (getTauCollectorFlags(tid).parallel>0) {
                 Tau_omp_stop_timer("OpenMP_PARALLEL_REGION", tid, 1, false);
-                Tau_collector_flags[tid].parallel--;
+                getTauCollectorFlags(tid).parallel--;
             }
-            region_name_cleanup(Tau_collector_flags[tid].regionid);
+            region_name_cleanup(getTauCollectorFlags(tid).regionid);
             break;
         case OMP_EVENT_THR_BEGIN_IDLE:
             // sometimes IDLE can be called twice in a row
-            if (Tau_collector_flags[tid].idle == 1 && 
-                    Tau_collector_flags[tid].busy == 0) {
+            if (getTauCollectorFlags(tid).idle == 1 && 
+                    getTauCollectorFlags(tid).busy == 0) {
                 break;
             }
-            if (Tau_collector_flags[tid].busy == 1) {
+            if (getTauCollectorFlags(tid).busy == 1) {
                 Tau_omp_stop_timer("OpenMP_PARALLEL_REGION", tid, 1, false);
-                Tau_collector_flags[tid].busy = 0;
+                getTauCollectorFlags(tid).busy = 0;
             }
-            Tau_collector_flags[tid].idle = 1;
+            getTauCollectorFlags(tid).idle = 1;
             break;
         case OMP_EVENT_THR_END_IDLE:
             Tau_omp_start_timer("OpenMP_PARALLEL_REGION", tid, 1, 1, false);
-            Tau_collector_flags[tid].busy = 1;
-            Tau_collector_flags[tid].idle = 0;
+            getTauCollectorFlags(tid).busy = 1;
+            getTauCollectorFlags(tid).idle = 0;
             break;
         case OMP_EVENT_THR_BEGIN_IBAR:
             Tau_omp_start_timer("OpenMP_IMPLICIT_BARRIER", tid, 1, 0, false);
@@ -727,16 +749,16 @@ extern "C" void Tau_omp_event_handler(OMP_COLLECTORAPI_EVENT event) {
             break;
         case OMP_EVENT_THR_BEGIN_ODWT:
             // for some reason, the ordered region wait is entered twice for some threads.
-            if (Tau_collector_flags[tid].ordered_region_wait == 0) {
+            if (getTauCollectorFlags(tid).ordered_region_wait == 0) {
                 Tau_omp_start_timer("OpenMP_ORDERED_REGION_WAIT", tid, 1, 0, false);
             }
-            Tau_collector_flags[tid].ordered_region_wait = 1;
+            getTauCollectorFlags(tid).ordered_region_wait = 1;
             break;
         case OMP_EVENT_THR_END_ODWT:
-            if (Tau_collector_flags[tid].ordered_region_wait == 1) {
+            if (getTauCollectorFlags(tid).ordered_region_wait == 1) {
                 Tau_omp_stop_timer("OpenMP_ORDERED_REGION_WAIT", tid, 1, false);
             }
-            Tau_collector_flags[tid].ordered_region_wait = 0;
+            getTauCollectorFlags(tid).ordered_region_wait = 0;
             break;
         case OMP_EVENT_THR_BEGIN_MASTER:
             Tau_omp_start_timer("OpenMP_MASTER_REGION", tid, 1, 0, false);
@@ -752,16 +774,16 @@ extern "C" void Tau_omp_event_handler(OMP_COLLECTORAPI_EVENT event) {
             break;
         case OMP_EVENT_THR_BEGIN_ORDERED:
             // for some reason, the ordered region is entered twice for some threads.
-            if (Tau_collector_flags[tid].ordered_region == 0) {
+            if (getTauCollectorFlags(tid).ordered_region == 0) {
                 Tau_omp_start_timer("OpenMP_ORDERED_REGION", tid, 1, 0, false);
-                Tau_collector_flags[tid].ordered_region = 1;
+                getTauCollectorFlags(tid).ordered_region = 1;
             }
             break;
         case OMP_EVENT_THR_END_ORDERED:
-            if (Tau_collector_flags[tid].ordered_region == 1) {
+            if (getTauCollectorFlags(tid).ordered_region == 1) {
                 Tau_omp_stop_timer("OpenMP_ORDERED_REGION", tid, 1, false);
             }
-            Tau_collector_flags[tid].ordered_region = 0;
+            getTauCollectorFlags(tid).ordered_region = 0;
             break;
         case OMP_EVENT_THR_BEGIN_ATWT:
             Tau_omp_start_timer("OpenMP_ATOMIC_REGION_WAIT", tid, 1, 0, false);
@@ -832,16 +854,16 @@ extern "C" void Tau_omp_event_handler(OMP_COLLECTORAPI_EVENT event) {
             // query the ORA to get the task id
             Tau_get_task_id (tid);
             Tau_omp_start_timer("OpenMP_EXECUTE_TASK", tid, 1, 0, false);
-            Tau_collector_flags[tid].task_exec += 1;
+            getTauCollectorFlags(tid).task_exec += 1;
             break;
         case OMP_EVENT_THR_BEGIN_FINISH_TASK:
             // query the ORA to get the task id
             //Tau_get_task_id (tid);
             // When we get a "finish task", there might be a task executing...
             // or there might not.
-            if (Tau_collector_flags[tid].task_exec > 0) {
+            if (getTauCollectorFlags(tid).task_exec > 0) {
                 Tau_omp_stop_timer("OpenMP_EXECUTE_TASK", tid, 0, false);
-                Tau_collector_flags[tid].task_exec -= 1;
+                getTauCollectorFlags(tid).task_exec -= 1;
             }
             Tau_omp_start_timer("OpenMP_FINISH_TASK", tid, 0, 0, false);
             break;
@@ -915,7 +937,7 @@ extern "C" int Tau_initialize_collector_api(void) {
     memcpy(&Tau_collector_api, &temp_fptr, sizeof(temp_fptr));
     if (Tau_collector_api == NULL) {
 
-#if defined (__INTEL_COMPILER)
+#if defined (TAU_INTEL_COMPILER)
         char * libname = "libiomp5.so";
 #elif defined (__GNUC__) && defined (__GNUC_MINOR__) && defined (__GNUC_PATCHLEVEL__)
 
@@ -999,26 +1021,26 @@ extern "C" int Tau_initialize_collector_api(void) {
     int task_rsz = sizeof(int);
     for(i=0;i<omp_get_max_threads();i++) {  
         // for getting thread state
-        Tau_collector_flags[i].signal_message = (int*)malloc(OMP_COLLECTORAPI_HEADERSIZE+state_rsz+sizeof(int));
-        memset(Tau_collector_flags[i].signal_message, 0, (OMP_COLLECTORAPI_HEADERSIZE+state_rsz+sizeof(int)));
-        Tau_collector_flags[i].signal_message[0] = OMP_COLLECTORAPI_HEADERSIZE+state_rsz;
-        Tau_collector_flags[i].signal_message[1] = OMP_REQ_STATE;
-        Tau_collector_flags[i].signal_message[2] = OMP_ERRCODE_OK;
-        Tau_collector_flags[i].signal_message[3] = state_rsz;
+        getTauCollectorFlags(i).signal_message = (int*)malloc(OMP_COLLECTORAPI_HEADERSIZE+state_rsz+sizeof(int));
+        memset(getTauCollectorFlags(i).signal_message, 0, (OMP_COLLECTORAPI_HEADERSIZE+state_rsz+sizeof(int)));
+        getTauCollectorFlags(i).signal_message[0] = OMP_COLLECTORAPI_HEADERSIZE+state_rsz;
+        getTauCollectorFlags(i).signal_message[1] = OMP_REQ_STATE;
+        getTauCollectorFlags(i).signal_message[2] = OMP_ERRCODE_OK;
+        getTauCollectorFlags(i).signal_message[3] = state_rsz;
         // for getting region id
-        Tau_collector_flags[i].region_message = (int *)malloc(OMP_COLLECTORAPI_HEADERSIZE+currentid_rsz+sizeof(int));
-        memset(Tau_collector_flags[i].region_message, 0, (OMP_COLLECTORAPI_HEADERSIZE+currentid_rsz+sizeof(int)));
-        Tau_collector_flags[i].region_message[0] = OMP_COLLECTORAPI_HEADERSIZE+currentid_rsz;
-        Tau_collector_flags[i].region_message[1] = OMP_REQ_CURRENT_PRID;
-        Tau_collector_flags[i].region_message[2] = OMP_ERRCODE_OK;
-        Tau_collector_flags[i].region_message[3] = currentid_rsz;
+        getTauCollectorFlags(i).region_message = (int *)malloc(OMP_COLLECTORAPI_HEADERSIZE+currentid_rsz+sizeof(int));
+        memset(getTauCollectorFlags(i).region_message, 0, (OMP_COLLECTORAPI_HEADERSIZE+currentid_rsz+sizeof(int)));
+        getTauCollectorFlags(i).region_message[0] = OMP_COLLECTORAPI_HEADERSIZE+currentid_rsz;
+        getTauCollectorFlags(i).region_message[1] = OMP_REQ_CURRENT_PRID;
+        getTauCollectorFlags(i).region_message[2] = OMP_ERRCODE_OK;
+        getTauCollectorFlags(i).region_message[3] = currentid_rsz;
         // for getting task id
-        Tau_collector_flags[i].task_message = (int *)malloc(OMP_COLLECTORAPI_HEADERSIZE+task_rsz+sizeof(int));
-        memset(Tau_collector_flags[i].task_message, 0, (OMP_COLLECTORAPI_HEADERSIZE+task_rsz+sizeof(int)));
-        Tau_collector_flags[i].task_message[0] = OMP_COLLECTORAPI_HEADERSIZE+task_rsz;
-        Tau_collector_flags[i].task_message[1] = OMP_REQ_CURRENT_PRID;
-        Tau_collector_flags[i].task_message[2] = OMP_ERRCODE_OK;
-        Tau_collector_flags[i].task_message[3] = task_rsz;
+        getTauCollectorFlags(i).task_message = (int *)malloc(OMP_COLLECTORAPI_HEADERSIZE+task_rsz+sizeof(int));
+        memset(getTauCollectorFlags(i).task_message, 0, (OMP_COLLECTORAPI_HEADERSIZE+task_rsz+sizeof(int)));
+        getTauCollectorFlags(i).task_message[0] = OMP_COLLECTORAPI_HEADERSIZE+task_rsz;
+        getTauCollectorFlags(i).task_message[1] = OMP_REQ_CURRENT_PRID;
+        getTauCollectorFlags(i).task_message[2] = OMP_ERRCODE_OK;
+        getTauCollectorFlags(i).task_message[3] = task_rsz;
     }
 
 #ifdef TAU_UNWIND
@@ -1112,8 +1134,8 @@ extern "C" int Tau_get_thread_omp_state(int tid) {
     if (Tau_collector_api == NULL) return -1;
     OMP_COLLECTOR_API_THR_STATE thread_state = THR_LAST_STATE;
     // query the thread state
-    (Tau_collector_api)(Tau_collector_flags[tid].signal_message);
-    thread_state = (OMP_COLLECTOR_API_THR_STATE)Tau_collector_flags[tid].signal_message[4];
+    (Tau_collector_api)(getTauCollectorFlags(tid).signal_message);
+    thread_state = (OMP_COLLECTOR_API_THR_STATE)getTauCollectorFlags(tid).signal_message[4];
     // return the thread state
     return (int)(thread_state);
 }
