@@ -84,13 +84,15 @@ typedef struct {
 
 
 /** Comparator class used to create a sort map for unification */
-class EventComparator : public binary_function<int, int, bool> {
+class EventComparator {
 
 private:
   EventLister *eventLister;
 
 public:
-
+  using result_type = bool;
+  using first_argument_type = int;
+  using second_argument_type = int;
   /** Constructor takes an EventLister, stores it for use with comparison */
   EventComparator(EventLister *eventLister) {
     this->eventLister = eventLister;
@@ -438,7 +440,7 @@ Tau_unify_object_t *Tau_unify_unifyEvents_MPI(EventLister *eventLister) {
     TAU_VERBOSE("TAU: Unifying Complete, duration = %.4G seconds\n",
 		((double)(end-start))/1000000.0f);
     char tmpstr[256];
-    sprintf(tmpstr, "%.4G seconds", ((double)(end-start))/1000000.0f);
+    snprintf(tmpstr, sizeof(tmpstr),  "%.4G seconds", ((double)(end-start))/1000000.0f);
     TAU_METADATA("TAU Unification Time", tmpstr);
   }
 
@@ -679,7 +681,7 @@ Tau_unify_object_t *Tau_unify_unifyEvents_SHMEM(EventLister *eventLister) {
     TAU_VERBOSE("TAU: Unifying Complete, duration = %.4G seconds\n",
 		((double)(end-start))/1000000.0f);
     char tmpstr[256];
-    sprintf(tmpstr, "%.4G seconds", ((double)(end-start))/1000000.0f);
+    snprintf(tmpstr, sizeof(tmpstr),  "%.4G seconds", ((double)(end-start))/1000000.0f);
     TAU_METADATA("TAU Unification Time", tmpstr);
   }
 
@@ -771,7 +773,7 @@ extern "C" int Tau_unify_unifyDefinitions_SHMEM() {
 #endif /* TAU_UNIFY */
 
 #ifdef TAU_MPC
-extern "C" int TauInitMpcThreads(int* rank) {
+/*extern "C" int TauInitMpcThreads(int* rank) {
   static bool firsttime = true;
   if (firsttime) {
     for (int i = 0; i < TAU_MAX_THREADS; i++) {
@@ -780,22 +782,31 @@ extern "C" int TauInitMpcThreads(int* rank) {
     firsttime = false;
   }
   return 0;
-}
+}*/
+struct RankList : vector<int>{ //TODO: DYNATHREAD Test this implementation with a working MPC + merged profile output
+ virtual ~RankList(){
+         //printf("Destroying RankList at %p, with size %ld\n", this, this->size());
+         Tau_destructor_trigger();
+     }
+};
 
 extern "C" int TauGetMpiRank(void) {
-  static int firsttime = 1;
-  static int *rank = NULL;
+  //static int firsttime = 1;
+  static RankList rank;//int *rank = NULL;
   int retval;
 
   RtsLayer::LockDB();
   int tid = RtsLayer::myThread();
-  if (firsttime) {
+  /*if (firsttime) {
     if (rank == NULL) {
-      rank = new int[TAU_MAX_THREADS];
+      rank = new int[TAU_MAX_THREADS];   
       firsttime = TauInitMpcThreads(rank);
     }
-  }
-  if (rank[tid] == -1) {
+  }*/
+  if (rank.size()<=tid||rank[tid] == -1) {
+    while(rank.size()<=tid){
+        rank.push_back(-1);
+    }
     PMPI_Comm_rank(MPI_COMM_WORLD, &rank[tid]);
   }
   retval = rank[tid];
