@@ -66,9 +66,9 @@
 #endif
 #endif /* TAU_INCLUDE_MPI_H_HEADER */
 
-#ifdef CUPTI
+#ifdef TAU_CUPTI
 #include <Profile/CuptiLayer.h>
-#endif
+#endif //TAU_CUPTI
 
 #define OTF2_EC(call) { \
     OTF2_ErrorCode ec = call; \
@@ -82,7 +82,7 @@
     OTF2_ErrorCode ec = call; \
     if (ec != OTF2_SUCCESS) { \
         printf("TAU: OTF2 Error (%s:%d): %s, %s\n", __FILE__, __LINE__, OTF2_Error_GetName(ec), OTF2_Error_GetDescription (ec)); \
-		printf("Prevous: %d, %lu tid = %d\n", previous_type[tid], previous_ts[tid], tid); \
+		printf("Prevous: %d, %lu tid = %d\n", getPreviousType(tid), getPreviousTS(tid), tid); \
         abort(); \
     } \
 }
@@ -117,8 +117,7 @@ static uint64_t start_time = 0;
 // Time of last event recorded
 static uint64_t end_time = 0;
 // Type, Time of previous event
-static int previous_type[TAU_MAX_THREADS] = {0};
-static uint64_t previous_ts[TAU_MAX_THREADS] = {0};
+//static int previous_type[TAU_MAX_THREADS] = {0};
 
 static uint64_t global_start_time = 0;
 
@@ -135,8 +134,100 @@ struct temp_buffer_entry {
 // Temporary buffer for events prior to initialization
 // pair.first = FunctionId
 // pair.second = timestamp
-static vector<temp_buffer_entry> * temp_buffers[TAU_MAX_THREADS] = {0};
-static bool buffers_written[TAU_MAX_THREADS] = {0};
+//static vector<temp_buffer_entry> * temp_buffers[TAU_MAX_THREADS] = {0};
+//static bool buffers_written[TAU_MAX_THREADS] = {0};
+
+typedef pair<pair<int,int>,int> rma_win_triple_t;
+typedef map<rma_win_triple_t,uint64_t> rma_win_map_t;
+
+//static rma_win_map_t * local_rma_win_maps[TAU_MAX_THREADS];
+//static uint64_t next_rma_win[TAU_MAX_THREADS];
+
+struct otf2_thread_data{
+    int previous_type = 0;
+    uint64_t previous_ts = 0;
+    bool buffers_written = false;
+    vector<temp_buffer_entry> * temp_buffers=NULL;
+    uint64_t next_rma_win;
+    rma_win_map_t * local_rma_win_maps;
+};
+
+struct OTF2ThreadList : vector<otf2_thread_data *>{
+    OTF2ThreadList (const OTF2ThreadList&) = delete;
+    OTF2ThreadList& operator= (const OTF2ThreadList&) = delete;
+    OTF2ThreadList(){
+         //printf("Creating OTF2ThreadList at %p\n", this);
+      }
+     virtual ~OTF2ThreadList(){
+         //printf("Destroying OTF2ThreadList at %p, with size %ld\n", this, this->size());
+         Tau_destructor_trigger();
+     }
+   };
+
+// Static holder for snapshot file handles
+static OTF2ThreadList & Tau_otf2_getThreadData() {
+  static OTF2ThreadList otf2ThreadList;
+  //TAU_VERBOSE("Tau_otf2_getThreadData() end: out=%p\n", &Tau_otf2_getThreadData);
+  return otf2ThreadList;
+}
+
+static inline void checkOtf2ThreadDataVector(int tid){
+	if(Tau_otf2_getThreadData().size()<=tid){
+		 RtsLayer::LockDB();
+		while(Tau_otf2_getThreadData().size()<=tid){
+			Tau_otf2_getThreadData().push_back(new otf2_thread_data());
+		}
+		RtsLayer::UnLockDB();
+	}
+}
+
+static inline int getPreviousType(int tid){
+    checkOtf2ThreadDataVector(tid);
+	return Tau_otf2_getThreadData()[tid]->previous_type;
+}
+static inline void setPreviousType(int tid, int value){
+    checkOtf2ThreadDataVector(tid);
+	Tau_otf2_getThreadData()[tid]->previous_type=value;
+}
+static inline uint64_t getPreviousTS(int tid){
+    checkOtf2ThreadDataVector(tid);
+	return Tau_otf2_getThreadData()[tid]->previous_ts;
+}
+static inline void setPreviousTS(int tid, uint64_t value){
+    checkOtf2ThreadDataVector(tid);
+	Tau_otf2_getThreadData()[tid]->previous_ts=value;
+}
+
+static inline bool getBuffersWritten(int tid){
+    checkOtf2ThreadDataVector(tid);
+	return Tau_otf2_getThreadData()[tid]->buffers_written;
+}
+static inline void setBuffersWritten(int tid, bool value){
+    checkOtf2ThreadDataVector(tid);
+	Tau_otf2_getThreadData()[tid]->buffers_written=value;
+}
+
+static inline vector<temp_buffer_entry>*& getTempBuffer(int tid){
+    checkOtf2ThreadDataVector(tid);
+    return Tau_otf2_getThreadData()[tid]->temp_buffers;
+}
+
+static inline rma_win_map_t * getLocalRMAWinMaps(int tid){
+    checkOtf2ThreadDataVector(tid);
+	return Tau_otf2_getThreadData()[tid]->local_rma_win_maps;
+}
+static inline void setLocalRMAWinMaps(int tid,rma_win_map_t * value){
+    checkOtf2ThreadDataVector(tid);
+	Tau_otf2_getThreadData()[tid]->local_rma_win_maps=value;
+}
+static inline uint64_t getNextRMAWin(int tid){
+    checkOtf2ThreadDataVector(tid);
+	return Tau_otf2_getThreadData()[tid]->next_rma_win;
+}
+static inline void setNextRMAWin(int tid, uint64_t value){
+    checkOtf2ThreadDataVector(tid);
+	Tau_otf2_getThreadData()[tid]->next_rma_win=value;
+}
 
 // For unification data
 static int * num_locations = NULL;
@@ -170,11 +261,7 @@ static metric_param_map_t global_metric_param_map;
 typedef set<uint64_t> metrics_seen_t;
 static metrics_seen_t metrics_seen;
 
-typedef pair<pair<int,int>,int> rma_win_triple_t;
-typedef map<rma_win_triple_t,uint64_t> rma_win_map_t;
 static rma_win_map_t rma_win_map;
-static rma_win_map_t * local_rma_win_maps[TAU_MAX_THREADS];
-static uint64_t next_rma_win[TAU_MAX_THREADS];
 static uint64_t total_rma_wins;
 
 static rma_win_triple_t make_triple(const int x, const int y, const int z) {
@@ -187,23 +274,35 @@ extern "C" void finalizeCallSites_if_necessary();
 extern "C" void Tau_ompt_resolve_callsite(FunctionInfo &fi, char * resolved_address);
 
 // Helper functions
-
+/*
 static inline OTF2_LocationRef my_location_offset() {
     const int64_t myNode = RtsLayer::myNode();
-    return myNode == -1 ? 0 : (myNode * TAU_MAX_THREADS);
+    const int64_t myThread = RtsLayer::myThread();
+    int totNodes=tau_totalnodes(0,0);
+    //printf("Max Nodes: %d\n",totNodes);
+    return myNode == -1 ? 0 : (myThread * tau_totalnodes(0,0));//(myNode * TAU_MAX_THREADS);//TODO: DYNATHREAD
 }
-
+*/
 static inline OTF2_LocationRef my_real_location( int64_t myNode, int64_t myThread ) {
      //const int64_t myNode = RtsLayer::myNode();
      //const int64_t myThread = RtsLayer::myThread();
-     return myNode == -1 ? myThread : (myNode * TAU_MAX_THREADS) + myThread;
+     //printf("OTF2 Location: myNode: %ld, myThread: %ld, global: %ld\n",myNode,myThread, myNode==-1 ? myThread : (myThread * tau_totalnodes(0,0)) + myNode);
+     //return myNode == -1 ? myThread : (myThread * tau_totalnodes(0,0)) + myNode; //(myNode * TAU_MAX_THREADS) + myThread;
+     //return myNode == -1 ? myThread : ((myNode+myThread-2)*(myNode+myThread-1)/2)+myNode;
+     //Generate unique location id using Szudzik's pairing function
+     if(myNode == -1) return myThread;
+     if(myNode>=myThread)
+       return myNode*myNode+myNode+myThread;
+     else
+       return myNode+myThread*myThread;
+
  }
 
 
 static inline OTF2_LocationRef my_location() {
     const int64_t myNode = RtsLayer::myNode();
     const int64_t myThread = RtsLayer::myThread();
-    return myNode == -1 ? myThread : (myNode * TAU_MAX_THREADS) + myThread;
+    return my_real_location(myNode,myThread);//myNode == -1 ? myThread : (myThread * tau_totalnodes(0,0)) + myNode; //(myNode * TAU_MAX_THREADS) + myThread;
 }
 
 static inline uint32_t my_node() {
@@ -231,7 +330,7 @@ static inline x_uint64 fix_zero_timestamp(x_uint64 my_ts, int tid) {
     my_ts = (x_uint64)(tmpTime[0]);
     // if so, the start time is possibly wrong, too.
     if (start_time == 0) {
-	  printf("Fixing Start! %" PRIu64 " = %" PRIu64 "\n", start_time, my_ts);
+	  printf("Fixing Start! %" PRIu64 " = %llu\n", start_time, my_ts);
       start_time = my_ts;
     }
   }
@@ -446,7 +545,7 @@ void remove_path(const char *pathname) {
         	FILE *file = NULL;
         	char abs_path[4096] = {0};
         	if(*(entry->d_name) != '.') {
-            	sprintf(abs_path, "%s/%s", pathname, entry->d_name);
+            	snprintf(abs_path, sizeof(abs_path),  "%s/%s", pathname, entry->d_name);
             	sub_dir = opendir(abs_path);
             	if(sub_dir != NULL) {
                 	closedir(sub_dir);
@@ -468,7 +567,7 @@ int TauTraceOTF2InitTS(int tid, x_uint64 ts)
 {
   TauInternalFunctionGuard protects_this_function;
 #ifdef TAU_OTF2_DEBUG
-  fprintf(stderr, "%u: TauTraceOTF2InitTS(%d, %" PRIu64 ")\n", my_node(), tid, ts);
+  fprintf(stderr, "%u: TauTraceOTF2InitTS(%d, %llu)\n", my_node(), tid, ts);
 #endif
   if(otf2_initialized || otf2_finished) {
       return 0;
@@ -566,12 +665,12 @@ void TauTraceOTF2WriteTempBuffer(int tid, int node_id) {
   fprintf(stderr, "%u: TauTraceOTF2WriteTempBuffer(%d, %d)\n", my_node(), tid, node_id);
 #endif
     TauInternalFunctionGuard protects_this_function;
-    buffers_written[tid] = true;
-    if(temp_buffers[tid] == NULL) {
+    setBuffersWritten(tid, true);
+    if(getTempBuffer(tid) == NULL) {
         return; // Nothing was saved for this thread
     }
     x_uint64 last_ts = 0;
-    for(vector<temp_buffer_entry>::const_iterator it = temp_buffers[tid]->begin(); it != temp_buffers[tid]->end(); ++it) {
+    for(vector<temp_buffer_entry>::const_iterator it = getTempBuffer(tid)->begin(); it != getTempBuffer(tid)->end(); ++it) {
       int kind = it->kind == TAU_TRACE_EVENT_KIND_USEREVENT ? TAU_TRACE_EVENT_KIND_TEMP_USEREVENT : TAU_TRACE_EVENT_KIND_TEMP_FUNC;
       TauTraceOTF2EventWithNodeId(it->ev, it->par, tid, it->ts, true, node_id, kind);
       last_ts = it->ts;
@@ -580,7 +679,7 @@ void TauTraceOTF2WriteTempBuffer(int tid, int node_id) {
 #if defined(TAU_SHMEM)
     OTF2_EvtWriter_RmaWinCreate(evt_writer, NULL, last_ts+1, TAU_OTF2_COMM_WIN);
 #endif
-    delete temp_buffers[tid];
+    delete getTempBuffer(tid);
 }
 
 extern "C" int Tau_is_thread_fake(int t);
@@ -592,7 +691,7 @@ void TauTraceOTF2EventWithNodeId(long int ev, x_int64 par, int tid, x_uint64 ts,
     return;
   }
 #ifdef TAU_OTF2_DEBUG
-  fprintf(stderr, "node=%u, tid=%d, loc=%d: TauTraceEventWithNodeId(ev=%ld, par=%" PRId64 ", tid=%d, ts=%" PRIu64 ", use_ts=%d, node_id=%d, kind=%d)\n", my_node(), tid, my_real_location(node_id, tid), ev, par, tid, ts, use_ts, node_id, kind);
+  fprintf(stderr, "node=%u, tid=%d, loc=%lu: TauTraceEventWithNodeId(ev=%ld, par=%lld, tid=%d, ts=%llu, use_ts=%d, node_id=%d, kind=%d)\n", my_node(), tid, my_real_location(node_id, tid), ev, par, tid, ts, use_ts, node_id, kind);
 #endif
 #ifdef CUPTI_disabled
   /* OK, this looks bad, but hear me out... CUDA events are in-order on a
@@ -604,7 +703,7 @@ void TauTraceOTF2EventWithNodeId(long int ev, x_int64 par, int tid, x_uint64 ts,
    * a timestamp should use the previous event timestamp just in case. */
   if (ts == 0 && Tau_is_thread_fake(tid)) {
     use_ts = 1;
-    ts = previous_ts[tid];
+    ts = getPreviousTS(tid);
   }
 #endif
   TauInternalFunctionGuard protects_this_function;
@@ -647,11 +746,11 @@ void TauTraceOTF2EventWithNodeId(long int ev, x_int64 par, int tid, x_uint64 ts,
    }
    else
    {
-    if(temp_buffers[tid] == NULL) {
-        temp_buffers[tid] = new vector<temp_buffer_entry>();
+    if(getTempBuffer(tid) == NULL) {
+        getTempBuffer(tid) = new vector<temp_buffer_entry>();
     }
     x_uint64 my_ts = fix_zero_timestamp(use_ts ? ts : TauTraceGetTimeStamp(tid), tid);
-    temp_buffers[tid]->push_back(temp_buffer_entry(ev, my_ts, par, kind));
+    getTempBuffer(tid)->push_back(temp_buffer_entry(ev, my_ts, par, kind));
     return;
    }
 #else
@@ -669,7 +768,7 @@ void TauTraceOTF2EventWithNodeId(long int ev, x_int64 par, int tid, x_uint64 ts,
 
 
   if(TauEnv_get_set_node()==-1){
-    if(!buffers_written[tid] && !otf2_comms_shutdown) {
+    if(!getBuffersWritten(tid) && !otf2_comms_shutdown) {
       TauTraceOTF2WriteTempBuffer(tid, node_id);
     }
   }
@@ -685,26 +784,26 @@ void TauTraceOTF2EventWithNodeId(long int ev, x_int64 par, int tid, x_uint64 ts,
     // in between the start and the stop, but the counter will get a timestamp
     // that could (will?) be after the timer stop.  So, to prevent out-of-order
     // events, make the counter timestamp the previous timer timestamp + 1.
-#if defined(CUPTI) || defined(TAU_ENABLE_ROCTRACER)
-    if (my_ts < previous_ts[tid]) {
-      TAU_VERBOSE("ERROR! Timestamps out of sequence. %lu < %lu on thread %d\nevent: node=%u, tid=%d, loc=%d: TauTraceEventWithNodeId(ev=%ld, par=%" PRId64 ", tid=%d, ts=%" PRIu64 ", use_ts=%d, node_id=%d, kind=%d)\n", my_ts, previous_ts[tid], tid, my_node(), tid, my_real_location(node_id, tid), ev, par, tid, ts, use_ts, node_id, kind);
-      my_ts = previous_ts[tid];
+#if defined(TAU_CUPTI) || defined(TAU_ENABLE_ROCTRACER) || defined(TAU_USE_OMPT_5_0)
+    if (my_ts < getPreviousTS(tid)) {
+      TAU_VERBOSE("ERROR! Timestamps out of sequence. %lu < %lu on thread %d\nevent: node=%u, tid=%d, loc=%d: TauTraceEventWithNodeId(ev=%ld, par=%" PRId64 ", tid=%d, ts=%" PRIu64 ", use_ts=%d, node_id=%d, kind=%d)\n", my_ts, getPreviousTS(tid), tid, my_node(), tid, my_real_location(node_id, tid), ev, par, tid, ts, use_ts, node_id, kind);
+      my_ts = getPreviousTS(tid);
     }
 #endif
   if(kind == TAU_TRACE_EVENT_KIND_FUNC || kind == TAU_TRACE_EVENT_KIND_CALLSITE) {
     OTF2_EvtWriter* evt_writer = OTF2_Archive_GetEvtWriter(otf2_archive, loc);
     if(par == 1) { // Enter
 #ifdef TAU_OTF2_DEBUG
-      fprintf(stderr, "%u: writing Enter event on loc %d\n", my_node(), loc);
+      fprintf(stderr, "%u: writing Enter event=%ld on loc %d\n", my_node(), ev, loc);
 #endif
       OTF2_EC2(OTF2_EvtWriter_Enter(evt_writer, NULL, my_ts, ev));
-      previous_type[tid] = 0;
+      setPreviousType(tid, 0);
     } else if(par == -1) { // Exit
 #ifdef TAU_OTF2_DEBUG
-      fprintf(stderr, "%u: writing Exit event on loc %d\n", my_node(), loc);
+      fprintf(stderr, "%u: writing Exit event=%ld on loc %d\n", my_node(), ev, loc);
 #endif
       OTF2_EC2(OTF2_EvtWriter_Leave(evt_writer, NULL, my_ts, ev));
-      previous_type[tid] = 1;
+      setPreviousType(tid, 1);
     }
   } else if(kind == TAU_TRACE_EVENT_KIND_USEREVENT) {
     if(otf2_comms_shutdown && metrics_seen.find(ev) == metrics_seen.end()) {
@@ -719,7 +818,7 @@ void TauTraceOTF2EventWithNodeId(long int ev, x_int64 par, int tid, x_uint64 ts,
     //printf ("%d %lu Counter: %d\n", tid, my_ts - start_time, ev);
     OTF2_EC2(OTF2_EvtWriter_Metric(evt_writer, NULL, my_ts, ev, 1, types, values))
   }
-  previous_ts[tid] = my_ts;
+  setPreviousTS(tid, my_ts);
   // fix the last timestamp, if necessary - might happen if the program has ended
   // but we are still flushing asynchronous GPU activity.
   if (otf2_flushing_at_exit && end_time > 0 && my_ts > end_time) {
@@ -733,8 +832,9 @@ extern "C" void TauTraceOTF2Msg(int send_or_recv, int type, int other_id, int le
     if(otf2_disable) {
         return;
     }
+
 #ifdef TAU_OTF2_DEBUG
-  fprintf(stderr, "%d: TauTraceOTF2Msg(%d, %d, %d, %d, %" PRIu64 ", %d, %d)\n", my_node(), send_or_recv, type, other_id, length, ts, use_ts, node_id);
+  fprintf(stderr, "my_node=%d: TauTraceOTF2Msg(send_or_recv=%d, type=%d, other_id=%d, length=%d, ts=%llu, use_ts=%d, node_id=%d)\n", my_node(), send_or_recv, type, other_id, length, ts, use_ts, node_id);
 #endif
     TauInternalFunctionGuard protects_this_function;
     if(!otf2_initialized) {
@@ -764,9 +864,11 @@ extern "C" void TauTraceOTF2Msg(int send_or_recv, int type, int other_id, int le
     }
 #else
     if(send_or_recv == TAU_MESSAGE_SEND) {
-        OTF2_EC(OTF2_EvtWriter_MpiSend(evt_writer, NULL, time, other_id, TAU_OTF2_COMM_WORLD, type, length));
+		//fprintf(stderr, "my_node=%d: OTF2 Writing MpiSend. Dest: %d\n", my_real_location(node_id,0),my_real_location(other_id,0));
+        OTF2_EC(OTF2_EvtWriter_MpiSend(evt_writer, NULL, time,other_id, TAU_OTF2_COMM_WORLD, type, length));//my_real_location(other_id,0)
     } else if(send_or_recv == TAU_MESSAGE_RECV) {
-        OTF2_EC(OTF2_EvtWriter_MpiRecv(evt_writer, NULL, time, other_id, TAU_OTF2_COMM_WORLD, type, length));
+		//fprintf(stderr, "my_node=%d: OTF2 Writing MpiRecv. Source: %d\n", my_real_location(node_id,0),my_real_location(other_id,0));
+        OTF2_EC(OTF2_EvtWriter_MpiRecv(evt_writer, NULL, time, other_id, TAU_OTF2_COMM_WORLD, type, length));// my_real_location(other_id,0)
     }
 #endif
 }
@@ -846,17 +948,18 @@ void TauTraceOTF2RMACollectiveBegin(int tag, int type, int start, int stride, in
     x_uint64 time = TauTraceGetTimeStamp(0);
     const int loc = my_location();
     const int tid = RtsLayer::myThread();
-    if(local_rma_win_maps[tid] == NULL) {
-        local_rma_win_maps[tid] = new rma_win_map_t;
-        next_rma_win[tid] = TAU_OTF2_WIN_FIRST_AVAILABLE;
+    if(getLocalRMAWinMaps(tid) == NULL) {
+       setLocalRMAWinMaps(tid,  new rma_win_map_t);
+        setNextRMAWin(tid, TAU_OTF2_WIN_FIRST_AVAILABLE);
     }
-    rma_win_map_t * local_map = local_rma_win_maps[tid];
+    rma_win_map_t * local_map = getLocalRMAWinMaps(tid);
     rma_win_triple_t triple = make_triple(start, stride, size);
     rma_win_map_t::const_iterator it = local_map->find(triple);
     OTF2_EvtWriter* evt_writer = OTF2_Archive_GetEvtWriter(otf2_archive, loc);
     if(it == local_map->end()) {
         // We haven't seen this active set before
-        int active_set_id = next_rma_win[tid]++;
+        int active_set_id = getNextRMAWin(tid)+1;
+		setNextRMAWin(tid,active_set_id);
 #ifdef TAU_OTF2_DEBUG
         fprintf(stderr, "%u: loc %d, new active set (%d, %d, %d) has num %d\n", my_node(), loc, start, stride, size, active_set_id);
 #endif
@@ -882,7 +985,7 @@ void TauTraceOTF2RMACollectiveEnd(int tag, int type, int start, int stride, int
     const int loc = my_location();
     OTF2_EvtWriter* evt_writer = OTF2_Archive_GetEvtWriter(otf2_archive, loc);
     const int tid = RtsLayer::myThread();
-    rma_win_map_t * local_map = local_rma_win_maps[tid];
+    rma_win_map_t * local_map = getLocalRMAWinMaps(tid);
     if(local_map == NULL) {
         fprintf(stderr, "TAU: Error: Got a collective end but have never encountered a start\n");
         abort();
@@ -926,8 +1029,11 @@ static void TauTraceOTF2WriteGlobalDefinitions() {
     //global_start_time -= trace_len * 0.02;
     //trace_len = end_time - global_start_time;
     //trace_len *= 1.02;
-    OTF2_GlobalDefWriter_WriteClockProperties(global_def_writer, TAU_OTF2_CLOCK_RES, global_start_time, trace_len);
-
+    OTF2_GlobalDefWriter_WriteClockProperties(global_def_writer, TAU_OTF2_CLOCK_RES, global_start_time, trace_len
+		    #if OTF2_VERSION_MAJOR > 2
+		    , global_start_time*1000
+		    #endif
+		    );
     // Write a Location for each thread within each Node (which has a LocationGroup and SystemTreeNode)
 
     int nextString = 1;
@@ -935,6 +1041,10 @@ static void TauTraceOTF2WriteGlobalDefinitions() {
     OTF2_EC(OTF2_GlobalDefWriter_WriteString(global_def_writer, 0, "" ));
 
     const int nodes = tau_totalnodes(0, 0);
+    int max_threads=*max_element(num_locations,num_locations+nodes);
+    #ifdef TAU_OTF2_DEBUG
+        fprintf(stderr, "Global max threads: %d\n", max_threads);
+    #endif
     for(int node = 0; node < nodes; ++node) {
         // System Tree Node
         char namebuf[256];
@@ -950,18 +1060,26 @@ static void TauTraceOTF2WriteGlobalDefinitions() {
         snprintf(namebuf, 256, "group %d", node);
         int groupName = nextString++;
         OTF2_EC(OTF2_GlobalDefWriter_WriteString(global_def_writer, groupName, namebuf));
-        OTF2_EC(OTF2_GlobalDefWriter_WriteLocationGroup(global_def_writer, node, groupName, OTF2_LOCATION_GROUP_TYPE_PROCESS, node));
+        OTF2_EC(OTF2_GlobalDefWriter_WriteLocationGroup(global_def_writer, node, groupName, OTF2_LOCATION_GROUP_TYPE_PROCESS, node
+				#if OTF2_VERSION_MAJOR > 2
+				, OTF2_UNDEFINED_LOCATION_GROUP
+				#endif
+				));
 
-        const int start_loc = node * TAU_MAX_THREADS;
-        const int end_loc = start_loc + num_locations[node];
+        //const int start_loc = my_real_location(node,0);//node + num_locations[node];//max_threads;//TAU_MAX_THREADS; //TODO: DYNATHREAD
+        //const int end_loc = start_loc + num_locations[node];
         int thread_num = 0;
-        for(int loc = start_loc; loc < end_loc; ++loc) {
+	int cputhreads = 0;
+        for(int it_thread = 0; it_thread < num_locations[node]; ++it_thread) {
+            int loc=my_real_location(node,it_thread);
 		    OTF2_LocationType_enum thread_type = OTF2_LOCATION_TYPE_CPU_THREAD;
-            if(nodes < 2 && thread_num == 0) {
-                snprintf(namebuf, 256, "Master thread 0");
-            } else if(thread_num == 0) {
-                snprintf(namebuf, 256, "Rank");
-            } else if(Tau_is_thread_fake(thread_num)) {
+            //if(nodes < 2 && thread_num == 0) {
+              //  snprintf(namebuf, 256, "Master thread 0");
+            //} else 
+			//if(thread_num == 0) {
+            //    snprintf(namebuf, 256, "Rank %d", node);
+            //} else 
+			if(Tau_is_thread_fake(thread_num)) {
                 /* Check for CUPTI */
                 char * test = Tau_metadata_get("CUDA Device", thread_num);
                 if (test != NULL && strcmp(test, "") != 0) {
@@ -985,11 +1103,20 @@ static void TauTraceOTF2WriteGlobalDefinitions() {
                         snprintf(namebuf, 256, "GPU thread %02d", gputhreads++);
                     }
                 }
-				thread_type = OTF2_LOCATION_TYPE_GPU;
-            } else {
-                static int cputhreads = 1;
-                int nodeThread=cputhreads%nodes;
-                snprintf(namebuf, 256, "CPU thread %02d", nodeThread);
+				thread_type = 
+					#if OTF2_VERSION_MAJOR > 2
+					OTF2_LOCATION_TYPE_ACCELERATOR_STREAM;
+					#else
+					OTF2_LOCATION_TYPE_GPU;
+				        #endif
+            } 
+			else {
+                //static int cputhreads = 1;
+                int nodeThread=cputhreads; //%((nodes > 0) ? nodes : 1);
+                snprintf(namebuf, 256, "Rank %d, CPU Thread %02d", node, nodeThread);
+		#ifdef TAU_OTF2_DEBUG
+		printf("nodes: %d, cputhreads: %d, thread_num (used): %d, nodeThread: %d\n", nodes, cputhreads, thread_num, nodeThread);
+		#endif
 		cputhreads++;
             }
 #ifdef TAU_ENABLE_ROCM
@@ -1004,17 +1131,17 @@ static void TauTraceOTF2WriteGlobalDefinitions() {
 		  name = it->first.name;
                   value = it->second;
 		  if (strcmp(name, "ROCM_GPU_ID") == 0) {
-                    sprintf(gpu_id, "%s", value->data.cval);
+                    snprintf(gpu_id, sizeof(gpu_id),  "%s", value->data.cval);
                   }
 		  if (strcmp(name, "ROCM_QUEUE_ID") == 0) {
-                    sprintf(queue_id, "%s", value->data.cval);
+                    snprintf(queue_id, sizeof(queue_id),  "%s", value->data.cval);
                   }
 		  if (strcmp(name, "TAU_TASK_ID") == 0) {
-                    sprintf(tau_task_id, "%s", value->data.cval);
+                    snprintf(tau_task_id, sizeof(tau_task_id),  "%s", value->data.cval);
                   }
          	}
                 if (strlen(gpu_id) > 0) {
-                  sprintf(namebuf, "GPU%s Queue%s", gpu_id, queue_id);
+                  snprintf(namebuf, sizeof(namebuf),  "GPU%s Queue%s", gpu_id, queue_id);
                   TAU_VERBOSE("name = %s\n", namebuf);
                 }
 
@@ -1024,7 +1151,7 @@ static void TauTraceOTF2WriteGlobalDefinitions() {
             OTF2_EC(OTF2_GlobalDefWriter_WriteString(global_def_writer, locName, namebuf));
             OTF2_EC(OTF2_GlobalDefWriter_WriteLocation(global_def_writer, loc, locName, thread_type, num_events_written[node], node));
 #ifdef TAU_OTF2_DEBUG
-            fprintf(stderr, "%u: wrote loc %d \"%s\" num events=%d\n", my_node(), loc, namebuf, num_events_written[node]);
+            fprintf(stderr, "%u (loop node %d): wrote loc %d \"%s\" num events=%ld\n", my_node(), node, loc, namebuf, num_events_written[node]);
 #endif
         }
 
@@ -1093,18 +1220,27 @@ static void TauTraceOTF2WriteGlobalDefinitions() {
     uint64_t nodes_list[nodes];
     uint64_t ranks_list[nodes];
     for(int i = 0; i < nodes; ++i) {
-        nodes_list[i] = i * TAU_MAX_THREADS;
+        //int nodelocs = num_locations[nodes];
+        nodes_list[i] = my_real_location(i,0); //i * nodelocs;//max_threads;//TAU_MAX_THREADS;//TODO: DYNATHREAD
         ranks_list[i] = i;
     }
     OTF2_EC(OTF2_GlobalDefWriter_WriteGroup(global_def_writer, TAU_OTF2_GROUP_LOCS, locsGroupName, OTF2_GROUP_TYPE_COMM_LOCATIONS, OTF2_PARADIGM_MPI, OTF2_GROUP_FLAG_NONE, nodes, nodes_list));
     OTF2_EC(OTF2_GlobalDefWriter_WriteGroup(global_def_writer, TAU_OTF2_GROUP_WORLD, worldGroupName, OTF2_GROUP_TYPE_COMM_GROUP, OTF2_PARADIGM_MPI, OTF2_GROUP_FLAG_NONE, nodes, ranks_list));
-    OTF2_EC(OTF2_GlobalDefWriter_WriteComm(global_def_writer, TAU_OTF2_COMM_WORLD, commName, TAU_OTF2_GROUP_WORLD, OTF2_UNDEFINED_COMM));
+    OTF2_EC(OTF2_GlobalDefWriter_WriteComm(global_def_writer, TAU_OTF2_COMM_WORLD, commName, TAU_OTF2_GROUP_WORLD, OTF2_UNDEFINED_COMM
+			    #if OTF2_VERSION_MAJOR > 2
+			    , OTF2_COMM_FLAG_NONE
+			    #endif
+			    ));
 
 #if defined(TAU_SHMEM)
     // Write global RMA window
     const int commWinName = nextString++;
     OTF2_EC(OTF2_GlobalDefWriter_WriteString(global_def_writer, commWinName, "RMA_WIN_WORLD"));
-    OTF2_EC(OTF2_GlobalDefWriter_WriteRmaWin(global_def_writer, TAU_OTF2_COMM_WIN, commWinName, TAU_OTF2_COMM_WORLD));
+    OTF2_EC(OTF2_GlobalDefWriter_WriteRmaWin(global_def_writer, TAU_OTF2_COMM_WIN, commWinName, TAU_OTF2_COMM_WORLD
+			    #if OTF2_VERSION_MAJOR > 2
+			    , OTF2_RMA_WIN_FLAG_NONE
+			    #endif
+                ));
 
     OTF2_GroupRef next_group = TAU_OTF2_GROUP_FIRST_AVAILABLE;
     OTF2_CommRef next_comm = TAU_OTF2_COMM_FIRST_AVAILABLE;
@@ -1134,8 +1270,16 @@ static void TauTraceOTF2WriteGlobalDefinitions() {
         const OTF2_GroupRef rmaGroupID = next_group++;
         OTF2_EC(OTF2_GlobalDefWriter_WriteGroup(global_def_writer, rmaGroupID, rmaGroupName, OTF2_GROUP_TYPE_COMM_GROUP, OTF2_PARADIGM_MPI, OTF2_GROUP_FLAG_NONE, pe_size, rma_win_list));
         const OTF2_CommRef rmaCommID = next_comm++;
-        OTF2_EC(OTF2_GlobalDefWriter_WriteComm(global_def_writer, rmaCommID, rmaGroupName, rmaGroupID, OTF2_UNDEFINED_COMM));
-        OTF2_EC(OTF2_GlobalDefWriter_WriteRmaWin(global_def_writer, rma_win_id, rmaGroupName, rmaCommID));
+        OTF2_EC(OTF2_GlobalDefWriter_WriteComm(global_def_writer, rmaCommID, rmaGroupName, rmaGroupID, OTF2_UNDEFINED_COMM
+			    #if OTF2_VERSION_MAJOR > 2
+			    , OTF2_COMM_FLAG_NONE
+			    #endif
+			    ));
+        OTF2_EC(OTF2_GlobalDefWriter_WriteRmaWin(global_def_writer, rma_win_id, rmaGroupName, rmaCommID
+			    #if OTF2_VERSION_MAJOR > 2
+			    , OTF2_RMA_WIN_FLAG_NONE
+			    #endif
+                ));
     }
 #else
     OTF2_GroupRef next_group = TAU_OTF2_GROUP_FIRST_AVAILABLE;
@@ -1180,10 +1324,11 @@ static void TauTraceOTF2WriteLocalDefinitions() {
             const uint64_t global_id = global_region_map_ref.find(string(fi->GetName()))->second;
             OTF2_EC(OTF2_IdMap_AddIdPair(loc_region_map, local_id, global_id));
         }
-        const int start_loc = my_location();
+        const int start_loc = 0;//my_location();
         const int end_loc = start_loc + RtsLayer::getTotalThreads();
         for(int loc = start_loc; loc < end_loc; ++loc) {
-            OTF2_DefWriter* def_writer = OTF2_Archive_GetDefWriter(otf2_archive, loc);
+            int gloc = my_real_location(my_node(),loc);
+            OTF2_DefWriter* def_writer = OTF2_Archive_GetDefWriter(otf2_archive, gloc);
             OTF2_EC(OTF2_DefWriter_WriteMappingTable(def_writer, OTF2_MAPPING_REGION, loc_region_map));
         }
         OTF2_IdMap_Free(loc_region_map);
@@ -1210,10 +1355,11 @@ static void TauTraceOTF2WriteLocalDefinitions() {
             const uint64_t global_id = global_id_iter->second;
             OTF2_EC(OTF2_IdMap_AddIdPair(loc_metric_map, local_id, global_id));
         }
-        const int start_loc = my_location();
+        const int start_loc = 0;//my_location();
         const int end_loc = start_loc + RtsLayer::getTotalThreads();
         for(int loc = start_loc; loc < end_loc; ++loc) {
-            OTF2_DefWriter* def_writer = OTF2_Archive_GetDefWriter(otf2_archive, loc);
+            int gloc = my_real_location(my_node(),loc);
+            OTF2_DefWriter* def_writer = OTF2_Archive_GetDefWriter(otf2_archive, gloc);
             OTF2_EC(OTF2_DefWriter_WriteMappingTable(def_writer, OTF2_MAPPING_METRIC, loc_metric_map));
         }
         OTF2_IdMap_Free(loc_metric_map);
@@ -1221,11 +1367,12 @@ static void TauTraceOTF2WriteLocalDefinitions() {
 
     // Write RMA window mapping table
     if(rma_win_map.size() > 0) {
-        const int start_loc = my_location();
+        const int start_loc = 0;//my_location();
         const int end_loc = start_loc + RtsLayer::getTotalThreads();
         int loc_tid = 0;
         for(int loc = start_loc; loc < end_loc; ++loc) {
-            rma_win_map_t * local_map = local_rma_win_maps[loc_tid++];
+            int gloc = my_real_location(my_node(),loc);
+            rma_win_map_t * local_map = getLocalRMAWinMaps(loc_tid++);
             if(local_map == NULL) {
                 continue;
             }
@@ -1246,16 +1393,17 @@ static void TauTraceOTF2WriteLocalDefinitions() {
                 const uint64_t global_id = global_it->second;
                 OTF2_EC(OTF2_IdMap_AddIdPair(loc_rma_map, local_id, global_id));
             }
-            OTF2_DefWriter* def_writer = OTF2_Archive_GetDefWriter(otf2_archive, loc);
+            OTF2_DefWriter* def_writer = OTF2_Archive_GetDefWriter(otf2_archive, gloc);
             OTF2_EC(OTF2_DefWriter_WriteMappingTable(def_writer, OTF2_MAPPING_RMA_WIN, loc_rma_map));
         }
     }
 
     // Close local definition writers
-    const int start_loc = my_location();
+    const int start_loc = 0;//my_location();
     const int end_loc = start_loc + RtsLayer::getTotalThreads();
     for(int loc = start_loc; loc < end_loc; ++loc) {
-        OTF2_DefWriter* def_writer = OTF2_Archive_GetDefWriter(otf2_archive, loc);
+        int gloc = my_real_location(my_node(),loc);
+        OTF2_DefWriter* def_writer = OTF2_Archive_GetDefWriter(otf2_archive, gloc);
         OTF2_EC(OTF2_Archive_CloseDefWriter(otf2_archive, def_writer));
     }
 
@@ -1310,13 +1458,13 @@ static void TauTraceOTF2ExchangeEventsWritten() {
     }
     const uint32_t my_num_threads = RtsLayer::getTotalThreads();
     uint64_t my_num_events[my_num_threads];
-    const int offset = my_location_offset();
+    //const int offset = my_location_offset();
     for(OTF2_LocationRef i = 0; i < my_num_threads; ++i) {
-        OTF2_EvtWriter * evt_writer = OTF2_Archive_GetEvtWriter(otf2_archive, offset + i);
+        OTF2_EvtWriter * evt_writer = OTF2_Archive_GetEvtWriter(otf2_archive, my_real_location(my_node(),i));
         TAU_ASSERT(evt_writer != NULL, "Failed to get event writer");
         OTF2_EC(OTF2_EvtWriter_GetNumberOfEvents(evt_writer, my_num_events + i));
 #ifdef TAU_OTF2_DEBUG
-        fprintf(stderr, "%u: loc %d has written %u events.\n", my_node(), i, my_num_events[i]);
+        fprintf(stderr, "%u: loc %ld has written %lu events.\n", my_node(), i, my_num_events[i]);
 #endif
     }
 
@@ -1632,14 +1780,14 @@ static void TauTraceOTF2ExchangeRmaWins() {
     uint64_t my_num_rma_wins[my_num_threads];
     uint64_t my_total_rma_wins = 0;
     for(OTF2_LocationRef i = 0; i < my_num_threads; ++i) {
-        if(local_rma_win_maps[i] == NULL) {
+        if(getLocalRMAWinMaps(i) == NULL) {
             my_num_rma_wins[i] = 0;
         } else {
-            my_num_rma_wins[i] = local_rma_win_maps[i]->size();
+            my_num_rma_wins[i] = getLocalRMAWinMaps(i)->size();
         }
         my_total_rma_wins += my_num_rma_wins[i];
 #ifdef TAU_OTF2_DEBUG
-        fprintf(stderr, "%u: thread %d has %" PRIu64 " RMA windows.\n", my_node(), i, my_num_rma_wins[i]);
+        fprintf(stderr, "%u: thread %lu has %" PRIu64 " RMA windows.\n", my_node(), i, my_num_rma_wins[i]);
 #endif
     }
 
@@ -1681,7 +1829,7 @@ static void TauTraceOTF2ExchangeRmaWins() {
     int * my_rma_win_data = new int[my_total_rma_wins * 4];
     size_t data_offset = 0;
     for(OTF2_LocationRef i = 0; i < my_num_threads; ++i) {
-        rma_win_map_t * local_map = local_rma_win_maps[i];
+        rma_win_map_t * local_map = getLocalRMAWinMaps(i);
         if(local_map == NULL) {
             continue;
         }
@@ -1772,7 +1920,7 @@ static void TauTraceOTF2DestroyRmaWins() {
     for(uint32_t i = 0; i < my_num_threads; ++i) {
         int loc = my_real_location(RtsLayer::myNode(), i);
         OTF2_EvtWriter* evt_writer = OTF2_Archive_GetEvtWriter(otf2_archive, loc);
-        rma_win_map_t * local_map = local_rma_win_maps[i];
+        rma_win_map_t * local_map = getLocalRMAWinMaps(i);
         if(local_map != NULL) {
             for(rma_win_map_t::const_iterator it = local_map->begin(); it != local_map->end(); ++it) {
                 OTF2_EC(OTF2_EvtWriter_RmaWinDestroy(evt_writer, NULL, TauTraceGetTimeStamp(0), it->second));
@@ -1805,8 +1953,9 @@ void TauTraceOTF2ShutdownComms(int tid) {
   // initialization, and end after MPI/OTF2 finalization.  We want to capture them.
   // The progress threads in Open MPI 4 are such threads.
   if(TauEnv_get_set_node()==-1){
-    for (int tid = 0; tid < TAU_MAX_THREADS; tid++) {
-      if(!buffers_written[tid]) {
+    int threadCount=Tau_otf2_getThreadData().size();
+    for (int tid = 0; tid < threadCount; tid++) {//TODO: DYNATHREAD
+      if(!getBuffersWritten(tid)) {
         TauTraceOTF2WriteTempBuffer(tid, my_node());
       }
     }
@@ -1859,7 +2008,7 @@ void TauTraceOTF2Close(int tid) {
     if(tid != 0 || otf2_finished || !otf2_initialized) {
         return;
     }
-#if defined(CUPTI) || defined(DTAU_ENABLE_ROCTRACER)
+#if defined(TAU_CUPTI) || defined(TAU_ENABLE_ROCTRACER)
     TauTraceOTF2ToggleFlushAtExit(true);
     Tau_flush_gpu_activity();
     TauTraceOTF2ToggleFlushAtExit(false);
