@@ -93,10 +93,10 @@ public class CallPathTextWindow extends JFrame implements ActionListener, MenuLi
 
         //Now set the title.
         //if (windowType == 0) {
-        if (thread.getNodeID() == -1) {
+        if (thread.getNodeID() == -1 || thread.getNodeID() == -6) {
             this.setTitle("TAU: ParaProf: Mean Call Path Data - "
                     + ppTrial.getTrialIdentifier(ParaProf.preferences.getShowPathTitleInReverse()));
-        } else if (thread.getNodeID() == -3) {
+        } else if (thread.getNodeID() == -3 || thread.getNodeID() == -7) {
             this.setTitle("TAU: ParaProf: Standard Deviation Call Path Data - "
                     + ppTrial.getTrialIdentifier(ParaProf.preferences.getShowPathTitleInReverse()));
         } else {
@@ -450,7 +450,7 @@ public class CallPathTextWindow extends JFrame implements ActionListener, MenuLi
     public String getHeaderString() {
         return "Metric Name: " + (dataSorter.getSelectedMetric().getName()) + "\n" + "Sorted By: "
                 + dataSorter.getValueType() + "\n" + "Units: "
-                + UtilFncs.getUnitsString(units, dataSorter.isTimeMetric(), dataSorter.isDerivedMetric()) + "\n";
+                + UtilFncs.getUnitsString(units, dataSorter.isTimeMetric(), dataSorter.isDerivedMetric(),dataSorter.getSelectedMetric().getName()) + "\n";
     }
 
     public void closeThisWindow() {
