@@ -8,11 +8,9 @@
 
 package edu.uoregon.tau.paraprof;
 
-import java.awt.*;
+import java.awt.Color;
 
-import edu.uoregon.tau.perfdmf.*;
-
-public class ColorPair implements Comparable {
+public class ColorPair implements Comparable<ColorPair> {
 
     private String name = null;
     private Color color = null;
@@ -30,8 +28,8 @@ public class ColorPair implements Comparable {
         return color;
     }
 
-    public int compareTo(Object inObject) {
-        String tmpString = ((ColorPair) inObject).getName();
+    public int compareTo(ColorPair inObject) {
+        String tmpString = inObject.getName();
         return name.compareTo(tmpString);
     }
 }
