@@ -8,22 +8,41 @@
 
 package edu.uoregon.tau.paraprof;
 
-import java.awt.*;
-import java.awt.event.*;
-import javax.swing.*;
+import java.awt.Component;
+import java.awt.Container;
+import java.awt.Dimension;
+import java.awt.EventQueue;
+import java.awt.GridBagConstraints;
+import java.awt.GridBagLayout;
+import java.awt.Insets;
+import java.awt.Point;
+import java.awt.event.ActionEvent;
+import java.awt.event.ActionListener;
 
-import edu.uoregon.tau.perfdmf.*;
+import javax.swing.JButton;
+import javax.swing.JFrame;
+import javax.swing.JLabel;
+import javax.swing.JProgressBar;
 
-import java.util.*;
+import edu.uoregon.tau.perfdmf.DataSource;
+import edu.uoregon.tau.perfdmf.DatabaseAPI;
+import edu.uoregon.tau.perfdmf.DatabaseException;
 
 public class LoadTrialProgressWindow extends JFrame implements ActionListener {
 
-    private ParaProfManagerWindow paraProfManager = null;
+    /**
+	 * 
+	 */
+	private static final long serialVersionUID = 7442413865642013151L;
+	//private ParaProfManagerWindow paraProfManager = null;
     private JProgressBar progressBar = null;
+    private JLabel progressMessage = null;// advaced progress load feature : progress status message
     private DataSource dataSource = null;
     private ParaProfTrial ppTrial = null;
     private boolean aborted = false;
     private JLabel label;
+
+
     private boolean dbUpload = false;
     private javax.swing.Timer jTimer;
 
@@ -32,11 +51,11 @@ public class LoadTrialProgressWindow extends JFrame implements ActionListener {
 
         this.dataSource = dataSource;
         this.ppTrial = ppTrial;
-        this.paraProfManager = paraProfManager;
+        //this.paraProfManager = paraProfManager;
 
         //Window Stuff.
-        int windowWidth = 300;
-        int windowHeight = 120;
+        int windowWidth = 800;
+        int windowHeight = 220;
 
         //Grab paraProfManager position and size.
         Point parentPosition = paraProfManager.getLocationOnScreen();
@@ -68,18 +87,18 @@ public class LoadTrialProgressWindow extends JFrame implements ActionListener {
         GridBagLayout gbl = new GridBagLayout();
         contentPane.setLayout(gbl);
         GridBagConstraints gbc = new GridBagConstraints();
-        gbc.insets = new Insets(2, 2, 2, 2);
+        gbc.insets = new Insets(1, 1, 1, 1);
 
         gbc.fill = GridBagConstraints.NONE;
         gbc.anchor = GridBagConstraints.CENTER;
         gbc.weightx = 0;
         gbc.weighty = 0;
         label = new JLabel("Loading Profile Data...");
+	label.setPreferredSize(new Dimension(760, 20));
         addCompItem(label, gbc, 0, 0, 1, 1);
 
         progressBar = new JProgressBar(0, 100);
-
-        progressBar.setPreferredSize(new Dimension(200, 14));
+        progressBar.setPreferredSize(new Dimension(760, 14));
         gbc.fill = GridBagConstraints.HORIZONTAL;
 
         addCompItem(progressBar, gbc, 0, 1, 1, 1);
@@ -91,7 +110,16 @@ public class LoadTrialProgressWindow extends JFrame implements ActionListener {
 
         JButton jButton = new JButton("Cancel");
         jButton.addActionListener(this);
-        addCompItem(jButton, gbc, 0, 3, 1, 1);
+        addCompItem(jButton, gbc, 0, 2, 1, 1);
+
+	if (dataSource.getClass().getName().equalsIgnoreCase("edu.uoregon.tau.perfdmf.CubeDataSource"))
+	{
+		progressMessage = new JLabel("");
+		addCompItem(progressMessage, gbc, 0, 3, 1, 1);
+		progressMessage.setPreferredSize(new Dimension(760, 100));
+		progressMessage.setMinimumSize(new Dimension(760, 100));
+		progressMessage.setVerticalAlignment(JLabel.TOP);
+	}
 
         jTimer = new javax.swing.Timer(200, this);
         jTimer.start();
@@ -112,7 +140,7 @@ public class LoadTrialProgressWindow extends JFrame implements ActionListener {
         progressBar.setValue(0);
         setDBUpload(true);
 
-        DatabaseAPI dbAPI = ParaProf.paraProfManagerWindow.getDatabaseAPI();
+        DatabaseAPI dbAPI = ParaProf.paraProfManagerWindow.getDatabaseAPI(ppTrial.getDatabase());
         ppTrial.setDatabaseAPI(dbAPI);
         if (dbAPI != null) {
             // this call will block until the entire thing is uploaded (could be a while)
@@ -198,10 +226,15 @@ public class LoadTrialProgressWindow extends JFrame implements ActionListener {
             }
 
         } else {
-            int progress = (int) dataSource.getProgress();
-            if (progress > 100)
-                progress = 100;
-            progressBar.setValue(progress);
+		if (dataSource.getClass().getName().equalsIgnoreCase("edu.uoregon.tau.perfdmf.CubeDataSource"))
+			progressMessage.setText("<html>"+((edu.uoregon.tau.perfdmf.CubeDataSource)(dataSource)).getProgressMessage().replaceAll("\n","<br>").replaceAll(" ", "&nbsp;") );
+// 			System.out.println(((edu.uoregon.tau.perfdmf.Cube4DataSource)(dataSource)).getProgressMessage());
+		int progress = (int) dataSource.getProgress();
+		if (progress > 100)
+			progress = 100;
+		progressBar.setValue(progress);
+
+		doLayout();
         }
     }
 
