@@ -2,6 +2,7 @@ package edu.uoregon.tau.paraprof;
 
 import jargs.gnu.CmdLineParser;
 
+import java.awt.Font;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.io.File;
@@ -13,10 +14,15 @@ import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
 import java.text.DecimalFormat;
 import java.util.ArrayList;
+import java.util.Enumeration;
 import java.util.Iterator;
+import java.util.LinkedHashSet;
 import java.util.List;
+import java.util.Scanner;
+import java.util.Set;
 
 import javax.swing.ToolTipManager;
+import javax.swing.UIManager;
 
 import edu.uoregon.tau.common.TauScripter;
 import edu.uoregon.tau.paraprof.interfaces.EclipseHandler;
@@ -71,6 +77,8 @@ public class ParaProf implements ActionListener {
     private static int numWindowsOpen = 0;
 
     private static int fileType = DataSource.TAUPROFILE;
+    //If this is set to true then the user has specified snapshot format and we may have a series of snapshots from one trial.
+    private static boolean seriesSnap = false;
     private static File sourceFiles[] = new File[0];
     private static boolean fixNames = false;
     private static boolean monitorProfiles;
@@ -126,15 +134,18 @@ public class ParaProf implements ActionListener {
 		+ "  -f, --filetype <filetype>       Specify type of performance data, options are:\n"
 		+ "                                    profiles (default), pprof, dynaprof, mpip,\n"
 		+ "                                    gprof, psrun, hpm, packed, cube, hpc, ompp\n"
-		+ "                                    snap, perixml, gptl, ipm, google\n"
+		+ "                                    snap, perixml, gptl, ipm, google, darshan\n"
 		+ "  --range a-b:c                   Load only profiles from the given range(s) of processes\n"
 		+ "                                    Seperate individual ids or dash-defined ranges with colons\n"
 		+ "  -h, --help                      Display this help message\n" + "\n"
 		+ "The following options will run only from the console (no GUI will launch):\n" + "\n"
 		+ "  --merge <file.gz>               Merges snapshot profiles\n"
 		+ "  --pack <file>                   Pack the data into packed (.ppk) format\n"
+		+ "  --text <file>                   Dump the data into text (.csv) format\n"
 		+ "  --dump                          Dump profile data to TAU profile format\n"
 		+ "  --dumprank <rank>               Dump profile data for <rank> to TAU profile format\n"
+		+ "                                  Specify multiple ranks in with a comma separated list. Use\n"
+		+ "                                  a dash between values for a range. Do not use spaces. E.g. 0,4,8-10,16\n"
 		+ "  -v, --dumpsummary               Dump derived statistical data to TAU profile format\n"
 		+ "  --overwrite                     Allow overwriting of profiles\n"
 		+ "  -o, --oss                       Print profile data in OSS style text output\n"
@@ -143,6 +154,8 @@ public class ParaProf implements ActionListener {
 		+ "  -x, --suppressmetrics           Exclude child calls and exclusive time from --dumpmpisummary\n"		
 		+ "  -s, --summary                   Print only summary statistics\n"
 		+ "                                    (only applies to OSS output)\n" 
+		+ "  -y, --control                   Enter command line control mode for exploring/loading/exporting database trials\n"
+		+ "  --writecomm <file.csv>          Write communication matrix data (if any) to the specified csv file.\n"
 		+ "\n" + "Notes:\n"
 		+ "  For the TAU profiles type, you can specify either a specific set of profile\n"
 		+ "files on the commandline, or you can specify a directory (by default the current\n"
@@ -196,9 +209,18 @@ public class ParaProf implements ActionListener {
 			filelist.add(new File(file));
 		    }
 		}
-		paraProfManagerWindow.addTrial(app, experiment, (File[]) filelist.toArray(sourceFiles), fileType, fixNames, monitorProfiles, range);
-	    }else {
-		paraProfManagerWindow.addTrial(app, experiment, sourceFiles, fileType, fixNames, monitorProfiles, range);
+		paraProfManagerWindow.addTrial(experiment, (File[]) filelist.toArray(sourceFiles), fileType, fixNames, monitorProfiles, range);
+	    }
+	    //If it's not a series but it is a snapshot then open the files successively.
+	    else if(!seriesSnap&&fileType==DataSource.SNAP){
+	    	for (int i = 0; i < sourceFiles.length; i++) {
+			    File files[] = new File[1];
+			    files[0] = sourceFiles[i];
+			    paraProfManagerWindow.addTrial(app, experiment, files, fileType, fixNames, monitorProfiles);
+			}
+	    }
+	    else {
+		paraProfManagerWindow.addTrial(experiment, sourceFiles, fileType, fixNames, monitorProfiles, range);
 	    }
 	} catch (java.security.AccessControlException ace) {
 	    // running as Java Web Start without permission
@@ -233,7 +255,7 @@ public class ParaProf implements ActionListener {
 		    colorChooser = new ColorChooser(ParaProf.preferences);
 		} catch (Exception e) {
 		    if (e instanceof FileNotFoundException) {
-			//System.out.println("No preference file present, using defaults!");
+			System.out.println("No preference file present, using defaults!");
 		    } else {
 			System.out.println("Error while trying to read the ParaProf preferences file, using defaults");
 			//System.out.println("Please delete this file, or replace it with a valid one!");
@@ -265,6 +287,8 @@ public class ParaProf implements ActionListener {
 	    //            ParaProf.preferences.setDatabaseConfigurationFile(path);
 	    preferences = new Preferences();
 	}
+	
+	setUIFont (new javax.swing.plaf.FontUIResource(new Font(preferences.getFontName(),preferences.getFontStyle(), preferences.getFontSize())));
 
 	if (colorChooser == null) {
 	    // we create one if ParaProf.conf wasn't properly read
@@ -358,7 +382,8 @@ public class ParaProf implements ActionListener {
 	try {
 	    savePreferences(new File(ParaProf.paraProfHomeDirectory.getPath() + "/ParaProf.conf"));
 	} catch (Exception e) {
-	    // we'll get an exception here if running under Java Web Start
+	    System.err.println("An error occured while trying to save ParaProf preferences.");
+	    e.printStackTrace();
 	}
 	if (!insideEclipse && !controlMode) {
 	    // never call System.exit when invoked by the eclipse plugin, it will close the whole JVM, including the user's eclipse!
@@ -371,7 +396,7 @@ public class ParaProf implements ActionListener {
 
 	ParaProf.colorChooser.setSavedColors();
 	ParaProf.preferences.setAssignedColors(ParaProf.colorMap.getMap());
-	ParaProf.preferences.setManagerWindowPosition(ParaProf.paraProfManagerWindow.getLocation());
+	//ParaProf.preferences.setManagerWindowPosition(ParaProf.paraProfManagerWindow.getLocation());
 	ParaProf.preferences.setSourceLocations(getDirectoryManager().getCurrentElements());
 	try {
 	    ObjectOutputStream prefsOut = new ObjectOutputStream(new FileOutputStream(file));
@@ -396,6 +421,7 @@ public class ParaProf implements ActionListener {
 
 	CmdLineParser.Option mergeOpt = parser.addStringOption('a', "merge");
 	CmdLineParser.Option packOpt = parser.addStringOption('a', "pack");
+	CmdLineParser.Option textOpt = parser.addStringOption('t', "text");
 	CmdLineParser.Option schemaLocationOpt = parser.addStringOption('c', "schemadir");
 	CmdLineParser.Option metadumpOpt = parser.addBooleanOption('d', "metadump");
 	CmdLineParser.Option typeOpt = parser.addStringOption('f', "filetype");
@@ -415,6 +441,7 @@ public class ParaProf implements ActionListener {
 	CmdLineParser.Option suppressOpt = parser.addBooleanOption('x', "suppressmetrics");
 	CmdLineParser.Option controlOpt = parser.addBooleanOption('y', "control");
 	CmdLineParser.Option demoOpt = parser.addBooleanOption('z', "demo");
+	CmdLineParser.Option writeCommOpt = parser.addStringOption('a', "writecomm");
 	
 	
 
@@ -432,6 +459,8 @@ public class ParaProf implements ActionListener {
 	Boolean fixNames = (Boolean) parser.getOptionValue(fixOpt);
 	String merge = (String) parser.getOptionValue(mergeOpt);
 	String pack = (String) parser.getOptionValue(packOpt);
+	String text = (String) parser.getOptionValue(textOpt);
+	String writeComm = (String) parser.getOptionValue(writeCommOpt);
 	Boolean unpack = (Boolean) parser.getOptionValue(unpackOpt);
 	Boolean unpackSumm = (Boolean) parser.getOptionValue(unpackSummOpt);
 	
@@ -488,6 +517,25 @@ public class ParaProf implements ActionListener {
 //	    	break;
 //	    }
 	}
+	
+	if(sourceFiles!=null&&sourceFiles.length>0){
+		if(sourceFiles[0]!=null&&sourceFiles[0].exists()){
+			if(sourceFiles[0].isDirectory()){
+				try {
+					LoadTrialWindow.lastDirectory=sourceFiles[0].getCanonicalPath();
+				} catch (IOException e) {
+				}
+			}
+			else{
+				try {
+					File pf=sourceFiles[0].getParentFile();
+					if(pf!=null&&pf.exists())
+						LoadTrialWindow.lastDirectory=pf.getCanonicalPath();
+				} catch (IOException e) {
+				}
+			}
+		}
+	}
 
 	if (fixNames != null)
 	    ParaProf.fixNames = fixNames.booleanValue();
@@ -516,7 +564,8 @@ public class ParaProf implements ActionListener {
 	    } else if (fileTypeString.equals("snapshot")) {
 		ParaProf.fileType = DataSource.SNAP;
 	    } else if (fileTypeString.equals("snap")) {
-		ParaProf.fileType = DataSource.SNAP;
+	    	seriesSnap=true;
+	    	ParaProf.fileType = DataSource.SNAP;
 	    } else if (fileTypeString.equals("ompp")) {
 		ParaProf.fileType = DataSource.OMPP;
 	    } else if (fileTypeString.equals("perixml")) {
@@ -529,7 +578,10 @@ public class ParaProf implements ActionListener {
 		ParaProf.fileType = DataSource.IPM;
 	    } else if (fileTypeString.equals("google")) {
 		ParaProf.fileType = DataSource.GOOGLE;
-	    } else {
+	    } else if (fileTypeString.equals("darshan")){
+	    ParaProf.fileType = DataSource.DARSHAN;
+	    }
+	    else {
 		System.err.println("Please enter a valid file type.");
 		ParaProf.usage();
 		System.exit(-1);
@@ -590,11 +642,65 @@ public class ParaProf implements ActionListener {
 	if (pack != null) {
 	    try {
 
+	    File outPPK = new File(pack);
+	    if(outPPK.exists()){
+	    	System.out.println("The file "+pack+" already exists. Do you want to overwrite?");
+	    	Scanner scin = new Scanner(System.in);
+	    	
+	    	
+	    	boolean valid = false;
+	    	boolean write = false;
+	    	while(!valid){
+	    	System.out.print("[yes/no]: ");
+	    	String response = scin.nextLine();
+	    	response = response.toLowerCase().trim();
+	    	
+	    	if(response.equals("yes")||response.equals("y")){
+	    		write=true;
+	    		valid=true;
+	    	}
+	    	else if(response.equals("no")||response.equals("n")){
+	    		valid=true;
+	    	}
+	    	}
+	    	scin.close();
+	    	if(!write){
+	    		System.exit(0);
+	    	}
+	    }
 		DataSource dataSource = UtilFncs.initializeDataSource(sourceFiles, fileType, ParaProf.fixNames);
 		System.out.println("Loading data...");
 		dataSource.load();
 		System.out.println("Packing data...");
-		DataSourceExport.writePacked(dataSource, new File(pack));
+		DataSourceExport.writePacked(dataSource, outPPK);
+
+	    } catch (Exception e) {
+		e.printStackTrace();
+	    }
+	    System.exit(0);
+	}
+	
+	if(writeComm!=null) {
+		try {
+			DataSource dataSource = UtilFncs.initializeDataSource(sourceFiles, fileType, ParaProf.fixNames);
+			System.out.println("Loading data...");
+			dataSource.load();
+			System.out.println("Writing data...");
+			CommunicationMatrixWindow.writeCommCSV(dataSource, new FileOutputStream(new File(writeComm)),-1);
+		}catch (Exception e) {
+		e.printStackTrace();
+	    }
+		System.exit(0);
+	}
+
+	if (text != null) {
+	    try {
+
+		DataSource dataSource = UtilFncs.initializeDataSource(sourceFiles, fileType, ParaProf.fixNames);
+		System.out.println("Loading data...");
+		dataSource.load();
+		System.out.println("Writing data...");
+		DataSourceExport.writeDelimited(dataSource, new FileOutputStream(new File(text)));
 
 	    } catch (Exception e) {
 		e.printStackTrace();
@@ -616,13 +722,28 @@ public class ParaProf implements ActionListener {
 		}
 
 		DataSource dataSource = UtilFncs.initializeDataSource(sourceFiles, fileType, ParaProf.fixNames);
+		Set<Integer> ranks = null;
 		System.out.println("Loading data...");
-		dataSource.load();
+		if (unpackrank != null) {
+		    ranks= getRankList(unpackrank);// = Integer.parseInt(unpackrank);
+		   dataSource.setSelectedRank(ranks);
+		}
+		
+			dataSource.load();
+		
 		System.out.println("Creating TAU Profile data...");
 		if (unpackrank != null) {
-		    int rank = Integer.parseInt(unpackrank);
-		    Node node = dataSource.getNode(rank);
-		    DataSourceExport.writeProfiles(dataSource, new File("."), node.getThreads());
+		    //int rank = Integer.parseInt(unpackrank);
+			for(Integer i:ranks) {
+				 Node node = dataSource.getNode(i);
+				 if(node==null) {
+					 System.out.println("No profile for rank "+i);
+				 }
+				 else{
+					 DataSourceExport.writeProfiles(dataSource, new File("."), node.getThreads());
+				 }
+			}
+		   
 		}else if(unpackSumm !=null){
 			DataSourceExport.writeAggProfiles(dataSource, new File("."));
 		}else if(unpackMPISumm !=null||metadump!=null){
@@ -656,6 +777,8 @@ public class ParaProf implements ActionListener {
 	    ParaProf.loadScripts();
 	    ExternalController.runController();
 	} else {
+		
+		
 	    javax.swing.SwingUtilities.invokeLater(new Runnable() {
 		public void run() {
 		    try {
@@ -669,4 +792,50 @@ public class ParaProf implements ActionListener {
 	    });
 	}
     }
+    
+    private static Set<Integer> getRankList(String ranks) {
+    	
+    	ranks=ranks.trim();
+    	String[] rankstrings = ranks.split(",");
+    	Set<Integer> rankSet=new LinkedHashSet<Integer>();
+    	for(String s:rankstrings) {
+    		s=s.trim();
+    		if(s.contains("-")) {
+    			String[] range = s.split("-");
+    			if(range.length!=2) {
+    				System.out.println("Invalid range specification: "+s);
+    				return null;
+    			}
+    			int from = Integer.parseInt(range[0]);
+    			int to = Integer.parseInt(range[1]);
+    			if(from>to) {
+    				int tmp=from;
+    				from=to;
+    				to=tmp;
+    			}
+    			for(int i=from;i<=to;i++) {
+    				rankSet.add(i);
+    			}
+    		}else {
+    			rankSet.add(Integer.parseInt(s));
+    		}
+    			
+    	}
+    	
+    	return rankSet;
+    }
+    
+    private static void setUIFont(javax.swing.plaf.FontUIResource f)
+    {
+        Enumeration<Object> keys = UIManager.getDefaults().keys();
+        while (keys.hasMoreElements())
+        {
+            Object key = keys.nextElement();
+            Object value = UIManager.get(key);
+            if (value instanceof javax.swing.plaf.FontUIResource)
+            {
+                UIManager.put(key, f);
+            }
+        }
+    }
 }
