@@ -73,6 +73,7 @@ public class StatWindow extends JFrame implements ActionListener, MenuListener,
     private JCheckBoxMenuItem descendingOrder = null;
     //private JCheckBoxMenuItem showPathTitleInReverse = null;
     private JCheckBoxMenuItem showMetaData = null;
+	private JCheckBoxMenuItem showTotal = null;
     private JCheckBoxMenuItem showFindPanelBox;
 
     private JScrollPane jScrollpane = null;
@@ -107,11 +108,11 @@ public class StatWindow extends JFrame implements ActionListener, MenuListener,
 
         String title;
         //Now set the title.
-        if (nodeID == -1) {
+        if (nodeID == -1 || nodeID == -6) {
             title = "TAU: ParaProf: Mean Data Statistics: ";
         } else if (nodeID == -2) {
             title = "TAU: ParaProf: Total Statistics: ";
-        } else if (nodeID == -3) {
+        } else if (nodeID == -3 || nodeID == -7) {
             title = "TAU: ParaProf: Standard Deviation Statistics: ";
         } else {
             title = "TAU: ParaProf: "+ ParaProfUtils.getThreadLabel(thread);//n,c,t, " + nodeID + "," + contextID + "," + threadID;
@@ -166,6 +167,12 @@ public class StatWindow extends JFrame implements ActionListener, MenuListener,
         showMetaData.addActionListener(this);
         optionsMenu.add(showMetaData);
 
+		// if (userEventWindow) {
+		// showTotal = new JCheckBoxMenuItem("Show Total", true);
+		// showTotal.addActionListener(this);
+		// optionsMenu.add(showTotal);
+		// }
+
         optionsMenu.add(new JSeparator());
 
         descendingOrder = new JCheckBoxMenuItem("Descending Order", true);
@@ -405,6 +412,9 @@ public class StatWindow extends JFrame implements ActionListener, MenuListener,
                     panel.repaint();
                 } else if (arg.equals("Show Meta Data in Panel")) {
                     this.setHeader();
+				} else if (arg.equals("Show Total")) {
+					panel.setShowTotal(showTotal.isSelected());
+					panel.repaint();
                 }
             }
         } catch (Exception e) {
@@ -563,10 +573,10 @@ public class StatWindow extends JFrame implements ActionListener, MenuListener,
             if (phase != null) {
                 return "Phase: " + phase.getName() + "\nMetric: " + ppTrial.getDefaultMetric().getName() + "\n" + "Sorted By: "
                         + dataSorter.getValueType() + "\n" + "Units: "
-                        + UtilFncs.getUnitsString(units, ppTrial.isTimeMetric(), ppTrial.isDerivedMetric()) + "\n";
+                        + UtilFncs.getUnitsString(units, ppTrial.isTimeMetric(), ppTrial.isDerivedMetric(),ppTrial.getDefaultMetric().getName()) + "\n";
             } else {
                 return "Metric: " + ppTrial.getDefaultMetric().getName() + "\n" + "Sorted By: " + dataSorter.getValueType()
-                        + "\n" + "Units: " + UtilFncs.getUnitsString(units, ppTrial.isTimeMetric(), ppTrial.isDerivedMetric())
+                        + "\n" + "Units: " + UtilFncs.getUnitsString(units, ppTrial.isTimeMetric(), ppTrial.isDerivedMetric(),ppTrial.getDefaultMetric().getName())
                         + "\n";
             }
         }
