@@ -12,6 +12,8 @@ import java.awt.event.AdjustmentListener;
 import java.io.File;
 import java.util.ArrayList;
 import java.util.List;
+import java.util.Set;
+import java.util.TreeSet;
 import java.util.Vector;
 
 import javax.swing.BorderFactory;
@@ -39,6 +41,7 @@ import javax.swing.plaf.basic.BasicComboPopup;
 import javax.swing.plaf.basic.ComboPopup;
 import javax.swing.plaf.metal.MetalComboBoxUI;
 
+import edu.uoregon.tau.common.MetaDataMap.MetaDataKey;
 import edu.uoregon.tau.common.Utility;
 import edu.uoregon.tau.paraprof.enums.UserEventValueType;
 import edu.uoregon.tau.paraprof.enums.ValueType;
@@ -435,7 +438,6 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
 //        return panel;
 //    }
 //    
-    //TODO: Make this swap from interval to atomic
     private JPanel createTopoIntervalSelectionPanel(String name, final int dex) {
         JPanel panel = new JPanel();
         panel.setLayout(new GridBagLayout());
@@ -446,9 +448,11 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
         gbc.anchor = GridBagConstraints.WEST;
         gbc.weightx = 0.1;
         gbc.weighty = 0.1;
-
-        final JCheckBox atomic = new JCheckBox("Atomic");
-        atomic.setToolTipText("Check for Atomic events, uncheck for interval events");
+        
+        String[] profDataTypes={"Timer","Atomic","Metadata"};
+        
+        final JComboBox atomic = new JComboBox(profDataTypes);
+        atomic.setToolTipText("Select type of values to use for this axis");
         //addCompItem(panel, new JLabel(name), gbc, 0, 0, 1, 2);
 
         final JTextField functionField;
@@ -491,22 +495,27 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
             public void actionPerformed(ActionEvent evt) {
             	
                 try {
-                	
-                	if(valueBox.getSelectedItem()==null)
+                	Object si=valueBox.getSelectedItem();
+                	if(si==null)
                 	{
                 		return;
                 	}
                 	
-                	if(!atomic.isSelected()){
+                	if(atomic.getSelectedIndex()==0&&si instanceof ValueType){//!atomic.isSelected()){
 
                     settings.setTopoValueType((ValueType) valueBox.getSelectedItem(),dex);
                     settings.setTopoMetric((Metric) metricBox.getSelectedItem(),dex);
                     settings.intervalETDex[dex]=valueBox.getSelectedIndex();
                 	
                 	}
-                	else{
+                	else if(atomic.getSelectedIndex()==1 && si instanceof UserEventValueType){
                 		 settings.setTopoUserEventValueType((UserEventValueType) valueBox.getSelectedItem(), dex);
                 		 settings.atomicETDex[dex]=valueBox.getSelectedIndex();
+                	}
+                	else if(atomic.getSelectedIndex()==2){
+                		//settings.setTopoMetadata(valueBox.getSelectedItem(),dex);
+                		//settings.metadataETDex[dex]=valueBox.getSelectedIndex();
+                		//TODO: Make sure we don't need to set anything else here.
                 	}
                 	 window.redraw();
                      minTopoField.setText(window.getSelectedMinTopoValue());
@@ -527,15 +536,16 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
         		
         		String fname=null;
         		
-        		if(atomic.isSelected()){
+        		if(atomic.getSelectedIndex()==1){
         			
         			int numUE = ppTrial.getDataSource().getNumUserEvents();
         			if(numUE<1){
-        				atomic.setSelected(false);
+        				atomic.setSelectedIndex(0);//.setSelected(false);
         				return;
         			}
         			
         			metricBox.setEnabled(false);
+        			valueBox.setEditable(true);
         			valueBox.removeAllItems();
         			for(int i=0;i<UserEventValueType.VALUES.length;i++){
         				valueBox.addItem(UserEventValueType.VALUES[i]);
@@ -543,13 +553,14 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
         			valueBox.setSelectedIndex(settings.atomicETDex[dex]);
         			if(settings.getTopoAtomic(dex)==null){
         				
-        				UserEvent tmpUE = ppTrial.getDataSource().getUserEvents().next();
+        				UserEvent tmpUE = ppTrial.getDataSource().getUserEventIterator().next();
         				settings.setTopoAtomic(tmpUE, dex);
         			}
         			fname=settings.getTopoAtomic(dex).getName();
         		}
-        		else{
+        		else if(atomic.getSelectedIndex()==0){
         			metricBox.setEnabled(true);
+        			valueBox.setEditable(true);
         			valueBox.removeAllItems();
         			for(int i=0;i<ValueType.VALUES.length;i++){
         				valueBox.addItem(ValueType.VALUES[i]);
@@ -557,9 +568,30 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
         			valueBox.setSelectedIndex(settings.intervalETDex[dex]);
         			fname = ParaProfUtils.getDisplayName(settings.getTopoFunction(dex));
         		}
+        		else if(atomic.getSelectedIndex()==2){
+        			metricBox.setEnabled(false);
+        			valueBox.removeAllItems();
+        			valueBox.setEditable(false);
+        			MetaDataKey mdk=settings.getTopoMetadata(dex);
+        			
+        			if(mdk==null){
+        				Thread t = ppTrial.getDataSource().getThread(0, 0, 0);
+                		if(t==null){
+                			t=ppTrial.getDataSource().getThreads().get(0);
+                		}
+                		mdk=t.getMetaData().keySet().iterator().next();
+                		
+                		settings.setTopoMetadata(mdk, dex);
+        			}
+        			
+        			fname=mdk.toString();
+        			//valueBox.setSelectedIndex(settings.meticETDex[dex]);
+        			//fname = settings.getTopoMetric(dex);
+        			//TODO: Metadata support
+        		}
         		
         		
-        		settings.setAtomic(atomic.isSelected(), dex);
+        		settings.setDataType(atomic.getSelectedIndex(), dex);
         		
         		functionField.setText(fname);
                 functionField.setToolTipText(fname);
@@ -582,7 +614,7 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
                 	if(fname==null||fname.length()==0) 
                 		fname = "   <none>";
                 	
-                	if(!atomic.isSelected()){
+                	if(atomic.getSelectedIndex()==0){
                 	
                     FunctionSelectorDialog fSelector = new FunctionSelectorDialog(window, true,
                             ppTrial.getDisplayedFunctions().iterator(), settings.getTopoFunction(dex), true, false);
@@ -600,9 +632,9 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
                     }
                     
                 	}
-                	else{
+                	else if(atomic.getSelectedIndex()==1){
                         FunctionSelectorDialog fSelector = new FunctionSelectorDialog(window, true,
-                        		ppTrial.getDataSource().getUserEvents(), settings.getTopoAtomic(dex), true, false);
+                        		ppTrial.getDataSource().getUserEventIterator(), settings.getTopoAtomic(dex), true, false);
 
                         if (fSelector.choose()) {
                             UserEvent selectedFunction = (UserEvent) fSelector.getSelectedObject();
@@ -616,7 +648,32 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
                             
                         }
                 	}
-                	
+                	else if(atomic.getSelectedIndex()==2){
+                		Thread t = ppTrial.getDataSource().getThread(0, 0, 0);
+                		if(t==null){
+                			t=ppTrial.getDataSource().getThreads().get(0);
+                		}
+                		
+                		Set<MetaDataKey> mdkSet= new TreeSet<MetaDataKey>();
+                		mdkSet.addAll(t.getMetaData().keySet());
+                		mdkSet.addAll(ppTrial.getDataSource().getMetaData().keySet());
+                		         		
+                		//t.getMetaData().keySet().iterator();
+                		FunctionSelectorDialog fSelector = new FunctionSelectorDialog(window, true,
+                				mdkSet.iterator(), settings.getTopoMetadata(dex), true, false);
+                		
+                		if (fSelector.choose()) {
+                            MetaDataKey metadataKey = (MetaDataKey) fSelector.getSelectedObject();
+                            settings.setTopoMetadata(metadataKey,dex);
+
+                            //String fname = "   <none>";
+                            if (settings.getTopoMetadata(dex) != null) {
+                                fname = settings.getTopoMetadata(dex).toString();
+                            }
+
+                		}
+                		//TODO: metadata support
+                	}
                 	functionField.setText(fname);
                     functionField.setToolTipText(fname);
                 	window.redraw();
@@ -683,7 +740,7 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
     }
     
     private boolean checkDisableTopoWidgets(String topo){
-    	boolean b = (topo.equals("Custom")||topo.equals("BGQ")||topo.equals("Map")||topo.startsWith("Topo"));
+    	boolean b = (topo.equals(CUSTOM)||topo.equals("BGQ")||topo.equals(MAP)||topo.startsWith("Topo"));
     	return !b;
     	
     }
@@ -709,6 +766,7 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
     }
     
     private static final String CUSTOM="Custom";
+    private static final String MAP="Map";
     private JPanel createTopoSelectionPanel(String name) {
         JPanel panel = new JPanel();
         panel.setLayout(new GridBagLayout());
@@ -733,7 +791,7 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
         //String[] a = new String[topos.size()];
         topos.add(CUSTOM);
         customTopoDex=topos.size()-1;
-        topos.add("Map");
+        topos.add(MAP);
 		//return topos;//topos.toArray(a);
         
         topoComboBox = new SteppedComboBox(topos);
@@ -781,10 +839,13 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
                     window.redraw();
                     resetTopoAxisSliders(true);
                     
-                    for(int i=0;i<customAxisSliders.length;i++)
+                    //Loop through axis spinners (3 of each) and activate or deactivate as needed.
+                    for(int i=0;i<3;i++)
                     {
-                    	if(customAxisSliders[i]!=null)
-                    		customAxisSliders[i].setEnabled(settings.getTopoCart().equals("Custom"));
+                    	if(customAxisSpinners[i]!=null)
+                    		customAxisSpinners[i].setEnabled(settings.getTopoCart().equals(CUSTOM));
+//                    	if(customCoreAxisSpinners[i]!=null)
+//                    		customCoreAxisSpinners[i].setEnabled(settings.getTopoCart().equals(MAP));
                     }
                 } catch (Exception e) {
                     ParaProfUtils.handleException(e);
@@ -824,7 +885,7 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
         topoFileButton.addActionListener(topoFileSelector);
 
         
-JButton mapFileButton = new JButton("map");
+        JButton mapFileButton = new JButton(MAP);
         
         ActionListener mapFileSelector = new ActionListener() {
             public void actionPerformed(ActionEvent evt) {
@@ -837,6 +898,7 @@ JButton mapFileButton = new JButton("map");
 //                		}
 //                	}
                 	//tsDialog.setVisible(true);
+                	tsDialog.setCurrentDirectory(new File(System.getProperty("user.dir")));
                     tsDialog.showOpenDialog(window);
                     File dFile = tsDialog.getSelectedFile();
                     if(dFile!=null&&dFile.exists()&&dFile.canRead()){
@@ -879,13 +941,23 @@ JButton mapFileButton = new JButton("map");
         gbc.weighty = 0;
         gbc.weightx = 0;
         addCompItem(panel, customAxisLabels [0]=new JLabel("X Axis"), gbc, 0, 3, 1, 1);
-        addCompItem(panel, createTopoCustomSliderPanel(0), gbc, 1, 3, 1, 1);
+        addCompItem(panel, createTopoCustomAxisSpinnerPanel(0, this.customAxisSpinners), gbc, 1, 3, 1, 1);
         gbc.weightx = 0;
         addCompItem(panel, customAxisLabels [1]=new JLabel("Y Axis"), gbc, 0, 4, 1, 1);
-        addCompItem(panel, createTopoCustomSliderPanel(1), gbc, 1, 4, 1, 1);
+        addCompItem(panel, createTopoCustomAxisSpinnerPanel(1, this.customAxisSpinners), gbc, 1, 4, 1, 1);
         gbc.weightx = 0;
         addCompItem(panel, customAxisLabels [2]=new JLabel("Z Axis"), gbc, 0, 5, 1, 1);
-        addCompItem(panel, createTopoCustomSliderPanel(2), gbc, 1, 5, 1, 1);
+        addCompItem(panel, createTopoCustomAxisSpinnerPanel(2, this.customAxisSpinners), gbc, 1, 5, 1, 1);
+        
+        
+//        addCompItem(panel, customCoreAxisLabels [0]=new JLabel("Core X Axis"), gbc, 0, 6, 1, 1);
+//        addCompItem(panel, createTopoCustomCoreAxisSpinnerPanel(0, this.customCoreAxisSpinners), gbc, 1, 6, 1, 1);
+//        gbc.weightx = 0;
+//        addCompItem(panel, customCoreAxisLabels [1]=new JLabel("Core Y Axis"), gbc, 0, 7, 1, 1);
+//        addCompItem(panel, createTopoCustomCoreAxisSpinnerPanel(1, this.customCoreAxisSpinners), gbc, 1, 7, 1, 1);
+//        gbc.weightx = 0;
+//        addCompItem(panel, customCoreAxisLabels [2]=new JLabel("Core Z Axis"), gbc, 0, 8, 1, 1);
+//        addCompItem(panel, createTopoCustomCoreAxisSpinnerPanel(2, this.customCoreAxisSpinners), gbc, 1, 8, 1, 1);
 
         return panel;
     }
@@ -1039,7 +1111,11 @@ JButton mapFileButton = new JButton("map");
     }
     
     JLabel[] customAxisLabels = new JLabel[3];
-    JSpinner[] customAxisSliders = new JSpinner[3]; 
+    JSpinner[] customAxisSpinners = new JSpinner[3]; 
+    
+    //For setting the dimensions of the core subsets in a mapped topology. Probably not needed.
+//    JLabel[] customCoreAxisLabels = new JLabel[3];
+//    JSpinner[] customCoreAxisSpinners = new JSpinner[3];
     
     JLabel[] selectAxisLabels = new JLabel[3];
     JSlider[] selectAxisSliders = new JSlider[3];
@@ -1100,12 +1176,12 @@ JButton mapFileButton = new JButton("map");
         gbc.weighty = 1.0;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         addCompItem(panel, selectAxisSliders[dex], gbc, 0, 0, 1, 1);
-
+//TODO: Fix these: Proper activation and length
         return panel;
     }
     
     
-    private JPanel createTopoCustomSliderPanel(int dex){
+    private JPanel createTopoCustomAxisSpinnerPanel(int dex, final JSpinner[] spinners){
         JPanel panel = new JPanel();
         panel.setLayout(new GridBagLayout());
         GridBagConstraints gbc = new GridBagConstraints();
@@ -1115,29 +1191,31 @@ JButton mapFileButton = new JButton("map");
         gbc.weightx = 0.1;
         gbc.weighty = 0.1;
         
-        customAxisSliders[dex] = new JSpinner();
+        spinners[dex] = new JSpinner();
         SpinnerModel model = new SpinnerNumberModel(20, //initial value
                                    0, //min
                                    1000, //max
                                    1);                //step
-        customAxisSliders[dex].setModel(model);
-        customAxisSliders[dex].setEnabled(((String)topoComboBox.getSelectedItem()).equals("Custom"));
+        spinners[dex].setModel(model);
+        spinners[dex].setEnabled(((String)topoComboBox.getSelectedItem()).equals(CUSTOM));
 
         final int idex = dex;
         int v = settings.getCustomTopoAxis(dex);
         if(v>0)
-        	customAxisSliders[dex].setValue(settings.getCustomTopoAxis(dex));
+        	spinners[dex].setValue(settings.getCustomTopoAxis(dex));
         else{
-        	settings.setCustomTopoAxis(50, dex);
+        	settings.setCustomTopoAxis(20, dex);
         }
         ChangeListener topoSelector = new ChangeListener() {
 			public void stateChanged(ChangeEvent e) {
 
 				 try {
 					 //if(!firstSet){
-						 int val = (Integer) customAxisSliders[idex].getModel().getValue();
+						 int val = (Integer) spinners[idex].getModel().getValue();
 						 settings.setCustomTopoAxis(val,idex);
+						 resetTopoAxisSliders(true);
 	                    window.redraw();
+						 resetTopoAxisSliders(true);
 //	                    if(val==-1){
 //	                    	selectAxisLabels[idex].setText(topoLabelStrings[idex]);
 //	                    }else selectAxisLabels[idex].setText(topoLabelStrings[idex]+": "+val);
@@ -1158,18 +1236,78 @@ JButton mapFileButton = new JButton("map");
 			}
         };
         
-        customAxisSliders[dex].addChangeListener(topoSelector);
+        spinners[dex].addChangeListener(topoSelector);
 
         gbc.insets = new Insets(1, 1, 1, 1);
 
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.fill = GridBagConstraints.HORIZONTAL;
-        addCompItem(panel, customAxisSliders[dex], gbc, 0, 0, 1, 1);
+        addCompItem(panel, spinners[dex], gbc, 0, 0, 1, 1);
 
         return panel;
     }
     
+    
+    /**
+     * Creates custom core axis spinner panel. Probably not needed.
+     * @return
+     */
+//    private JPanel createTopoCustomCoreAxisSpinnerPanel(int dex, JSpinner[] spinners){
+//        JPanel panel = new JPanel();
+//        panel.setLayout(new GridBagLayout());
+//        GridBagConstraints gbc = new GridBagConstraints();
+//
+//        gbc.fill = GridBagConstraints.NONE;
+//        gbc.anchor = GridBagConstraints.WEST;
+//        gbc.weightx = 0.1;
+//        gbc.weighty = 0.1;
+//        
+//        spinners[dex] = new JSpinner();
+//        SpinnerModel model = new SpinnerNumberModel(ThreeDeeSettings.customTopoCoreAxesDefaults[dex], //initial value
+//                                   0, //min
+//                                   1000, //max
+//                                   1);                //step
+//        spinners[dex].setModel(model);
+//        spinners[dex].setEnabled(((String)topoComboBox.getSelectedItem()).equals(MAP));
+//
+//        final int idex = dex;
+//        int v = settings.getCustomTopoCoreAxis(dex);
+//        if(v>0)
+//        	spinners[dex].setValue(settings.getCustomTopoCoreAxis(dex));
+//        else{
+//        	settings.setCustomTopoAxis(ThreeDeeSettings.customTopoCoreAxesDefaults[dex], dex);
+//        }
+//        ChangeListener topoSelector = new ChangeListener() {
+//			public void stateChanged(ChangeEvent e) {
+//
+//				 try {
+//					 
+//						 int val = (Integer) spinners[idex].getModel().getValue();
+//						 settings.setCustomTopoCoreAxis(val,idex);
+//						 resetTopoAxisSliders(true);
+//	                    window.redraw();
+//						 resetTopoAxisSliders(true);
+//	                } catch (Exception evt) {
+//	                    ParaProfUtils.handleException(evt);
+//	                }
+//			}
+//        };
+//        
+//        spinners[dex].addChangeListener(topoSelector);
+//
+//        gbc.insets = new Insets(1, 1, 1, 1);
+//
+//        gbc.weightx = 1.0;
+//        gbc.weighty = 1.0;
+//        gbc.fill = GridBagConstraints.HORIZONTAL;
+//        addCompItem(panel, spinners[dex], gbc, 0, 0, 1, 1);
+//
+//        return panel;
+//    }
+    
+    
+    
     private JPanel createCallGraphPanel() {
         JPanel panel = new JPanel();
         panel.setLayout(new GridBagLayout());
@@ -1487,7 +1625,7 @@ JButton mapFileButton = new JButton("map");
         	{
         		firstSet=true;
         		this.selectAxisSliders[i].setMaximum(window.tsizes[i]);
-        		if(window.tsizes[i]<=1){
+        		if(window.tsizes[i]<1){
         			selectAxisSliders[i].setEnabled(false);
         		}else
         			selectAxisSliders[i].setEnabled(true);
@@ -1710,7 +1848,14 @@ JButton mapFileButton = new JButton("map");
         addCompItem(selectionPanel, functionSelectorPanel, gbc, 1, 0, 1, 1);
         addCompItem(selectionPanel, nodeSelectorPanel, gbc, 1, 1, 1, 1);
         addCompItem(selectionPanel, heightValueField, gbc, 1, 2, 1, 1);
+        
+//        GridBagConstraints gbcCF=(GridBagConstraints) gbc.clone();
+//        
+//        gbcCF.fill = GridBagConstraints.NONE;
+//        gbcCF.weightx = 0.0;
         addCompItem(selectionPanel, colorValueField, gbc, 1, 3, 1, 1);
+        //gbc.fill = GridBagConstraints.BOTH;
+        //gbc.weightx = 1.0;
 
         addCompItem(regularPanel, selectionPanel, gbc, 0, 4, 2, 1);
 
@@ -1781,8 +1926,8 @@ JButton mapFileButton = new JButton("map");
         }else topoValLabel.setText(ACV);
         }
     }
-    private static final String CV = "Color Value: ";
-    private static final String ACV ="Avg Color Value: ";
+    private static final String CV = "Color Value (Event 3): ";
+    private static final String ACV ="Avg Color Value (Event 3): ";
     private boolean allAxesOn(){
     	int numdis=0;
     	int numSet=0;
