@@ -233,12 +233,14 @@ public class GlobalBarChartModel extends AbstractBarChartModel {
             //Return the name of the function
             name = getDisplayName(fp);
         }
+        
+        String htmlName=Common.HTMLEntityEncodeLimit(name,100);
 
         String metricName = dataSorter.getSelectedMetric().getName();
 
         Metric metric = dataSorter.getSelectedMetric();
 
-        String unitsString = UtilFncs.getUnitsString(window.units(), dataSorter.isTimeMetric(), false);
+        String unitsString = UtilFncs.getUnitsString(window.units(), dataSorter.isTimeMetric(), false,dataSorter.getSelectedMetric().getName());
 
         String exclusiveValue = UtilFncs.getOutputString(window.units(), fp.getExclusive(metric.getID()),
                 ParaProf.defaultNumberPrecision, metric.isTimeDenominator());
@@ -250,7 +252,7 @@ public class GlobalBarChartModel extends AbstractBarChartModel {
         String calls = "<br>Calls: " + fp.getNumCalls();
         String subr = "<br>SubCalls: " + fp.getNumSubr();
 
-        return "<html>" + Common.HTMLEntityEncode(name) + exclusive + inclusive + calls + subr + "</html>";
+        return "<html>" + htmlName + exclusive + inclusive + calls + subr + "</html>";
 
     }
 
@@ -262,12 +264,12 @@ public class GlobalBarChartModel extends AbstractBarChartModel {
 
         if (ParaProf.getHelpWindow().isShowing()) {
             ParaProf.getHelpWindow().clearText();
-            if (thread.getNodeID() == -1) {
+            if (thread.getNodeID() == -1 || thread.getNodeID() == -6) {
                 ParaProf.getHelpWindow().writeText("This line represents the mean statistics (over all threads).\n");
 
             } else if (thread.getNodeID() == -2) {
 
-            } else if (thread.getNodeID() == -3) {
+            } else if (thread.getNodeID() == -3 || thread.getNodeID() == -7) {
                 ParaProf.getHelpWindow().writeText(
                         "This line represents the standard deviation of each function (over threads).\n");
 
