@@ -5,9 +5,9 @@ package edu.uoregon.tau.paraprof.enums;
  *    
  * TODO : nothing, this class is complete
  *
- * <P>CVS $Id: VisType.java,v 1.1 2005/09/26 21:12:46 amorris Exp $</P>
+ * <P>CVS $Id: VisType.java,v 1.2 2007/12/07 02:05:21 amorris Exp $</P>
  * @author  Alan Morris
- * @version $Revision: 1.1 $
+ * @version $Revision: 1.2 $
  */
 public class VisType {
 
@@ -18,6 +18,8 @@ public class VisType {
     public static final VisType TRIANGLE_MESH_PLOT = new VisType("Triangle Mesh");
     public static final VisType BAR_PLOT = new VisType("Bar Plot");
     public static final VisType SCATTER_PLOT = new VisType("Scatter Plot");
+    public static final VisType CALLGRAPH = new VisType("Callgraph");
+    public static final VisType TOPO_PLOT = new VisType("Topology Plot");
     //public static final VisType KIVIAT_TUBE = new VisType("Kiviat Tube");
     
     
