@@ -174,7 +174,10 @@ public class TreeTableNode extends DefaultMutableTreeNode implements Comparable<
                     } else {
                         node = new TreeTableNode((FunctionProfile) obj, model, null);
                     }
-                    children.add(node);
+                    /*
+                     * SOM This is causing problem with view CUBE callpaths. 
+                     */
+                   // children.add(node);
                 }
             }
         }
@@ -262,7 +265,19 @@ public class TreeTableNode extends DefaultMutableTreeNode implements Comparable<
             Comparable<Comparable> a = (Comparable<Comparable>) column.getValueFor(this, true);
             Comparable b = (Comparable) column.getValueFor((TreeTableNode) o, true);
             try {
-                result = a.compareTo(b);
+            	if(a==null&&b==null){
+            		result=0;
+            	}
+            	else if(a==null){
+            		result = -1;
+            	}
+            	else if(b==null){
+            		result = 1;
+            	}
+            	else
+                {
+            		result = a.compareTo(b);
+                }
 
             } catch (Exception e) {
                 e.printStackTrace();
