@@ -61,7 +61,7 @@ public class HPCToolkitDataSource extends DataSource {
             // now subract out children inclusive values from parent exclusive values
             for (int i = 0; i < this.getNumberOfMetrics(); i++) {
                 Thread thread = this.getThread(0, 0, 0);
-                for (Iterator<Function> it = this.getFunctions(); it.hasNext();) {
+                for (Iterator<Function> it = this.getFunctionIterator(); it.hasNext();) {
                     Function function = it.next();
 
                     FunctionProfile fp = thread.getFunctionProfile(function);
@@ -83,7 +83,7 @@ public class HPCToolkitDataSource extends DataSource {
 
             // Set flat profile data (D is combined from A=>D, B=>D, etc)
             Thread thread = this.getThread(0, 0, 0);
-            for (Iterator<Function> it = this.getFunctions(); it.hasNext();) {
+            for (Iterator<Function> it = this.getFunctionIterator(); it.hasNext();) {
                 Function function = it.next();
                 FunctionProfile fp = thread.getFunctionProfile(function);
                 if (fp != null) {
