@@ -6,8 +6,10 @@ import java.sql.ResultSet;
 import java.sql.ResultSetMetaData;
 import java.sql.SQLException;
 import java.util.ArrayList;
+import java.util.HashMap;
 import java.util.Hashtable;
 import java.util.List;
+import java.util.Map;
 import java.util.Vector;
 
 import edu.uoregon.tau.perfdmf.database.DB;
@@ -41,168 +43,14 @@ import edu.uoregon.tau.perfdmf.database.DB;
  * @see		IntervalLocationProfile
  */
 public class IntervalEvent {
-    private int intervalEventID;
-    private String name;
-    private String group;
-    private int trialID;
-    private IntervalLocationProfile meanSummary = null;
-    private IntervalLocationProfile totalSummary = null;
-    private DatabaseAPI dataSession = null;
-
-    public IntervalEvent(DatabaseAPI dataSession) {
-        this.dataSession = dataSession;
-    }
-
-    /**
-     * Gets the unique identifier of this intervalEvent object.
-     *
-     * @return	the unique identifier of the intervalEvent
-     */
-    public int getID() {
-        return this.intervalEventID;
-    }
-
-    /**
-     * Gets the intervalEvent name.
-     *
-     * @return	the name of the intervalEvent
-     */
-    public String getName() {
-        return this.name;
-    }
-
-    /**
-     * Gets the TAU group of this intervalEvent object.
-     *
-     * @return	the TAU group the intervalEvent is in.
-     */
-    public String getGroup() {
-        return this.group;
-    }
-
-    /**
-     * Gets the trial ID of this intervalEvent object.
-     *
-     * @return	the trial ID for the intervalEvent.
-     */
-    public int getTrialID() {
-        return this.trialID;
-    }
-
-    /**
-     * Gets mean summary data for the intervalEvent object.
-     * The Trial object associated with this intervalEvent has a vector of metric
-     * names that represent the metric data stored for each intervalEvent in the
-     * application.  Multiple metrics can be recorded for each trial.  If the
-     * user has selected a metric before calling getIntervalEvents(), then this method
-     * can be used to return the mean metric data for this 
-     * intervalEvent/trial/experiment/application combination.
-     * The mean data is averaged across all locations, defined as any combination
-     * of node/context/thread.
-     *
-     * @return	the IntervalLocationProfile containing the mean data for this intervalEvent/metric combination.
-     * @see		Trial
-     * @see		IntervalLocationProfile
-     * @see		DataSession#getIntervalEvents
-     * @see		DataSession#setMetric(Metric)
-     */
-    public IntervalLocationProfile getMeanSummary() throws SQLException {
-        if (this.meanSummary == null)
-            dataSession.getIntervalEventDetail(this);
-        return (this.meanSummary);
-    }
-
-    /**
-     * Gets total summary data for the intervalEvent object.
-     * The Trial object associated with this intervalEvent has a vector of metric
-     * names that represent the metric data stored for each intervalEvent in the
-     * application.  Multiple metrics can be recorded for each trial.  If the
-     * user has selected a metric before calling getIntervalEvents(), then this method
-     * can be used to return the total metric data for this 
-     * intervalEvent/trial/experiment/application combination.
-     * The total data is summed across all locations, defined as any combination
-     * of node/context/thread.
-     *
-     * @return	the IntervalLocationProfile containing the total data for this intervalEvent/metric combination.
-     * @see		Trial
-     * @see		IntervalLocationProfile
-     * @see		DataSession#getIntervalEvents
-     */
-    public IntervalLocationProfile getTotalSummary() throws SQLException {
-        if (this.totalSummary == null)
-            dataSession.getIntervalEventDetail(this);
-        return (this.totalSummary);
-    }
-
-    /**
-     * Sets the unique ID associated with this interval event.
-     * <i> NOTE: This method is used by the DataSession object to initialize
-     * the object.  Not currently intended for use by any other code.</i>
-     *
-     * @param	id unique ID associated with this intervalEvent
-     */
-    public void setID(int id) {
-        this.intervalEventID = id;
-    }
-
-    /**
-     * Sets the intervalEvent name.
-     * <i> NOTE: This method is used by the DataSession object to initialize
-     * the object.  Not currently intended for use by any other code.</i>
-     *
-     * @param	name the name of the intervalEvent
-     */
-    public void setName(String name) {
-        this.name = name;
-    }
-
-    /**
-     * Sets the TAU group of this intervalEvent object.
-     * <i> NOTE: This method is used by the DataSession object to initialize
-     * the object.  Not currently intended for use by any other code.</i>
-     *
-     * @param	group the TAU group the intervalEvent is in.
-     */
-    public void setGroup(String group) {
-        this.group = group;
-    }
-
-    /**
-     * Sets the trial ID of this intervalEvent object.
-     * <i> NOTE: This method is used by the DataSession object to initialize
-     * the object.  Not currently intended for use by any other code.</i>
-     *
-     * @param	id the trial ID for the intervalEvent.
-     */
-    public void setTrialID(int id) {
-        this.trialID = id;
-    }
-
-    /**
-     * Adds a IntervalLocationProfile to the intervalEvent as a mean summary.
-     * <i> NOTE: This method is used by the DataSession object to initialize
-     * the object.  Not currently intended for use by any other code.</i>
-     *
-     * @param	meanSummary the mean summary object for the intervalEvent.
-     */
-    public void setMeanSummary(IntervalLocationProfile meanSummary) {
-        this.meanSummary = meanSummary;
-    }
-
-    /**
-     * Adds a IntervalLocationProfile to the intervalEvent as a total summary.
-     * <i> NOTE: This method is used by the DataSession object to initialize
-     * the object.  Not currently intended for use by any other code.</i>
-     *
-     * @param	totalSummary the total summary object for the intervalEvent.
-     */
-    public void setTotalSummary(IntervalLocationProfile totalSummary) {
-        this.totalSummary = totalSummary;
+	// can't instantiate this class!
+	private IntervalEvent() {
     }
 
     // returns a Vector of IntervalEvents
-    public static Vector<IntervalEvent> getIntervalEvents(DatabaseAPI dataSession, DB db, String whereClause) {
-        Vector<IntervalEvent> events = new Vector<IntervalEvent>();
+    public static Map<Integer, Function> getIntervalEvents(DatabaseAPI dataSession, DB db, String whereClause, DataSource dataSource, int numberOfMetrics) {
+    	if(db.getSchemaVersion() >0) return getTAUdbIntervalEvents(dataSession,db,  whereClause, dataSource, numberOfMetrics);
+        Map<Integer, Function> events = new HashMap<Integer, Function>();
         // create a string to hit the database
         StringBuffer buf = new StringBuffer();
         buf.append("select id, name, group_name, trial ");
@@ -227,12 +75,14 @@ public class IntervalEvent {
 
             //IntervalEvent tmpIntervalEvent = null;
             while (resultSet.next() != false) {
-                IntervalEvent event = new IntervalEvent(dataSession);
-                event.setID(resultSet.getInt(1));
-                event.setName(resultSet.getString(2));
-                event.setGroup(resultSet.getString(3));
-                event.setTrialID(resultSet.getInt(4));
-                events.addElement(event);
+                int id = resultSet.getInt(1);
+                String name = resultSet.getString(2);
+                String groups = resultSet.getString(3);
+                int trialID = resultSet.getInt(4);
+                Function function = dataSource.addFunction(name, numberOfMetrics);
+//                function.setID(id);
+                dataSource.addGroups(groups, function);
+                events.put(id, function);
             }
             resultSet.close();
         } catch (Exception ex) {
@@ -243,8 +93,112 @@ public class IntervalEvent {
         return events;
     }
 
-    public int saveIntervalEvent(DB db, int newTrialID, Hashtable<Integer, Integer> newMetHash, int saveMetricIndex)
+    private static Map<Integer, Function> getTAUdbIntervalEvents(
+			DatabaseAPI dataSession, DB db, String whereClause, DataSource dataSource, int numberOfMetrics) {
+//    	SELECT timer.id, timer.trial, timer.name, timer_group.group_name 
+//    	FROM timer
+//    	LEFT JOIN timer_group
+//    	ON timer.id=timer_group.timer
+    	
+    	 Map<Integer, Function> events = new HashMap<Integer, Function>();
+         // create a string to hit the database
+         StringBuffer buf = new StringBuffer();
+         buf.append("with recursive cp (id, parent, timer, name) as (");
+         /* flat timer part */
+         buf.append("SELECT tc.id, tc.parent, tc.timer, t.name FROM ");
+         buf.append(db.getSchemaPrefix());
+         buf.append("timer_callpath tc INNER JOIN ");
+         buf.append(db.getSchemaPrefix());
+         buf.append("timer t on tc.timer = t.id WHERE ");
+         if (dataSession.getTrial() != null) {
+        	 buf.append(" t.trial = " + dataSession.getTrial().getID() + " AND ");
+         }
+         buf.append(" tc.parent is null ");
+         buf.append("UNION ALL ");
+         /* recursive part */
+         buf.append("SELECT d.id, d.parent, d.timer, ");
+		        if (db.getDBType().compareTo("h2") == 0) {
+					buf.append("concat (cp.name, ' => ', dt.name) FROM ");
+		        } else {
+					buf.append("cp.name || ' => ' || dt.name FROM ");
+		        }
+				buf.append(db.getSchemaPrefix());
+         buf.append("timer_callpath AS d JOIN cp on (d.parent = cp.id) JOIN ");
+         buf.append(db.getSchemaPrefix());
+         buf.append("timer dt on d.timer = dt.id ");
+         if (dataSession.getTrial() != null) {
+        	 buf.append("where dt.trial = " + dataSession.getTrial().getID() + " ");
+         }
+         buf.append(") ");
+         buf.append("SELECT distinct cp.id, cp.timer, cp.name, t.short_name, t.source_file, t.line_number, ");
+         buf.append("t.line_number_end, t.column_number, t.column_number_end, g.group_name, t.trial FROM cp join ");
+         buf.append(db.getSchemaPrefix());
+         buf.append("timer t on cp.timer = t.id join ");
+         buf.append(db.getSchemaPrefix());
+         buf.append("timer_group g on t.id = g.timer ");
+         buf.append(whereClause);
+
+         if (db.getDBType().compareTo("oracle") == 0) {
+             buf.append(" order by dbms_lob.substr(name) asc");
+         } else if (db.getDBType().compareTo("derby") == 0) {
+             buf.append(" order by cast (name as varchar(4000)) asc");
+         } else if (db.getDBType().compareTo("db2") == 0) {
+             buf.append(" order by cast (name as varchar(256)) asc");
+         } else {
+             buf.append(" order by name asc ");
+         }
+
+         // get the results
+         try {
+        	 //System.out.println(buf.toString());
+             ResultSet resultSet = db.executeQuery(buf.toString());
+             //IntervalEvent tmpIntervalEvent = null;
+             Function last = null;
+             while (resultSet.next() != false) {
+                 int id = resultSet.getInt(1);
+                 String name = resultSet.getString(3);
+                 String group = resultSet.getString(10);
+                 int trialID = resultSet.getInt(11);
+                 last = events.get(id);
+                 
+                 boolean isCallpath = false;
+                 if (name!=null&&name.contains(" => ")) {
+                 	isCallpath = true;
+                 	if(group!=null&&!group.contains("TAU_CALLPATH")){
+                 		group = group+"|TAU_CALLPATH";
+                 	}
+                 }
+                 
+                 if (last != null) {
+                	 dataSource.addGroups(group, last);
+                 } else {
+                    Function function = dataSource.addFunction(name, numberOfMetrics);
+                    function.setDatabaseID(id);
+                    dataSource.addGroups(group, function);
+                    events.put(id, function);
+                    SourceRegion sourceRegion = new SourceRegion();
+                    sourceRegion.setFilename(resultSet.getString(5));
+                    sourceRegion.setStartLine(resultSet.getInt(6));
+                    sourceRegion.setEndLine(resultSet.getInt(7));
+                    sourceRegion.setStartColumn(resultSet.getInt(8));
+                    sourceRegion.setEndColumn(resultSet.getInt(9));
+                    function.callpathFunction = isCallpath;
+                    function.setShortName(resultSet.getString(4));
+                    function.setSourceRegion(sourceRegion);
+                 }
+             }
+             resultSet.close();
+         } catch (Exception ex) {
+             ex.printStackTrace();
+             return null;
+         }
+
+         return events;
+	}
+
+	public static int saveIntervalEvent(DB db, int newTrialID, Function function, Hashtable<Integer, Integer> newMetHash, int saveMetricIndex)
             throws SQLException {
+    	if(db.getSchemaVersion() >0) return saveTAUdbIntervalEvent(db, newTrialID, function, newMetHash, saveMetricIndex);
         int newIntervalEventID = -1;
 
         PreparedStatement statement = null;
@@ -253,8 +207,8 @@ public class IntervalEvent {
             statement = db.prepareStatement("INSERT INTO " + db.getSchemaPrefix()
                     + "interval_event (trial, name, group_name) VALUES (?, ?, ?)");
             statement.setInt(1, newTrialID);
-            statement.setString(2, name);
-            statement.setString(3, group);
+            statement.setString(2, function.getName());
+            statement.setString(3, function.getGroupString());
             statement.executeUpdate();
             statement.close();
 
@@ -263,6 +217,8 @@ public class IntervalEvent {
                 tmpStr = "select LAST_INSERT_ID();";
             else if (db.getDBType().compareTo("db2") == 0)
                 tmpStr = "select IDENTITY_VAL_LOCAL() FROM interval_event";
+            else if (db.getDBType().compareTo("sqlite") == 0)
+                tmpStr = "select seq from sqlite_sequence where name = 'interval_event'";
             else if (db.getDBType().compareTo("derby") == 0)
                 tmpStr = "select IDENTITY_VAL_LOCAL() FROM interval_event";
             else if (db.getDBType().compareTo("h2") == 0)
@@ -284,7 +240,7 @@ public class IntervalEvent {
                 statement = db.prepareStatement("SELECT id FROM " + db.getSchemaPrefix()
                         + "interval_event where name = ? and trial = ?");
 
-            statement.setString(1, name);
+            statement.setString(1, function.getName());
             statement.setInt(2, newTrialID);
             ResultSet resultSet = statement.executeQuery();
             while (resultSet.next() != false) {
@@ -295,10 +251,106 @@ public class IntervalEvent {
         }
         
         if (newIntervalEventID == -1) {
-            throw new RuntimeException("Unable to find event in database, event: " + name, null);
+            throw new RuntimeException("Unable to find event in database, event: " + function.getName(), null);
         }
 
         // save the intervalEvent mean summary
+        if (function.getMeanProfile() != null) {
+        	System.out.println("TODO! SAVE THE MEAN SUMMARY!");
+            //meanSummary.saveMeanSummary(db, newIntervalEventID, newMetHash, saveMetricIndex);
+        }
+
+        // save the intervalEvent total summary
+        if (function.getTotalProfile() != null) {
+        	System.out.println("TODO! SAVE THE TOTAL SUMMARY!");
+            //totalSummary.saveTotalSummary(db, newIntervalEventID, newMetHash, saveMetricIndex);
+        }
+        return newIntervalEventID;
+    }
+    
+	public static int saveTAUdbIntervalEvent(DB db, int newTrialID, Function function, Hashtable<Integer, Integer> newMetHash, int saveMetricIndex)
+            throws SQLException {
+        int newIntervalEventID = -1;
+
+        PreparedStatement statement = null;
+        if (saveMetricIndex < 0) {
+            statement = db.prepareStatement("INSERT INTO " + db.getSchemaPrefix()
+                    + "timer (trial, name) VALUES (?, ?)");
+            statement.setInt(1, newTrialID);
+            statement.setString(2, function.getName());
+            // TODO: What about short_name, file, column, line, etc?
+            statement.executeUpdate();
+            statement.close();
+
+            String tmpStr = new String();
+            if (db.getDBType().compareTo("mysql") == 0)
+                tmpStr = "select LAST_INSERT_ID();";
+            else if (db.getDBType().compareTo("db2") == 0)
+                tmpStr = "select IDENTITY_VAL_LOCAL() FROM interval_event";
+            else if (db.getDBType().compareTo("sqlite") == 0)
+                tmpStr = "select seq from sqlite_sequence where name = 'interval_event'";
+            else if (db.getDBType().compareTo("derby") == 0)
+                tmpStr = "select IDENTITY_VAL_LOCAL() FROM interval_event";
+            else if (db.getDBType().compareTo("h2") == 0)
+                tmpStr = "select IDENTITY_VAL_LOCAL() FROM interval_event";
+            else if (db.getDBType().compareTo("oracle") == 0)
+                tmpStr = "select " + db.getSchemaPrefix() + "interval_event_id_seq.currval FROM dual";
+            else // postgres
+                tmpStr = "select currval('interval_event_id_seq');";
+            newIntervalEventID = Integer.parseInt(db.getDataItem(tmpStr));
+
+			// save the groups
+            List<Group> groups = function.getGroups();
+            statement = db.prepareStatement("INSERT INTO " + db.getSchemaPrefix()
+               	+ "timer_group (timer, group_name) VALUES (?, ?)");
+			for (int i = 0 ; i < groups.size() ; i++) {
+				statement.setInt(1,newIntervalEventID);
+				statement.setString(2,groups.get(i).getName().trim());
+				statement.addBatch();
+			}
+			statement.executeBatch();
+			statement.close();
+        } else {
+        	StringBuilder query = new StringBuilder();
+        	query.append("with recursive cp (id, parent, timer, name) as " +
+        			"(SELECT tc.id, tc.parent, tc.timer, t.name FROM timer_callpath tc " + 
+        			"INNER JOIN timer t on tc.timer = t.id WHERE t.trial = " + 
+        			newTrialID + " and tc.parent is null " +
+        			"UNION ALL SELECT d.id, d.parent, d.timer, ");
+		        if (db.getDBType().compareTo("h2") == 0) {
+					query.append("concat (cp.name, ' => ', dt.name) FROM ");
+		        } else {
+					query.append("cp.name || ' => ' || dt.name FROM ");
+		        }
+				query.append("timer_callpath AS d JOIN cp on (d.parent = cp.id) " +
+        			"JOIN timer dt on d.timer = dt.id " +
+					"where dt.trial = " + newTrialID + " ) " +
+        			"SELECT distinct cp.id, cp.timer, cp.name, t.trial " +
+        			"FROM cp join timer t on cp.timer = t.id " +
+        			"join timer_group g on t.id = g.timer WHERE trial = ? and ");
+            if (db.getDBType().compareTo("oracle") == 0)
+                statement = db.prepareStatement(query.toString() + "dbms_lob.instr(cp.name, ?) > 0");
+            else if (db.getDBType().compareTo("derby") == 0)
+                statement = db.prepareStatement(query.toString() + "cast(cp.name as varchar(4000)) = ?");
+            else
+            	statement = db.prepareStatement(query.toString() + "cp.name = ?");
+
+            statement.setInt(1, newTrialID);
+            statement.setString(2, function.getName());
+            ResultSet resultSet = statement.executeQuery();
+            while (resultSet.next() != false) {
+                newIntervalEventID = resultSet.getInt(1);
+            }
+            resultSet.close();
+            statement.close();
+        }
+        
+        if (newIntervalEventID == -1) {
+            throw new RuntimeException("Unable to find event in database, event: " + function.getName(), null);
+        }
+
+        // TODO: mean, total. etc should all be saved when aggregate threads are saved.
+/*        // save the intervalEvent mean summary
         if (meanSummary != null) {
             meanSummary.saveMeanSummary(db, newIntervalEventID, newMetHash, saveMetricIndex);
         }
@@ -307,6 +359,7 @@ public class IntervalEvent {
         if (totalSummary != null) {
             totalSummary.saveTotalSummary(db, newIntervalEventID, newMetHash, saveMetricIndex);
         }
+*/
         return newIntervalEventID;
     }
     
