@@ -30,6 +30,7 @@ import java.util.NoSuchElementException;
 import java.util.StringTokenizer;
 
 import edu.uoregon.tau.common.LineCountBufferedReader;
+import edu.uoregon.tau.common.MetaDataMap;
 import edu.uoregon.tau.common.Utility;
 
 public class TauDataSource extends DataSource {
@@ -242,9 +243,16 @@ public class TauDataSource extends DataSource {
                                 int end = inputString.indexOf("</metadata>") + 11;
                                 String metadata = inputString.substring(start, end);
                                 try {
-                                    MetaDataParser.parse(thread.getMetaData(), metadata);
+                                	MetaDataMap mdm = thread.getMetaData();
+                                	if(mdm==null)
+                                    {
+                                		thread.setMetaData(MetaDataParser.parse(metadata, thread));
+                                    }
+                                	else{
+                                		mdm.putAll(MetaDataParser.parse(metadata, thread));
+                                	}
                                 } catch (Exception exception) {
-                                    //exception.printStackTrace();
+                                    exception.printStackTrace();
                                     throw new CorruptFileException("Unable to parse metadata block");
                                 }
                             }
@@ -332,8 +340,10 @@ public class TauDataSource extends DataSource {
 
                                         inputString = br.readLine();
                                         if (inputString == null) {
-                                            throw new CorruptFileException("Unexpected end of file: Only found " + (j - 2)
-                                                    + " of " + numUserEvents + " User Event Lines");
+                                            //throw new CorruptFileException("Unexpected end of file: Only found " + (j - 2)
+                                              //      + " of " + numUserEvents + " User Event Lines");
+						System.out.println("WARNING: Unexpected end of file: Only found " + (j - 2)+ " of " + numUserEvents + " User Event Lines");
+						break;
                                         }
 
                                         this.processUserEventLine(inputString, thread);
@@ -549,12 +559,12 @@ public class TauDataSource extends DataSource {
         }
 
         //if (numcalls != 0) {
-            Function func = this.addFunction(name, 1);
+            Function func = this.addFunction(name, this.dirs.size());
 
             FunctionProfile functionProfile = thread.getFunctionProfile(func);
 
             if (functionProfile == null) {
-                functionProfile = new FunctionProfile(func);
+                functionProfile = new FunctionProfile(func, this.dirs.size());
                 thread.addFunctionProfile(functionProfile);
             }
 
