@@ -12,10 +12,15 @@ import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
+import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.List;
 import java.util.Map;
 import java.util.Properties;
 
+
+//import common.Logger;
+
 import edu.uoregon.tau.perfdmf.Database;
 
 /*******************************************************
@@ -27,6 +32,14 @@ import edu.uoregon.tau.perfdmf.Database;
  *******************************************************/
 
 public class DBConnector implements DB {
+    public static final String DEFAULT_TRUSTSTORE_PATH =
+        System.getProperty("user.home") + File.separator + ".ParaProf" + File.separator + "keystore.taudb";
+    public static final String DEFAULT_TRUSTSTORE_PASSWORD = "changeit";
+
+    public static final String DEFAULT_KEYSTORE_PATH =
+        System.getProperty("user.home") + File.separator + ".ParaProf" + File.separator + "keystore.taudb";
+    public static final String DEFAULT_KEYSTORE_PASSWORD = "changeit";
+
 
     private Statement statement;
     private Connection conn;
@@ -39,16 +52,28 @@ public class DBConnector implements DB {
 
     private String driverName;
     private String JDBCjarFileName;
+	private String dbServerHostname;
+
+	private boolean dbUseSSL = false;
+	private boolean dbUseKeys = false;
+	private boolean dbUseTrust = false;
+
+	private String dbKeystore;
+	private String dbKeystorePassword;
+
+	private String dbTruststore;
+	private String dbTruststorePassword;
 
     private Database database;
+    private int schemaVersion = -1;
+
 
-    
     private static Map<String, String> passwordMap = new HashMap<String, String>();
-    
+
     private static PasswordCallback passwordCallback;
-    
-    
-    
+
+
+
     /*
      * This class is here because the DriverManager refuses to use a driver that is not loaded
      * by the system ClassLoader.  So we wrap it with this.
@@ -84,6 +109,11 @@ public class DBConnector implements DB {
         public boolean jdbcCompliant() {
             return this.driver.jdbcCompliant();
         }
+        public java.util.logging.Logger getParentLogger(){
+        	//throw new Exception();
+        	return null;
+        }
+
     }
 
     // it should be "org.postgresql.Driver" in PostgreSQL.
@@ -113,10 +143,36 @@ public class DBConnector implements DB {
         }
     }
 
+
+    public String setDefault(String s, String default_value) {
+        if (s == null || s.equals("")) {
+            return default_value;
+        }
+        return s;
+    }
+
+
     public void setJDBC(ParseConfig parser) {
         driverName = parser.getJDBCDriver();
         dbaddress = parser.getConnectionString();
         JDBCjarFileName = parser.getJDBCJarFile();
+		dbServerHostname = parser.getDBHost();
+
+        // SSL keystore and truststore parameters
+		dbUseSSL = parser.getDBUseSSL();
+		dbKeystore =
+            setDefault(parser.getDBKeystore(), DEFAULT_KEYSTORE_PATH);
+		dbKeystorePassword =
+            setDefault(parser.getDBKeystorePasswd(), DEFAULT_KEYSTORE_PASSWORD);
+
+		dbTruststore =
+            setDefault(parser.getDBTruststore(), DEFAULT_TRUSTSTORE_PATH);
+		dbTruststorePassword =
+            setDefault(parser.getDBTruststorePasswd(), DEFAULT_TRUSTSTORE_PASSWORD);
+
+        // Check whether the keystore and trust store exist, disable if they don't
+        dbUseKeys = new File(dbKeystore).exists();
+        dbUseTrust = new File(dbTruststore).exists();
     }
 
     public void close() {
@@ -144,7 +200,7 @@ public class DBConnector implements DB {
     public void rollback() throws SQLException {
         conn.rollback();
     }
-    
+
     private static String findPassword(ParseConfig config) {
 
         //System.out.println("finding password, path: " + config.getPath());
@@ -157,22 +213,60 @@ public class DBConnector implements DB {
     }
 
     public boolean connect(String user, String password) throws SQLException {
-        String cs = "";
+        StringBuffer cs = new StringBuffer();
         try {
             if (conn != null) {
                 return true;
             }
-            cs = getConnectString();
+            cs.append(getConnectString());
+			Properties props = new Properties();
+            
 
-            if (password == null) {
-                password = findPassword(config);
-            }
-            conn = DriverManager.getConnection(cs, user, password);
+			if (dbUseSSL) {
+                // Use SSL and set that up first.
+			    props.setProperty("user", user);
+			    props.setProperty("ssl","true");
+                props.setProperty("sslfactory", "edu.uoregon.tau.perfdmf.database.CustomSSLSocketFactory");
+
+                if (dbUseTrust) {
+                    System.setProperty("javax.net.ssl.trustStore", dbTruststore);
+                    System.setProperty("javax.net.ssl.trustStorePassword", dbTruststorePassword);
+                }
+
+                if (dbUseKeys) {
+                    // Tell Java about the keystore and trust store.  User can
+                    // add trust certs if needed, but all we really need is
+                    // the client key.
+                    System.setProperty("javax.net.ssl.keyStore", dbKeystore);
+                    System.setProperty("javax.net.ssl.keyStorePassword", dbKeystorePassword);
+
+                    // require that the client alias is user@example.com, for
+                    // a DB hosted at example.com
+                    CustomX509KeyManager.setClientAlias(user + "@" + dbServerHostname);
+
+                } else {
+                	if (password == null) {
+                        password = findPassword(config);
+                    }
+                    props.setProperty("password", password);
+                }
+
+			} else {
+                // Do not use SSL, just username/password
+				if (password == null) {
+	                password = findPassword(config);
+	            }
+			    props.setProperty("user",user);
+			    props.setProperty("password",password);
+			}
+            conn = DriverManager.getConnection(cs.toString(), props);
             return true;
+
         } catch (SQLException ex) {
             System.err.println("Cannot connect to server.");
             System.err.println("Connection String: " + cs);
             System.err.println("Exception Message: " + ex.getMessage());
+            ex.printStackTrace();
             throw ex;
         }
     }
@@ -182,7 +276,7 @@ public class DBConnector implements DB {
         try {
             cs.append(getConnectString());
             cs.append(";create=true");
-            
+
             if (password == null) {
                 password = findPassword(config);
             }
@@ -210,7 +304,7 @@ public class DBConnector implements DB {
         }
         //        conn.setAutoCommit(false);
         //        statement.setFetchSize(100);
-        //	System.out.println ("executing query: " + query.trim());
+        //System.out.println ("executing query: " + query.trim());
         return statement.executeQuery(query.trim());
     }
 
@@ -370,7 +464,7 @@ public class DBConnector implements DB {
     }
 
     public PreparedStatement prepareStatement(String statement) throws SQLException {
-        //System.out.println("statement = " + statement);
+//        System.out.println("statement = " + statement);
         return getConnection().prepareStatement(statement);
     }
 
@@ -405,7 +499,7 @@ public class DBConnector implements DB {
     // REF          : 2006
 
     //     public static boolean isReadWriteType(int type) {
-    // 	if (type == java.sql.Types.VARCHAR 
+    // 	if (type == java.sql.Types.VARCHAR
     // 	    || type == java.sql.Types.CLOB
     // 	    || type == java.sql.Types.INTEGER
     // 	    || type == java.sql.Types.DECIMAL
@@ -415,19 +509,19 @@ public class DBConnector implements DB {
     //     }
 
     public static boolean isReadAbleType(int type) {
-        if (type == java.sql.Types.VARCHAR 
-            || type == java.sql.Types.CLOB 
+        if (type == java.sql.Types.VARCHAR
+            || type == java.sql.Types.CLOB
             || type == java.sql.Types.INTEGER
             || type == java.sql.Types.BIGINT
-            || type == java.sql.Types.DECIMAL 
-            || type == java.sql.Types.DOUBLE 
+            || type == java.sql.Types.DECIMAL
+            || type == java.sql.Types.DOUBLE
             || type == java.sql.Types.FLOAT
-            || type == java.sql.Types.LONGVARCHAR 
-            || type == java.sql.Types.TIME 
+            || type == java.sql.Types.LONGVARCHAR
+            || type == java.sql.Types.TIME
             || type == java.sql.Types.TIMESTAMP
                 // added binary types for XML_METADATA_GZ processing
-            || type == java.sql.Types.BINARY 
-            || type == java.sql.Types.VARBINARY 
+            || type == java.sql.Types.BINARY
+            || type == java.sql.Types.VARBINARY
             || type == java.sql.Types.LONGVARBINARY
             || type == java.sql.Types.BLOB)
             return true;
@@ -462,17 +556,17 @@ public class DBConnector implements DB {
     }
 
     //     public static boolean isReadOnlyType(int type) {
-    // 	if (type == java.sql.Types.TIME 
+    // 	if (type == java.sql.Types.TIME
     // 	    || type == java.sql.Types.TIMESTAMP)
     // 	    return true;
     // 	return false;
     //     }
 
-    public int checkTable(DatabaseMetaData dbMeta, String tableName, String columns[]) throws SQLException {
+   	public int checkTable(DatabaseMetaData dbMeta, String tableName, Object columns[]) throws SQLException {
         boolean checks[] = new boolean[columns.length];
 
         ResultSet resultSet = null;
-        if ((this.getDBType().compareTo("oracle") == 0) 
+        if ((this.getDBType().compareTo("oracle") == 0)
 		|| (this.getDBType().compareTo("derby") == 0)
                 || (this.getDBType().compareTo("h2") == 0)
                 || (this.getDBType().compareTo("db2") == 0)) {
@@ -493,7 +587,7 @@ public class DBConnector implements DB {
 
                 for (int i = 0; i < columns.length; i++) {
             		//System.out.println (cname.toUpperCase() + ", " + columns[i].toUpperCase());
-                    if (columns[i].toUpperCase().compareTo(cname.toUpperCase()) == 0) {
+                    if (columns[i].toString().toUpperCase().compareTo(cname.toUpperCase()) == 0) {
                         checks[i] = true;
                     }
                 }
@@ -512,6 +606,9 @@ public class DBConnector implements DB {
     }
 
     public int checkSchema() throws SQLException {
+    	if (getSchemaVersion() > 0) {
+    		return checkTAUdbSchema();
+    	}
 
         //ResultSet resultSet = null;
         DatabaseMetaData dbMeta = this.getMetaData();
@@ -579,12 +676,159 @@ public class DBConnector implements DB {
         return 0;
     }
 
-    public Database getDatabase() {
+    private int checkTAUdbSchema() throws SQLException {
+        //ResultSet resultSet = null;
+        DatabaseMetaData dbMeta = this.getMetaData();
+
+        List<String> columns = new ArrayList<String>();
+        columns.add("version");
+        if (checkTable(dbMeta, "schema_version", columns.toArray()) != 0)
+            return -1;
+
+        columns.clear();
+        columns.add("id");
+        columns.add("name");
+        columns.add("description");
+        if (checkTable(dbMeta, "data_source", columns.toArray()) != 0)
+            return -1;
+
+        columns.clear();
+        columns.add("id");
+        columns.add("name");
+        if (checkTable(dbMeta, "trial", columns.toArray()) != 0)
+            return -1;
+        if (checkTable(dbMeta, "derived_thread_type", columns.toArray()) != 0)
+            return -1;
+
+        columns.clear();
+        columns.add("trial");
+        columns.add("name");
+        columns.add("value");
+        if (checkTable(dbMeta, "primary_metadata", columns.toArray()) != 0)
+            return -1;
+        if (checkTable(dbMeta, "secondary_metadata", columns.toArray()) != 0)
+            return -1;
+
+        columns.clear();
+        columns.add("id");
+        columns.add("trial");
+        if (checkTable(dbMeta, "thread", columns.toArray()) != 0)
+            return -1;
+        columns.add("name");
+        if (checkTable(dbMeta, "metric", columns.toArray()) != 0)
+            return -1;
+        if (checkTable(dbMeta, "timer", columns.toArray()) != 0)
+            return -1;
+        if (checkTable(dbMeta, "counter", columns.toArray()) != 0)
+            return -1;
+
+        columns.clear();
+        columns.add("timer");
+        if (checkTable(dbMeta, "timer_group", columns.toArray()) != 0)
+            return -1;
+        if (checkTable(dbMeta, "timer_parameter", columns.toArray()) != 0)
+            return -1;
+        if (checkTable(dbMeta, "timer_callpath", columns.toArray()) != 0)
+            return -1;
+        columns.clear();
+        columns.add("timer_callpath");
+        columns.add("thread");
+        if (checkTable(dbMeta, "timer_call_data", columns.toArray()) != 0)
+            return -1;
+
+
+        columns.clear();
+        columns.add("timer_call_data");
+        columns.add("metric");
+        columns.add("inclusive_value");
+        columns.add("exclusive_value");
+        columns.add("inclusive_percent");
+        columns.add("exclusive_percent");
+        columns.add("sum_exclusive_squared");
+
+        if (checkTable(dbMeta, "timer_value", columns.toArray()) != 0)
+            return -1;
+
+        columns.clear();
+        columns.add("counter");
+        columns.add("thread");
+        columns.add("sample_count");
+        columns.add("minimum_value");
+        columns.add("maximum_value");
+        columns.add("mean_value");
+        columns.add("standard_deviation");
+        if (checkTable(dbMeta, "counter_value", columns.toArray()) != 0)
+            return -1;
+
+        columns.clear();
+        columns.add("id");
+        columns.add("parent");
+        columns.add("name");
+        if (checkTable(dbMeta, "taudb_view", columns.toArray()) != 0)
+            return -1;
+
+        columns.clear();
+        columns.add("taudb_view");
+        columns.add("table_name");
+        columns.add("column_name");
+        if (checkTable(dbMeta, "taudb_view_parameter", columns.toArray()) != 0)
+            return -1;
+
+        return 0;
+	}
+
+	public Database getDatabase() {
         return database;
     }
-    
+
     public static void setPasswordCallback(PasswordCallback callback) {
         passwordCallback = callback;
     }
 
+	public int getSchemaVersion()  {
+		if (schemaVersion == -1){
+			setSchemaVersion();
+		}
+		return schemaVersion;
+	}
+
+
+	private void setSchemaVersion() {
+			boolean hasSchemaVersionTable = false;
+			DatabaseMetaData meta;
+			try {
+				meta = getMetaData();
+
+                String tmpStr = null;
+                if (this.getDBType().compareTo("h2") == 0) {
+                    tmpStr = "SCHEMA_VERSION";
+                } else {
+                    tmpStr = "schema_version";
+                }
+                ResultSet r = meta.getTables(null, null, tmpStr, null);
+
+                while (r.next()) {
+                    String tablename = r.getString("TABLE_NAME");
+                    if (tablename.equalsIgnoreCase("schema_version")) {
+                        hasSchemaVersionTable = true;
+                    }
+			    }
+
+			    if (hasSchemaVersionTable) {
+				    String query = "SELECT version FROM " + getSchemaPrefix()
+						    + "schema_version";
+				    ResultSet resultSet = executeQuery(query);
+
+				    if (resultSet.next()){
+					    schemaVersion = resultSet.getInt(1);
+					    return;
+				    }
+			    }
+			} catch (SQLException e) {
+				// TODO Auto-generated catch block
+				e.printStackTrace();
+			}
+			schemaVersion = 0;
+		}
+
 }
