@@ -5,7 +5,13 @@ import jargs.gnu.CmdLineParser;
 import java.io.BufferedReader;
 import java.io.File;
 import java.io.FileReader;
+import java.sql.SQLException;
+import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
 
+import edu.uoregon.tau.common.MetaDataMap;
 import edu.uoregon.tau.perfdmf.DataSource;
 import edu.uoregon.tau.perfdmf.DataSourceException;
 import edu.uoregon.tau.perfdmf.DatabaseAPI;
@@ -13,65 +19,85 @@ import edu.uoregon.tau.perfdmf.DatabaseException;
 import edu.uoregon.tau.perfdmf.Experiment;
 import edu.uoregon.tau.perfdmf.Trial;
 import edu.uoregon.tau.perfdmf.UtilFncs;
+import edu.uoregon.tau.perfdmf.SnapshotDataSource;
+import edu.uoregon.tau.perfdmf.View;
+import edu.uoregon.tau.perfdmf.View.ViewRule;
+import edu.uoregon.tau.perfdmf.View.ViewRule.NumericViewComparator;
+import edu.uoregon.tau.perfdmf.View.ViewRule.StringViewComparator;
+import edu.uoregon.tau.perfdmf.taudb.TAUdbDatabaseAPI;
 
 public class LoadTrial {
 
     private String sourceFiles[];
     private String metadataFile;
+    private String metadataString;
     private Experiment exp;
     private boolean fixNames;
     private boolean summaryOnly;
+    private boolean useNulls;
     private int expID;
     public int trialID;
     private DataSource dataSource;
     public String trialName;
+    public String appName;
+    public String expName;
     public String problemFile;
     public String configuration;
     public boolean terminate = true;
 
     private DatabaseAPI databaseAPI;
     private Trial trial;
+	private Double reducePercentage;
 
     public static void usage() {
-        System.err.println("Usage: perfdmf_loadtrial -a <appName> -x <expName> -n <name> [options] <files>\n\n"
-                + "try `perfdmf_loadtrial --help' for more information");
+        //System.err.println("Usage: taudb_loadtrial -a <appName> -x <expName> -n <name> [options] <file>\n\n"
+        System.err.println("Usage: taudb_loadtrial -n <name> [options] <file>\n\n"
+                + "try `taudb_loadtrial --help' for more information");
     }
 
     public static void outputHelp() {
 
-        System.err.println("Usage: perfdmf_loadtrial -a <appName> -x <expName> -n <name> [options] <files>\n\n"
+        //System.err.println("Usage: taudb_loadtrial -a <appName> -x <expName> -n <name> [options] <file>\n\n"
+        System.err.println("Usage: taudb_loadtrial -n <name> [options] <file>\n\n"
                 + "Required Arguments:\n\n" + "  -n, --name <text>               Specify the name of the trial\n"
-                + "  -a, --applicationname <string>  Specify associated application name\n"
-                + "                                    for this trial\n"
-                + "  -x, --experimentname <string>   Specify associated experiment name\n"
-                + "                                    for this trial\n" + "               ...or...\n\n"
-                + "  -n, --name <text>               Specify the name of the trial\n"
-                + "  -e, --experimentid <number>     Specify associated experiment ID\n"
-                + "                                    for this trial\n" + "\n" + "Optional Arguments:\n\n"
+                //+ "  -a, --applicationname <string>  Specify associated application name\n"
+                //+ "                                    for this trial\n"
+                //+ "  -x, --experimentname <string>   Specify associated experiment name\n"
+                //+ "                                    for this trial\n" + "               ...or...\n\n"
+                //+ "  -n, --name <text>               Specify the name of the trial\n"
+                //+ "  -e, --experimentid <number>     Specify associated experiment ID\n"
+                //+ "                                    for this trial\n" 
                 + "  -c, --config <name>             Specify the name of the configuration to use\n"
+				+ "\n" + "Optional Arguments:\n\n"
                 + "  -g, --configFile <file>         Specify the configuration file to use\n"
                 + "                                    (overrides -c)\n"
                 + "  -f, --filetype <filetype>       Specify type of performance data, options are:\n"
                 + "                                    profiles (default), pprof, dynaprof, mpip,\n"
                 + "                                    gprof, psrun, hpm, packed, cube, hpc, ompp,\n"
                 + "                                    snap, perixml, gptl, paraver, ipm, google\n"
-                + "  -t, --trialid <number>          Specify trial ID\n"
+                //+ "  -t, --trialid <number>          Specify trial ID\n"
                 + "  -i, --fixnames                  Use the fixnames option for gprof\n"
-                + "  -m, --metadata <filename>       XML metadata for the trial\n\n" + "Notes:\n"
+                + "  -z, --usenull                   Include NULL values as 0 for mean calculation\n"
+                + "  -r, --reduce <percentage>       Aggregate all timers less than percentage as \"other\"\n"
+                + "  -d, --metadata-file <filename>  XML metadata for the trial\n" 
+                + "  -m, --metadata                  Colon seperated metadata name/value pairs \n" 
+                + "                                  <foo1=bar1:foo2=bar2>\n\n" + "Notes:\n"
+                
+                + "  -a, --applicationname <string>  Specify associated application name\n"
+                + "                                    for this trial. Creates a view.\n"
+                + "  -x, --experimentname <string>   Specify associated experiment name\n"
+                + "                                    for this trial. Creates a view.\n"
+                
                 + "  For the TAU profiles type, you can specify either a specific set of profile\n"
                 + "files on the commandline, or you can specify a directory (by default the current\n"
                 + "directory).  The specified directory will be searched for profile.*.*.* files,\n"
                 + "or, in the case of multiple counters, directories named MULTI_* containing\n" + "profile data.\n\n"
-                + "Examples:\n\n" + "  perfdmf_loadtrial -e 12 -n \"Batch 001\"\n"
+                + "Examples:\n\n" + "  taudb_loadtrial -c mydb -n \"Batch 001\"\n"
                 + "    This will load profile.* (or multiple counters directories MULTI_*) into\n"
-                + "    experiment 12 and give the trial the name \"Batch 001\"\n\n"
-                + "  perfdmf_loadtrial -e 12 -n \"HPM data 01\" -f hpm perfhpm*\n"
-                + "    This will load perfhpm* files of type HPMToolkit into experiment 12 and give\n"
-                + "    the trial the name \"HPM data 01\"\n\n"
-                + "  perfdmf_loadtrial -a \"NPB2.3\" -x \"parametric\" -n \"64\" par64.ppk\n"
-                + "    This will load packed profile par64.ppk into the experiment named\n"
-                + "    \"parametric\" under the application named \"NPB2.3\" and give the trial\n"
-                + "    the name \"64\".  The application and experiment will be created if not found.\n");
+                + "    database config mydb and give the trial the name \"Batch 001\"\n\n"
+                + "  taudb_loadtrial -c mydb -n \"HPM data 01\" -f hpm perfhpm*\n"
+                + "    This will load perfhpm* files of type HPMToolkit into datatabase mydb and give\n"
+                + "    the trial the name \"HPM data 01\"\n\n");
     }
 
     public LoadTrial(String configFileName, String sourceFiles[]) {
@@ -80,6 +106,12 @@ public class LoadTrial {
         databaseAPI = new DatabaseAPI();
         try {
             databaseAPI.initialize(configFileName, true);
+			if (databaseAPI.db().getSchemaVersion() > 0) {
+				// copy the DatabaseAPI object data into a new TAUdbDatabaseAPI object
+				
+				databaseAPI = new TAUdbDatabaseAPI(databaseAPI);
+			}
+
         } catch (Exception e) {
             e.printStackTrace();
             System.exit(-1);
@@ -88,6 +120,12 @@ public class LoadTrial {
     }
 
     public boolean checkForExp(String expid, String appName, String expName) {
+    	if(databaseAPI.db().getSchemaVersion() >0){
+    		System.err.println("Ignore Exp in load trial");
+//            Experiment exp = databaseAPI.getExperiment(appName, expName, true);
+//            return exp != null;
+    		return true;
+    	}
         if (expid != null) {
             this.expID = Integer.parseInt(expid);
 
@@ -133,6 +171,7 @@ public class LoadTrial {
 
         try {
             dataSource = UtilFncs.initializeDataSource(files, fileType, fixNames);
+			DataSource.setMeanIncludeNulls(useNulls);
         } catch (DataSourceException e) {
             e.printStackTrace();
             System.err.println("Error: Unable to initialize datasource!");
@@ -145,6 +184,18 @@ public class LoadTrial {
         // set the metadata file name before loading the data, because
         // aggregateData() is called at the end of the dataSource.load()
         // and this file has to be set before then.
+        
+        if(metadataString!=null&&metadataString.length()>0){
+        	MetaDataMap mdMap = new MetaDataMap();
+        	String[] pairs = metadataString.split(":");
+        	for(int i=0;i<pairs.length;i++){
+        		String[] pair = pairs[i].split("=");
+        		if(pair.length==2)
+        			mdMap.put(pair[0], pair[1]);
+        	}
+        	dataSource.setMetaData(mdMap);
+        }
+        
         try {
             if (metadataFile != null) {
                 dataSource.setMetadataFile(metadataFile);
@@ -156,7 +207,14 @@ public class LoadTrial {
         }
 
         try {
+		    // generate all the statistics - unless we already have them!
+            if (databaseAPI.db().getSchemaVersion()>=0 && !(dataSource instanceof SnapshotDataSource)) {
+            	dataSource.setGenerateTAUdbStatistics(true);
+            }
             dataSource.load();
+            if (reducePercentage != null && reducePercentage > 0.0) {
+            	dataSource.reduceTrial(reducePercentage);
+            }
         } catch (Exception e) {
             System.err.println("Error Loading Trial:");
             e.printStackTrace();
@@ -178,6 +236,64 @@ public class LoadTrial {
 
     public void saveTrial() {
         trial.setName(trialName);
+        int appView=-2;
+        String NAME="NAME";
+        String ID="ID";
+        String appViewName="Application-"+this.appName;
+        List<ViewRule> rules = new ArrayList<ViewRule>(2);
+        boolean createAppView=true;
+        if (this.appName != null)
+        {
+        	//First see if this view already exists
+        	List<View>checkViews = View.getViews(0, this.databaseAPI.db());
+        	if(checkViews!=null){
+        		for(View topView:checkViews){
+        			String vname = topView.getField(NAME);
+        			if(vname.equals(appViewName)){
+        				createAppView=false;
+        				appView=Integer.parseInt(topView.getField(ID));
+        				break;
+        			}
+        		}
+        	}
+        	//If it doesn't then make it, otherwise use its id when making the experiment view
+        	if(createAppView){
+        	rules.add(ViewRule.createStringViewRule("Application", this.appName, StringViewComparator.EXACTLY));
+        	try {
+				appView=View.createView(this.databaseAPI.db(), appViewName, true, -1, rules);
+			} catch (SQLException e) {
+				e.printStackTrace();
+			}
+        	}
+        	trial.getMetaData().put("Application", this.appName);
+        }
+        if (this.expName != null)
+        {
+        	if(appView>=0)
+        	{
+        		String expViewName="Experiment-"+this.expName;
+        		boolean createExpView=true;
+        		List<View>checkViews = View.getViews(appView, this.databaseAPI.db());
+               	if(checkViews!=null){
+            		for(View expView:checkViews){
+            			String vname = expView.getField(NAME);
+            			if(vname.equals(expViewName)){
+            				createExpView=false;
+            				break;
+            			}
+            		}
+            	}
+        		if(createExpView){
+        		rules.add(ViewRule.createStringViewRule("Experiment", this.expName, StringViewComparator.EXACTLY));
+        		try {
+					View.createView(this.databaseAPI.db(), expViewName, true, appView, rules);
+				} catch (SQLException e) {
+					e.printStackTrace();
+				}
+        		}
+        	}
+        	trial.getMetaData().put("Experiment", this.expName);
+        }
 
         System.err.println("TrialName: " + trialName);
         trial.setExperimentID(expID);
@@ -259,10 +375,13 @@ public class LoadTrial {
         CmdLineParser.Option trialOpt = parser.addStringOption('t', "trialid");
         CmdLineParser.Option typeOpt = parser.addStringOption('f', "filetype");
         CmdLineParser.Option fixOpt = parser.addBooleanOption('i', "fixnames");
+        CmdLineParser.Option metadataFileOpt = parser.addStringOption('l', "metadataFile");
         CmdLineParser.Option metadataOpt = parser.addStringOption('m', "metadata");
         CmdLineParser.Option appNameOpt = parser.addStringOption('a', "applicationname");
         CmdLineParser.Option expNameOpt = parser.addStringOption('x', "experimentname");
         CmdLineParser.Option summaryOpt = parser.addBooleanOption('s', "summaryonly");
+        CmdLineParser.Option percentageOpt = parser.addDoubleOption('r', "reduce");
+        CmdLineParser.Option useNullOpt = parser.addBooleanOption('z', "usenull");
 
         try {
             parser.parse(args);
@@ -280,13 +399,16 @@ public class LoadTrial {
         String trialName = (String) parser.getOptionValue(nameOpt);
         String appName = (String) parser.getOptionValue(appNameOpt);
         String expName = (String) parser.getOptionValue(expNameOpt);
+        Double percentage = (Double) parser.getOptionValue(percentageOpt);
 
         //String problemFile = (String)parser.getOptionValue(problemOpt);
         String trialID = (String) parser.getOptionValue(trialOpt);
         String fileTypeString = (String) parser.getOptionValue(typeOpt);
         Boolean fixNames = (Boolean) parser.getOptionValue(fixOpt);
-        String metadataFile = (String) parser.getOptionValue(metadataOpt);
+        String metadataFile = (String) parser.getOptionValue(metadataFileOpt);
+        String metadataString = (String) parser.getOptionValue(metadataOpt);
         Boolean summaryOnly = (Boolean) parser.getOptionValue(summaryOpt);
+        Boolean useNull = (Boolean) parser.getOptionValue(useNullOpt);
 
         if (help != null && help.booleanValue()) {
             LoadTrial.outputHelp();
@@ -304,6 +426,7 @@ public class LoadTrial {
         String sourceFiles[] = parser.getRemainingArgs();
         
         boolean multippk=false;
+        LoadTrial tmpLT = new LoadTrial(configFile, sourceFiles);
         if (trialName == null) {
         	
         	if(sourceFiles!=null&&sourceFiles.length>0){
@@ -320,7 +443,8 @@ public class LoadTrial {
             LoadTrial.usage();
             System.exit(-1);
         	}
-        } else if (experimentID == null && expName == null) {
+			/*
+        } else if (experimentID == null && expName == null && tmpLT.getDatabaseAPI().getDb().getSchemaVersion()==0) {
             System.err.println("Error: Missing experiment id or name\n");
             LoadTrial.usage();
             System.exit(-1);
@@ -328,6 +452,7 @@ public class LoadTrial {
             System.err.println("Error: Missing application name\n");
             LoadTrial.usage();
             System.exit(-1);
+			*/
         }
 
         //String sourceFiles[] = parser.getRemainingArgs();
@@ -393,6 +518,12 @@ public class LoadTrial {
         if (summaryOnly == null) {
             summaryOnly = new Boolean(false);
         }
+        if (useNull == null) {
+            useNull = new Boolean(false);
+        }
+        if (percentage == null) {
+        	percentage = new Double(0.0);
+        }
         
         
         if(multippk){
@@ -413,14 +544,20 @@ public class LoadTrial {
             //trans.problemFile = problemFile;
             trans.fixNames = fixNames.booleanValue();
             trans.metadataFile = metadataFile;
+            trans.metadataString = metadataString;
             trans.summaryOnly = summaryOnly.booleanValue();
+            trans.reducePercentage = percentage;
             trans.loadTrial(fileType);
         	}
         }
         else
         {
         LoadTrial trans = new LoadTrial(configFile, sourceFiles);
-        trans.checkForExp(experimentID, appName, expName);
+        if (trans.databaseAPI.db().getSchemaVersion()==0) {
+            trans.checkForExp(experimentID, appName, expName);
+        } else {
+        	//System.err.println("Warning - this is the TAUdb schema, there is no Experiment table");
+        }
         if (trialID != null) {
             trans.checkForTrial(trialID);
             trans.trialID = Integer.parseInt(trialID);
@@ -430,7 +567,16 @@ public class LoadTrial {
         //trans.problemFile = problemFile;
         trans.fixNames = fixNames.booleanValue();
         trans.metadataFile = metadataFile;
+        trans.metadataString = metadataString;
         trans.summaryOnly = summaryOnly.booleanValue();
+        if (appName != null) {
+		  trans.appName = appName;
+		}
+        if (expName != null) {
+		  trans.expName = expName;
+		}
+        trans.useNulls = useNull.booleanValue();
+        trans.reducePercentage = percentage;
         trans.loadTrial(fileType);
         // the trial will be saved when the load is finished (update is called)
         }
@@ -439,6 +585,10 @@ public class LoadTrial {
     public void setMetadataFile(String metadataFile) {
         this.metadataFile = metadataFile;
     }
+    
+    public void setMetadataString(String metadataString) {
+        this.metadataString = metadataString;
+    }
 
     public void setFixNames(boolean fixNames) {
         this.fixNames = fixNames;
