@@ -1,6 +1,7 @@
 package edu.uoregon.tau.perfexplorer.client;
 
-import javax.swing.event.*;
+import javax.swing.event.ChangeEvent;
+import javax.swing.event.ChangeListener;
 
 public class PerfExplorerJTabbedPaneListener implements ChangeListener {
 
@@ -16,9 +17,12 @@ public class PerfExplorerJTabbedPaneListener implements ChangeListener {
 			PerformanceExplorerPane.getPane().updateImagePanel();
 		} else if (index == 2) {
 			PerfExplorerCorrelationPane.getPane().updateImagePanel();
-		} else {
+		} else if (index == 3){
 			ChartPane.getPane().refreshDynamicControls(true, true, true);
 			ChartPane.getPane().drawChart();
+		} else {
+			DeriveMetricsPane.getPane();//.refreshDynamicControls(true, true, true);
+			//DeriveMetricsPane.getPane().drawChart();
 		}
 	}
 
