@@ -15,8 +15,8 @@ import edu.uoregon.tau.perfdmf.Application;
 import edu.uoregon.tau.perfdmf.Experiment;
 import edu.uoregon.tau.perfdmf.IntervalEvent;
 import edu.uoregon.tau.perfdmf.Metric;
+import edu.uoregon.tau.perfdmf.View;
 import edu.uoregon.tau.perfdmf.Trial;
-import edu.uoregon.tau.perfexplorer.common.RMIView;
 
 
 public class PerfExplorerTreeExpansionListener implements TreeExpansionListener, TreeWillExpandListener {
@@ -28,6 +28,7 @@ public class PerfExplorerTreeExpansionListener implements TreeExpansionListener,
 	}
 
 	public void treeWillExpand (TreeExpansionEvent e) {
+
 		TreePath path = e.getPath();
 		if (path == null)
 			return;
@@ -45,10 +46,22 @@ public class PerfExplorerTreeExpansionListener implements TreeExpansionListener,
 		if (node.isRoot()) {
 			PerfExplorerJTree.refreshDatabases();
 		} else if (node.toString() != null && node.toString().startsWith("jdbc:")) {
-			PerfExplorerJTree.addApplicationNodes(node, false);
-			PerfExplorerJTree.getTree().addViewNode(node);
+			// get the schema version for the connection
+			int index = PerfExplorerJTree.getConnectionIndex(node);
+			int schemaVersion = PerfExplorerConnection.getConnection().getSchemaVersion(index);
+			if (object instanceof ConnectionNodeObject) {
+				ConnectionNodeObject tmp = (ConnectionNodeObject)object;
+				tmp.setString(PerfExplorerConnection.getConnection().getConnectionStrings().get(index));
+				PerfExplorerJTree.nodeChanged(node);
+			}
+            if (schemaVersion < 1) {
+            	PerfExplorerJTree.addApplicationNodes(node, false);
+            	PerfExplorerJTree.getTree().addViewNode(node);
+            } else {
+				PerfExplorerJTree.addTAUdbViewNodes (node, 0);            	
+            }
 		} else if (node.toString() != null && node.toString().equals("Views")) {
-			PerfExplorerJTree.addViewNodes(node, "0");
+			PerfExplorerJTree.addViewNodes(node, 0);
 		} else {
 			if (object instanceof Application) {
 				Application app = (Application)object;
@@ -76,9 +89,9 @@ public class PerfExplorerTreeExpansionListener implements TreeExpansionListener,
 				}
 			} else if (object instanceof IntervalEvent) {
 				// do nothing
-			} else if (object instanceof RMIView) {
-				RMIView view = (RMIView) object;
-				PerfExplorerJTree.addViewNodes(node, view.getField("id"));
+			} else if (object instanceof View) {
+				View view = (View) object;
+				PerfExplorerJTree.addViewNodes(node, view.getID());
 			} else {
 				System.out.println("unknown!");
 			}
