@@ -2,7 +2,7 @@
  * Created on Mar 16, 2005
  *
  */
-package clustering;
+package edu.uoregon.tau.perfexplorer.clustering;
 
 import java.util.List;
 
@@ -12,7 +12,7 @@ import java.util.List;
  * cluster output data, or PCA output data.
  * 
  * @author khuck
- * <P>CVS $Id: RawDataInterface.java,v 1.6 2007/01/04 21:20:01 khuck Exp $</P>
+ * <P>CVS $Id: RawDataInterface.java,v 1.9 2009/02/25 19:51:46 wspear Exp $</P>
  * @version 0.1
  * @since 0.1
  */
@@ -72,7 +72,7 @@ public interface RawDataInterface {
 	 *
      * @return
      */
-	public List getEventNames();
+	public List<String> getEventNames();
 	
 	/**
      * Returns the number of vectors in this data set.
@@ -142,4 +142,12 @@ public interface RawDataInterface {
      * @return
      */
 	public String getMainEventName();
+
+	/**
+	 * Sets an attribute in the instance, where the attribute is a string.
+	 * @param thread
+	 * @param i
+	 * @param name
+	 */
+	public void addValue(int vectorIndex, int dimensionIndex, String value);
 }
