@@ -11,7 +11,7 @@ int main (int argc, char** argv) {
    const char* home = getenv("HOME");
    printf("Found $HOME as: %s\n", home);
    char config_dir[256];
-   sprintf(config_dir, "%s/.ParaProf", home);
+   snprintf(config_dir, sizeof(config_dir),  "%s/.ParaProf", home);
    printf("Looking for config files in: %s\n", config_dir);
    // get the config files in the home directory
    DIR *dp = NULL;
@@ -22,7 +22,7 @@ int main (int argc, char** argv) {
      ep = readdir(dp);
      while (ep != NULL) {
 	   if (strncmp(ep->d_name, config_prefix, 11) == 0) {
-          sprintf(config_file, "%s/%s", config_dir, ep->d_name);
+          snprintf(config_file, sizeof(config_file),  "%s/%s", config_dir, ep->d_name);
           printf("Parsing config file %s...\n", config_file);
           TAUDB_CONFIGURATION* config = taudb_parse_config_file(config_file);
 		  printf ("Database: %s, %s, %s\n", config->db_hostname, config->db_portnum, config->db_dbname);
