@@ -14,7 +14,6 @@ import java.util.Observable;
 
 import javax.media.opengl.GL;
 import javax.media.opengl.GLAutoDrawable;
-import javax.media.opengl.GLAutoDrawable;
 import javax.media.opengl.glu.GLU;
 import javax.media.opengl.glu.GLUquadric;
 import javax.swing.BorderFactory;
@@ -60,6 +59,7 @@ public class ScatterPlot implements Plot {
     public ScatterPlot() {
     }
 
+    
     public void setSize(float xSize, float ySize, float zSize) {
         this.xSize = xSize;
         this.ySize = ySize;
@@ -73,13 +73,28 @@ public class ScatterPlot implements Plot {
         this.dirty = true;
     }
 
+//    public void clearValues(){
+//    	this.values=null;
+//    }
+//    float[][] origValues=null;
     /**
      * Sets the values.  The 2nd dimension must be of size 4 (one value for each axis).
      * @param values
      */
     public void setValues(float values[][]) {
         this.values = values;
+        
+//        if(isTopo)
+//        {
+//        	origValues=new float[values.length][];
+//        	for(int i=0;i<values.length;i++){
+//        		origValues[i]=values[i].clone();
+//        	}
+//        }
+//        else
+//        	origValues=null;
         processValues();
+//        isTopo=false;
         this.dirty = true;
     }
 
@@ -185,7 +200,7 @@ public class ScatterPlot implements Plot {
         gbc.weighty = 0.2;
 
         final JSlider sphereSizeSlider = new JSlider(0, 20, (int) (sphereSize * 10));
-        final JSlider sphereDetailSlider = new JSlider(3, 30, sphereDetail);
+        final JSlider sphereDetailSlider = new JSlider(2, 30, sphereDetail);
 
         sphereSizeSlider.addChangeListener(new ChangeListener() {
             public void stateChanged(ChangeEvent event) {
@@ -219,7 +234,7 @@ public class ScatterPlot implements Plot {
 
     public void render(VisRenderer visRenderer) {
         GLAutoDrawable glDrawable = visRenderer.getGLAutoDrawable();
-        Vec direction = visRenderer.getViewDirection();
+        //Vec direction = visRenderer.getViewDirection();
 
 //    public void render(GLAutoDrawable glDrawable, Vec direction) {
         if (axes != null) {
@@ -252,16 +267,68 @@ public class ScatterPlot implements Plot {
         for (int f = 0; f < 4; f++) {
             float maxValue = Float.MIN_VALUE;
             float minValue = Float.MAX_VALUE;
-
             for (int i = 0; i < values.length; i++) {
                 maxValue = Math.max(maxValue, values[i][f]);
                 minValue = Math.min(minValue, values[i][f]);
             }
+            
+//            if(maxValue!=tmpMax)
+//            	System.out.println("Bad Max: "+maxValue+" vs. "+tmpMax);
+//            if(minValue!=tmpMin)
+//            	System.out.println("Bad Min: "+minValue+" vs. "+tmpMin);
+
+//            if(isTopo&&f==3)
+//            {
+//            	if(minVis>0)
+//            	{
+//            		useMinCutoff=true;
+//            		float tmp = maxValue-minValue;
+//            		mincut=minValue+tmp*(minVis/100.0f);
+//            	}
+//            	else
+//            		mincut=minValue;
+//            	if(maxVis<100){
+//            		useMaxCutoff=true;
+//            		float tmp = maxValue-minValue;
+//            		maxcut=minValue+tmp*(maxVis/100.0f);
+//            	}
+//            	else
+//            		maxcut=maxValue;
+//            
+//
+//            }
 
             for (int i = 0; i < values.length; i++) {
                 if (maxValue - minValue == 0) {
                     values[i][f] = 0;
-                } else {
+                    continue;
+                } 
+//                
+//                if(useMinCutoff&&values[i][f]<mincut){
+//                	if(useMaxCutoff&&values[i][f]<maxcut&&mincut>maxcut){
+//                		
+//                	}
+//                	else
+//                	{
+//                		values[i][f]=Float.NaN;
+//                		continue;
+//                	}
+//                }
+//                else if(useMaxCutoff&&values[i][f]>maxcut){
+//                	if(useMinCutoff&&values[i][f]>mincut&&mincut>maxcut){
+//                		
+//                	}
+//                	else
+//                	{
+//                		values[i][f]=Float.NaN;
+//                		continue;
+//                	}
+//                }
+//                if(isTopo&&i>=0&&i<4){
+//                	continue;
+//                }
+//                else
+                {
                     if (normalized) {
                         values[i][f] = (values[i][f] - minValue) / (maxValue - minValue) * norms[f];
                     } else {
@@ -273,6 +340,35 @@ public class ScatterPlot implements Plot {
         }
     }
 
+
+    
+//    private boolean showCoord(int dex){
+//    	boolean use = checkCoord(dex);
+//    	if(use&&origValues!=null){
+//    		statCount++;
+//    		statAcc+=origValues[dex][3];
+//    		statMin=Math.min(origValues[dex][3], statMin);
+//    		statMax=Math.max(origValues[dex][3], statMax);
+//    	}
+//    	return use;
+//    }
+    
+//    private boolean checkCoord(int dex){
+//    	if(Float.compare(values[dex][3],Float.NaN)==0)
+//    		return false;
+//    	if(topoVis==null||origValues==null)
+//    		return true;
+//    	
+//    	//System.out.println(topoVis[0]+" "+topoVis[1]+" "+topoVis[2]+" vs "+origValues[dex][0]+" "+origValues[dex][1]+" "+origValues[dex][2]);
+//    	
+//    	for(int i=0;i<3;i++){
+//    		if(topoVis[i]!=-1&&topoVis[i]!=origValues[dex][i])
+//    			return false;
+//    	}
+//    	
+//    	return true;
+//    }
+    
     private void privateRender(GLAutoDrawable glDrawable) {
         if (values == null)
             return;
@@ -285,16 +381,23 @@ public class ScatterPlot implements Plot {
         // Set to red, in case there is no colorScale
         gl.glColor3f(1.0f, 0, 0);
 
-        if (sphereSize < 0.1f) {
+        if (sphereSize < 0.1f||sphereDetail<3) {
             gl.glDisable(GL.GL_LIGHTING);
-            gl.glPointSize(2.5f);
+            float actualSize=2.5f;
+            if(sphereSize>=0.1f){
+            	actualSize=sphereSize*30;
+            }
+            gl.glPointSize(actualSize);
             gl.glBegin(GL.GL_POINTS);
             for (int i = 0; i < values.length; i++) {
+            	//if(showCoord(i)){
                 if (colorScale != null) {
                     Color color = colorScale.getColor(values[i][3]);
                     gl.glColor3f(color.getRed() / 255.0f, color.getGreen() / 255.0f, color.getBlue() / 255.0f);
                 }
                 gl.glVertex3f(values[i][0], values[i][1], values[i][2]);
+            	//}
+
             }
             gl.glEnd();
         } else {
@@ -308,6 +411,7 @@ public class ScatterPlot implements Plot {
             glu.gluQuadricNormals(qobj, GLU.GLU_SMOOTH);
 
             for (int i = 0; i < values.length; i++) {
+            	//if(showCoord(i)){
                 gl.glPushMatrix();
                 gl.glTranslatef(values[i][0], values[i][1], values[i][2]);
                 if (colorScale != null) {
@@ -316,6 +420,7 @@ public class ScatterPlot implements Plot {
                 }
                 glu.gluSphere(qobj, sphereSize, sphereDetail, sphereDetail);
                 gl.glPopMatrix();
+            	//}
             }
         }
 
