@@ -93,6 +93,9 @@ public class Tau2Slog2
             slog_outs.setTreeLeafByteSize( leaf_bytesize );
         if ( num_children_per_node > 0 )
             slog_outs.setNumChildrenPerNode( num_children_per_node );
+        
+	//Bypasses error detected by newer java implementations
+        System.setProperty("java.util.Arrays.useLegacyMergeSort", "true");
 
         treetrunk = new TreeTrunk( slog_outs, shadefs );
         /* */    Date time2 = new Date();
@@ -100,7 +103,7 @@ public class Tau2Slog2
             if ( next_kind == Kind.TOPOLOGY ) {
                 topo = dobj_ins.getNextTopology();
                 objdef = Category.getShadowCategory( topo );
-                objdefs.put( new Integer( objdef.getIndex() ), objdef );
+                objdefs.put( Integer.valueOf(objdef.getIndex()), objdef );
                 shadefs.put( topo, objdef );
             }
             else if ( next_kind == Kind.YCOORDMAP ) {
@@ -109,7 +112,7 @@ public class Tau2Slog2
             }
             else if ( next_kind == Kind.CATEGORY ) {
                 objdef = dobj_ins.getNextCategory();
-                objdefs.put( new Integer(objdef.getIndex() ),objdef );
+                objdefs.put( Integer.valueOf(objdef.getIndex()), objdef );
             }
             else if ( next_kind == Kind.PRIMITIVE ) {
                 prime_obj = dobj_ins.getNextPrimitive();
